@@ -30,7 +30,7 @@
 
 #ifdef QT4
 
-#include <QDialog>
+#include <QtGui/QDialog>
 
 #else
 
