@@ -2,7 +2,7 @@
  * arp_and_chords_tab_widget.h - declaration of class arpAndChordWidget which
  *                               provides code for using arpeggio and chords
  *
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -31,7 +31,7 @@
 
 #ifdef QT4
 
-#include <QWidget>
+#include <QtGui/QWidget>
 
 #else
 
@@ -43,29 +43,31 @@
 #include <config.h>
 #endif
 
-#include "settings.h"
+#include "journalling_object.h"
 #include "types.h"
+#include "engine.h"
 
 
-class QComboBox;
 class QPixmap;
 
-class channelTrack;
+class automatableButtonGroup;
+class flpImport;
+class instrumentTrack;
+class comboBox;
 class groupBox;
 class knob;
 class notePlayHandle;
-class pixmapButton;
 class tempoSyncKnob;
 
 
 const int MAX_CHORD_POLYPHONY = 10;
 
 
-class arpAndChordsTabWidget : public QWidget, public settings
+class arpAndChordsTabWidget : public QWidget, public journallingObject
 {
 	Q_OBJECT
 public:
-	arpAndChordsTabWidget( channelTrack * _channel_track );
+	arpAndChordsTabWidget( instrumentTrack * _channel_track );
 	virtual ~arpAndChordsTabWidget();
 
 	static struct chord
@@ -95,14 +97,6 @@ public:
 	}
 
 
-
-protected slots:
-	void arpUpToggled( bool );
-	void arpDownToggled( bool );
-	void arpUpAndDownToggled( bool );
-	void arpRandomToggled( bool );
-
-
 private:
 
 	enum arpDirections
@@ -112,25 +106,33 @@ private:
 		DOWN,
 		UP_AND_DOWN,
 		RANDOM
-	} m_arpDirection;
+	} ;
+
+	enum arpModes
+	{
+		FREE,
+		SORT,
+		SYNC
+	} ;
 
 	// chord-stuff
 	groupBox * m_chordsGroupBox;
-	QComboBox * m_chordsComboBox;
+	comboBox * m_chordsComboBox;
 	knob * m_chordRangeKnob;
 
 	// arpeggio-stuff
 	groupBox * m_arpGroupBox;
-	QComboBox * m_arpComboBox;
+	comboBox * m_arpComboBox;
 	knob * m_arpRangeKnob;
 	tempoSyncKnob * m_arpTimeKnob;
 	knob * m_arpGateKnob;
 	QLabel * m_arpDirectionLbl;
 
-	pixmapButton * m_arpUpBtn;
-	pixmapButton * m_arpDownBtn;
-	pixmapButton * m_arpUpAndDownBtn;
-	pixmapButton * m_arpRandomBtn;
+	automatableButtonGroup * m_arpDirectionBtnGrp;
+
+	comboBox * m_arpModeComboBox;
+
+	friend class flpImport;
 
 } ;
 
