@@ -1,7 +1,7 @@
 /*
  * audio_dummy.h - dummy-audio-device
  *
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -27,19 +27,23 @@
 #define _AUDIO_DUMMY_H
 
 #include "audio_device.h"
+#include "micro_timer.h"
 
 
-class audioDummy : public audioDevice
+class audioDummy : public audioDevice, public QThread
 {
 public:
-	audioDummy( Uint32 _sample_rate, bool & _success_ful ) :
-		audioDevice( _sample_rate, DEFAULT_CHANNELS )
+	audioDummy( const sample_rate_t _sample_rate, bool & _success_ful,
+							mixer * _mixer ) :
+		audioDevice( _sample_rate, DEFAULT_CHANNELS, _mixer ),
+		m_quit( FALSE )
 	{
 		_success_ful = TRUE;
 	}
 
 	virtual ~audioDummy()
 	{
+		stopProcessing();
 	}
 
 	inline static QString name( void )
@@ -68,8 +72,42 @@ public:
 
 
 private:
-	// TODO: derive from QThread and call getNextBuffer() in an
-	// endless loop
+	virtual void startProcessing( void )
+	{
+		start();
+	}
+
+	virtual void stopProcessing( void )
+	{
+		if( isRunning() )
+		{
+			m_quit = TRUE;
+			wait( 1000 );
+			terminate();
+		}
+	}
+
+	virtual void run( void )
+	{
+		microTimer timer;
+		while( m_quit == FALSE )
+		{
+			timer.reset();
+			processNextBuffer();
+			const Sint32 microseconds = static_cast<Sint32>(
+					getMixer()->framesPerAudioBuffer() *
+					1000000.0f / getMixer()->sampleRate() -
+							timer.elapsed() );
+			if( microseconds > 0 )
+			{
+				usleep( microseconds );
+			}
+		}
+	}
+
+
+	volatile bool m_quit;
+
 } ;
 
 
