@@ -1,7 +1,7 @@
 /*
  * bb_editor.h - declaration of class bbEditor, a basic-component of LMMS
  *
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -27,32 +27,22 @@
 #define _BB_EDITOR_H
 
 #include "qt3support.h"
-
-
 #include "track_container.h"
-#include "lmms_main_win.h"
 
 
-class toolButton;
-class songEditor;
 class QPixmap;
 
+class comboBox;
+class toolButton;
+
 
 class bbEditor : public trackContainer
 {
 	Q_OBJECT
 public:
-	static inline bbEditor * inst( void )
-	{
-		if( s_instanceOfMe == NULL )
-		{
-			s_instanceOfMe = new bbEditor();
-		}
-		return( s_instanceOfMe );
-	}
-
-	virtual bool FASTCALL play( midiTime _start, Uint32 _start_frame,
-					Uint32 _frames, Uint32 _frame_base,
+	virtual bool FASTCALL play( midiTime _start, const f_cnt_t _start_frame,
+						const f_cnt_t _frames,
+						const f_cnt_t _frame_base,
 							Sint16 _tco_num = -1 );
 
 	virtual void FASTCALL saveSettings( QDomDocument & _doc,
@@ -71,49 +61,51 @@ public:
 	}
 
 	csize currentBB( void ) const;
-	void FASTCALL setCurrentBB( csize _bb );
-	tact FASTCALL lengthOfBB( csize _bb );
+	tact FASTCALL lengthOfBB( const csize _bb );
 	inline tact lengthOfCurrentBB( void )
 	{
 		return( lengthOfBB( currentBB() ) );
 	}
-	void FASTCALL removeBB( csize _bb );
+	void FASTCALL removeBB( const csize _bb );
 	csize numOfBBs( void ) const;
 
-	void FASTCALL swapBB( csize _bb1, csize _bb2 );
+	void FASTCALL swapBB( const csize _bb1, const csize _bb2 );
 
 	void updateBBTrack( trackContentObject * _tco );
 
 
+public slots:
+	void play( void );
+	void stop( void );
+	void updateComboBox( void );
+	void setCurrentBB( int _bb );
+
+
 protected:
 	virtual void closeEvent( QCloseEvent * _ce );
 	virtual void keyPressEvent( QKeyEvent * _ke );
 	virtual void resizeEvent( QResizeEvent * _re );
 
-
-protected slots:
-	void play( void );
-	void stop( void );
+	virtual QRect scrollAreaRect( void ) const;
 
 
 private:
-	bbEditor();
+	bbEditor( engine * _engine );
 	//bbEditor( const bbEditor & );
-	~bbEditor();
+	virtual ~bbEditor();
 
-	void FASTCALL createTCOsForBB( csize _bb );
+	void FASTCALL createTCOsForBB( const csize _bb );
 
 
-	static bbEditor * s_instanceOfMe;
-
 	QWidget * m_toolBar;
 
 	toolButton * m_playButton;
 	toolButton * m_stopButton;
 
+	comboBox * m_bbComboBox;
 
-	friend class songEditor;
-	friend lmmsMainWin::~lmmsMainWin();
+
+	friend class engine;
 
 } ;
 
