@@ -2,7 +2,7 @@
  * bb_track.h - class bbTrack, a wrapper for using bbEditor
  *              (which is a singleton-class) as track
  *
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -31,8 +31,8 @@
 
 #ifdef QT4
 
-#include <QObject>
-#include <QMap>
+#include <QtCore/QObject>
+#include <QtCore/QMap>
 
 #else
 
@@ -55,9 +55,6 @@ public:
 	bbTCO( track * _track, const QColor & _c = QColor() );
 	virtual ~bbTCO();
 
-	virtual void FASTCALL movePosition( const midiTime & _pos );
-	virtual void FASTCALL changeLength( const midiTime & _length );
-
 	virtual void FASTCALL saveSettings( QDomDocument & _doc,
 							QDomElement & _parent );
 	virtual void FASTCALL loadSettings( const QDomElement & _this );
@@ -103,8 +100,9 @@ public:
 
 	virtual trackTypes type( void ) const;
 	virtual bool FASTCALL play( const midiTime & _start,
-					Uint32 _start_frame, Uint32 _frames,
-					Uint32 _frame_base,
+					const f_cnt_t _start_frame,
+					const fpab_t _frames,
+					const f_cnt_t _frame_base,
 							Sint16 _tco_num = -1 );
 	virtual trackContentObject * FASTCALL createTCO( const midiTime &
 									_pos );
@@ -114,7 +112,7 @@ public:
 	virtual void FASTCALL loadTrackSpecificSettings( const QDomElement &
 									_this );
 
-	static bbTrack * FASTCALL findBBTrack( csize _bb_num );
+	static bbTrack * FASTCALL findBBTrack( csize _bb_num, engine * _engine );
 	static csize FASTCALL numOfBBTrack( track * _track );
 	static void FASTCALL swapBBTracks( track * _track1, track * _track2 );
 
@@ -138,12 +136,8 @@ protected:
 private:
 	nameLabel * m_trackLabel;
 
-	struct bbInfoStruct
-	{
-		csize num;
-		QString name;
-	} ;
-	static QMap<bbTrack *, bbInfoStruct> s_bbNums;
+	typedef QMap<bbTrack *, csize> infoMap;
+	static infoMap s_infoMap;
 
 } ;
 
