@@ -1,7 +1,7 @@
 /*
  * config_mgr.h - class configManager, a class for managing LMMS-configuration
  *
- * Copyright (c) 2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2005-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -36,10 +36,10 @@
 
 #ifdef QT4
 
-#include <QDialog>
-#include <QMap>
-#include <QVector>
-#include <QPair>
+#include <QtGui/QDialog>
+#include <QtCore/QMap>
+#include <QtCore/QVector>
+#include <QtCore/QPair>
 
 #else
 
@@ -51,9 +51,6 @@
 #endif
 
 
-#include "lmms_main_win.h"
-
-
 class QLineEdit;
 class QLabel;
 class QRadioButton;
@@ -61,11 +58,14 @@ class QHBoxLayout;
 class QVBoxLayout;
 class QFrame;
 
+class engine;
+
 
 const QString PROJECTS_PATH = "projects/";
 const QString PRESETS_PATH = "presets/";
 const QString SAMPLES_PATH = "samples/";
-const QString ARTWORK_PATH = "artwork/";
+const QString DEFAULT_THEME_PATH = "themes/default/";
+const QString TRACK_ICON_PATH = "track_icons/";
 const QString LOCALE_PATH = "locale/";
 
 
@@ -82,35 +82,80 @@ public:
 		return( s_instanceOfMe );
 	}
 
-	const QString & lmmsWorkingDir( void ) const
+	const QString & dataDir( void ) const
 	{
-		return( m_lmmsWorkingDir );
-	}
-	QString projectsDir( void ) const
-	{
-		return( m_lmmsWorkingDir + PROJECTS_PATH );
-	}
-	QString presetsDir( void ) const
-	{
-		return( m_lmmsWorkingDir + PRESETS_PATH );
-	}
-	QString samplesDir( void ) const
-	{
-		return( m_lmmsWorkingDir + SAMPLES_PATH );
-	}
-	QString artworkDir( void ) const
-	{
-		return( m_lmmsDataDir + ARTWORK_PATH );
-	}
-	QString localeDir( void ) const
-	{
-		return( m_lmmsDataDir + LOCALE_PATH );
-	}
-	const QString & pluginDir( void ) const
-	{
-		return( m_lmmsPluginDir );
+		return( m_dataDir );
 	}
 
+	const QString & workingDir( void ) const
+	{
+		return( m_workingDir );
+	}
+
+	QString userProjectsDir( void ) const
+	{
+		return( workingDir() + PROJECTS_PATH );
+	}
+
+	QString userPresetsDir( void ) const
+	{
+		return( workingDir() + PRESETS_PATH );
+	}
+
+	QString userSamplesDir( void ) const
+	{
+		return( workingDir() + SAMPLES_PATH );
+	}
+
+	QString factoryProjectsDir( void ) const
+	{
+		return( dataDir() + PROJECTS_PATH );
+	}
+
+	QString factoryPresetsDir( void ) const
+	{
+		return( dataDir() + PRESETS_PATH );
+	}
+
+	QString factorySamplesDir( void ) const
+	{
+		return( dataDir() + SAMPLES_PATH );
+	}
+
+	QString defaultArtworkDir( void ) const
+	{
+		return( m_dataDir + DEFAULT_THEME_PATH );
+	}
+
+	QString artworkDir( void ) const
+	{
+		return( m_artworkDir );
+	}
+
+	QString trackIconsDir( void ) const
+	{
+		return( m_dataDir + TRACK_ICON_PATH );
+	}
+
+	QString localeDir( void ) const
+	{
+		return( m_dataDir + LOCALE_PATH );
+	}
+
+	const QString & pluginDir( void ) const
+	{
+		return( m_pluginDir );
+	}
+
+	const QString & vstDir( void ) const
+	{
+		return( m_vstDir );
+	}
+
+	const QString & flDir( void ) const
+	{
+		return( m_flDir );
+	}
 
 	const QString & value( const QString & _class,
 					const QString & _attribute ) const;
@@ -121,9 +166,15 @@ public:
 	void saveConfigFile( void );
 
 
+public slots:
+	void setWorkingDir( const QString & _wd );
+	void setVSTDir( const QString & _vd );
+	void setArtworkDir( const QString & _ad );
+	void setFLDir( const QString & _fd );
+
+
 protected slots:
 	void openWorkingDir( void );
-	void setWorkingDir( const QString & _wd );
 
 	virtual void accept( void );
 
@@ -145,16 +196,19 @@ private:
 
 	void FASTCALL addPage( QWidget * _w, const QString & _title );
 
-	static void processFilesRecursively( const QString & _src_dir,
+/*	static void processFilesRecursively( const QString & _src_dir,
 						const QString & _dst_dir,
 		void( * _proc_func )( const QString & _src, const QString &
-								_dst ) );
+								_dst ) );*/
 
 
 	const QString m_lmmsRcFile;
-	QString m_lmmsWorkingDir;
-	QString m_lmmsDataDir;
-	QString m_lmmsPluginDir;
+	QString m_workingDir;
+	QString m_dataDir;
+	QString m_artworkDir;
+	QString m_pluginDir;
+	QString m_vstDir;
+	QString m_flDir;
 
 	typedef vvector<QPair<QString, QString> > stringPairVector;
 	typedef QMap<QString, stringPairVector> settingsMap;
@@ -186,8 +240,7 @@ private:
 	QVBoxLayout * m_contentLayout;
 
 
-	// lmmsMainWin-dtor is allowed to destroy us...
-	friend lmmsMainWin::~lmmsMainWin();
+	friend class engine;
 
 } ;
 
