@@ -3,7 +3,7 @@
  *                             is used by envelope/lfo/filter-tab of
  *                              channel-window
  *
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -36,16 +36,20 @@
 
 #ifdef QT4
 
-#include <QWidget>
+#include <QtGui/QWidget>
+#include <QtCore/QMap>
+#include <QtCore/QVector>
 
 #else
 
 #include <qwidget.h>
+#include <qmap.h>
+#include <qvaluevector.h>
 
 #endif
 
 
-#include "settings.h"
+#include "journalling_object.h"
 #include "types.h"
 #include "spc_bg_hndl_widget.h"
 #include "sample_buffer.h"
@@ -54,31 +58,38 @@
 class QPaintEvent;
 class QPixmap;
 
+class automatableButtonGroup;
 class envelopeTabWidget;
 class knob;
 class ledCheckBox;
 class pixmapButton;
 class tempoSyncKnob;
+class track;
 
 
+class flpImport;
 
-class envelopeAndLFOWidget : public QWidget, public settings,
+
+class envelopeAndLFOWidget : public QWidget, public journallingObject,
 				public specialBgHandlingWidget
 {
 	Q_OBJECT
 public:
-	envelopeAndLFOWidget( float _value_for_zero_amount, QWidget * _parent );
-	~envelopeAndLFOWidget();
+	envelopeAndLFOWidget( float _value_for_zero_amount, QWidget * _parent,
+					engine * _engine, track * _track );
+	virtual ~envelopeAndLFOWidget();
 
 	static inline float expKnobVal( float val )
 	{
 		return( ( ( val < 0 ) ? -1 : 1 ) * val*val );
 	}
-	static void triggerLFO( void );
-	static void resetLFO( void );
 
-	float FASTCALL level( Uint32 _frame, Uint32 _release_begin,
-						Uint32 _frame_offset ) const;
+	static void triggerLFO( engine * _engine );
+	static void resetLFO( engine * _engine );
+
+	float FASTCALL level( f_cnt_t _frame,
+				const f_cnt_t _release_begin,
+				const f_cnt_t _frame_offset ) const;
 
 	inline bool used( void ) const
 	{
@@ -105,15 +116,13 @@ protected:
 	virtual void mousePressEvent( QMouseEvent * _me );
 	virtual void paintEvent( QPaintEvent * _pe );
 
-	float FASTCALL lfoLevel( Uint32 _frame, Uint32 _frame_offset ) const;
+	float FASTCALL lfoLevel( f_cnt_t _frame,
+					const f_cnt_t _frame_offset ) const;
 
 
 protected slots:
 	void updateAfterKnobChange( float );
-	void lfoSinWaveCh( bool );
-	void lfoTriangleWaveCh( bool );
-	void lfoSawWaveCh( bool );
-	void lfoSquareWaveCh( bool );
+	void lfoWaveCh( int );
 	void lfoUserWaveCh( bool );
 	void x100Toggled( bool );
 
@@ -122,10 +131,11 @@ private:
 	static QPixmap * s_envGraph;
 	static QPixmap * s_lfoGraph;
 
-	static Uint32 s_lfoFrame;
+	static QMap<engine *, vvector<envelopeAndLFOWidget *> > s_EaLWidgets;
 
 	bool   m_used;
 
+
 	// envelope-stuff
 	knob * m_predelayKnob;
 	knob * m_attackKnob;
@@ -139,31 +149,29 @@ private:
 	float  m_amount;
 	float  m_valueForZeroAmount;
 	float  m_amountAdd;
-	Uint32 m_pahdFrames;
-	Uint32 m_rFrames;
-	float * m_pahdEnv;
-	float * m_rEnv;
+	f_cnt_t m_pahdFrames;
+	f_cnt_t m_rFrames;
+	sample_t * m_pahdEnv;
+	sample_t * m_rEnv;
 
 	// LFO-stuff
 	knob * m_lfoPredelayKnob;
 	knob * m_lfoAttackKnob;
 	tempoSyncKnob * m_lfoSpeedKnob;
 	knob * m_lfoAmountKnob;
-	pixmapButton * m_sinLfoBtn;
-	pixmapButton * m_triangleLfoBtn;
-	pixmapButton * m_sawLfoBtn;
-	pixmapButton * m_sqrLfoBtn;
-	pixmapButton * m_usrLfoBtn;
+	pixmapButton * m_userLfoBtn;
+	automatableButtonGroup * m_lfoWaveBtnGrp;
 
 	ledCheckBox * m_x100Cb;
 	ledCheckBox * m_controlEnvAmountCb;
 
-	Uint32 m_lfoPredelayFrames;
-	Uint32 m_lfoAttackFrames;
-	Uint32 m_lfoOscillationFrames;
+	f_cnt_t m_lfoPredelayFrames;
+	f_cnt_t m_lfoAttackFrames;
+	f_cnt_t m_lfoOscillationFrames;
+	f_cnt_t m_lfoFrame;
 	float m_lfoAmount;
 	bool m_lfoAmountIsZero;
-	float * m_lfoShapeData;
+	sample_t * m_lfoShapeData;
 	sampleBuffer m_userWave;
 
 	enum lfoShapes
@@ -180,6 +188,7 @@ private:
 
 
 	friend class envelopeTabWidget;
+	friend class flpImport;
 
 } ;
 
