@@ -3,7 +3,7 @@
  *                         provides UI- and DSP-code for using envelopes, LFOs
  *                         and a filter
  *
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -32,7 +32,7 @@
 
 #ifdef QT4
 
-#include <QWidget>
+#include <QtGui/QWidget>
 
 #else
 
@@ -41,29 +41,32 @@
 #endif
 
 
-#include "settings.h"
 #include "basic_filters.h"
 #include "envelope_and_lfo_widget.h"
 
 
-class QComboBox;
-class groupBox;
-class tabWidget;
 class QLabel;
+
+class instrumentTrack;
+class comboBox;
+class groupBox;
 class knob;
-class pixmapButton;
-class channelTrack;
 class notePlayHandle;
+class pixmapButton;
+class tabWidget;
+
+class flpImport;
 
 
-class envelopeTabWidget : public QWidget, public settings
+class envelopeTabWidget : public QWidget, public journallingObject
 {
 	Q_OBJECT
 public:
-	envelopeTabWidget( channelTrack * _channel_track );
+	envelopeTabWidget( instrumentTrack * _channel_track );
 	virtual ~envelopeTabWidget();
 
-	void FASTCALL processAudioBuffer( sampleFrame * _ab, Uint32 _frames,
+	void FASTCALL processAudioBuffer( sampleFrame * _ab,
+							const fpab_t _frames,
 							notePlayHandle * _n );
 
 	enum targets
@@ -76,10 +79,10 @@ public:
 		TARGET_COUNT
 	} ;
 
-	Uint32 envFrames( void );
-	Uint32 releaseFrames( void );
+	f_cnt_t envFrames( const bool _only_vol = FALSE );
+	f_cnt_t releaseFrames( const bool _only_vol = FALSE );
 
-	float FASTCALL volumeLevel( notePlayHandle * _n, Uint32 _frame );
+	float FASTCALL volumeLevel( notePlayHandle * _n, const f_cnt_t _frame );
 
 
 	virtual void FASTCALL saveSettings( QDomDocument & _doc,
@@ -97,10 +100,12 @@ private:
 
 	// filter-stuff
 	groupBox * m_filterGroupBox;
-	QComboBox * m_filterComboBox;
+	comboBox * m_filterComboBox;
 	knob * m_filterCutKnob;
 	knob * m_filterResKnob;
 
+	friend class flpImport;
+
 } ;
 
 #endif
