@@ -1,7 +1,7 @@
 /*
  * group_box.h - LMMS-groupbox
  *
- * Copyright (c) 2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2005-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -30,7 +30,7 @@
 
 #ifdef QT4
 
-#include <QWidget>
+#include <QtGui/QWidget>
 
 #else
 
@@ -40,23 +40,21 @@
 
 #include "pixmap_button.h"
 
+
 class QPixmap;
 
 
-class groupBox : public QWidget
+class groupBox : public QWidget, public engineObject
 {
 	Q_OBJECT
 public:
-	groupBox( const QString & _caption, QWidget * _parent );
-	~groupBox();
+	groupBox( const QString & _caption, QWidget * _parent,
+					engine * _engine, track * _track );
+	virtual ~groupBox();
 
 	bool isActive( void ) const
 	{
-#ifdef QT4
 		return( m_led->isChecked() );
-#else
-		return( m_led->isOn() );
-#endif
 	}
 
 
