@@ -1,8 +1,8 @@
 /***************************************************************************
                           kmultitabbar.h -  description
                              -------------------
-    begin              :  2001
-    copyright          : (C) 2001,2002,2003 by Joseph Wenninger <jowenn@kde.org>
+    begin                :  2001
+    copyright            : (C) 2001,2002,2003 by Joseph Wenninger <jowenn@kde.org>
  ***************************************************************************/
 
 /***************************************************************************
@@ -18,50 +18,24 @@
 
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
  ***************************************************************************/
 
 #ifndef _KMultitabbar_h_
 #define _KMultitabbar_h_
 
-#include "qt3support.h"
-
-class KMultiTabBarButton;
-class KMultiTabBarTab;
-
-#ifdef QT4
-
-#include <QWidget>
-#include <QMenu>
-#include <QString>
-#include <QBoxLayout>
-#include <QList>
-#include <QScrollArea>
-#include <QPushButton>
-
-typedef QList<KMultiTabBarButton *> buttonList;
-typedef QList<KMultiTabBarTab *> tabList;
-
-#else
-
-#include <qscrollview.h>
-#include <qvbox.h>
-#include <qhbox.h>
-#include <qabstractlayout.h>
-#include <qstring.h>
-#include <qptrlist.h>
-#include <qpushbutton.h>
-
-typedef QPtrList<KMultiTabBarButton> buttonList;
-typedef QPtrList<KMultiTabBarTab> tabList;
-
-#endif
+#ifndef QT3
 
+#include <QtCore/QString>
+#include <QtCore/QList>
+#include <QtGui/QLayout>
+#include <QtGui/QPushButton>
 
 class QPixmap;
 class QPainter;
 class QFrame;
+class QMenu;
 
 class KMultiTabBarPrivate;
 class KMultiTabBarTabPrivate;
@@ -79,18 +53,18 @@ class KMultiTabBar: public QWidget
 {
 	Q_OBJECT
 public:
-	//enum KMultiTabBarMode{ Horizontal, Vertical };
-	enum KMultiTabBarPosition{DockLeft, DockRight, DockTop, DockBottom};
+       enum KMultiTabBarMode{Horizontal, Vertical};
+       enum KMultiTabBarPosition{Left, Right, Top, Bottom};
 
 	/**
-	 * VSNET == Visual Studio .Net like (only show the text of active tabs
-	 * KDEV3 == Kdevelop 3 like (always show the text)
-	 * KONQSBC == konqy's classic sidebar style (unthemed), this one is disabled
-	 * 	at the moment, but will be renabled soon too
+	 * The list of available styles for KMultiTabBar
+	 *   - VSNET - Visual Studio .Net like (only show the text of active tabs
+	 *   - KDEV3 - Kdevelop 3 like (always show the text)
+	 *   - KONQSBC - konqy's classic sidebar style (unthemed) (currently disabled)
 	 */
 	enum KMultiTabBarStyle{VSNET=0, KDEV3=1, KONQSBC=2, KDEV3ICON=3,STYLELAST=0xffff};
 
-	KMultiTabBar(Qt::Orientation o,QWidget *parent=0);
+        KMultiTabBar(KMultiTabBarMode bm,QWidget *parent=0 );
 	virtual ~KMultiTabBar();
 
 	/**
@@ -102,8 +76,8 @@ public:
 	 * @param popup A popup menu which should be displayed if the button is clicked
 	 * @param not_used_yet will be used for a popup text in the future
 	 */
- 	int appendButton(const QPixmap &pic,int id=-1,QMenu* popup=0,const QString& not_used_yet=QString::null);
-	/** 
+ 	int appendButton(const QPixmap &pic,int id=-1,QMenu* popup=0,const QString& not_used_yet=QString());
+	/**
          * remove a button with the given ID
 	 */
 	void removeButton(int id);
@@ -113,7 +87,7 @@ public:
 	 * @param id an arbitrary ID which can be used later on to identify the tab
 	 * @param text if a mode with text is used it will be the tab text, otherwise a mouse over hint
 	 */
-	int appendTab(const QPixmap &pic,int id=-1,const QString& text=QString::null);
+	int appendTab(const QPixmap &pic,int id=-1,const QString& text=QString());
 	/**
 	 * remove a tab with a given ID
 	 */
@@ -142,18 +116,28 @@ public:
 	 * @param pos if the mode is horizontal, only use top, bottom, if it is vertical use left or right
 	 */
 	void setPosition(KMultiTabBarPosition pos);
+	/**
+	 * get the tabbar position.
+	 * @return position
+	 */
+	KMultiTabBarPosition position() const;
 	/**
 	 * set the display style of the tabs
 	 */
 	void setStyle(KMultiTabBarStyle style);
+	/**
+	 * get the display style of the tabs
+	 * @return display style
+	 */
+	KMultiTabBarStyle tabStyle() const;
 	/**
 	 * be carefull, don't delete tabs yourself and don't delete the list itself
 	 */
-        tabList * tabs();
+        QList<KMultiTabBarTab *>* tabs();
 	/**
 	 * be carefull, don't delete buttons yourself and don't delete the list itself
 	 */
-	buttonList * buttons();
+	QList<KMultiTabBarButton *>* buttons();
 
 	/**
 	 * might vanish, not sure yet
@@ -167,7 +151,7 @@ private:
 	class KMultiTabBarInternal *m_internal;
 	QBoxLayout *m_l;
 	QFrame *m_btnTabSep;
-	buttonList m_buttons;
+	QList<KMultiTabBarButton *> m_buttons;
 	KMultiTabBarPosition m_position;
 	KMultiTabBarPrivate *d;
 };
@@ -245,12 +229,6 @@ public:
 	 */
 	void showActiveTabText(bool show);
 	void resize(){ setSize( neededSize() ); }
-#ifndef QT4
-	virtual inline const QColorGroup & palette( void ) const
-	{
-		return( colorGroup() );
-	}
-#endif 
 private:
 	bool m_showActiveTabText;
 	int m_expandedSize;
@@ -260,6 +238,7 @@ protected:
 	void setSize(int);
 	int neededSize();
 	void updateState();
+	virtual void paintEvent(QPaintEvent *);
 	virtual void drawButton(QPainter *);
 	virtual void drawButtonLabel(QPainter *);
 	void drawButtonStyled(QPainter *);
@@ -272,48 +251,66 @@ public slots:
 	virtual void setIcon(const QString&);
 	virtual void setIcon(const QPixmap&);
 };
+/***************************************************************************
+                          kmultitabbar_p.h -  description
+                             -------------------
+    begin                :  2003
+    copyright            : (C) 2003 by Joseph Wenninger <jowenn@kde.org>
+ ***************************************************************************/
+
+/***************************************************************************
+    This library is free software; you can redistribute it and/or
+    modify it under the terms of the GNU Library General Public
+    License as published by the Free Software Foundation; either
+    version 2 of the License, or (at your option) any later version.
+
+    This library is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+    Library General Public License for more details.
+
+    You should have received a copy of the GNU Library General Public License
+    along with this library; see the file COPYING.LIB.  If not, write to
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
+ ***************************************************************************/
 
 
-
-class KMultiTabBarInternal: public Q3ScrollView
+class KMultiTabBarInternal: public QWidget
 {
         Q_OBJECT
 public:
-        KMultiTabBarInternal(QWidget *parent,Qt::Orientation o);
-        int appendTab(const QPixmap &,int=-1,const QString& =QString::null);
+        KMultiTabBarInternal(QWidget *parent,KMultiTabBar::KMultiTabBarMode bm);
+        int appendTab(const QPixmap &,int=-1,const QString& =QString());
         KMultiTabBarTab *tab(int) const;
         void removeTab(int);
         void setPosition(enum KMultiTabBar::KMultiTabBarPosition pos);
         void setStyle(enum KMultiTabBar::KMultiTabBarStyle style);
         void showActiveTabTexts(bool show);
-        tabList * tabs(){return &m_tabs;}
-#ifndef QT4
-	virtual inline const QColorGroup & palette( void ) const
-	{
-		return( colorGroup() );
-	}
-#endif 
+        QList<KMultiTabBarTab *>* tabs(){return &m_tabs;}
 private:
         friend class KMultiTabBar;
         QWidget *box;
 	QBoxLayout *mainLayout;
-        tabList m_tabs;
+        QList<KMultiTabBarTab *> m_tabs;
         enum KMultiTabBar::KMultiTabBarPosition m_position;
         bool m_showActiveTabTexts;
         enum  KMultiTabBar::KMultiTabBarStyle m_style;
 	int m_expandedTabSize;
 	int m_lines;
+	KMultiTabBar::KMultiTabBarMode m_barMode;
 protected:
-        virtual void drawContents ( QPainter *, int, int, int, int);
+	virtual bool eventFilter(QObject *,QEvent*);
+//        virtual void paintEvent( QPaintEvent * );
 
-        /**
-         * [contentsM|m]ousePressEvent are reimplemented from QScrollView
-         * in order to ignore all mouseEvents on the viewport, so that the
-         * parent can handle them.
-         */
-        virtual void contentsMousePressEvent(QMouseEvent *);
-        virtual void mousePressEvent(QMouseEvent *);
 	virtual void resizeEvent(QResizeEvent *);
 };
 
+
+#else
+
+#include "kmultitabbar-qt3.h"
+
+#endif
+
 #endif
