@@ -4,7 +4,7 @@
  * This file is based on the knob-widget of the Qwt Widget Library by
  * Josef Wilgen
  *
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -33,8 +33,8 @@
 
 #ifdef QT4
 
-#include <QWidget>
-#include <QPoint>
+#include <QtGui/QWidget>
+#include <QtCore/QPoint>
 
 #else
 
@@ -44,22 +44,27 @@
 #endif
 
 
+#include "engine.h"
+#include "automatable_object.h"
+
+
 class QPixmap;
 class textFloat;
 
 
 enum knobTypes
 {
-	knobDark_28, knobBright_26, knobSmall_17
+	knobDark_28, knobBright_26, knobSmall_17, knobGreen_17
 } ;
 
 
 
-class knob : public QWidget
+class knob : public QWidget, public automatableObject<float>
 {
 	Q_OBJECT
 public:
-	knob( int _knob_num, QWidget * _parent, const QString & _name );
+	knob( int _knob_num, QWidget * _parent, const QString & _name,
+					engine * _engine, track * _track );
 	virtual ~knob();
 
     
@@ -69,50 +74,34 @@ public:
 
 	void setTotalAngle( float _angle );
 
-	void setRange( float _vmin, float _vmax, float _vstep = 0.0,
-							int _pagesize = 1 );
-
-	inline float value( void ) const
+	inline virtual void setInitValue( const float _val )
 	{
-		return( m_value );
+		m_initValue = _val;
+		autoObj::setInitValue( _val );
 	}
 
-	void setStep( float );
-
-	inline float maxValue( void ) const
-	{
-		return( m_maxValue );
-	}
-	inline float minValue( void ) const
-	{
-		return( m_minValue );
-	}
-
-	inline void incPages( int _n_pages )
-	{
-		setNewValue( m_value + float( _n_pages ) * m_pageSize, 1 );
-	}
+	virtual void setValue( const float _x );
 
+	virtual void setRange( const float _min, const float _max,
+						const float _step = 0.0 );
 
 
 public slots:
-	void setValue( float _val, bool _is_init_value = FALSE );
-	void fitValue( float _val );
-	void incValue( int _steps );
 	void reset( void );
 	void copyValue( void );
 	void pasteValue( void );
-	void enterValue( void );
+	virtual void enterValue( void );
 	void connectToMidiDevice( void );
 	void displayHelp( void );
 
 
 signals:
 	void valueChanged( float value );
+	void valueChanged( const QVariant & _data );
 	void sliderPressed( void );
 	void sliderReleased( void );
 	void sliderMoved( float value );
-    
+
 
 protected:
 	virtual void contextMenuEvent( QContextMenuEvent * _me );
@@ -144,7 +133,6 @@ protected:
 
 	void buttonReleased( void );
 
-	void setNewValue( float _x, bool _align = FALSE );
 
 
 	static float s_copiedValue;
@@ -156,9 +144,11 @@ protected:
 	bool m_buttonPressed;
 
 
+	float m_pageSize;
 	float m_angle;
 	float m_totalAngle;
 
+
 	QPixmap * m_knobPixmap;
 	int m_knobNum;
 	QString m_hintTextBeforeValue;
@@ -166,14 +156,6 @@ protected:
 	QString m_label;
 
 
-	float m_minValue;
-	float m_maxValue;
-	float m_step;
-	float m_pageSize;
-	float m_value;
-	float m_exactValue;
-	float m_exactPrevValue;
-	float m_prevValue;
 	float m_initValue;
 
 } ;
