@@ -1,7 +1,7 @@
 /*
  * midi_dummy.h - dummy MIDI-driver
  *
- * Copyright (c) 2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2005-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -35,11 +35,11 @@
 class midiDummy : public midiClientRaw
 {
 public:
-	midiDummy() :
-		midiClientRaw()
+	midiDummy( engine * _engine ) :
+		midiClientRaw( _engine )
 	{
 	}
-	~midiDummy()
+	virtual ~midiDummy()
 	{
 	}
 
