@@ -1,7 +1,7 @@
 /*
  * mixer.h - audio-device-independent mixer for LMMS
  *
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -34,8 +34,8 @@
 
 #ifdef QT4
 
-#include <QMutex>
-#include <QVector>
+#include <QtCore/QMutex>
+#include <QtCore/QVector>
 
 #else
 
@@ -52,35 +52,39 @@
 #include "note.h"
 #include "play_handle.h"
 #include "effect_board.h"
+#include "engine.h"
 
 
 class audioDevice;
 class midiClient;
-class lmmsMainWin;
-class plugin;
 class audioPort;
 
 
-const int DEFAULT_BUFFER_SIZE = 512;
+const fpab_t DEFAULT_BUFFER_SIZE = 512;
 
-const Uint8  DEFAULT_CHANNELS = 2;
+const ch_cnt_t DEFAULT_CHANNELS = 2;
 
-const Uint8  SURROUND_CHANNELS =
+const ch_cnt_t SURROUND_CHANNELS =
 #ifndef DISABLE_SURROUND
 				4;
 #else
 				2;
 #endif
 
-const Uint8  QUALITY_LEVELS = 2;
-const Uint32 DEFAULT_QUALITY_LEVEL = 0;
-const Uint32 HIGH_QUALITY_LEVEL = DEFAULT_QUALITY_LEVEL+1;
-extern Uint32 SAMPLE_RATES[QUALITY_LEVELS];
-const Uint32 DEFAULT_SAMPLE_RATE = 44100;
+
+enum qualityLevels
+{
+	DEFAULT_QUALITY_LEVEL,
+	HIGH_QUALITY_LEVEL,
+	QUALITY_LEVELS
+} ;
+
+extern sample_rate_t SAMPLE_RATES[QUALITY_LEVELS];
+const sample_rate_t DEFAULT_SAMPLE_RATE = 44100;
 
 
-typedef sampleType sampleFrame[DEFAULT_CHANNELS];
-typedef sampleType surroundSampleFrame[SURROUND_CHANNELS];
+typedef sample_t sampleFrame[DEFAULT_CHANNELS];
+typedef sample_t surroundSampleFrame[SURROUND_CHANNELS];
 
 typedef struct
 {
@@ -88,10 +92,10 @@ typedef struct
 } volumeVector;
 
 
-const Uint32 BYTES_PER_SAMPLE = sizeof( sampleType );
-const Uint32 BYTES_PER_FRAME = sizeof( sampleFrame );
-const Uint32 BYTES_PER_SURROUND_FRAME = sizeof( surroundSampleFrame );
-const Uint32 BYTES_PER_OUTPUT_SAMPLE = sizeof( outputSampleType );
+const Uint8 BYTES_PER_SAMPLE = sizeof( sample_t );
+const Uint8 BYTES_PER_INT_SAMPLE = sizeof( int_sample_t );
+const Uint8 BYTES_PER_FRAME = sizeof( sampleFrame );
+const Uint8 BYTES_PER_SURROUND_FRAME = sizeof( surroundSampleFrame );
 
 const float OUTPUT_SAMPLE_MULTIPLIER = 32767.0f;
 
@@ -102,45 +106,14 @@ const octaves BASE_OCTAVE = OCTAVE_4;
 
 
 
-class mixer : public QObject
+class mixer : public QObject, public engineObject
 {
 	Q_OBJECT
 public:
-	static inline mixer * inst( void )
-	{
-		if( s_instanceOfMe == NULL )
-		{
-			s_instanceOfMe = new mixer();
-		}
-		return( s_instanceOfMe );
-	}
+	void initDevices( void );
+	void FASTCALL clear( const bool _everything = FALSE );
 
 
-	void FASTCALL bufferToPort( sampleFrame * _buf, Uint32 _frames,
-						Uint32 _framesAhead,
-						volumeVector & _volumeVector,
-						audioPort * _port );
-	inline Uint32 framesPerAudioBuffer( void ) const
-	{
-		return( m_framesPerAudioBuffer );
-	}
-
-	inline Uint8 cpuLoad( void ) const
-	{
-		return( m_cpuLoad );
-	}
-
-	inline bool highQuality( void ) const
-	{
-		return( m_qualityLevel > DEFAULT_QUALITY_LEVEL );
-	}
-
-
-	inline const surroundSampleFrame * currentAudioBuffer( void ) const
-	{
-		return( m_curBuf );
-	}
-
 	// audio-device-stuff
 	inline const QString & audioDevName( void ) const
 	{
@@ -174,6 +147,7 @@ public:
 		}
 	}
 
+
 	// MIDI-client-stuff
 	inline const QString & midiClientName( void ) const
 	{
@@ -186,9 +160,16 @@ public:
 	}
 
 
-	inline void addPlayHandle( playHandle * _ph )
+	// play-handle stuff
+	inline bool addPlayHandle( playHandle * _ph )
 	{
-		m_playHandles.push_back( _ph );
+		if( criticalXRuns() == FALSE )
+		{
+			m_playHandles.push_back( _ph );
+			return( TRUE );
+		}
+		delete _ph;
+		return( FALSE );
 	}
 
 	inline void removePlayHandle( playHandle * _ph )
@@ -196,11 +177,43 @@ public:
 		m_playHandlesToRemove.push_back( _ph );
 	}
 
+	inline const playHandleVector & playHandles( void ) const
+	{
+		return( m_playHandles );
+	}
+
 	void checkValidityOfPlayHandles( void );
 
+	inline bool haveNoRunningNotes( void ) const
+	{
+		return( m_playHandles.size() == 0 );
+	}
 
 
-	inline Uint32 sampleRate( void )
+	// methods providing information for other classes
+	inline fpab_t framesPerAudioBuffer( void ) const
+	{
+		return( m_framesPerAudioBuffer );
+	}
+
+	inline const surroundSampleFrame * currentAudioBuffer( void ) const
+	{
+		return( m_writeBuf );
+	}
+
+
+	inline Uint8 cpuLoad( void ) const
+	{
+		return( m_cpuLoad );
+	}
+
+	inline bool highQuality( void ) const
+	{
+		return( m_qualityLevel > DEFAULT_QUALITY_LEVEL );
+	}
+
+
+	inline sample_rate_t sampleRate( void ) const
 	{
 		return( SAMPLE_RATES[m_qualityLevel] );
 	}
@@ -211,13 +224,13 @@ public:
 		return( m_masterGain );
 	}
 
-	inline void setMasterGain( float _mo )
+	inline void setMasterGain( const float _mo )
 	{
 		m_masterGain = _mo;
 	}
 
 
-	static inline sampleType clip( sampleType _s )
+	static inline sample_t clip( const sample_t _s )
 	{
 		if( _s > 1.0f )
 		{
@@ -231,55 +244,65 @@ public:
 	}
 
 
+	// methods for controlling mixer-state
 	void pause( void )
 	{
-		m_mixMutex.lock();
+		if( m_mixMutexLockLevel == 0 )
+		{
+			m_mixMutex.lock();
+		}
+		++m_mixMutexLockLevel;
 	}
 
 	void play( void )
 	{
-		m_mixMutex.unlock();
+		if( m_mixMutexLockLevel == 1 )
+		{
+			m_mixMutex.unlock();
+		}
+		--m_mixMutexLockLevel;
 	}
 
 
-	void clear( void );
+	// audio-buffer-mgm
+	void FASTCALL bufferToPort( const sampleFrame * _buf,
+					const fpab_t _frames,
+					const fpab_t _framesAhead,
+					const volumeVector & _volumeVector,
+							audioPort * _port );
 
-
-	void FASTCALL clearAudioBuffer( sampleFrame * _ab, Uint32 _frames );
+	void FASTCALL clearAudioBuffer( sampleFrame * _ab,
+							const f_cnt_t _frames );
 #ifndef DISABLE_SURROUND
 	void FASTCALL clearAudioBuffer( surroundSampleFrame * _ab,
-							Uint32 _frames );
+							const f_cnt_t _frames );
 #endif
 
-	inline bool haveNoRunningNotes( void ) const
-	{
-		return( m_playHandles.size() == 0 );
-	}
-
+	bool criticalXRuns( void ) const;
 
 	const surroundSampleFrame * renderNextBuffer( void );
 
+
 public slots:
 	void setHighQuality( bool _hq_on = FALSE );
-
+	void setClipScaling( bool _state );
 
 signals:
 	void sampleRateChanged( void );
-	void nextAudioBuffer( const surroundSampleFrame *, Uint32 _frames );
+	void nextAudioBuffer( const surroundSampleFrame *, int _frames );
 
 
 private:
-
-	static mixer * s_instanceOfMe;
-
-	mixer();
+	mixer( engine * _engine );
 	~mixer();
 
+	void startProcessing( void );
 	void stopProcessing( void );
 
 
 	// we don't allow to create mixer by using copy-ctor
-	mixer( const mixer & )
+	mixer( const mixer & ) :
+		engineObject( NULL )
 	{
 	}
 
@@ -288,23 +311,38 @@ private:
 	audioDevice * tryAudioDevices( void );
 	midiClient * tryMIDIClients( void );
 
-	void processBuffer( surroundSampleFrame * _buf, fxChnl _fx_chnl );
-
+	void processBuffer( const surroundSampleFrame * _buf,
+						const fx_ch_t _fx_chnl );
 
+	void FASTCALL scaleClip( fpab_t _frame, ch_cnt_t _chnl );
 
 	vvector<audioPort *> m_audioPorts;
 
-	Uint32 m_framesPerAudioBuffer;
-
-	surroundSampleFrame * m_curBuf;
-	surroundSampleFrame * m_nextBuf;
+	fpab_t m_framesPerAudioBuffer;
 
+	surroundSampleFrame * m_readBuf;
+	surroundSampleFrame * m_writeBuf;
+	
+	vvector<surroundSampleFrame *> m_bufferPool;
+	Uint8 m_readBuffer;
+	Uint8 m_writeBuffer;
+	Uint8 m_analBuffer;
+	Uint8 m_poolDepth;
+	
+	bool m_scaleClip;
+	surroundSampleFrame m_maxClip;
+	surroundSampleFrame m_previousSample;
+	fpab_t m_halfStart[SURROUND_CHANNELS];
+	bool m_clipped[SURROUND_CHANNELS];
+	bool m_oldBuffer[SURROUND_CHANNELS];
+	bool m_newBuffer[SURROUND_CHANNELS];
+	
 	Uint8 m_cpuLoad;
 
 	playHandleVector m_playHandles;
 	playHandleVector m_playHandlesToRemove;
 
-	Uint8 m_qualityLevel;
+	qualityLevels m_qualityLevel;
 	float m_masterGain;
 
 
@@ -318,9 +356,10 @@ private:
 
 
 	QMutex m_mixMutex;
+	Uint8 m_mixMutexLockLevel;
 
 
-	friend class lmmsMainWin;
+	friend class engine;
 
 } ;
 
