@@ -1,7 +1,7 @@
 /*
  * mmp.h - class for reading and writing multimedia-project-files
  *
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -50,12 +50,13 @@ public:
 		UNKNOWN,
 		SONG_PROJECT,
 		SONG_PROJECT_TEMPLATE,
-		CHANNEL_SETTINGS,
+		INSTRUMENT_TRACK_SETTINGS,
 		DRAG_N_DROP_DATA,
+		JOURNAL_DATA,
 		EFFECT_SETTINGS,
-		VIDEO_PROJECT,		// will come later...
-		BURN_PROJECT,		// will come later...
-		PLAYLIST,		// will come later...
+		VIDEO_PROJECT,		// might come later...
+		BURN_PROJECT,		// might come later...
+		PLAYLIST,		// might come later...
 		PROJ_TYPE_COUNT
 	} ;
 
@@ -89,6 +90,9 @@ private:
 	static projectTypes FASTCALL type( const QString & _type_name );
 	static QString FASTCALL typeName( projectTypes _project_type );
 
+	void cleanMetaNodes( QDomElement _de );
+
+
 	struct typeDescStruct
 	{
 		projectTypes m_type;
