@@ -2,7 +2,7 @@
  * note_play_handle.h - declaration of class notePlayHandle which is needed
  *                      by LMMS-Play-Engine
  *
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -27,29 +27,33 @@
 #ifndef _NOTE_PLAY_HANDLE_H
 #define _NOTE_PLAY_HANDLE_H
 
-#include <qpair.h>
+#include <qobject.h>
 
 #include "play_handle.h"
 #include "basic_filters.h"
 #include "note.h"
 
 
-class channelTrack;
+class instrumentTrack;
 class notePlayHandle;
 
 typedef vvector<notePlayHandle *> notePlayHandleVector;
+typedef vvector<const notePlayHandle *> constNotePlayHandleVector;
 
 
-class notePlayHandle : public playHandle, public note
+class notePlayHandle : public QObject, public playHandle, public note
 {
+	Q_OBJECT
 public:
 	void * m_pluginData;
 	basicFilters<> * m_filter;
 
-	notePlayHandle( channelTrack * _chnl_trk, Uint32 _frames_ahead,
-					Uint32 _frames, note * n,
-					bool _arp_note = FALSE ) FASTCALL;
-	~notePlayHandle();
+	float m_frequency;
+
+	notePlayHandle( instrumentTrack * _chnl_trk, const f_cnt_t _frames_ahead,
+					const f_cnt_t _frames, const note & _n,
+					const bool _arp_note = FALSE );
+	virtual ~notePlayHandle();
 
 
 	virtual void play( void );
@@ -57,41 +61,41 @@ public:
 	virtual inline bool done( void ) const
 	{
 		return( ( m_released && m_framesBeforeRelease == 0 &&
-			m_releaseFramesDone >= m_releaseFramesToDo  ) ||
-						m_channelTrack == NULL );
+			m_releaseFramesDone >= m_releaseFramesToDo ) ||
+						m_instrumentTrack == NULL );
 	}
 
 	virtual void checkValidity( void );
 
 
-	void FASTCALL noteOff( Uint32 _s = 0 );
+	void FASTCALL noteOff( const f_cnt_t _s = 0 );
 
-	inline Uint32 framesBeforeRelease( void ) const
+	inline f_cnt_t framesBeforeRelease( void ) const
 	{
 		return( m_framesBeforeRelease );
 	}
 
-	inline Uint32 releaseFramesDone( void ) const
+	inline f_cnt_t releaseFramesDone( void ) const
 	{
 		return( m_releaseFramesDone );
 	}
 
-	Uint32 actualReleaseFramesToDo( void ) const;
+	f_cnt_t actualReleaseFramesToDo( void ) const;
 
 	// returns how many samples this note is aligned ahead, i.e.
 	// at which position it is inserted in the according buffer
-	inline Uint32 framesAhead( void ) const
+	inline f_cnt_t framesAhead( void ) const
 	{
 		return ( m_framesAhead );
 	}
 
 	// returns total numbers of frames to play
-	inline Uint32 frames( void ) const
+	inline f_cnt_t frames( void ) const
 	{
 		return( m_frames );
 	}
 
-	void setFrames( Uint32 _frames );
+	void setFrames( const f_cnt_t _frames );
 
 	// returns whether note was released
 	inline bool released( void ) const
@@ -100,13 +104,13 @@ public:
 	}
 
 	// returns total numbers of played frames
-	inline Uint32 totalFramesPlayed( void ) const
+	inline f_cnt_t totalFramesPlayed( void ) const
 	{
 		return( m_totalFramesPlayed );
 	}
 
 	// returns volume-level at frame _frame (envelope/LFO)
-	float FASTCALL volumeLevel( Uint32 _frame );
+	float FASTCALL volumeLevel( const f_cnt_t _frame );
 
 	// adds note-play-handle _n as subnote
 	inline void addSubNote( notePlayHandle * _n )
@@ -119,10 +123,10 @@ public:
 		m_arpNote = _n->arpNote() && baseNote();
 	}
 
-	// returns channel-track this note-play-handle plays
-	inline channelTrack * getChannelTrack( void )
+	// returns instrument-track this note-play-handle plays
+	inline instrumentTrack * getInstrumentTrack( void )
 	{
-		return( m_channelTrack );
+		return( m_instrumentTrack );
 	}
 
 	// returns whether note is a base-note, e.g. is not part of an arpeggio
@@ -138,6 +142,11 @@ public:
 		return( m_arpNote );
 	}
 
+	inline void setArpNote( const bool _on )
+	{
+		m_arpNote = _on;
+	}
+
 	// returns whether note is base-note for arpeggio
 	inline bool arpBaseNote( void ) const
 	{
@@ -151,21 +160,32 @@ public:
 
 	void mute( void );
 
+	// returns index of note-play-handle in vector of note-play-handles 
+	// belonging to this channel
+	int index( void ) const;
+
+	// note-play-handles belonging to given channel
+	static constNotePlayHandleVector nphsOfInstrumentTrack(
+						const instrumentTrack * _ct );
+
+	// return whether given note-play-handle is equal to *this
+	bool operator==( const notePlayHandle & _nph ) const;
+
 
 private:
-	channelTrack * m_channelTrack;	// needed for calling
-					// channelTrack::playNote
-	Uint32 m_frames;		// total frames to play
-	Uint32 m_framesAhead;		// numbers of frames ahead in buffer
+	instrumentTrack * m_instrumentTrack;	// needed for calling
+					// instrumentTrack::playNote
+	f_cnt_t m_frames;		// total frames to play
+	f_cnt_t m_framesAhead;		// numbers of frames ahead in buffer
 					// to mix in
-	Uint32 m_totalFramesPlayed;	// total frame-counter - used for
+	f_cnt_t m_totalFramesPlayed;	// total frame-counter - used for
 					// figuring out whether a whole note
 					// has been played
-	Uint32 m_framesBeforeRelease;	// number of frames after which note
+	f_cnt_t m_framesBeforeRelease;	// number of frames after which note
 					// is released
-	Uint32 m_releaseFramesToDo;	// total numbers of frames to be
+	f_cnt_t m_releaseFramesToDo;	// total numbers of frames to be
 					// played after release
-	Uint32 m_releaseFramesDone;	// number of frames done after
+	f_cnt_t m_releaseFramesDone;	// number of frames done after
 					// release of note
 	notePlayHandleVector m_subNotes;// used for chords and arpeggios
 	bool m_released;		// indicates whether note is released
@@ -176,6 +196,11 @@ private:
 					// sub-note)
 	bool m_muted;			// indicates whether note is muted
 
+
+private slots:
+	void processMidiTime( const midiTime & _time );
+	void updateFrequency( void );
+
 } ;
 
 #endif
