@@ -2,7 +2,7 @@
  * piano_widget.h - declaration of class pianoWidget, a widget which provides
  *                  an interactive piano/keyboard-widget
  *
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -31,9 +31,9 @@
 
 #ifdef QT4
 
-#include <QWidget>
-#include <QPixmap>
-#include <QScrollBar>
+#include <QtGui/QWidget>
+#include <QtGui/QPixmap>
+#include <QtGui/QScrollBar>
 
 #else
 
@@ -44,10 +44,12 @@
 #endif
 
 
+#include "knob.h"
 #include "note.h"
+#include "templates.h"
 
 
-class channelTrack;
+class instrumentTrack;
 class notePlayHandle;
 
 
@@ -62,17 +64,32 @@ class pianoWidget : public QWidget
 {
 	Q_OBJECT
 public:
-	pianoWidget( channelTrack * _channel_track );
+	pianoWidget( instrumentTrack * _channel_track );
 	virtual ~pianoWidget();
 
+	inline void setKeyState( int _key, bool _on = FALSE )
+	{
+		m_pressedKeys[tLimit( _key, 0, NOTES_PER_OCTAVE *
+							OCTAVES -1 )] = _on;
+		update();
+	}
+
+	virtual void saveSettings( QDomDocument & _doc, QDomElement & _this,
+							const QString & _name );
+	virtual void loadSettings( const QDomElement & _this,
+							const QString & _name );
+
+	virtual void keyPressEvent( QKeyEvent * ke );
+	virtual void keyReleaseEvent( QKeyEvent * ke );
+	virtual bool x11Event( XEvent * _xe );
+
 
 protected:
+	virtual void contextMenuEvent( QContextMenuEvent * _me );
 	virtual void paintEvent( QPaintEvent * );
 	virtual void mousePressEvent( QMouseEvent * me );
 	virtual void mouseReleaseEvent( QMouseEvent * me );
 	virtual void mouseMoveEvent( QMouseEvent * me );
-	virtual void keyPressEvent( QKeyEvent * ke );
-	virtual void keyReleaseEvent( QKeyEvent * ke );
 	virtual void focusOutEvent( QFocusEvent * _fe );
 
 
@@ -89,15 +106,19 @@ private:
 	bool m_pressedKeys[NOTES_PER_OCTAVE * OCTAVES];
 
 	QScrollBar * m_pianoScroll;
-	channelTrack * m_channelTrack;
+	instrumentTrack * m_instrumentTrack;
 	tones m_startTone;			// first key when drawing
 	octaves m_startOctave;
 
 	int m_lastKey;
+	unsigned int m_keycode;
+
+	knob * m_noteKnob;
 
 
 private slots:
 	void pianoScrolled( int _new_pos );
+	void updateBaseNote( void );
 
 } ;
 
