@@ -1,7 +1,7 @@
 /*
  * pixmap_button.h - declaration of class pixmapButton
  *
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -26,42 +26,33 @@
 #ifndef _PIXMAP_BUTTON_H
 #define _PIXMAP_BUTTON_H
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #include "qt3support.h"
 
 #ifdef QT4
 
-#include <QPushButton>
-#include <QPixmap>
+#include <QtGui/QPixmap>
 
 #else
 
-#include <qpushbutton.h>
 #include <qpixmap.h>
 
 #endif
 
 
+#include "automatable_button.h"
 
-class pixmapButton : public QPushButton
+
+class pixmapButton : public automatableButton
 {
 	Q_OBJECT
 public:
-	pixmapButton( QWidget * _parent );
+	pixmapButton( QWidget * _parent, const QString & _name,
+					engine * _engine, track * _track );
 	virtual ~pixmapButton();
-	void FASTCALL setActiveGraphic( const QPixmap & _pm );
-	void FASTCALL setInactiveGraphic( const QPixmap & _pm,
-							bool _update = TRUE );
-	void FASTCALL setBgGraphic( const QPixmap & _pm );
-#ifndef QT4
-	inline void setChecked( bool _on )
-	{
-		setOn( _on );
-	}
-#endif
+
+	void setActiveGraphic( const QPixmap & _pm );
+	void setInactiveGraphic( const QPixmap & _pm, bool _update = TRUE );
+	void setBgGraphic( const QPixmap & _pm );
 
 
 signals:
@@ -76,9 +67,9 @@ protected:
 
 
 private:
-	QPixmap * m_activePixmap;
-	QPixmap * m_inactivePixmap;
-	QPixmap * m_bgPixmap;
+	QPixmap m_activePixmap;
+	QPixmap m_inactivePixmap;
+	QPixmap m_bgPixmap;
 
 } ;
 
