@@ -2,7 +2,7 @@
  * play_handle.h - base-class playHandle which is needed by
  *                 LMMS-Player-Engine
  *
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -35,7 +35,7 @@
 
 #ifdef QT4
 
-#include <QVector>
+#include <QtCore/QVector>
 
 #else
 
@@ -47,12 +47,26 @@
 class playHandle
 {
 public:
-	inline playHandle( void )
+	enum types
+	{
+		NOTE_PLAY_HANDLE, INSTRUMENT_PLAY_HANDLE, SAMPLE_PLAY_HANDLE,
+		PRESET_PREVIEW_PLAY_HANDLE
+	} ;
+
+	playHandle( const types _type ) :
+		m_type( _type )
 	{
 	}
+
 	virtual inline ~playHandle()
 	{
 	}
+
+	inline types type( void ) const
+	{
+		return( m_type );
+	}
+
 	virtual void play( void ) = 0;
 	virtual bool done( void ) const = 0;
 
@@ -67,11 +81,13 @@ public:
 
 
 private:
+	types m_type;
 
 } ;
 
 
 typedef vvector<playHandle *> playHandleVector;
+typedef vvector<const playHandle *> constPlayHandleVector;
 
 
 #endif
