@@ -2,7 +2,7 @@
  * preset_preview_play_handle.h - play-handle for playing a short preview-sound
  *                                of a preset
  *
- * Copyright (c) 2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2005-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -27,31 +27,49 @@
 #ifndef _PRESET_PREVIEW_PLAY_HANDLE_H
 #define _PRESET_PREVIEW_PLAY_HANDLE_H
 
-#include "play_handle.h"
+#include "qt3support.h"
+
+#ifdef QT4
+
+#include <QtCore/QMap>
+
+#else
+
+#include <qmap.h>
+
+#endif
 
 
-class channelTrack;
+#include "note_play_handle.h"
+#include "engine.h"
+
+
+class instrumentTrack;
 class notePlayHandle;
-class QMutex;
+class previewTrackContainer;
 
 
-class presetPreviewPlayHandle : public playHandle
+class presetPreviewPlayHandle : public playHandle, public engineObject
 {
 public:
-	presetPreviewPlayHandle( const QString & _preset_file );
+	presetPreviewPlayHandle( const QString & _preset_file,
+						engine * _engine );
 	virtual ~presetPreviewPlayHandle();
 
-	static void cleanUp( void );
-
-
 	virtual void play( void );
 	virtual bool done( void ) const;
 
+	static void cleanUp( engine * _engine );
+	static constNotePlayHandleVector nphsOfInstrumentTrack(
+						const instrumentTrack * _ct );
 
 private:
-	static channelTrack * s_globalChannelTrack;
-	static notePlayHandle * s_globalPreviewNote;
-	static QMutex * s_globalDataMutex;
+	inline previewTrackContainer * previewTC( void )
+	{
+		return( s_previewTCs[eng()] );
+	}
+
+	static QMap<const engine *, previewTrackContainer *> s_previewTCs;
 
 	notePlayHandle * m_previewNote;
 
