@@ -1,7 +1,7 @@
 /*
  * project_notes.h - header for project-notes-editor
  *
- * Copyright (c) 2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2005-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -30,15 +30,17 @@
 
 #ifdef QT4
 
-#include <QMainWindow>
+#include <QtGui/QMainWindow>
 
 #else
 
 #include <qmainwindow.h>
+#define textColor color
 
 #endif
 
-#include "settings.h"
+#include "journalling_object.h"
+#include "engine.h"
 
 
 class QAction;
@@ -46,12 +48,15 @@ class QComboBox;
 class QTextEdit;
 
 
-class projectNotes : public QMainWindow, public settings
+class projectNotes : public QMainWindow, public journallingObject
 {
 	Q_OBJECT
 public:
-	projectNotes();
+	projectNotes( engine * _engine );
+	virtual ~projectNotes();
+
 	void clear( void );
+	void setText( const QString & _text );
 
 	virtual void FASTCALL saveSettings( QDomDocument & _doc,
 							QDomElement & _parent );
