@@ -1,7 +1,7 @@
 /*
  * qt3support.h - layer for supporting Qt3
  *
- * Copyright (c) 2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2005-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -34,9 +34,17 @@
 #include <qpair.h>
 
 #if QT_VERSION >= 0x040000
+
 #ifndef QT4
 #define QT4
 #endif
+
+#else
+
+#ifndef QT3
+#define QT3
+#endif
+
 #endif
 
 
@@ -49,10 +57,12 @@ class QColorGroup;
 
 typedef int csize;
 #define QListViewItem Q3ListViewItem
-#define vvector QVector
-#define vlist QList 
 
-#include <QFileInfo>
+#define vlist QList 
+#define vstack QStack
+#define vvector QVector
+
+#include <QtCore/QFileInfo>
 
 inline QString baseName( const QString & _file )
 {
@@ -61,11 +71,13 @@ inline QString baseName( const QString & _file )
 }
 
 
-#else
+#else /* QT3 */
 
 
-#define vvector QValueVector
 #define vlist QValueList 
+#define vstack QValueStack
+#define vvector QValueVector
+
 
 #define QMenu QPopupMenu
 #define QAbstractButton QButton
@@ -77,11 +89,13 @@ inline QString baseName( const QString & _file )
 
 #define QMatrix QWMatrix
 #define QIcon QIconSet
+#define QAbstractButton QButton
 
 
 // QWidget
 #define setWindowTitle setCaption
 #define setWindowIcon setIcon
+#define windowIcon icon
 #define isExplicitlyHidden isHidden
 #define accessibleName name
 #define ensurePolished constPolish
@@ -107,6 +121,7 @@ inline QString baseName( const QString & _file )
 
 // QScrollView/QScrollArea
 #define setHorizontalScrollBarPolicy setHScrollBarMode
+#define setVerticalScrollBarPolicy setVScrollBarMode
 
 
 // QScrollBar
@@ -115,7 +130,6 @@ inline QString baseName( const QString & _file )
 
 
 // QAbstractButton/QButton
-#define setCheckable setToggleButton
 #define setShortcut setAccel
 
 
@@ -123,10 +137,6 @@ inline QString baseName( const QString & _file )
 #define modifiers state
 
 
-// QButtonGroup
-#define addButton insert
-
-
 // QProgressBar
 #define setTextVisible setPercentageVisible
 
@@ -137,6 +147,7 @@ inline QString baseName( const QString & _file )
 
 // QString
 #define toLower lower
+#define toUpper upper
 
 
 // QTextEdit
@@ -177,6 +188,14 @@ inline QString baseName( const QString & _file )
 #define setColumnMinimumWidth setColSpacing
 
 
+// QPointer
+#define QPointer QGuardedPtr
+
+
+// QRect
+#define normalized normalize
+
+
 // Qt-namespace
 #define ShiftModifier ShiftButton
 #define ControlModifier ControlButton
