@@ -1,7 +1,7 @@
 /*
  * setup_dialog.h - dialog for setting up LMMS
  *
- * Copyright (c) 2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2005-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -30,8 +30,8 @@
 
 #ifdef QT4
 
-#include <QDialog>
-#include <QMap>
+#include <QtGui/QDialog>
+#include <QtCore/QMap>
 
 #else
 
@@ -47,22 +47,25 @@
 
 class QComboBox;
 class QLabel;
+class QLineEdit;
 class QSlider;
 
 class tabBar;
 
 
-class setupDialog : public QDialog
+class setupDialog : public QDialog, public engineObject
 {
 	Q_OBJECT
 public:
 	enum configTabs
 	{
-		GENERAL_SETTINGS, AUDIO_SETTINGS, MIDI_SETTINGS
+		GENERAL_SETTINGS, DIRECTORY_SETTINGS, PERFORMANCE_SETTINGS,
+		AUDIO_SETTINGS, MIDI_SETTINGS
 	} ;
 
-	setupDialog( configTabs _tab_to_open = GENERAL_SETTINGS );
-	~setupDialog();
+	setupDialog( engine * _engine,
+				configTabs _tab_to_open = GENERAL_SETTINGS );
+	virtual ~setupDialog();
 
 
 protected slots:
@@ -75,6 +78,12 @@ private slots:
 	void resetBufSize( void );
 	void displayBufSizeHelp( void );
 
+	// directory settings widget
+	void setWorkingDir( const QString & _wd );
+	void setVSTDir( const QString & _vd );
+	void setArtworkDir( const QString & _ad );
+	void setFLDir( const QString & _fd );
+
 	// audio settings widget
 	void audioInterfaceChanged( const QString & _driver );
 	void displayAudioHelp( void );
@@ -87,6 +96,18 @@ private slots:
 	void toggleToolTips( bool _disabled );
 	void toggleKnobUsability( bool _classical );
 	void toggleGIMPLikeWindows( bool _enabled );
+	void toggleNoWizard( bool _enabled );
+	void toggleNoMsgAfterSetup( bool _enabled );
+	void toggleDisplaydBV( bool _enabled );
+
+	void openWorkingDir( void );
+	void openVSTDir( void );
+	void openArtworkDir( void );
+	void openFLDir( void );
+
+
+	void toggleDisableChActInd( bool _disabled );
+	void toggleManualChPiano( bool _enabled );
 
 
 private:
@@ -99,6 +120,24 @@ private:
 	bool m_disableToolTips;
 	bool m_classicalKnobUsability;
 	bool m_gimpLikeWindows;
+	bool m_noWizard;
+	bool m_noMsgAfterSetup;
+	bool m_displaydBV;
+
+
+	QLineEdit * m_wdLineEdit;
+	QLineEdit * m_vdLineEdit;
+	QLineEdit * m_adLineEdit;
+	QLineEdit * m_fdLineEdit;
+
+	QString m_workingDir;
+	QString m_vstDir;
+	QString m_artworkDir;
+	QString m_flDir;
+
+
+	bool m_disableChActInd;
+	bool m_manualChPiano;
 
 
 	typedef QMap<QString, audioDevice::setupWidget *> aswMap;
