@@ -1,7 +1,7 @@
 /*
  * side_bar_widget.h - base-class for all side-bar-widgets
  *
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -30,9 +30,9 @@
 
 #ifdef QT4
 
-#include <QWidget>
-#include <QVBoxLayout>
-#include <QPixmap>
+#include <QtGui/QPixmap>
+#include <QtGui/QVBoxLayout>
+#include <QtGui/QWidget>
 
 #else
 
