@@ -2,7 +2,7 @@
  * song_editor.h - declaration of class songEditor, a window where you can
  *                 setup your songs
  *
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -29,29 +29,26 @@
 
 #include "track_container.h"
 #include "types.h"
+#include "tool_button.h"
 
 
-class QComboBox;
 class QLabel;
 class QScrollBar;
-class QSlider;
 
+class automatableSlider;
+class comboBox;
 class lcdSpinBox;
-class lmmsMainWin;
 class pattern;
-class projectNotes;
 class textFloat;
 class timeLine;
-class toolButton;
 
 
-const int MIN_BPM = 10;
-const int DEFAULT_BPM = 140;
-const int MAX_BPM = 999;
+const bpm_t MIN_BPM = 10;
+const bpm_t DEFAULT_BPM = 140;
+const bpm_t MAX_BPM = 999;
 const Uint16 MAX_SONG_LENGTH = 9999;
 
 
-
 class songEditor : public trackContainer
 {
 	Q_OBJECT
@@ -62,6 +59,7 @@ public:
 		PLAY_TRACK,
 		PLAY_BB,
 		PLAY_PATTERN,
+		PLAY_AUTOMATION_PATTERN,
 		PLAY_MODE_CNT
 	} ;
 
@@ -76,11 +74,11 @@ public:
 			m_currentFrame( 0 )
 		{
 		}
-		inline void setCurrentFrame( Uint32 _f )
+		inline void setCurrentFrame( const f_cnt_t _f )
 		{
 			m_currentFrame = _f;
 		}
-		inline Uint32 currentFrame( void ) const
+		inline f_cnt_t currentFrame( void ) const
 		{
 			return( m_currentFrame );
 		}
@@ -88,20 +86,11 @@ public:
 		bool m_timeLineUpdate;
 
 	private:
-		Uint32 m_currentFrame;
+		f_cnt_t m_currentFrame;
 	} ;
 
 
 
-	static inline songEditor * inst( void )
-	{
-		if( s_instanceOfMe == NULL )
-		{
-			s_instanceOfMe = new songEditor();
-		}
-		return( s_instanceOfMe );
-	}
-
 	void processNextBuffer( void );
 
 
@@ -120,6 +109,8 @@ public:
 		return( m_exporting );
 	}
 
+	bool realTimeTask( void ) const;
+
 	inline bool exportDone( void ) const
 	{
 		return( m_exporting == TRUE &&
@@ -139,7 +130,8 @@ public:
 	tact lengthInTacts( void ) const;
 
 
-	int getBPM( void );
+	bpm_t getTempo( void );
+
 
 	// every function that replaces current file (e.g. creates new file,
 	// opens another file...) has to call this before and may only process
@@ -159,9 +151,7 @@ public:
 	{
 		return( m_fileName );
 	}
-	virtual void FASTCALL saveSettings( QDomDocument & _doc,
-							QDomElement & _parent );
-	virtual void FASTCALL loadSettings( const QDomElement & _this );
+
 	inline virtual QString nodeName( void ) const
 	{
 		return( "songeditor" );
@@ -174,11 +164,6 @@ public:
 
 	int masterPitch( void ) const;
 
-	projectNotes * getProjectNotesWindow( void )
-	{
-		return( m_projectNotes );
-	}
-
 
 public slots:
 	void play( void );
@@ -194,21 +179,33 @@ public slots:
 
 	void startExport( void );
 	void stopExport( void );
-	// set BPM (beats per minute)
-	void setBPM( int _new_bpm = DEFAULT_BPM );
+
+	// set tempo in BPM (beats per minute)
+	void setTempo( int _new_bpm = DEFAULT_BPM );
+	void setMasterVolume( volume _vol );
+	void setMasterPitch( int _master_pitch );
 
 	inline void setModified( void )
 	{
 		m_modified = TRUE;
 	}
 
+	void clearProject( void );
+
 
 protected:
-	void closeEvent( QCloseEvent * _ce );
-	void resizeEvent( QResizeEvent * _re );
-	void keyPressEvent( QKeyEvent * _ke );
-	void wheelEvent( QWheelEvent * _we );
-	void paintEvent( QPaintEvent * _pe );
+	virtual void closeEvent( QCloseEvent * _ce );
+	virtual void resizeEvent( QResizeEvent * _re );
+	virtual void keyPressEvent( QKeyEvent * _ke );
+	virtual void wheelEvent( QWheelEvent * _we );
+	virtual void paintEvent( QPaintEvent * _pe );
+
+	virtual QRect scrollAreaRect( void ) const;
+
+	virtual bool allowRubberband( void ) const
+	{
+		return( m_editModeButton->isChecked() );
+	}
 
 
 protected slots:
@@ -232,14 +229,14 @@ protected slots:
 
 	void zoomingChanged( const QString & _zfac );
 
+	void doActions( void );
+
 
 private:
-	songEditor();
+	songEditor( engine * _engine );
 	songEditor( const songEditor & );
 	virtual ~songEditor();
 
-	void clearProject( void );
-
 
 	inline tact currentTact( void ) const
 	{
@@ -256,8 +253,6 @@ private:
 
 
 
-	static songEditor * s_instanceOfMe;
-
 	QScrollBar * m_leftRightScroll;
 
 	QWidget * m_toolBar;
@@ -266,17 +261,18 @@ private:
 	toolButton * m_stopButton;
 	lcdSpinBox * m_bpmSpinBox;
 
-	QSlider * m_masterVolumeSlider;
-	QSlider * m_masterPitchSlider;
+	automatableSlider * m_masterVolumeSlider;
+	automatableSlider * m_masterPitchSlider;
 	textFloat * m_mvsStatus;
 	textFloat * m_mpsStatus;
 
 	toolButton * m_addBBTrackButton;
 	toolButton * m_addSampleTrackButton;
-	toolButton * m_insertBarButton;
-	toolButton * m_removeBarButton;
 
-	QComboBox * m_zoomingComboBox;
+	toolButton * m_drawModeButton;
+	toolButton * m_editModeButton;
+
+	comboBox * m_zoomingComboBox;
 
 
 	QString m_fileName;
@@ -298,8 +294,9 @@ private:
 
 	bool m_scrollBack;
 
+	track * m_automation_track;
 
-	projectNotes * m_projectNotes;
+	bool m_destroyed;
 
 
 
@@ -310,15 +307,12 @@ private:
 	} ;
 	vvector<ACTIONS> m_actions;
 
-	void doActions( void );
-
-
-	friend class lmmsMainWin;
 
+	friend class engine;
 
 
 signals:
-	void bpmChanged( int _new_bpm );
+	void tempoChanged( bpm_t _new_bpm );
 
 } ;
 
