@@ -2,7 +2,7 @@
  * track.h - declaration of classes concerning tracks -> neccessary for all
  *           track-like objects (beat/bassline, sample-track...)
  *
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -35,9 +35,9 @@
 
 #ifdef QT4
 
-#include <QVector>
-#include <QList>
-#include <QWidget>
+#include <QtCore/QVector>
+#include <QtCore/QList>
+#include <QtGui/QWidget>
 
 #else
 
@@ -54,11 +54,15 @@
 
 #include "types.h"
 #include "midi_time.h"
-#include "settings.h"
+#include "rubberband.h"
+#include "engine.h"
+#include "journalling_object.h"
 
 
 class QMenu;
+class QPushButton;
 
+class automationPattern;
 class pixmapButton;
 class textFloat;
 class track;
@@ -66,7 +70,7 @@ class trackContainer;
 class trackContentWidget;
 class trackWidget;
 
-typedef QWidget trackOperationsWidget;
+typedef QWidget trackSettingsWidget;
 
 
 
@@ -75,7 +79,8 @@ const Uint16 TRACK_OP_WIDTH = 70;
 const Uint16 TCO_BORDER_WIDTH = 1;
 
 
-class trackContentObject : public QWidget, public settings
+class trackContentObject : public selectableObject,
+			   public journallingObject
 {
 	Q_OBJECT
 public:
@@ -98,7 +103,14 @@ public:
 	{
 		return( m_length );
 	}
+
+	bool muted( void ) const
+	{
+		return( m_muted );
+	}
+
 	bool fixedTCOs( void );
+
 	virtual void FASTCALL movePosition( const midiTime & _pos );
 	virtual void FASTCALL changeLength( const midiTime & _length );
 
@@ -111,41 +123,56 @@ protected:
 	virtual void constructContextMenu( QMenu * )
 	{
 	}
+
 	virtual void contextMenuEvent( QContextMenuEvent * _cme );
 	virtual void dragEnterEvent( QDragEnterEvent * _dee );
 	virtual void dropEvent( QDropEvent * _de );
 	virtual void leaveEvent( QEvent * _e );
-	virtual void mouseMoveEvent( QMouseEvent * _me );
 	virtual void mousePressEvent( QMouseEvent * _me );
+	virtual void mouseMoveEvent( QMouseEvent * _me );
 	virtual void mouseReleaseEvent( QMouseEvent * _me );
 
 	void setAutoResizeEnabled( bool _e = FALSE );
 	float pixelsPerTact( void );
 
+	virtual void undoStep( journalEntry & _je );
+	virtual void redoStep( journalEntry & _je );
+
 
 protected slots:
 	void cut( void );
 	void copy( void );
 	void paste( void );
+	void toggleMute( void );
 
 
 private:
+	enum actions
+	{
+		NONE, MOVE, MOVE_SELECTION, RESIZE
+	} ;
+
 	static textFloat * s_textFloat;
 
 	track * m_track;
 	midiTime m_startPosition;
 	midiTime m_length;
-	bool m_moving;
-	bool m_resizing;
+	actions m_action;
 	bool m_autoResize;
 	Sint16 m_initialMouseX;
 
+	bool m_muted;
+
+	textFloat * m_hint;
+
+	midiTime m_oldTime;// used for undo/redo while mouse-button is pressed
+
 } ;
 
 
 
 
-class trackContentWidget : public QWidget
+class trackContentWidget : public QWidget, public journallingObject
 {
 	Q_OBJECT
 public:
@@ -165,10 +192,12 @@ public:
 	{
 		return( m_pixelsPerTact );
 	}
+
 	inline void setPixelsPerTact( Uint16 _ppt )
 	{
 		m_pixelsPerTact = _ppt;
 	}
+
 	tact length( void ) const;
 
 
@@ -182,12 +211,24 @@ protected:
 	virtual void dragEnterEvent( QDragEnterEvent * _dee );
 	virtual void dropEvent( QDropEvent * _de );
 	virtual void mousePressEvent( QMouseEvent * _me );
-	virtual void mouseMoveEvent( QMouseEvent * _me );
-	virtual void mouseReleaseEvent( QMouseEvent * _me );
+	virtual void paintEvent( QPaintEvent * _pe );
 	virtual void resizeEvent( QResizeEvent * _re );
 
+	virtual QString nodeName( void ) const
+	{
+		return( "trackcontentwidget" );
+	}
+
+	virtual void undoStep( journalEntry & _je );
+	virtual void redoStep( journalEntry & _je );
+
 
 private:
+	enum actions
+	{
+		ADD_TCO, REMOVE_TCO
+	} ;
+
 	track * getTrack( void );
 	midiTime getPosition( int _mouse_x );
 
@@ -203,20 +244,40 @@ private:
 
 
 
-class trackSettingsWidget : public QWidget
+class trackOperationsWidget : public QWidget
 {
+	Q_OBJECT
 public:
-	trackSettingsWidget( trackWidget * _parent );
-	~trackSettingsWidget();
+	trackOperationsWidget( trackWidget * _parent );
+	~trackOperationsWidget();
+
+	bool muted( void ) const;
+
+
+public slots:
+	void setMuted( bool _muted );
 
 
 protected:
 	virtual void mousePressEvent( QMouseEvent * _me );
+	virtual void paintEvent( QPaintEvent * _pe );
+
+
+private slots:
+	void cloneTrack( void );
+	void removeTrack( void );
+	void removeTrackTimer( void );
+	void muteBtnRightClicked( void );
 
 
 private:
+	static QPixmap * s_grip;
+
 	trackWidget * m_trackWidget;
 
+	QPushButton * m_trackOps;
+	pixmapButton * m_muteBtn;
+
 } ;
 
 
@@ -225,7 +286,7 @@ private:
 
 
 // actual widget shown in trackContainer
-class trackWidget : public QWidget
+class trackWidget : public QWidget, public journallingObject
 {
 	Q_OBJECT
 public:
@@ -242,6 +303,12 @@ public:
 		return( m_track );
 	}
 
+	inline const trackOperationsWidget & getTrackOperationsWidget( void )
+									const
+	{
+		return( m_trackOperationsWidget );
+	}
+
 	inline const trackSettingsWidget & getTrackSettingsWidget( void ) const
 	{
 		return( m_trackSettingsWidget );
@@ -252,6 +319,11 @@ public:
 		return( m_trackContentWidget );
 	}
 
+	inline trackOperationsWidget & getTrackOperationsWidget( void )
+	{
+		return( m_trackOperationsWidget );
+	}
+
 	inline trackSettingsWidget & getTrackSettingsWidget( void )
 	{
 		return( m_trackSettingsWidget );
@@ -262,22 +334,33 @@ public:
 		return( m_trackContentWidget );
 	}
 
-	bool muted( void ) const;
+	bool isMovingTrack( void ) const
+	{
+		return( m_action == MOVE_TRACK );
+	}
+	
+	virtual void repaint( void );
 
 
 public slots:
 	void changePosition( const midiTime & _new_pos = -1 );
-	void cloneTrack( void );
-	void removeTrack( void );
-	void moveTrackUp( void );
-	void moveTrackDown( void );
-	void setMuted( bool _muted );
-	void muteBtnRightClicked( void );
 
 
 protected:
+	virtual void undoStep( journalEntry & _je );
+	virtual void redoStep( journalEntry & _je );
+
+	virtual QString nodeName( void ) const
+	{
+		return( "trackwidget" );
+	}
+
+
 	virtual void dragEnterEvent( QDragEnterEvent * _dee );
 	virtual void dropEvent( QDropEvent * _de );
+	virtual void mousePressEvent( QMouseEvent * _me );
+	virtual void mouseMoveEvent( QMouseEvent * _me );
+	virtual void mouseReleaseEvent( QMouseEvent * _me );
 	virtual void paintEvent( QPaintEvent * _pe );
 	virtual void resizeEvent( QResizeEvent * _re );
 
@@ -285,13 +368,17 @@ protected:
 
 
 private:
+	enum actions
+	{
+		NONE, MOVE_TRACK, RESIZE_TRACK
+	} ;
 	track * m_track;
 
 	trackOperationsWidget m_trackOperationsWidget;
 	trackSettingsWidget m_trackSettingsWidget;
 	trackContentWidget m_trackContentWidget;
 
-	pixmapButton * m_muteBtn;
+	actions m_action;
 
 } ;
 
@@ -299,7 +386,7 @@ private:
 
 
 // base-class for all tracks
-class track : public settings
+class track : public journallingObject
 {
 public:
 	enum trackTypes
@@ -310,11 +397,13 @@ public:
 		EVENT_TRACK,
 		VIDEO_TRACK,
 		NULL_TRACK,
+		AUTOMATION_TRACK,
 		TOTAL_TRACK_TYPES
 	} ;
 
-	track( trackContainer * _tc );
+	track( trackContainer * _tc, bool _create_widget = TRUE );
 	virtual ~track();
+
 	static track * FASTCALL create( trackTypes _tt, trackContainer * _tc );
 	static track * FASTCALL create( const QDomElement & _this,
 							trackContainer * _tc );
@@ -324,16 +413,22 @@ public:
 
 	inline bool muted( void ) const
 	{
-		return( m_trackWidget->muted() );
+		return( m_trackWidget->getTrackOperationsWidget().muted() );
 	}
 
+	inline void setMuted( bool _muted )
+	{
+		m_trackWidget->getTrackOperationsWidget().setMuted( _muted );
+	}
+
+
 	// pure virtual functions
 	virtual trackTypes type( void ) const = 0;
 
 	virtual bool FASTCALL play( const midiTime & _start,
-						Uint32 _start_frame,
-						Uint32 _frames,
-						Uint32 _frame_base,
+						const f_cnt_t _start_frame,
+						const fpab_t _frames,
+						const f_cnt_t _frame_base,
 						Sint16 _tco_num = -1 ) = 0;
 
 
@@ -347,7 +442,7 @@ public:
 
 
 	virtual void FASTCALL saveSettings( QDomDocument & _doc,
-						QDomElement & _parent );
+							QDomElement & _this );
 	virtual void FASTCALL loadSettings( const QDomElement & _this );
 
 
@@ -391,10 +486,30 @@ public:
 		return( &m_trackWidget->getTrackContentWidget() );
 	}
 
+	void addAutomationPattern( automationPattern * _pattern );
+	void removeAutomationPattern( automationPattern * _pattern );
+
+	// name-stuff
+	inline virtual const QString & name( void ) const
+	{
+		return( m_name );
+	}
+	inline virtual void setName( const QString & _new_name )
+	{
+		m_name = _new_name;
+	}
+
+
+protected:
+	bool sendMidiTime( const midiTime & _time );
+	QString m_name;
+
 
 private:
 	trackContainer * m_trackContainer;
 	trackWidget * m_trackWidget;
+	QPtrList<automationPattern> m_automation_patterns;
+	midiTime m_last_time_sent;
 
 } ;
 
