@@ -2,7 +2,7 @@
  * track_container.h - base-class for all track-containers like Song-Editor,
  *                     BB-Editor...
  *
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -31,9 +31,9 @@
 
 #ifdef QT4
 
-#include <QScrollArea>
-#include <QVector>
-#include <QMainWindow>
+#include <QtGui/QScrollArea>
+#include <QtCore/QVector>
+#include <QtGui/QMainWindow>
 
 #else
 
@@ -45,39 +45,51 @@
 
 
 #include "track.h"
-#include "settings.h"
+#include "journalling_object.h"
+#include "rubberband.h"
+#include "engine.h"
+
 
 
 const Uint16 DEFAULT_PIXELS_PER_TACT = 16;
+const Uint16 DEFAULT_SCROLLBAR_SIZE = 16;
 
 
-class trackContainer : public QMainWindow, public settings
+
+class trackContainer : public QMainWindow, public journallingObject
 {
 	Q_OBJECT
 public:
-	trackContainer();
-	~trackContainer();
+	trackContainer( engine * _engine );
+	virtual ~trackContainer();
+
 	inline QWidget * containerWidget( void )
 	{
 		return( m_scrollArea );
 	}
+
 	virtual void FASTCALL saveSettings( QDomDocument & _doc,
 							QDomElement & _parent );
+
 	virtual void FASTCALL loadSettings( const QDomElement & _this );
 
+
 	inline float pixelsPerTact( void ) const
 	{
 		return( m_ppt );
 	}
+
 	inline const midiTime & currentPosition( void ) const
 	{
 		return( m_currentPosition );
 	}
+
 	virtual bool fixedTCOs( void ) const
 	{
 		return( FALSE );
 	}
-	unsigned int FASTCALL countTracks( track::trackTypes _tt =
+
+	Uint16 FASTCALL countTracks( track::trackTypes _tt =
 					track::TOTAL_TRACK_TYPES ) const;
 
 	void FASTCALL setMutedOfAllTracks( bool _muted );
@@ -90,17 +102,50 @@ public:
 	void FASTCALL removeTrack( track * _track );
 	void FASTCALL moveTrackUp( track * _track );
 	void FASTCALL moveTrackDown( track * _track );
+
 	void FASTCALL realignTracks( bool _complete_update = FALSE );
+	void clearAllTracks( void );
+
+	const trackWidget * trackWidgetAt( const int _y ) const;
+
+
+	virtual bool allowRubberband( void ) const;
+
+	inline bool rubberBandActive( void ) const
+	{
+		return( m_rubberBand->isVisible() );
+	}
+
+	inline vvector<selectableObject *> selectedObjects( void )
+	{
+		if( allowRubberband() == TRUE )
+		{
+			return( m_rubberBand->selectedObjects() );
+		}
+		//return( vvector<selectableObject *>() );
+		vvector<selectableObject *> foo;
+		return( foo );
+	}
 
 
 protected:
+	virtual void undoStep( journalEntry & _je );
+	virtual void redoStep( journalEntry & _je );
+
 	virtual void dragEnterEvent( QDragEnterEvent * _dee );
 	virtual void dropEvent( QDropEvent * _de );
+	virtual void mousePressEvent( QMouseEvent * _me );
+	virtual void mouseMoveEvent( QMouseEvent * _me );
+	virtual void mouseReleaseEvent( QMouseEvent * _me );
 	virtual void resizeEvent( QResizeEvent * );
 
 	constTrackVector tracks( void ) const;
 	trackVector tracks( void );
 
+	virtual QRect scrollAreaRect( void ) const
+	{
+		return( rect() );
+	}
 
 	midiTime m_currentPosition;
 
@@ -110,6 +155,10 @@ protected slots:
 
 
 private:
+	enum actions
+	{
+		ADD_TRACK, REMOVE_TRACK
+	} ;
 
 	class scrollArea : public QScrollArea
 	{
@@ -132,6 +181,9 @@ private:
 	trackWidgetVector m_trackWidgets;
 	float m_ppt;
 
+	rubberBand * m_rubberBand;
+	QPoint m_origin;
+
 
 	friend class scrollArea;
 
