@@ -1,7 +1,7 @@
 /*
  * audio_file_processor.cpp - instrument for using audio-files
  *
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -27,30 +27,27 @@
 
 #ifdef QT4
 
-#include <QPainter>
-#include <QButtonGroup>
-#include <QBitmap>
+#include <QtGui/QPainter>
+#include <QtGui/QBitmap>
 #include <Qt/QtXml>
-#include <QFileInfo>
-#include <QDropEvent>
+#include <QtCore/QFileInfo>
+#include <QtGui/QDropEvent>
 
 #else
 
 #include <qpainter.h>
-#include <qbuttongroup.h>
 #include <qbitmap.h>
 #include <qdom.h>
 #include <qfileinfo.h>
-
-#define isChecked isOn
-#define setChecked setOn
+#include <qcursor.h>
+#include <qwhatsthis.h>
 
 #endif
 
 
 #include "audio_file_processor.h"
 #include "song_editor.h"
-#include "channel_track.h"
+#include "instrument_track.h"
 #include "note_play_handle.h"
 #include "paths.h"
 #include "interpolation.h"
@@ -61,8 +58,9 @@
 #include "string_pair_drag.h"
 #include "mmp.h"
 
+#undef SINGLE_SOURCE_COMPILE
 #include "embed.cpp"
-
+#include "volume_knob.h"
 
 
 extern "C"
@@ -74,11 +72,12 @@ plugin::descriptor audiofileprocessor_plugin_descriptor =
 	"AudioFileProcessor",
 	QT_TRANSLATE_NOOP( "pluginBrowser",
 				"simple sampler with various settings for "
-				"using samples (e.g. drums) in a channel" ),
+				"using samples (e.g. drums) in an "
+				"instrument-track" ),
 	"Tobias Doerffel <tobydox/at/users.sf.net>",
 	0x0100,
 	plugin::INSTRUMENT,
-	PLUGIN_NAME::findEmbeddedData( "logo.png" )
+	new QPixmap( PLUGIN_NAME::getIconPixmap( "logo" ) )
 } ;
 
 }
@@ -88,11 +87,10 @@ QPixmap * audioFileProcessor::s_artwork = NULL;
 
 
 
-audioFileProcessor::audioFileProcessor( channelTrack * _channel_track ) :
-	instrument( _channel_track,
-			audiofileprocessor_plugin_descriptor.public_name ),
+audioFileProcessor::audioFileProcessor( instrumentTrack * _channel_track ) :
+	instrument( _channel_track, &audiofileprocessor_plugin_descriptor ),
 	specialBgHandlingWidget( PLUGIN_NAME::getIconPixmap( "artwork" ) ),
-	m_sampleBuffer( "" ),
+	m_sampleBuffer( eng(), "" ),
 	m_drawMethod( sampleBuffer::LINE_CONNECT )
 {
 	connect( &m_sampleBuffer, SIGNAL( sampleUpdated() ), this,
@@ -105,8 +103,7 @@ audioFileProcessor::audioFileProcessor( channelTrack * _channel_track ) :
 	}
 
 
-	m_openAudioFileButton = new pixmapButton( this );
-	m_openAudioFileButton->setCheckable( FALSE );
+	m_openAudioFileButton = new pixmapButton( this, NULL, eng(), NULL );
 	m_openAudioFileButton->setCursor( QCursor( Qt::PointingHandCursor ) );
 	m_openAudioFileButton->move( 200, 90 );
 	m_openAudioFileButton->setActiveGraphic( embed::getIconPixmap(
@@ -131,7 +128,8 @@ audioFileProcessor::audioFileProcessor( channelTrack * _channel_track ) :
 			"are not reset, so please don't wonder if your sample "
 			"doesn't sound like the original one..." ) );
 	
-	m_reverseButton = new pixmapButton( this );
+	m_reverseButton = new pixmapButton( this, NULL, eng(), NULL );
+	m_reverseButton->setCheckable( TRUE );
 	m_reverseButton->move( 160, 124 );
 	m_reverseButton->setActiveGraphic( PLUGIN_NAME::getIconPixmap(
 							"reverse_on" ) );
@@ -150,7 +148,8 @@ audioFileProcessor::audioFileProcessor( channelTrack * _channel_track ) :
 			"This is useful for cool effects, e.g. a reversed "
 			"crash." ) );
 
-	m_loopButton = new pixmapButton( this );
+	m_loopButton = new pixmapButton( this, NULL, eng(), NULL );
+	m_loopButton->setCheckable( TRUE );
 	m_loopButton->move( 180, 124 );
 	m_loopButton->setActiveGraphic( PLUGIN_NAME::getIconPixmap(
 								"loop_on" ) );
@@ -170,10 +169,11 @@ audioFileProcessor::audioFileProcessor( channelTrack * _channel_track ) :
 			"This is useful for things like string- and choir-"
 			"samples." ) );
 
-	m_ampKnob = new knob( knobDark_28, this, tr( "Amplify" ) );
+	m_ampKnob = new volumeKnob( knobDark_28, this, tr( "Amplify" ), eng(),
+									NULL );
 	m_ampKnob->setRange( 0, 500, 1.0f );
 	m_ampKnob->move( 6, 114 );
-	m_ampKnob->setValue( 100.0f, TRUE );
+	m_ampKnob->setInitValue( 100.0f );
 	m_ampKnob->setHintText( tr( "Amplify:" )+" ", "%" );
 	m_ampKnob->setLabel( tr( "AMP" ) );
 	connect( m_ampKnob, SIGNAL( valueChanged( float ) ), this,
@@ -188,10 +188,11 @@ audioFileProcessor::audioFileProcessor( channelTrack * _channel_track ) :
 			"Otherwise it will be amplified up or down (your "
 			"actual sample-file isn't touched!)" ) );
 
-	m_startKnob = new knob( knobDark_28, this, tr( "Start of sample" ) );
+	m_startKnob = new knob( knobDark_28, this, tr( "Start of sample" ),
+								eng(), NULL );
 	m_startKnob->setRange( 0.0f, 1.0f, 0.00001f );
 	m_startKnob->move( 46, 114 );
-	m_startKnob->setValue( 0.0f, TRUE );
+	m_startKnob->setInitValue( 0.0f );
 	m_startKnob->setHintText( tr( "Startpoint:" )+" ", "" );
 	m_startKnob->setLabel( tr( "START" ) );
 	connect( m_startKnob, SIGNAL( valueChanged( float ) ), this,
@@ -207,10 +208,11 @@ audioFileProcessor::audioFileProcessor( channelTrack * _channel_track ) :
 			"which AudioFileProcessor returns if a note is longer "
 			"than the sample between start- and end-point." ) );
 
-	m_endKnob = new knob( knobDark_28, this, tr( "End of sample" ) );
+	m_endKnob = new knob( knobDark_28, this, tr( "End of sample" ), eng(),
+									NULL );
 	m_endKnob->setRange( 0.0f, 1.0f, 0.00001f );
 	m_endKnob->move( 84, 114 );
-	m_endKnob->setValue( 1.0f, TRUE );
+	m_endKnob->setInitValue( 1.0f );
 	m_endKnob->setHintText( tr( "Endpoint:" )+" ", "" );
 	m_endKnob->setLabel( tr( "END" ) );
 	connect( m_endKnob, SIGNAL( valueChanged( float ) ), this,
@@ -226,16 +228,12 @@ audioFileProcessor::audioFileProcessor( channelTrack * _channel_track ) :
 			"AudioFileProcessor returns if a note is longer than "
 			"the sample between start- and end-point." ) );
 
-	m_viewLinesPB = new pixmapButton( this );
+	m_viewLinesPB = new pixmapButton( this, NULL, eng(), NULL );
 	m_viewLinesPB->move( 154, 158 );
 	m_viewLinesPB->setBgGraphic( getBackground( m_viewLinesPB ) );
 	if( m_drawMethod == sampleBuffer::LINE_CONNECT )
 	{
-#ifdef QT4
 		m_viewLinesPB->setChecked( TRUE );
-#else
-		m_viewLinesPB->setOn( TRUE );
-#endif
 	}
 	connect( m_viewLinesPB, SIGNAL( toggled( bool ) ), this,
 					SLOT( lineDrawBtnToggled( bool ) ) );
@@ -249,16 +247,12 @@ audioFileProcessor::audioFileProcessor( channelTrack * _channel_track ) :
 			"sound itself. It just gives you another view to your "
 			"sample." ) );
 
-	m_viewDotsPB = new pixmapButton( this );
+	m_viewDotsPB = new pixmapButton( this, NULL, eng(), NULL );
 	m_viewDotsPB->move( 204, 158 );
 	m_viewDotsPB->setBgGraphic( getBackground( m_viewDotsPB ) );
 	if( m_drawMethod == sampleBuffer::DOTS )
 	{
-#ifdef QT4
 		m_viewDotsPB->setChecked( TRUE );
-#else
-		m_viewDotsPB->setOn( TRUE );
-#endif
 	}
 	connect( m_viewDotsPB, SIGNAL( toggled( bool ) ), this,
 					SLOT( dotDrawBtnToggled( bool ) ) );
@@ -271,13 +265,12 @@ audioFileProcessor::audioFileProcessor( channelTrack * _channel_track ) :
 			"with dots. This doesn't change the sound itself. "
 			"It just gives you another view to your sample." ) );
 	
-	QButtonGroup * view_group = new QButtonGroup( this );
+	automatableButtonGroup * view_group = new automatableButtonGroup( this,
+							NULL, eng(), NULL );
 	view_group->addButton( m_viewLinesPB );
 	view_group->addButton( m_viewDotsPB );
-	view_group->setExclusive( TRUE );
-#ifndef QT4
-	view_group->hide();
 
+#ifndef QT4
 	setBackgroundMode( Qt::NoBackground );
 #endif
 	setAcceptDrops( TRUE );
@@ -294,22 +287,23 @@ audioFileProcessor::~audioFileProcessor()
 
 
 void audioFileProcessor::saveSettings( QDomDocument & _doc,
-							QDomElement & _parent )
+							QDomElement & _this )
 {
-	QDomElement afp_de = _doc.createElement( nodeName() );
-	afp_de.setAttribute( "src", m_sampleBuffer.audioFile() );
-	afp_de.setAttribute( "sframe", QString::number(
+	_this.setAttribute( "src", m_sampleBuffer.audioFile() );
+	if( m_sampleBuffer.audioFile() == "" )
+	{
+		QString s;
+		_this.setAttribute( "sampledata", m_sampleBuffer.toBase64( s ) );
+	}
+	_this.setAttribute( "sframe", QString::number(
 						m_sampleBuffer.startFrame() /
 					(float)m_sampleBuffer.frames() ) );
-	afp_de.setAttribute( "eframe", QString::number(
+	_this.setAttribute( "eframe", QString::number(
 						m_sampleBuffer.endFrame() /
 					(float)m_sampleBuffer.frames() ) );
-	afp_de.setAttribute( "reversed", QString::number(
-					m_reverseButton->isChecked() ) );
-	afp_de.setAttribute( "looped", QString::number(
-					m_loopButton->isChecked() ) );
-	afp_de.setAttribute( "amp", QString::number( m_ampKnob->value() ) );
-	_parent.appendChild( afp_de );
+	m_reverseButton->saveSettings( _doc, _this, "reversed" );
+	m_loopButton->saveSettings( _doc, _this, "looped" );
+	m_ampKnob->saveSettings( _doc, _this, "amp" );
 }
 
 
@@ -317,12 +311,35 @@ void audioFileProcessor::saveSettings( QDomDocument & _doc,
 
 void audioFileProcessor::loadSettings( const QDomElement & _this )
 {
-	setAudioFile( _this.attribute( "src" ) );
+	if( _this.attribute( "src" ) != "" )
+	{
+		setAudioFile( _this.attribute( "src" ), FALSE );
+	}
+	else if( _this.attribute( "sampledata" ) != "" )
+	{
+		m_sampleBuffer.loadFromBase64( _this.attribute( "srcdata" ) );
+	}
 	setStartAndEndKnob( _this.attribute( "sframe" ).toFloat(),
 				_this.attribute( "eframe" ).toFloat() );  
-	m_reverseButton->setChecked( _this.attribute( "reversed" ).toInt() );
-	m_loopButton->setChecked( _this.attribute( "looped" ).toInt() );
-	m_ampKnob->setValue( _this.attribute( "amp" ).toFloat() );
+	m_reverseButton->loadSettings( _this, "reversed" );
+	m_loopButton->loadSettings( _this, "looped" );
+	m_ampKnob->loadSettings( _this, "amp" );
+}
+
+
+
+
+void audioFileProcessor::setParameter( const QString & _param,
+							const QString & _value )
+{
+	if( _param == "samplefile" )
+	{
+		setAudioFile( _value );
+	}
+	else if( _param == "sampledata" )
+	{
+		m_sampleBuffer.loadFromBase64( _value );
+	}
 }
 
 
@@ -336,24 +353,12 @@ QString audioFileProcessor::nodeName( void ) const
 
 
 
-void audioFileProcessor::setParameter( const QString & _param,
-							const QString & _value )
-{
-	if( _param == "audiofile" )
-	{
-		setAudioFile( _value );
-	}
-}
-
-
-
-
 Uint32 audioFileProcessor::getBeatLen( notePlayHandle * _n ) const
 {
 	const float freq_factor = BASE_FREQ /
-				( getChannelTrack()->frequency( _n ) *
+				( getInstrumentTrack()->frequency( _n ) *
 						DEFAULT_SAMPLE_RATE /
-						mixer::inst()->sampleRate() );
+					eng()->getMixer()->sampleRate() );
 
 	return( static_cast<Uint32>( floorf( ( m_sampleBuffer.endFrame() -
 						m_sampleBuffer.startFrame() ) *
@@ -363,15 +368,16 @@ Uint32 audioFileProcessor::getBeatLen( notePlayHandle * _n ) const
 
 
 
-void audioFileProcessor::setAudioFile( const QString & _audio_file )
+void audioFileProcessor::setAudioFile( const QString & _audio_file, bool _rename )
 {
 	// is current channel-name equal to previous-filename??
-	if( getChannelTrack()->name() ==
+	if( _rename && 
+		( getInstrumentTrack()->name() ==
 			QFileInfo( m_sampleBuffer.audioFile() ).fileName() ||
-		m_sampleBuffer.audioFile() == "" )
+		   	m_sampleBuffer.audioFile() == "" ) )
 	{
 		// then set it to new one
-		getChannelTrack()->setName( QFileInfo( _audio_file
+		getInstrumentTrack()->setName( QFileInfo( _audio_file
 								).fileName() );
 	}
 	// else we don't touch the channel-name, because the user named it self
@@ -386,19 +392,19 @@ void audioFileProcessor::setAudioFile( const QString & _audio_file )
 
 void audioFileProcessor::playNote( notePlayHandle * _n )
 {
-	const Uint32 frames = mixer::inst()->framesPerAudioBuffer();
+	const Uint32 frames = eng()->getMixer()->framesPerAudioBuffer();
 	sampleFrame * buf = bufferAllocator::alloc<sampleFrame>( frames );
 
 	// calculate frequency of note
-	const float note_freq = getChannelTrack()->frequency( _n ) /
-						( mixer::inst()->sampleRate() /
+	const float note_freq = getInstrumentTrack()->frequency( _n ) /
+					( eng()->getMixer()->sampleRate() /
 							DEFAULT_SAMPLE_RATE );
 	if( m_sampleBuffer.play( buf, _n->totalFramesPlayed(),
 					frames, note_freq,
 					m_loopButton->isChecked(),
 					&_n->m_pluginData ) == TRUE )
 	{
-		getChannelTrack()->processAudioBuffer( buf, frames, _n );
+		getInstrumentTrack()->processAudioBuffer( buf, frames, _n );
 	}
 	bufferAllocator::free( buf );
 }
@@ -419,8 +425,12 @@ void audioFileProcessor::deleteNotePluginData( notePlayHandle * _n )
 
 void audioFileProcessor::dragEnterEvent( QDragEnterEvent * _dee )
 {
-	stringPairDrag::processDragEnterEvent( _dee,
-		QString( "samplefile,tco_%1" ).arg( track::SAMPLE_TRACK ) );
+	if( stringPairDrag::processDragEnterEvent( _dee,
+		QString( "samplefile,tco_%1" ).arg( track::SAMPLE_TRACK ) ) ==
+									FALSE )
+	{
+		_dee->ignore();
+	}
 }
 
 
@@ -432,7 +442,7 @@ void audioFileProcessor::dropEvent( QDropEvent * _de )
 	QString value = stringPairDrag::decodeValue( _de );
 	if( type == "samplefile" )
 	{
-		setAudioFile( stringPairDrag::decodeValue( _de ) );
+		setAudioFile( value );
 		_de->accept();
 	}
 	else if( type == QString( "tco_%1" ).arg( track::SAMPLE_TRACK ) )
@@ -442,6 +452,10 @@ void audioFileProcessor::dropEvent( QDropEvent * _de )
 							attribute( "src" ) );
 		_de->accept();
 	}
+	else
+	{
+		_de->ignore();
+	}
 }
 
 
@@ -494,8 +508,8 @@ void audioFileProcessor::paintEvent( QPaintEvent * )
 
 	p.setPen( QColor( 0xFF, 0xAA, 0x00 ) );
 	const QRect graph_rect( 4, 174, 241, 70 );
-	const Uint32 frames = tMax( m_sampleBuffer.frames(),
-						static_cast<Uint32>( 1 ) );
+	const f_cnt_t frames = tMax( m_sampleBuffer.frames(),
+						static_cast<f_cnt_t>( 1 ) );
 	const Uint16 start_frame_x = m_sampleBuffer.startFrame() *
 						graph_rect.width() / frames;
 	const Uint16 end_frame_x = m_sampleBuffer.endFrame() *
@@ -528,7 +542,7 @@ void audioFileProcessor::sampleUpdated( void )
 	QPainter p( &m_graph );
 #endif
 	p.setPen( QColor( 64, 255, 160 ) );
-	m_sampleBuffer.drawWaves( p, QRect( 2, 2, m_graph.width() - 4,
+	m_sampleBuffer.visualize( p, QRect( 2, 2, m_graph.width() - 4,
 							m_graph.height() - 4 ),
 								m_drawMethod );
 	update();
@@ -540,7 +554,7 @@ void audioFileProcessor::sampleUpdated( void )
 void audioFileProcessor::reverseBtnToggled( bool _on )
 {
 	m_sampleBuffer.setReversed( _on );
-	songEditor::inst()->setModified();
+	eng()->getSongEditor()->setModified();
 }
 
 
@@ -548,7 +562,7 @@ void audioFileProcessor::reverseBtnToggled( bool _on )
 
 void audioFileProcessor::lineDrawBtnToggled( bool _on )
 {
-	if( _on )
+	if( _on == TRUE )
 	{
 		m_drawMethod = sampleBuffer::LINE_CONNECT;
 		sampleUpdated();
@@ -560,7 +574,7 @@ void audioFileProcessor::lineDrawBtnToggled( bool _on )
 
 void audioFileProcessor::dotDrawBtnToggled( bool _on )
 {
-	if( _on )
+	if( _on == TRUE )
 	{
 		m_drawMethod = sampleBuffer::DOTS;
 		sampleUpdated();
@@ -572,7 +586,7 @@ void audioFileProcessor::dotDrawBtnToggled( bool _on )
 
 void audioFileProcessor::ampKnobChanged( float _val )
 {
-	m_sampleBuffer.setAmplification( _val/100.0f );
+	m_sampleBuffer.setAmplification( _val / 100.0f );
 }
 
 
@@ -608,7 +622,7 @@ void audioFileProcessor::startKnobChanged( float _new_value )
 	}
 	else
 	{
-		m_startKnob->setValue( m_endKnob->value() - 0.01 );
+		m_startKnob->setValue( m_endKnob->value() - 0.01f );
 	}
 	update();
 }
@@ -633,7 +647,7 @@ void audioFileProcessor::endKnobChanged( float _new_value )
 	}
 	else
 	{
-		m_endKnob->setValue( m_startKnob->value() + 0.01 );
+		m_endKnob->setValue( m_startKnob->value() + 0.01f );
 	}
 	update();
 }
@@ -647,7 +661,7 @@ void audioFileProcessor::openAudioFile( void )
 	if( af != "" )
 	{
 		setAudioFile( af );
-		songEditor::inst()->setModified();
+		eng()->getSongEditor()->setModified();
 	}
 }
 
@@ -661,16 +675,12 @@ extern "C"
 plugin * lmms_plugin_main( void * _data )
 {
 	return( new audioFileProcessor(
-				static_cast<channelTrack *>( _data ) ) );
+				static_cast<instrumentTrack *>( _data ) ) );
 }
 
 
 }
 
 
-
-#undef isChecked
-#undef isOn
-
 #include "audio_file_processor.moc"
 
