@@ -1,7 +1,9 @@
+#ifndef SINGLE_SOURCE_COMPILE
+
 /*
  * audio_alsa.cpp - device-class which implements ALSA-PCM-output
  *
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -28,8 +30,8 @@
 
 #ifdef QT4
 
-#include <QLineEdit>
-#include <QLabel>
+#include <QtGui/QLineEdit>
+#include <QtGui/QLabel>
 
 #else
 
@@ -54,10 +56,12 @@
 
 
 
-audioALSA::audioALSA( Uint32 _sample_rate, bool & _success_ful ) :
-	audioDevice( _sample_rate, tLimit<int>( configManager::inst()->value(
-					"audioalsa", "channels" ).toInt(),
-					DEFAULT_CHANNELS, SURROUND_CHANNELS ) ),
+audioALSA::audioALSA( const sample_rate_t _sample_rate, bool & _success_ful,
+							mixer * _mixer ) :
+	audioDevice( _sample_rate, tLimit<ch_cnt_t>(
+		configManager::inst()->value( "audioalsa", "channels" ).toInt(),
+					DEFAULT_CHANNELS, SURROUND_CHANNELS ),
+								_mixer ),
 	m_handle( NULL ),
 	m_hwParams( NULL ),
 	m_swParams( NULL ),
@@ -181,7 +185,15 @@ void audioALSA::startProcessing( void )
 {
 	if( !isRunning() )
 	{
-		start();
+		start(
+#ifdef QT4
+			QThread::HighPriority
+#else
+#if QT_VERSION >= 0x030505
+			QThread::HighestPriority
+#endif
+#endif
+							);
 	}
 }
 
@@ -205,22 +217,21 @@ void audioALSA::run( void )
 {
 	surroundSampleFrame * temp =
 			bufferAllocator::alloc<surroundSampleFrame>(
-					mixer::inst()->framesPerAudioBuffer() );
-	outputSampleType * outbuf =
-				bufferAllocator::alloc<outputSampleType>(
-					mixer::inst()->framesPerAudioBuffer() *
+					getMixer()->framesPerAudioBuffer() );
+	int_sample_t * outbuf = bufferAllocator::alloc<int_sample_t>(
+					getMixer()->framesPerAudioBuffer() *
 								channels() );
 	m_quit = FALSE;
 
 	while( m_quit == FALSE )
 	{
-		const Uint32 frames = getNextBuffer( temp );
+		const f_cnt_t frames = getNextBuffer( temp );
 
-		convertToS16( temp, frames, mixer::inst()->masterGain(), outbuf,
-					m_littleEndian != isLittleEndian() );
+		convertToS16( temp, frames, getMixer()->masterGain(), outbuf,
+					m_littleEndian );
 
-		Uint32 frame = 0;
-		outputSampleType * ptr = outbuf;
+		f_cnt_t frame = 0;
+		int_sample_t * ptr = outbuf;
 
 		while( frame < frames )
 		{
@@ -253,7 +264,8 @@ void audioALSA::run( void )
 
 
 
-int audioALSA::setHWParams( Uint32 _sample_rate, Uint32 _channels,
+int audioALSA::setHWParams( const sample_rate_t _sample_rate,
+						const ch_cnt_t _channels,
 						snd_pcm_access_t _access )
 {
 	int err, dir;
@@ -328,7 +340,7 @@ int audioALSA::setHWParams( Uint32 _sample_rate, Uint32 _channels,
 		}
 	}
 
-	m_periodSize = mixer::inst()->framesPerAudioBuffer();
+	m_periodSize = getMixer()->framesPerAudioBuffer();
 	m_bufferSize = m_periodSize * 8;
 	dir = 0;
 	err = snd_pcm_hw_params_set_period_size_near( m_handle, m_hwParams,
@@ -447,7 +459,7 @@ audioALSA::setupWidget::setupWidget( QWidget * _parent ) :
 	dev_lbl->setGeometry( 10, 40, 160, 10 );
 
 	m_channels = new lcdSpinBox( DEFAULT_CHANNELS, SURROUND_CHANNELS, 1,
-									this );
+						this, NULL, NULL, NULL );
 	m_channels->setStep( 2 );
 	m_channels->setLabel( tr( "CHANNELS" ) );
 	m_channels->setValue( configManager::inst()->value( "audioalsa",
@@ -478,3 +490,5 @@ void audioALSA::setupWidget::saveSettings( void )
 
 #endif
 
+
+#endif
