@@ -1,3 +1,5 @@
+#ifndef SINGLE_SOURCE_COMPILE
+
 /*
  * audio_file_ogg.cpp - audio-device which encodes wave-stream and writes it
  *                      into an OGG-file. This is used for song-export.
@@ -5,7 +7,7 @@
  * This file is based on encode.c from vorbis-tools-source, for more information
  * see below.
  *
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -39,12 +41,17 @@
 #include <cstring>
 
 
-audioFileOgg::audioFileOgg( Uint32 _sample_rate, Uint32 _channels,
-				bool & _success_ful, const QString & _file,
-				bool _use_vbr, Uint16 _nom_bitrate,
-				Uint16 _min_bitrate, Uint16 _max_bitrate ) :
+audioFileOgg::audioFileOgg( const sample_rate_t _sample_rate,
+				const ch_cnt_t _channels,
+				bool & _success_ful,
+				const QString & _file,
+				const bool _use_vbr,
+				const bitrate_t _nom_bitrate,
+				const bitrate_t _min_bitrate,
+				const bitrate_t _max_bitrate,
+				mixer * _mixer ) :
 	audioFileDevice( _sample_rate, _channels, _file, _use_vbr,
-				_nom_bitrate, _min_bitrate, _max_bitrate )
+			_nom_bitrate, _min_bitrate, _max_bitrate, _mixer )
 {
 	_success_ful = startEncoding();
 }
@@ -60,9 +67,9 @@ audioFileOgg::~audioFileOgg()
 
 
 
-inline int audioFileOgg::writePage( void )
+inline Sint32 audioFileOgg::writePage( void )
 {
-	int written = writeData( m_og.header, m_og.header_len );
+	Sint32 written = writeData( m_og.header, m_og.header_len );
 	written += writeData( m_og.body, m_og.body_len );
 	return( written );
 }
@@ -75,7 +82,7 @@ bool audioFileOgg::startEncoding( void )
 	vorbis_comment vc;
 	char * comments = "Cool=This song has been made using Linux "
 							"MultiMedia Studio";
-	int comment_length = strlen( comments );
+	Sint32 comment_length = strlen( comments );
 
 	vc.user_comments = &comments;
 	vc.comment_lengths = &comment_length;
@@ -86,18 +93,15 @@ bool audioFileOgg::startEncoding( void )
 	// vbr enabled?
 	if( useVBR() == 0 )
 	{
-		m_managed 	= -1;		// we don't want vbr
 		m_minBitrate 	= nominalBitrate();	// min for vbr
 		m_maxBitrate 	= nominalBitrate();	// max for vbr
 	}
 	else
 	{
-		m_managed 	= 0;		// let's use vbr
 		m_minBitrate 	= minBitrate();	// min for vbr
 		m_maxBitrate 	= maxBitrate();	// max for vbr
 	}
 
-	m_bitrate 	= nominalBitrate();	// nominal bitrate
 	m_rate 		= sampleRate();		// default-samplerate
 	m_serialNo 	= 0;			// track-num?
 	m_comments 	= &vc;			// comments for ogg-file
@@ -107,7 +111,7 @@ bool audioFileOgg::startEncoding( void )
 
 	if( vorbis_encode_setup_managed( &m_vi, m_channels, m_rate,
 			( m_maxBitrate > 0 )? m_maxBitrate * 1000 : -1,
-						m_bitrate * 1000, 
+						nominalBitrate() * 1000, 
 			( m_minBitrate > 0 )? m_minBitrate * 1000 : -1 ) )
 	{
 		printf( "Mode initialization failed: invalid parameters for "
@@ -116,11 +120,11 @@ bool audioFileOgg::startEncoding( void )
 		return( FALSE );
 	}
     
-	if( m_managed && m_bitrate < 0 )
+	if( useVBR() == FALSE )
 	{
 		vorbis_encode_ctl( &m_vi, OV_ECTL_RATEMANAGE_AVG, NULL );
 	}
-	else if( !m_managed )
+	else if( useVBR() == TRUE )
 	{
 		// Turn off management entirely (if it was turned on).
 		vorbis_encode_ctl( &m_vi, OV_ECTL_RATEMANAGE_SET, NULL );
@@ -159,7 +163,7 @@ bool audioFileOgg::startEncoding( void )
 		{
 			break;
 		}
-		int ret = writePage();
+		Sint32 ret = writePage();
 		if( ret != m_og.header_len + m_og.body_len )
 		{
 			// clean up
@@ -174,18 +178,18 @@ bool audioFileOgg::startEncoding( void )
 
 
 
-void FASTCALL audioFileOgg::writeBuffer( surroundSampleFrame * _ab,
-						Uint32 _frames,
-						float _master_gain )
+void FASTCALL audioFileOgg::writeBuffer( const surroundSampleFrame * _ab,
+						const fpab_t _frames,
+						const float _master_gain )
 {
 	int eos = 0;
 
 	float * * buffer = vorbis_analysis_buffer( &m_vd, _frames *
 							BYTES_PER_SAMPLE *
 								channels() );
-	for( Uint32 frame = 0; frame < _frames; ++frame )
+	for( fpab_t frame = 0; frame < _frames; ++frame )
 	{
-		for( Uint8 chnl = 0; chnl < channels(); ++chnl )
+		for( ch_cnt_t chnl = 0; chnl < channels(); ++chnl )
 		{
 			buffer[chnl][frame] = _ab[frame][chnl] * _master_gain;
 		}
@@ -255,3 +259,5 @@ void audioFileOgg::finishEncoding( void )
 
 #endif
 
+
+#endif
