@@ -1,7 +1,9 @@
+#ifndef SINGLE_SOURCE_COMPILE
+
 /*
  * audio_jack.cpp - support for JACK-transport
  *
- * Copyright (c) 2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2005-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -30,9 +32,9 @@
 
 #ifdef QT4
 
-#include <QLineEdit>
-#include <QLabel>
-#include <QMessageBox>
+#include <QtGui/QLineEdit>
+#include <QtGui/QLabel>
+#include <QtGui/QMessageBox>
 
 #else
 
@@ -43,6 +45,9 @@
 #endif
 
 
+#include <stdlib.h>
+
+
 #include "debug.h"
 #include "templates.h"
 #include "gui_templates.h"
@@ -53,15 +58,17 @@
 
 
 
-audioJACK::audioJACK( Uint32 _sample_rate, bool & _success_ful ) :
+audioJACK::audioJACK( const sample_rate_t _sample_rate, bool & _success_ful,
+							mixer * _mixer ) :
 	audioDevice( _sample_rate, tLimit<int>( configManager::inst()->value(
 					"audiojack", "channels" ).toInt(),
-					DEFAULT_CHANNELS, SURROUND_CHANNELS ) ),
+					DEFAULT_CHANNELS, SURROUND_CHANNELS ),
+								_mixer ),
 	m_client( NULL ),
 	m_stopped( FALSE ),
 	m_processCallbackMutex(),
 	m_outBuf( bufferAllocator::alloc<surroundSampleFrame>(
-				mixer::inst()->framesPerAudioBuffer() ) ),
+				getMixer()->framesPerAudioBuffer() ) ),
 	m_framesDoneInCurBuf( 0 ),
 	m_framesToDoInCurBuf( 0 )
 {
@@ -105,7 +112,9 @@ audioJACK::audioJACK( Uint32 _sample_rate, bool & _success_ful ) :
 #endif
 					jack_get_client_name( m_client ) );
 	}
-#else
+
+#else /* OLD_JACK */
+
 	m_client = jack_client_new( client_name.
 #ifdef QT4
 						toAscii().constData()
@@ -113,6 +122,12 @@ audioJACK::audioJACK( Uint32 _sample_rate, bool & _success_ful ) :
 						ascii()
 #endif
 					);
+	if( m_client == NULL )
+	{
+		printf( "jack_client_new() failed\n" );
+		return;
+	}
+
 #endif
 
 	// set process-callback
@@ -166,7 +181,7 @@ audioJACK::audioJACK( Uint32 _sample_rate, bool & _success_ful ) :
 
 
 	// try to sync JACK's and LMMS's buffer-size
-	jack_set_buffer_size( m_client, mixer::inst()->framesPerAudioBuffer() );
+	jack_set_buffer_size( m_client, getMixer()->framesPerAudioBuffer() );
 
 
 
@@ -335,7 +350,7 @@ int audioJACK::processCallback( jack_nframes_t _nframes, void * _udata )
 	}
 
 /*	const Uint32 frames = tMin<Uint32>( _nframes,
-					mixer::inst()->framesPerAudioBuffer() );
+					getMixer()->framesPerAudioBuffer() );
 	for( jackPortMap::iterator it = _this->m_portMap.begin();
 					it != _this->m_portMap.end(); ++it )
 	{
@@ -367,11 +382,12 @@ int audioJACK::processCallback( jack_nframes_t _nframes, void * _udata )
 		{
 			for( Uint8 chnl = 0; chnl < _this->channels(); ++chnl )
 			{
-				for( Uint32 frame = 0; frame < todo; ++frame )
+				for( jack_nframes_t frame = 0; frame < todo;
+								++frame )
 				{
 					outbufs[chnl][done+frame] = 
 		_this->m_outBuf[_this->m_framesDoneInCurBuf+frame][chnl] *
-						mixer::inst()->masterGain();
+						_this->getMixer()->masterGain();
 				}
 			}
 		}
@@ -435,7 +451,7 @@ audioJACK::setupWidget::setupWidget( QWidget * _parent ) :
 	cn_lbl->setGeometry( 10, 40, 160, 10 );
 
 	m_channels = new lcdSpinBox( DEFAULT_CHANNELS, SURROUND_CHANNELS, 1,
-									this );
+						this, NULL, NULL, NULL );
 	m_channels->setStep( 2 );
 	m_channels->setLabel( tr( "CHANNELS" ) );
 	m_channels->setValue( configManager::inst()->value( "audiojack",
@@ -463,4 +479,6 @@ void audioJACK::setupWidget::saveSettings( void )
 }
 
 
+#endif
+
 #endif
