@@ -1,7 +1,9 @@
+#ifndef SINGLE_SOURCE_COMPILE
+
 /*
  * audio_oss.cpp - device-class that implements OSS-PCM-output
  *
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -32,9 +34,9 @@
 
 #ifdef QT4
 
-#include <QFileInfo>
-#include <QLineEdit>
-#include <QLabel>
+#include <QtCore/QFileInfo>
+#include <QtGui/QLabel>
+#include <QtGui/QLineEdit>
 
 #else
 
@@ -91,10 +93,12 @@
 
 
 
-audioOSS::audioOSS( Uint32 _sample_rate, bool & _success_ful ) :
-	audioDevice( _sample_rate, tLimit<int>( configManager::inst()->value(
-					"audiooss", "channels" ).toInt(),
-					DEFAULT_CHANNELS, SURROUND_CHANNELS ) ),
+audioOSS::audioOSS( const sample_rate_t _sample_rate, bool & _success_ful,
+							mixer * _mixer ) :
+	audioDevice( _sample_rate, tLimit<ch_cnt_t>(
+		configManager::inst()->value( "audiooss", "channels" ).toInt(),
+					DEFAULT_CHANNELS, SURROUND_CHANNELS ),
+								_mixer ),
 	m_convertEndian( FALSE ),
 	m_quit( FALSE )
 {
@@ -124,9 +128,9 @@ audioOSS::audioOSS( Uint32 _sample_rate, bool & _success_ful ) :
 	}
 
 	int frag_spec;
-	for( frag_spec = 0; static_cast<unsigned int>( 0x01 << frag_spec ) <
-		mixer::inst()->framesPerAudioBuffer() * channels() *
-							BYTES_PER_OUTPUT_SAMPLE;
+	for( frag_spec = 0; static_cast<int>( 0x01 << frag_spec ) <
+		getMixer()->framesPerAudioBuffer() * channels() *
+							BYTES_PER_INT_SAMPLE;
 		++frag_spec )
 	{
 	}
@@ -284,7 +288,15 @@ void audioOSS::startProcessing( void )
 {
 	if( !isRunning() )
 	{
-		start();
+		start(
+#ifdef QT4
+			QThread::HighPriority
+#else
+#if QT_VERSION >= 0x030505
+			QThread::HighestPriority
+#endif
+#endif
+							);
 	}
 }
 
@@ -308,19 +320,18 @@ void audioOSS::run( void )
 {
 	surroundSampleFrame * temp =
 			bufferAllocator::alloc<surroundSampleFrame>(
-					mixer::inst()->framesPerAudioBuffer() );
-	outputSampleType * outbuf =
-				bufferAllocator::alloc<outputSampleType>(
-					mixer::inst()->framesPerAudioBuffer() *
+					getMixer()->framesPerAudioBuffer() );
+	int_sample_t * outbuf = bufferAllocator::alloc<int_sample_t>(
+					getMixer()->framesPerAudioBuffer() *
 								channels() );
 	m_quit = FALSE;
 
 	while( m_quit == FALSE )
 	{
-		const Uint32 frames = getNextBuffer( temp );
+		const fpab_t frames = getNextBuffer( temp );
 
 		int bytes = convertToS16( temp, frames,
-				mixer::inst()->masterGain(), outbuf,
+				getMixer()->masterGain(), outbuf,
 							m_convertEndian );
 		write( m_audioFD, outbuf, bytes );
 	}
@@ -343,7 +354,7 @@ audioOSS::setupWidget::setupWidget( QWidget * _parent ) :
 	dev_lbl->setGeometry( 10, 40, 160, 10 );
 
 	m_channels = new lcdSpinBox( DEFAULT_CHANNELS, SURROUND_CHANNELS, 1,
-									this );
+						this, NULL, NULL, NULL );
 	m_channels->setStep( 2 );
 	m_channels->setLabel( tr( "CHANNELS" ) );
 	m_channels->setValue( configManager::inst()->value( "audiooss",
@@ -374,3 +385,5 @@ void audioOSS::setupWidget::saveSettings( void )
 
 #endif
 
+
+#endif
