@@ -1,7 +1,9 @@
+#ifndef SINGLE_SOURCE_COMPILE
+
 /*
  * audio_sdl.cpp - device-class that performs PCM-output via SDL
  *
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -30,8 +32,8 @@
 
 #ifdef QT4
 
-#include <QLineEdit>
-#include <QLabel>
+#include <QtGui/QLabel>
+#include <QtGui/QLineEdit>
 
 #else
 
@@ -48,10 +50,11 @@
 
 
 
-audioSDL::audioSDL( Uint32 _sample_rate, bool & _success_ful ) :
-	audioDevice( _sample_rate, DEFAULT_CHANNELS ),
+audioSDL::audioSDL( const sample_rate_t _sample_rate, bool & _success_ful,
+							mixer * _mixer ) :
+	audioDevice( _sample_rate, DEFAULT_CHANNELS, _mixer ),
 	m_outBuf( bufferAllocator::alloc<surroundSampleFrame>(
-				mixer::inst()->framesPerAudioBuffer() ) ),
+				getMixer()->framesPerAudioBuffer() ) ),
 	m_convertEndian( FALSE )
 {
 	_success_ful = FALSE;
@@ -81,7 +84,7 @@ audioSDL::audioSDL( Uint32 _sample_rate, bool & _success_ful ) :
 						// of system, so we don't have
 						// to convert the buffers
 	m_audioHandle.channels = channels();
-	m_audioHandle.samples = mixer::inst()->framesPerAudioBuffer();
+	m_audioHandle.samples = getMixer()->framesPerAudioBuffer();
 
 	m_audioHandle.callback = sdlAudioCallback;
 	m_audioHandle.userdata = this;
@@ -142,11 +145,11 @@ void audioSDL::sdlAudioCallback( void * _udata, Uint8 * _buf, int _len )
 	assert( _this != NULL );
 #endif
 
-	const Uint32 frames = _this->getNextBuffer( _this->m_outBuf );
+	const fpab_t frames = _this->getNextBuffer( _this->m_outBuf );
 
 	_this->convertToS16( _this->m_outBuf, frames,
-						mixer::inst()->masterGain(),
-						(outputSampleType *)( _buf ),
+						_this->getMixer()->masterGain(),
+						(int_sample_t *)( _buf ),
 						_this->m_convertEndian );
 }
 
@@ -171,7 +174,6 @@ audioSDL::setupWidget::setupWidget( QWidget * _parent ) :
 
 audioSDL::setupWidget::~setupWidget()
 {
-
 }
 
 
@@ -184,4 +186,6 @@ void audioSDL::setupWidget::saveSettings( void )
 }
 
 
+#endif
+
 #endif
