@@ -1,8 +1,10 @@
+#ifndef SINGLE_SOURCE_COMPILE
+
 /*
  * arp_and_chords_tab_widget.cpp - widget for use in arp/chord-tab of 
- *                                 channel-window
+ *                                 instrument-track-window
  *
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -29,26 +31,20 @@
 #ifdef QT4
 
 #include <Qt/QtXml>
-#include <QButtonGroup>
-#include <QLabel>
-#include <QComboBox>
+#include <QtGui/QLabel>
 
 #else
 
-#include <qbuttongroup.h>
 #include <qdom.h>
 #include <qlabel.h>
-#include <qcombobox.h>
 #include <qwhatsthis.h>
 
-#define setChecked setOn
-
 #endif
 
 
-#ifdef HAVE_STDLIB_H
+/*#ifdef HAVE_STDLIB_H*/
 #include <stdlib.h>
-#endif
+/*#endif*/
 
 
 #include "arp_and_chords_tab_widget.h"
@@ -61,15 +57,15 @@
 #include "tooltip.h"
 #include "gui_templates.h"
 #include "tempo_sync_knob.h"
-#include "channel_track.h"
+#include "instrument_track.h"
+#include "led_checkbox.h"
+#include "preset_preview_play_handle.h"
+#include "combobox.h"
 
 
 
 arpAndChordsTabWidget::chord arpAndChordsTabWidget::s_chords[] =
 {
-	// thanks to the FL-team for this chords *lol* ;-) took me at least 3
-	// hours to get them all out of FL-arpeggiator...
-
 	{ arpAndChordsTabWidget::tr( "octave" ), { 0, -1 } },
 	{ arpAndChordsTabWidget::tr( "Major" ), { 0, 4, 7, -1 } },
 	{ arpAndChordsTabWidget::tr( "Majb5" ), { 0, 4, 6, -1 } },
@@ -153,9 +149,9 @@ arpAndChordsTabWidget::chord arpAndChordsTabWidget::s_chords[] =
 	{ arpAndChordsTabWidget::tr( "Diminished" ), { 0, 2, 3, 5, 6, 8, 9,
 								11, -1 } },
 	{ arpAndChordsTabWidget::tr( "Major pentatonic" ), { 0, 2, 4, 7, 10,
-								-1 } },
+									-1 } },
 	{ arpAndChordsTabWidget::tr( "Minor pentatonic" ), { 0, 3, 5, 7, 10,
-								-1 } },
+									-1 } },
 	{ arpAndChordsTabWidget::tr( "Jap in sen" ), { 0, 1, 5, 7, 10, -1 } },
 	{ arpAndChordsTabWidget::tr( "Major bebop" ), { 0, 2, 4, 5, 7, 8, 9,
 								11, -1 } },
@@ -164,7 +160,7 @@ arpAndChordsTabWidget::chord arpAndChordsTabWidget::s_chords[] =
 	{ arpAndChordsTabWidget::tr( "Blues" ), { 0, 3, 5, 6, 7, 10, -1 } },
 	{ arpAndChordsTabWidget::tr( "Arabic" ), { 0, 1, 4, 5, 7, 8, 11, -1 } },
 	{ arpAndChordsTabWidget::tr( "Enigmatic" ), { 0, 1, 4, 6, 8, 10, 11,
-								-1 } },
+									-1 } },
 	{ arpAndChordsTabWidget::tr( "Neopolitan" ), { 0, 1, 3, 5, 7, 9, 11,
 								-1 } },
 	{ arpAndChordsTabWidget::tr( "Neopolitan minor" ), { 0, 1, 3, 5, 7, 9,
@@ -173,14 +169,14 @@ arpAndChordsTabWidget::chord arpAndChordsTabWidget::s_chords[] =
 								11, -1 } },
 	{ arpAndChordsTabWidget::tr( "Dorian" ), { 0, 2, 3, 5, 7, 9, 10, -1 } },
 	{ arpAndChordsTabWidget::tr( "Phrygolydian" ), { 0, 1, 3, 5, 7, 8, 10,
-								-1 } },
+									-1 } },
 	{ arpAndChordsTabWidget::tr( "Lydian" ), { 0, 2, 4, 6, 7, 9, 11, -1 } },
 	{ arpAndChordsTabWidget::tr( "Mixolydian" ), { 0, 2, 4, 5, 7, 9, 10,
-								-1 } },
+									-1 } },
 	{ arpAndChordsTabWidget::tr( "Aeolian" ), { 0, 2, 3, 5, 7, 8, 10,
-								-1 } },
+									-1 } },
 	{ arpAndChordsTabWidget::tr( "Locrian" ), { 0, 1, 3, 5, 6, 8, 10,
-								-1 } },
+									-1 } },
 
 	{ "", { -1, -1 } }
 
@@ -194,22 +190,23 @@ const int CHORDS_GROUPBOX_HEIGHT = 65;
 const int ARP_GROUPBOX_X = CHORDS_GROUPBOX_X;
 const int ARP_GROUPBOX_Y = 10 + CHORDS_GROUPBOX_Y + CHORDS_GROUPBOX_HEIGHT;
 const int ARP_GROUPBOX_WIDTH = CHORDS_GROUPBOX_WIDTH;
-const int ARP_GROUPBOX_HEIGHT = 200 - ARP_GROUPBOX_Y;
+const int ARP_GROUPBOX_HEIGHT = 240 - ARP_GROUPBOX_Y;
 
 
 
-arpAndChordsTabWidget::arpAndChordsTabWidget( channelTrack * _channel_track ) :
-	QWidget( _channel_track->tabWidgetParent() ),
-	settings(),
-	m_arpDirection( UP )
+arpAndChordsTabWidget::arpAndChordsTabWidget( instrumentTrack * _instrument_track ) :
+	QWidget( _instrument_track->tabWidgetParent() ),
+	journallingObject( _instrument_track->eng() )
 {
-	m_chordsGroupBox = new groupBox( tr( "CHORDS" ), this );
+	m_chordsGroupBox = new groupBox( tr( "CHORDS" ), this, eng(),
+							_instrument_track );
 	m_chordsGroupBox->setGeometry( CHORDS_GROUPBOX_X, CHORDS_GROUPBOX_Y,
 						CHORDS_GROUPBOX_WIDTH,
 						CHORDS_GROUPBOX_HEIGHT );
 
-	m_chordsComboBox = new QComboBox( m_chordsGroupBox );
-	m_chordsComboBox->setFont( pointSize<9>( m_chordsComboBox->font() ) );
+	m_chordsComboBox = new comboBox( m_chordsGroupBox, tr( "Chord type" ),
+						eng(), _instrument_track );
+	m_chordsComboBox->setFont( pointSize<8>( m_chordsComboBox->font() ) );
 	m_chordsComboBox->setGeometry( 10, 25, 140, 22 );
 
 	for( int i = 0; s_chords[i].interval[0] != -1; ++i )
@@ -222,10 +219,11 @@ arpAndChordsTabWidget::arpAndChordsTabWidget( channelTrack * _channel_track ) :
 	}
 
 	m_chordRangeKnob = new knob( knobBright_26, m_chordsGroupBox,
-							tr( "Chord range" ) );
+						tr( "Chord range" ),
+						eng(), _instrument_track );
 	m_chordRangeKnob->setLabel( tr( "RANGE" ) );
-	m_chordRangeKnob->setRange( 1.0, 9.0, 1.0 );
-	m_chordRangeKnob->setValue( 1.0, TRUE );
+	m_chordRangeKnob->setRange( 1.0f, 9.0f, 1.0f );
+	m_chordRangeKnob->setInitValue( 1.0f );
 	m_chordRangeKnob->move( 164, 24 );
 	m_chordRangeKnob->setHintText( tr( "Chord range:" ) + " ", " " +
 							tr( "octave(s)" ) );
@@ -241,7 +239,8 @@ arpAndChordsTabWidget::arpAndChordsTabWidget( channelTrack * _channel_track ) :
 
 
 
-	m_arpGroupBox = new groupBox( tr( "ARPEGGIO" ), this );
+	m_arpGroupBox = new groupBox( tr( "ARPEGGIO" ), this, eng(),
+							_instrument_track );
 	m_arpGroupBox->setGeometry( ARP_GROUPBOX_X, ARP_GROUPBOX_Y,
 					ARP_GROUPBOX_WIDTH,
 					ARP_GROUPBOX_HEIGHT );
@@ -259,8 +258,9 @@ arpAndChordsTabWidget::arpAndChordsTabWidget( channelTrack * _channel_track ) :
 			"not played at the same time. Typical arpeggios are "
 			"major or minor triads. But there're a lot of other "
 			"possible chords, you can select." ) );
-	m_arpComboBox = new QComboBox( m_arpGroupBox );
-	m_arpComboBox->setFont( pointSize<9>( m_arpComboBox->font() ) );
+	m_arpComboBox = new comboBox( m_arpGroupBox, tr( "Arpeggio type" ),
+						eng(), _instrument_track );
+	m_arpComboBox->setFont( pointSize<8>( m_arpComboBox->font() ) );
 	m_arpComboBox->setGeometry( 10, 25, 140, 22 );
 
 	for( int i = 0; s_chords[i].interval[0] != -1; ++i )
@@ -271,11 +271,14 @@ arpAndChordsTabWidget::arpAndChordsTabWidget( channelTrack * _channel_track ) :
 #endif
 						) );
 	}
+
+
 	m_arpRangeKnob = new knob( knobBright_26, m_arpGroupBox,
-						tr( "Arpeggio range" ) );
+						tr( "Arpeggio range" ),
+						eng(), _instrument_track );
 	m_arpRangeKnob->setLabel( tr( "RANGE" ) );
-	m_arpRangeKnob->setRange( 1.0, 9.0, 1.0 );
-	m_arpRangeKnob->setValue( 1.0, TRUE );
+	m_arpRangeKnob->setRange( 1.0f, 9.0f, 1.0f );
+	m_arpRangeKnob->setInitValue( 1.0f );
 	m_arpRangeKnob->move( 164, 24 );
 	m_arpRangeKnob->setHintText( tr( "Arpeggio range:" ) + " ", " " +
 							tr( "octave(s)" ) );
@@ -289,10 +292,11 @@ arpAndChordsTabWidget::arpAndChordsTabWidget( channelTrack * _channel_track ) :
 			"amount of octaves." ) );
 
 	m_arpTimeKnob = new tempoSyncKnob( knobBright_26, m_arpGroupBox,
-							tr( "Arpeggio time" ) );
+						tr( "Arpeggio time" ),
+						eng(), _instrument_track );
 	m_arpTimeKnob->setLabel( tr( "TIME" ) );
-	m_arpTimeKnob->setRange( 10.0, 1000.0, 1.0 );
-	m_arpTimeKnob->setValue( 100.0, TRUE );
+	m_arpTimeKnob->setRange( 25.0f, 2000.0f, 1.0f );
+	m_arpTimeKnob->setInitValue( 100.0f );
 	m_arpTimeKnob->move( 164, 70 );
 	m_arpTimeKnob->setHintText( tr( "Arpeggio time:" ) + " ", " " +
 								tr( "ms" ) );
@@ -306,10 +310,11 @@ arpAndChordsTabWidget::arpAndChordsTabWidget( channelTrack * _channel_track ) :
 			"each arpeggio-tone should be played." ) );
 
 	m_arpGateKnob = new knob( knobBright_26, m_arpGroupBox,
-							tr( "Arpeggio gate" ) );
+						tr( "Arpeggio gate" ),
+						eng(), _instrument_track );
 	m_arpGateKnob->setLabel( tr( "GATE" ) );
-	m_arpGateKnob->setRange( 1.0, 200.0, 1.0 );
-	m_arpGateKnob->setValue( 100.0, TRUE );
+	m_arpGateKnob->setRange( 1.0f, 200.0f, 1.0f );
+	m_arpGateKnob->setInitValue( 100.0f );
 	m_arpGateKnob->move( 204, 70 );
 	m_arpGateKnob->setHintText( tr( "Arpeggio gate:" ) + " ", tr( "%" ) );
 #ifdef QT4
@@ -322,76 +327,88 @@ arpAndChordsTabWidget::arpAndChordsTabWidget( channelTrack * _channel_track ) :
 			"arpeggio-tone that should be played. With this you "
 			"can make cool staccato-arpeggios." ) );
 
-	m_arpDirectionLbl = new QLabel( tr( "DIRECTION:" ), m_arpGroupBox );
-	m_arpDirectionLbl->setGeometry( 10, 70, 64, 8 );
-	m_arpDirectionLbl->setFont( pointSize<6>( m_arpDirectionLbl->font() ) );
+	m_arpDirectionLbl = new QLabel( tr( "Direction:" ), m_arpGroupBox );
+	m_arpDirectionLbl->setGeometry( 10, 60, 64, 10 );
+	m_arpDirectionLbl->setFont( pointSize<7>( m_arpDirectionLbl->font() ) );
 
 
 
-	m_arpUpBtn = new pixmapButton( m_arpGroupBox );
-	m_arpUpBtn->move( 70, 70 );
-	m_arpUpBtn->setActiveGraphic( embed::getIconPixmap( "arp_up_on" ) );
-	m_arpUpBtn->setInactiveGraphic( embed::getIconPixmap( "arp_up_off" ) );
-#ifdef QT4
-	m_arpUpBtn->setChecked( TRUE );
-#else
-	m_arpUpBtn->setOn( TRUE );
+	pixmapButton * arp_up_btn = new pixmapButton( m_arpGroupBox, NULL,
+								eng(), NULL );
+	arp_up_btn->move( 10, 74 );
+	arp_up_btn->setActiveGraphic( embed::getIconPixmap( "arp_up_on" ) );
+	arp_up_btn->setInactiveGraphic( embed::getIconPixmap( "arp_up_off" ) );
+#ifdef QT3
+	arp_up_btn->setBackgroundMode( Qt::PaletteBackground );
 #endif
-#ifndef QT4
-	m_arpUpBtn->setBackgroundMode( Qt::PaletteBackground );
-#endif
-	toolTip::add( m_arpUpBtn, tr( "arpeggio direction = up" ) );
-	connect( m_arpUpBtn, SIGNAL( toggled( bool ) ), this,
-						SLOT( arpUpToggled( bool ) ) );
+	toolTip::add( arp_up_btn, tr( "arpeggio direction = up" ) );
 
-	m_arpDownBtn = new pixmapButton( m_arpGroupBox );
-	m_arpDownBtn->move( 90, 70 );
-	m_arpDownBtn->setActiveGraphic( embed::getIconPixmap( "arp_down_on" ) );
-	m_arpDownBtn->setInactiveGraphic( embed::getIconPixmap(
+
+	pixmapButton * arp_down_btn = new pixmapButton( m_arpGroupBox, NULL,
+								eng(), NULL );
+	arp_down_btn->move( 30, 74 );
+	arp_down_btn->setActiveGraphic( embed::getIconPixmap( "arp_down_on" ) );
+	arp_down_btn->setInactiveGraphic( embed::getIconPixmap(
 							"arp_down_off" ) );
-#ifndef QT4
-	m_arpDownBtn->setBackgroundMode( Qt::PaletteBackground );
+#ifdef QT3
+	arp_down_btn->setBackgroundMode( Qt::PaletteBackground );
 #endif
-	toolTip::add( m_arpDownBtn, tr( "arpeggio direction = down" ) );
-	connect( m_arpDownBtn, SIGNAL( toggled( bool ) ), this,
-					SLOT( arpDownToggled( bool ) ) );
+	toolTip::add( arp_down_btn, tr( "arpeggio direction = down" ) );
 
-	m_arpUpAndDownBtn = new pixmapButton( m_arpGroupBox );
-	m_arpUpAndDownBtn->move( 110, 70 );
-	m_arpUpAndDownBtn->setActiveGraphic( embed::getIconPixmap(
+
+	pixmapButton * arp_up_and_down_btn = new pixmapButton( m_arpGroupBox,
+							NULL, eng(), NULL );
+	arp_up_and_down_btn->move( 50, 74 );
+	arp_up_and_down_btn->setActiveGraphic( embed::getIconPixmap(
 						"arp_up_and_down_on" ) );
-	m_arpUpAndDownBtn->setInactiveGraphic( embed::getIconPixmap(
+	arp_up_and_down_btn->setInactiveGraphic( embed::getIconPixmap(
 						"arp_up_and_down_off" ) );
-#ifndef QT4
-	m_arpUpAndDownBtn->setBackgroundMode( Qt::PaletteBackground );
+#ifdef QT3
+	arp_up_and_down_btn->setBackgroundMode( Qt::PaletteBackground );
 #endif
-	toolTip::add( m_arpUpAndDownBtn,
+	toolTip::add( arp_up_and_down_btn,
 				tr( "arpeggio direction = up and down" ) );
-	connect( m_arpUpAndDownBtn, SIGNAL( toggled( bool ) ), this,
-					SLOT( arpUpAndDownToggled( bool ) ) );
 
-	m_arpRandomBtn = new pixmapButton( m_arpGroupBox );
-	m_arpRandomBtn->move( 130, 70 );
-	m_arpRandomBtn->setActiveGraphic( embed::getIconPixmap(
+
+	pixmapButton * arp_random_btn = new pixmapButton( m_arpGroupBox, NULL,
+								eng(), NULL );
+	arp_random_btn->move( 70, 74 );
+	arp_random_btn->setActiveGraphic( embed::getIconPixmap(
 							"arp_random_on" ) );
-	m_arpRandomBtn->setInactiveGraphic( embed::getIconPixmap(
+	arp_random_btn->setInactiveGraphic( embed::getIconPixmap(
 							"arp_random_off" ) );
-#ifndef QT4
-	m_arpRandomBtn->setBackgroundMode( Qt::PaletteBackground );
+#ifdef QT3
+	arp_random_btn->setBackgroundMode( Qt::PaletteBackground );
 #endif
-	toolTip::add( m_arpRandomBtn, tr( "arpeggio direction = random" ) );
-	connect( m_arpRandomBtn, SIGNAL( toggled( bool ) ), this,
-					SLOT( arpRandomToggled( bool ) ) );
+	toolTip::add( arp_random_btn, tr( "arpeggio direction = random" ) );
 
-	QButtonGroup * m_arpDirections_group = new QButtonGroup( this );
-	m_arpDirections_group->addButton( m_arpUpBtn );
-	m_arpDirections_group->addButton( m_arpDownBtn );
-	m_arpDirections_group->addButton( m_arpUpAndDownBtn );
-	m_arpDirections_group->addButton( m_arpRandomBtn );
-	m_arpDirections_group->setExclusive( TRUE );
-#ifndef QT4
-	m_arpDirections_group->hide();
-#endif
+	m_arpDirectionBtnGrp = new automatableButtonGroup( this,
+						tr( "Arpeggio direction" ),
+						eng(), _instrument_track );
+	m_arpDirectionBtnGrp->addButton( arp_up_btn );
+	m_arpDirectionBtnGrp->addButton( arp_down_btn );
+	m_arpDirectionBtnGrp->addButton( arp_up_and_down_btn );
+	m_arpDirectionBtnGrp->addButton( arp_random_btn );
+
+	m_arpDirectionBtnGrp->setInitValue( UP - 1 );
+
+
+	QLabel * mode_lbl = new QLabel( tr( "Mode:" ), m_arpGroupBox );
+	mode_lbl->setGeometry( 10, 104, 64, 10 );
+	mode_lbl->setFont( pointSize<7>( mode_lbl->font() ) );
+
+	m_arpModeComboBox = new comboBox( m_arpGroupBox, tr( "Arpeggio mode" ),
+						eng(), _instrument_track );
+	m_arpModeComboBox->setFont( pointSize<8>( m_arpModeComboBox->font() ) );
+	m_arpModeComboBox->setGeometry( 10, 118, 128, 22 );
+
+	m_arpModeComboBox->addItem( tr( "Free" ),
+					embed::getIconPixmap( "arp_free" ) );
+	m_arpModeComboBox->addItem( tr( "Sort" ),
+					embed::getIconPixmap( "arp_sort" ) );
+	m_arpModeComboBox->addItem( tr( "Sync" ),
+					embed::getIconPixmap( "arp_sync" ) );
+	//m_arpModeComboBox->setValue( 0 );
 }
 
 
@@ -418,11 +435,8 @@ void arpAndChordsTabWidget::processNote( notePlayHandle * _n )
 					m_chordsGroupBox->isActive() == TRUE )
 	{
 		// then insert sub-notes for chord
-#ifdef QT4
-		const int selected_chord = m_chordsComboBox->currentIndex();
-#else
-		const int selected_chord = m_chordsComboBox->currentItem();
-#endif
+		const int selected_chord = m_chordsComboBox->value();
+
 		for( int octave_cnt = 0;
 			octave_cnt < m_chordRangeKnob->value(); ++octave_cnt )
 		{
@@ -447,19 +461,21 @@ void arpAndChordsTabWidget::processNote( notePlayHandle * _n )
 					break;
 				}
 				// create copy of base-note
-				note note_copy( 0, 0, (tones)( sub_note_key %
+				note note_copy( NULL, 0, 0,
+						(tones)( sub_note_key %
 							NOTES_PER_OCTAVE ),
 						(octaves)( sub_note_key /
 							NOTES_PER_OCTAVE ),
 							_n->getVolume(),
 							_n->getPanning() );
+				note_copy.setDetuning( _n->detuning() );
 				// duplicate note-play-handle, only note is
 				// different
 				notePlayHandle * note_play_handle =
 					new notePlayHandle(
-						_n->getChannelTrack(),
+						_n->getInstrumentTrack(),
 						_n->framesAhead(),
-						_n->frames(), &note_copy );
+						_n->frames(), note_copy );
 				// add sub-note to base-note, now all stuff is
 				// done by notePlayHandle::play_note()
 				_n->addSubNote( note_play_handle );
@@ -470,44 +486,59 @@ void arpAndChordsTabWidget::processNote( notePlayHandle * _n )
 
 	// now follows code for arpeggio
 
-	if( _n->baseNote() == FALSE || m_arpDirection == OFF ||
-		!m_arpGroupBox->isActive() ||
-		( _n->released() && _n->releaseFramesDone() >=
+	if( _n->baseNote() == FALSE ||
+		( m_arpDirectionBtnGrp->value() + 1) == OFF ||
+			!m_arpGroupBox->isActive() ||
+			( _n->released() && _n->releaseFramesDone() >=
 					_n->actualReleaseFramesToDo() ) )
 	{
 		return;
 	}
 
 
-#ifdef QT4
-	const int selected_arp = m_arpComboBox->currentIndex();
-#else
-	const int selected_arp = m_arpComboBox->currentItem();
-#endif
+	const int selected_arp = m_arpComboBox->value();
+
+	constNotePlayHandleVector cnphv = notePlayHandle::nphsOfInstrumentTrack(
+						_n->getInstrumentTrack() );
+	if( m_arpModeComboBox->value() != FREE && cnphv.size() == 0 )
+	{
+		// maybe we're playing only a preset-preview-note?
+		cnphv = presetPreviewPlayHandle::nphsOfInstrumentTrack(
+						_n->getInstrumentTrack() );
+		if( cnphv.size() == 0 )
+		{
+			// still nothing found here, so lets return
+			//return;
+			cnphv.push_back( _n );
+		}
+	}
 
 	const int cur_chord_size = getChordSize( s_chords[selected_arp] );
-	const int total_range = (int)( cur_chord_size *
-						m_arpRangeKnob->value() );
+	const int range = (int)( cur_chord_size * m_arpRangeKnob->value() );
+	const int total_range = range * cnphv.size();
+
 	// number of frames that every note should be played
-	const Uint32 arp_frames = (Uint32)( m_arpTimeKnob->value() / 1000.0f *
-						mixer::inst()->sampleRate() );
-	const Uint32 gated_frames = (Uint32)( m_arpGateKnob->value() *
+	const f_cnt_t arp_frames = (f_cnt_t)( m_arpTimeKnob->value() / 1000.0f *
+					eng()->getMixer()->sampleRate() );
+	const f_cnt_t gated_frames = (f_cnt_t)( m_arpGateKnob->value() *
 							arp_frames / 100.0f );
+
 	// used for calculating remaining frames for arp-note, we have to add
 	// arp_frames-1, otherwise the first arp-note will not be setup
 	// correctly... -> arp_frames frames silence at the start of every note!
-	int cur_frame = _n->totalFramesPlayed() + arp_frames - 1;
+	int cur_frame = ( ( m_arpModeComboBox->value() != FREE ) ?
+				cnphv.first()->totalFramesPlayed() :
+				_n->totalFramesPlayed() ) + arp_frames - 1;
 	// used for loop
-	Uint32 frames_processed = 0;
-	// len for all arp-notes (depending on arp-time)
+	f_cnt_t frames_processed = 0;
 
-	while( frames_processed < mixer::inst()->framesPerAudioBuffer() )
+	while( frames_processed < eng()->getMixer()->framesPerAudioBuffer() )
 	{
-		const Uint32 remaining_frames_for_cur_arp = arp_frames -
+		const f_cnt_t remaining_frames_for_cur_arp = arp_frames -
 						( cur_frame % arp_frames );
 		// does current arp-note fill whole audio-buffer?
-		if( remaining_frames_for_cur_arp >=
-					mixer::inst()->framesPerAudioBuffer() )
+		if( remaining_frames_for_cur_arp >
+				eng()->getMixer()->framesPerAudioBuffer() )
 		{
 			// then we don't have to do something!
 			break;
@@ -518,36 +549,50 @@ void arpAndChordsTabWidget::processNote( notePlayHandle * _n )
 		// init with zero
 		int cur_arp_idx = 0;
 
+		// in sorted mode: is it our turn or do we have to be quiet for
+		// now?
+		if( m_arpModeComboBox->value() == SORT &&
+				( ( cur_frame / arp_frames ) % total_range ) /
+						range != (f_cnt_t) _n->index() )
+		{
+			// update counters
+			frames_processed += arp_frames;
+			cur_frame += arp_frames;
+			continue;
+		}
+
+		arpDirections dir = static_cast<arpDirections>(
+					m_arpDirectionBtnGrp->value() + 1 );
 		// process according to arpeggio-direction...
-		if( m_arpDirection == UP )
+		if( dir == UP )
 		{
-			cur_arp_idx = ( cur_frame / arp_frames ) % total_range;
+			cur_arp_idx = ( cur_frame / arp_frames ) % range;
 		}
-		else if( m_arpDirection == DOWN )
+		else if( dir == DOWN )
 		{
-			cur_arp_idx = total_range - ( cur_frame / arp_frames ) %
-						total_range - 1;
+			cur_arp_idx = range - ( cur_frame / arp_frames ) %
+								range - 1;
 		}
-		else if( m_arpDirection == UP_AND_DOWN && total_range > 1 )
+		else if( dir == UP_AND_DOWN && range > 1 )
 		{
 			// imagine, we had to play the arp once up and then
-			// once down -> makes 2 * total_range possible notes...
+			// once down -> makes 2 * range possible notes...
 			// because we don't play the lower and upper notes
 			// twice, we have to subtract 2
 			cur_arp_idx = ( cur_frame / arp_frames ) %
-							( total_range * 2 - 2 );
-			// if greater than total_range, we have to play down...
+							( range * 2 - 2 );
+			// if greater than range, we have to play down...
 			// looks like the code for arp_dir==DOWN... :)
-			if( cur_arp_idx >= total_range )
+			if( cur_arp_idx >= range )
 			{
-				cur_arp_idx = total_range - cur_arp_idx %
-							( total_range - 1 ) - 1;
+				cur_arp_idx = range - cur_arp_idx %
+							( range - 1 ) - 1;
 			}
 		}
-		else if( m_arpDirection == RANDOM )
+		else if( dir == RANDOM )
 		{
 			// just pick a random chord-index
-			cur_arp_idx = (int)( total_range * ( (float) rand() /
+			cur_arp_idx = (int)( range * ( (float) rand() /
 							(float) RAND_MAX ) );
 		}
 
@@ -559,7 +604,7 @@ void arpAndChordsTabWidget::processNote( notePlayHandle * _n )
 
 		// range-checking
 		if( sub_note_key >= NOTES_PER_OCTAVE * OCTAVES ||
-							sub_note_key < 0 )
+			sub_note_key < 0 || eng()->getMixer()->criticalXRuns() )
 		{
 			continue;
 		}
@@ -571,7 +616,7 @@ void arpAndChordsTabWidget::processNote( notePlayHandle * _n )
 		}
 
 		// create new arp-note
-		note new_note( midiTime( 0 ), midiTime( 0 ),
+		note new_note( NULL, midiTime( 0 ), midiTime( 0 ),
 				static_cast<tones>( sub_note_key %
 							NOTES_PER_OCTAVE ),
 				static_cast<octaves>( sub_note_key /
@@ -579,68 +624,58 @@ void arpAndChordsTabWidget::processNote( notePlayHandle * _n )
 				static_cast<volume>( _n->getVolume() *
 								vol_level ),
 				_n->getPanning() );
+		new_note.setDetuning( _n->detuning() );
 
 		// duplicate note-play-handle, only ptr to note is different
 		// and is_arp_note=TRUE
 		notePlayHandle * note_play_handle = new notePlayHandle(
-						_n->getChannelTrack(),
-						_n->framesAhead() +
+						_n->getInstrumentTrack(),
+					( ( m_arpModeComboBox->value() !=
+					    				FREE ) ?
+						cnphv.first()->framesAhead() :
+						_n->framesAhead() ) +
 							frames_processed,
 						gated_frames,
-						&new_note,
+						new_note,
 						TRUE );
 
-		// add sub-note to base-note, now all stuff is done by
-		// notePlayHandle::play_note()
+		// add sub-note to base-note - now all stuff is done by
+		// notePlayHandle::playNote()
 		_n->addSubNote( note_play_handle );
 
 		// update counters
 		frames_processed += arp_frames;
 		cur_frame += arp_frames;
 	}
+
+	// make sure, note is handled as arp-base-note, even if we didn't add a
+	// sub-note so far
+	if( m_arpModeComboBox->value() != FREE )
+	{
+		_n->setArpNote( TRUE );
+	}
 }
 
 
 
 
 void arpAndChordsTabWidget::saveSettings( QDomDocument & _doc,
-							QDomElement & _parent )
+							QDomElement & _this )
 {
-	QDomElement act_de = _doc.createElement( nodeName() );
-	act_de.setAttribute( "chorddisabled", QString::number(
-					!m_chordsGroupBox->isActive() ) );
-#ifdef QT4
-	act_de.setAttribute( "chord", QString::number(
-					m_chordsComboBox->currentIndex() ) );
-#else
-	act_de.setAttribute( "chord", QString::number(
-					m_chordsComboBox->currentItem() ) );
-#endif
-	act_de.setAttribute( "chordrange", QString::number(
-					m_chordRangeKnob->value() ) );
+	_this.setAttribute( "chorddisabled", !m_chordsGroupBox->isActive() );
+	m_chordsComboBox->saveSettings( _doc, _this, "chord" );
+	m_chordRangeKnob->saveSettings( _doc, _this, "chordrange" );
 
-	act_de.setAttribute( "arpdisabled", QString::number(
-						!m_arpGroupBox->isActive() ) );
-#ifdef QT4
-	act_de.setAttribute( "arp", QString::number(
-					m_arpComboBox->currentIndex() ) );
-#else
-	act_de.setAttribute( "arp", QString::number(
-					m_arpComboBox->currentItem() ) );
-#endif
-	act_de.setAttribute( "arprange", QString::number(
-					m_arpRangeKnob->value() ) );
-	act_de.setAttribute( "arptime", QString::number(
-					m_arpTimeKnob->value() ) );
-	act_de.setAttribute( "arpgate", QString::number(
-					m_arpGateKnob->value() ) );
-	act_de.setAttribute( "arpdir", QString::number(
-					m_arpDirection ) );
-	act_de.setAttribute( "arpsyncmode", QString::number(
-				( int ) m_arpTimeKnob->getSyncMode() ) );
+	_this.setAttribute( "arpdisabled", !m_arpGroupBox->isActive() );
+	m_arpComboBox->saveSettings( _doc, _this, "arp" );
+	m_arpRangeKnob->saveSettings( _doc, _this, "arprange" );
+	m_arpTimeKnob->saveSettings( _doc, _this, "arptime" );
+	m_arpGateKnob->saveSettings( _doc, _this, "arpgate" );
+	m_arpDirectionBtnGrp->saveSettings( _doc, _this, "arpdir" );
+	_this.setAttribute( "arpsyncmode",
+					( int ) m_arpTimeKnob->getSyncMode() );
 
-
-	_parent.appendChild( act_de );
+	m_arpModeComboBox->saveSettings( _doc, _this, "arpmode" );
 }
 
 
@@ -650,99 +685,37 @@ void arpAndChordsTabWidget::loadSettings( const QDomElement & _this )
 {
 	m_chordsGroupBox->setState( !_this.attribute
 						( "chorddisabled" ).toInt() );
-#ifdef QT4
-	m_chordsComboBox->setCurrentIndex( _this.attribute( "chord" ).toInt() );
-#else
-	m_chordsComboBox->setCurrentItem( _this.attribute( "chord" ).toInt() );
-#endif
-	m_chordRangeKnob->setValue( _this.attribute( "chordrange" ).toFloat() );
-#ifdef QT4
-	m_arpComboBox->setCurrentIndex( _this.attribute( "arp" ).toInt() );
-#else
-	m_arpComboBox->setCurrentItem( _this.attribute( "arp" ).toInt() );
-#endif
-	m_arpRangeKnob->setValue( _this.attribute( "arprange" ).toFloat() );
-	m_arpTimeKnob->setValue( _this.attribute( "arptime" ).toFloat() );
-	m_arpGateKnob->setValue( _this.attribute( "arpgate" ).toFloat() );
-	m_arpDirection = static_cast<arpDirections>(
-					_this.attribute( "arpdir" ).toInt() );
+	m_chordsComboBox->loadSettings( _this, "chord" );
+	m_chordRangeKnob->loadSettings( _this, "chordrange" );
+	m_arpComboBox->loadSettings( _this, "arp" );
+	m_arpRangeKnob->loadSettings( _this, "arprange" );
+	m_arpTimeKnob->loadSettings( _this, "arptime" );
+	m_arpGateKnob->loadSettings( _this, "arpgate" );
+	if( _this.hasAttribute( "arpdir" ) )
+	{
+		m_arpDirectionBtnGrp->setInitValue(
+				_this.attribute( "arpdir" ).toInt() - 1 );
+	}
+	else
+	{
+		m_arpDirectionBtnGrp->loadSettings( _this, "arpdir" );
+	}
 	m_arpTimeKnob->setSyncMode( 
 		( tempoSyncKnob::tempoSyncMode ) _this.attribute(
 						 "arpsyncmode" ).toInt() );
 
-	m_arpGroupBox->setState( m_arpDirection != OFF &&
+	m_arpModeComboBox->loadSettings( _this, "arpmode" );
+
+	m_arpGroupBox->setState( _this.attribute( "arpdir" ).toInt() != OFF &&
 				!_this.attribute( "arpdisabled" ).toInt() );
-	switch( m_arpDirection )
-	{
-		case DOWN:
-			m_arpDownBtn->setChecked( TRUE );
-			break;
-		case UP_AND_DOWN:
-			m_arpUpAndDownBtn->setChecked( TRUE );
-			break;
-		case RANDOM:
-			m_arpRandomBtn->setChecked( TRUE );
-			break;
-		case UP:
-		default:
-			m_arpUpBtn->setChecked( TRUE );
-			m_arpDirection = UP;
-			break;
-	}
 }
 
 
 
 
-void arpAndChordsTabWidget::arpUpToggled( bool _on )
-{
-	if( _on )
-	{
-		m_arpDirection = UP;
-	}
-	songEditor::inst()->setModified();
-}
 
 
-
-
-void arpAndChordsTabWidget::arpDownToggled( bool _on )
-{
-	if( _on )
-	{
-		m_arpDirection = DOWN;
-	}
-	songEditor::inst()->setModified();
-}
-
-
-
-
-void arpAndChordsTabWidget::arpUpAndDownToggled( bool _on )
-{
-	if( _on )
-	{
-		m_arpDirection = UP_AND_DOWN;
-	}
-	songEditor::inst()->setModified();
-}
-
-
-
-
-void arpAndChordsTabWidget::arpRandomToggled( bool _on )
-{
-	if( _on )
-	{
-		m_arpDirection = RANDOM;
-	}
-	songEditor::inst()->setModified();
-}
-
-
-
-
-#undef setChecked
-
 #include "arp_and_chords_tab_widget.moc"
 
+
+#endif
