@@ -1,7 +1,9 @@
+#ifndef SINGLE_SOURCE_COMPILE
+
 /*
  * bb_editor.cpp - basic main-window for editing of beats and basslines
  *
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -27,15 +29,16 @@
 
 #ifdef QT4
 
-#include <QPainter>
-#include <QKeyEvent>
-#include <QCloseEvent>
-#include <QLayout>
+#include <QtGui/QPainter>
+#include <QtGui/QKeyEvent>
+#include <QtGui/QCloseEvent>
+#include <QtGui/QLayout>
 
 #else
 
 #include <qpainter.h>
 #include <qlayout.h>
+#include <qwhatsthis.h>
 
 #endif
 
@@ -50,28 +53,27 @@
 #include "templates.h"
 #include "debug.h"
 #include "tooltip.h"
-
+#include "combobox.h"
+#include "main_window.h"
 
 
 const int BBE_PPT = 192;
 
 
 
-bbEditor * bbEditor::s_instanceOfMe = NULL;
 
-
-
-bbEditor::bbEditor() :
-	trackContainer()
+bbEditor::bbEditor( engine * _engine ) :
+	trackContainer( _engine )
 {
 	// create toolbar
 	m_toolBar = new QWidget( this );
 	m_toolBar->setFixedHeight( 32 );
 	m_toolBar->move( 0, 0 );
 #ifdef QT4
+	m_toolBar->setAutoFillBackground( TRUE );
 	QPalette pal;
-	pal.setBrush( m_toolBar->backgroundRole(), QBrush(
-				embed::getIconPixmap( "toolbar_bg" ) ) );
+	pal.setBrush( m_toolBar->backgroundRole(),
+					embed::getIconPixmap( "toolbar_bg" ) );
 	m_toolBar->setPalette( pal );
 #else
 	m_toolBar->setPaletteBackgroundPixmap( embed::getIconPixmap(
@@ -83,16 +85,21 @@ bbEditor::bbEditor() :
 
 
 	setWindowIcon( embed::getIconPixmap( "bb_track" ) );
-	setWindowTitle( tr( "Beat+Bassline Editor" ) );
+	setWindowTitle( tr( "Beat+Baseline Editor" ) );
 	setMinimumWidth( TRACK_OP_WIDTH + DEFAULT_SETTINGS_WIDGET_WIDTH +
-					BBE_PPT + 2 * TCO_BORDER_WIDTH );
-	if( lmmsMainWin::inst()->workspace() != NULL )
+				BBE_PPT + 2 * TCO_BORDER_WIDTH +
+				DEFAULT_SCROLLBAR_SIZE );
+
+	QWidget * w = ( parentWidget() != NULL ) ? parentWidget() : this;
+	if( eng()->getMainWindow()->workspace() != NULL )
 	{
-		setGeometry( 10, 340, minimumWidth(), 300 );
+		resize( minimumWidth(), 300 );
+		w->move( 10, 340 );
 	}
 	else
 	{
-		setGeometry( 210, 340, minimumWidth(), 300 );
+		resize( minimumWidth(), 300 );
+		w->move( 210, 340 );
 	}
 
 	containerWidget()->move( 0, 32 );
@@ -107,6 +114,12 @@ bbEditor::bbEditor() :
 			tr( "Stop playing of current beat/bassline (Space)" ),
 					this, SLOT( stop() ), m_toolBar );
 
+	toolButton * add_bb_track = new toolButton(
+					embed::getIconPixmap( "add_bb_track" ),
+						tr( "Add beat/bassline" ),
+				eng()->getSongEditor(), SLOT( addBBTrack() ),
+								m_toolBar );
+
 
 #ifdef QT4
 	m_playButton->setWhatsThis(
@@ -127,9 +140,18 @@ bbEditor::bbEditor() :
 	QLabel * l = new QLabel( m_toolBar );
 	l->setPixmap( embed::getIconPixmap( "drum" ) );
 
+	m_bbComboBox = new comboBox( m_toolBar, NULL, eng(), NULL );
+	m_bbComboBox->setFixedSize( 200, 22 );
+	connect( m_bbComboBox, SIGNAL( valueChanged( int ) ),
+				this, SLOT( setCurrentBB( int ) ) );
+
 	tb_layout->addSpacing( 5 );
 	tb_layout->addWidget( m_playButton );
 	tb_layout->addWidget( m_stopButton );
+	tb_layout->addSpacing( 20 );
+	tb_layout->addWidget( m_bbComboBox );
+	tb_layout->addSpacing( 10 );
+	tb_layout->addWidget( add_bb_track );
 	tb_layout->addStretch();
 	tb_layout->addWidget( l );
 	tb_layout->addSpacing( 15 );
@@ -155,21 +177,27 @@ csize bbEditor::currentBB( void ) const
 
 
 
-void bbEditor::setCurrentBB( csize _bb )
+void bbEditor::setCurrentBB( int _bb )
 {
+	if( m_bbComboBox->value() != _bb )
+	{
+		m_bbComboBox->setValue( _bb );
+	}
+
 	// first make sure, all channels have a TCO at current BB
-	createTCOsForBB( _bb );
+	createTCOsForBB( static_cast<csize>( _bb ) );
 
 	realignTracks();
 
 	// now update all track-labels (the current one has to become white,
 	// the others green)
-	for( csize i = 0; i < bbEditor::inst()->numOfBBs(); ++i )
+	for( csize i = 0; i < numOfBBs(); ++i )
 	{
-		bbTrack::findBBTrack( i )->trackLabel()->update();
+		bbTrack::findBBTrack( i, eng() )->trackLabel()->update();
 	}
 
-	emit positionChanged( m_currentPosition = midiTime( _bb, 0 ) );
+	emit positionChanged( m_currentPosition = midiTime(
+					static_cast<csize>( _bb ), 0 ) );
 }
 
 
@@ -178,7 +206,7 @@ void bbEditor::setCurrentBB( csize _bb )
 tact bbEditor::lengthOfBB( csize _bb )
 {
 	midiTime max_length;
-	
+
 	trackVector tv = tracks();
 	for( trackVector::iterator it = tv.begin(); it != tv.end(); ++it )
 	{
@@ -199,8 +227,8 @@ tact bbEditor::lengthOfBB( csize _bb )
 
 
 
-bool FASTCALL bbEditor::play( midiTime _start, Uint32 _start_frame,
-				Uint32 _frames, Uint32 _frame_base,
+bool FASTCALL bbEditor::play( midiTime _start, f_cnt_t _start_frame,
+				f_cnt_t _frames, f_cnt_t _frame_base,
 							Sint16 _tco_num )
 {
 	bool played_a_note = FALSE;
@@ -229,7 +257,7 @@ bool FASTCALL bbEditor::play( midiTime _start, Uint32 _start_frame,
 
 csize bbEditor::numOfBBs( void ) const
 {
-	return( songEditor::inst()->countTracks( track::BB_TRACK ) );
+	return( eng()->getSongEditor()->countTracks( track::BB_TRACK ) );
 }
 
 
@@ -243,24 +271,17 @@ void bbEditor::removeBB( csize _bb )
 		( *it )->removeTCO( _bb );
 		( *it )->getTrackContentWidget()->removeTact( _bb * 64 );
 	}
-/*	if( _bb == currentBB() && numOfBBs() > 0 )
-	{*/
-		if( _bb > 0)
-		{
-			setCurrentBB( _bb - 1 );
-		}
-		else
-		{
-			setCurrentBB( 0 );
-		}
-//	}
+	if( _bb <= currentBB() )
+	{
+		setCurrentBB( tMax( (int)currentBB() - 1, 0 ) );
+	}
 }
 
 
 
 void bbEditor::updateBBTrack( trackContentObject * _tco )
 {
-	bbTrack * t = bbTrack::findBBTrack( _tco->startPosition() / 64 );
+	bbTrack * t = bbTrack::findBBTrack( _tco->startPosition() / 64, eng() );
 	if( t != NULL )
 	{
 		t->getTrackContentWidget()->updateTCOs();
@@ -270,6 +291,30 @@ void bbEditor::updateBBTrack( trackContentObject * _tco )
 
 
 
+void bbEditor::updateComboBox( void )
+{
+	disconnect( m_bbComboBox, SIGNAL( valueChanged( int ) ),
+					this, SLOT( setCurrentBB( int ) ) );
+
+	csize current_bb = currentBB();
+
+	m_bbComboBox->clear();
+
+	for( csize i = 0; i < numOfBBs(); ++i )
+	{
+		bbTrack * bbt = bbTrack::findBBTrack( i, eng() );
+		m_bbComboBox->addItem( bbt->trackLabel()->text(),
+					bbt->trackLabel()->pixmap() );
+	}
+	m_bbComboBox->setValue( current_bb );
+
+	connect( m_bbComboBox, SIGNAL( valueChanged( int ) ),
+					this, SLOT( setCurrentBB( int ) ) );
+}
+
+
+
+
 // close-handler for bb-editor-window because closing of bb-editor isn't allowed
 // instead of closing it's being hidden
 void bbEditor::closeEvent( QCloseEvent * _ce )
@@ -285,7 +330,7 @@ void bbEditor::keyPressEvent( QKeyEvent * _ke )
 {
 	if ( _ke->key() == Qt::Key_Space )
 	{
-		if( songEditor::inst()->playing() )
+		if( eng()->getSongEditor()->playing() )
 		{
 			stop();
 		}
@@ -323,7 +368,8 @@ void bbEditor::resizeEvent( QResizeEvent * _re )
 {
 	setPixelsPerTact( width() - ( TRACK_OP_WIDTH +
 					DEFAULT_SETTINGS_WIDGET_WIDTH + 2 *
-					TCO_BORDER_WIDTH ) );
+					TCO_BORDER_WIDTH +
+					DEFAULT_SCROLLBAR_SIZE ) );
 	trackContainer::resizeEvent( _re );
 	m_toolBar->setFixedWidth( width() );
 }
@@ -331,33 +377,42 @@ void bbEditor::resizeEvent( QResizeEvent * _re )
 
 
 
+QRect bbEditor::scrollAreaRect( void ) const
+{
+	return( QRect( 0, 0, (int) pixelsPerTact(),
+					height() - m_toolBar->height() ) );
+}
+
+
+
+
 void bbEditor::play( void )
 {
-	if( songEditor::inst()->playing() )
+	if( eng()->getSongEditor()->playing() )
 	{
-		if( songEditor::inst()->playMode() != songEditor::PLAY_BB )
+		if( eng()->getSongEditor()->playMode() != songEditor::PLAY_BB )
 		{
-			songEditor::inst()->stop();
-			songEditor::inst()->playBB();
+			eng()->getSongEditor()->stop();
+			eng()->getSongEditor()->playBB();
 			m_playButton->setIcon( embed::getIconPixmap(
 								"pause" ) );
 		}
 		else
 		{
-			songEditor::inst()->pause();
+			eng()->getSongEditor()->pause();
 			m_playButton->setIcon( embed::getIconPixmap(
 								"play" ) );
 		}
 	}
-	else if( songEditor::inst()->paused() )
+	else if( eng()->getSongEditor()->paused() )
 	{
-		songEditor::inst()->resumeFromPause();
+		eng()->getSongEditor()->resumeFromPause();
 		m_playButton->setIcon( embed::getIconPixmap( "pause" ) );
 	}
 	else
 	{
 		m_playButton->setIcon( embed::getIconPixmap( "pause" ) );
-		songEditor::inst()->playBB();
+		eng()->getSongEditor()->playBB();
 	}
 
 }
@@ -367,7 +422,7 @@ void bbEditor::play( void )
 
 void bbEditor::stop( void )
 {
-	songEditor::inst()->stop();
+	eng()->getSongEditor()->stop();
 	m_playButton->setIcon( embed::getIconPixmap( "play" ) );
 	m_playButton->update();
 }
@@ -431,9 +486,12 @@ void bbEditor::swapBB( csize _bb1, csize _bb2 )
 	{
 		( *it )->swapPositionOfTCOs( _bb1, _bb2 );
 	}
+	updateComboBox();
 }
 
 
 
 #include "bb_editor.moc"
 
+
+#endif
