@@ -1,7 +1,9 @@
+#ifndef SINGLE_SOURCE_COMPILE
+
 /*
  * config_mgr.cpp - implementation of class configManager
  *
- * Copyright (c) 2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2005-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -31,18 +33,18 @@
 
 #ifdef QT4
 
-#include <QDir>
 #include <Qt/QtXml>
-#include <QFile>
-#include <QMessageBox>
-#include <QLabel>
-#include <QLayout>
-#include <QLineEdit>
-#include <QPushButton>
-#include <QFileDialog>
-#include <QRadioButton>
-#include <QButtonGroup>
-#include <QApplication>
+#include <QtCore/QDir>
+#include <QtCore/QFile>
+#include <QtGui/QMessageBox>
+#include <QtGui/QLabel>
+#include <QtGui/QLayout>
+#include <QtGui/QLineEdit>
+#include <QtGui/QPushButton>
+#include <QtGui/QFileDialog>
+#include <QtGui/QRadioButton>
+#include <QtGui/QButtonGroup>
+#include <QtGui/QApplication>
 
 #else
 
@@ -59,6 +61,9 @@
 #include <qradiobutton.h>
 #include <qapplication.h>
 
+#define absolutePath absPath
+#define addButton insert
+
 #ifndef __USE_XOPEN_EXTENDED
 #define __USE_XOPEN_EXTENDED
 #endif
@@ -103,7 +108,7 @@ void mkPath( const QString & _path )
 
 
 
-
+/*
 void linkFile( const QString & _src, const QString & _dst )
 {
 #ifdef QT4
@@ -144,7 +149,7 @@ void copyFile( const QString & _src, const QString & _dst )
 	}
 #endif
 }
-
+*/
 
 
 
@@ -153,31 +158,29 @@ configManager * configManager::s_instanceOfMe = NULL;
 
 configManager::configManager( void ) :
 	QDialog(),
-#ifdef QT4
 	m_lmmsRcFile( QDir::home().absolutePath() + "/.lmmsrc.xml" ),
-	m_lmmsWorkingDir( QDir::home().absolutePath() + "/lmms" ),
-#else
-	m_lmmsRcFile( QDir::home().absPath() + "/.lmmsrc.xml" ),
-	m_lmmsWorkingDir( QDir::home().absPath() + "/lmms" ),
-#endif
+	m_workingDir( QDir::home().absolutePath() + "/lmms" ),
 #if QT_VERSION >= 0x030200
-	m_lmmsDataDir( qApp->applicationDirPath().section( '/', 0, -2 ) +
+	m_dataDir( qApp->applicationDirPath().section( '/', 0, -2 ) +
 							"/share/lmms/" ),
 #else
 	// hardcode since qt < 3.2 doesn't know something like
 	// applicationDirPath and implementing own function would be senseless
 	// since real support for qt < 3.2 is senseless too ;-)
-	m_lmmsDataDir( "/usr/share/lmms" ),
+	m_dataDir( "/usr/share/lmms/" ),
 #endif
+	m_artworkDir( defaultArtworkDir() ),
 #if QT_VERSION >= 0x030200
-	m_lmmsPluginDir( qApp->applicationDirPath().section( '/', 0, -2 ) +
+	m_pluginDir( qApp->applicationDirPath().section( '/', 0, -2 ) +
 							"/lib/lmms/" ),
 #else
 	// hardcode since qt < 3.2 doesn't know something like
 	// applicationDirPath and implementing own function would be senseless
 	// since real support for qt < 3.2 is senseless too ;-)
-	m_lmmsPluginDir( "/usr/lib/lmms" ),
+	m_pluginDir( "/usr/lib/lmms" ),
 #endif
+	m_vstDir( QDir::home().absolutePath() ),
+	m_flDir( QDir::home().absolutePath() ),
 	m_currentPage( 0 )
 {
 }
@@ -335,8 +338,7 @@ void configManager::createWidgets( void )
 	workingdir_input_fields_layout->setSpacing( 10 );
 	workingdir_input_fields_layout->setMargin( 0 );
 
-	m_wdLineEdit = new QLineEdit( m_lmmsWorkingDir,
-						workingdir_input_fields );
+	m_wdLineEdit = new QLineEdit( m_workingDir, workingdir_input_fields );
 	connect( m_wdLineEdit, SIGNAL( textChanged( const QString & ) ), this,
 				SLOT( setWorkingDir( const QString & ) ) );
 
@@ -357,7 +359,7 @@ void configManager::createWidgets( void )
 	workingdir_layout->addWidget( workingdir_content );
 
 
-
+/*
 	// page for files-management
 	m_pageFiles = new QWidget( m_contentWidget );
 	QHBoxLayout * files_layout = new QHBoxLayout( m_pageFiles );
@@ -500,12 +502,12 @@ void configManager::createWidgets( void )
 
 	files_layout->addWidget( files_logo_lbl );
 	files_layout->addWidget( files_content );
-
+*/
 
 
 	addPage( m_pageIntro, tr( "Welcome to LMMS" ) );
 	addPage( m_pageWorkingDir, tr( "Select working directory" ) );
-	addPage( m_pageFiles, tr( "Copy or link files" ) );
+	//addPage( m_pageFiles, tr( "Copy or link files" ) );
 	switchPage( static_cast<csize>( 0 ) );
 }
 
@@ -516,12 +518,11 @@ void configManager::openWorkingDir( void )
 {
 #ifdef QT4
 	QString new_dir = QFileDialog::getExistingDirectory( this,
-					tr( "Choose LMMS-working-directory" ),
-							m_lmmsWorkingDir );
+					tr( "Choose LMMS working directory" ),
+								m_workingDir );
 #else
-	QString new_dir = QFileDialog::getExistingDirectory( m_lmmsWorkingDir,
-									0, 0,
-					tr( "Choose LMMS-working-directory" ),
+	QString new_dir = QFileDialog::getExistingDirectory( m_workingDir, 0, 0,
+					tr( "Choose LMMS working directory" ),
 									TRUE );
 #endif
 	if( new_dir != QString::null )
@@ -535,7 +536,31 @@ void configManager::openWorkingDir( void )
 
 void configManager::setWorkingDir( const QString & _wd )
 {
-	m_lmmsWorkingDir = _wd;
+	m_workingDir = _wd;
+}
+
+
+
+
+void configManager::setVSTDir( const QString & _vd )
+{
+	m_vstDir = _vd;
+}
+
+
+
+
+void configManager::setArtworkDir( const QString & _ad )
+{
+	m_artworkDir = _ad;
+}
+
+
+
+
+void configManager::setFLDir( const QString & _fd )
+{
+	m_flDir = _fd;
 }
 
 
@@ -543,11 +568,11 @@ void configManager::setWorkingDir( const QString & _wd )
 
 void configManager::accept( void )
 {
-	if( m_lmmsWorkingDir.right( 1 ) != "/" )
+	if( m_workingDir.right( 1 ) != "/" )
 	{
-		m_lmmsWorkingDir += '/';
+		m_workingDir += '/';
 	}
-	if( !QDir( m_lmmsWorkingDir ).exists() )
+	if( !QDir( m_workingDir ).exists() )
 	{
 		if( QMessageBox::
 #if QT_VERSION >= 0x030200
@@ -562,7 +587,7 @@ void configManager::accept( void )
 					QMessageBox::Yes, QMessageBox::No )
 			== QMessageBox::Yes )
 		{
-			mkPath( m_lmmsWorkingDir );
+			mkPath( m_workingDir );
 		}
 		else
 		{
@@ -570,23 +595,25 @@ void configManager::accept( void )
 			return;
 		}
 	}
-	processFilesRecursively( m_lmmsDataDir + "samples/", m_lmmsWorkingDir +
+
+	mkPath( userProjectsDir() );
+	mkPath( userSamplesDir() );
+	mkPath( userPresetsDir() );
+/*	processFilesRecursively( m_dataDir + "samples/", m_workingDir +
 								"samples/",
 					m_samplesCopyRB->isChecked() ?
 								&copyFile :
 								&linkFile );
-	processFilesRecursively( m_lmmsDataDir + "presets/", m_lmmsWorkingDir +
+	processFilesRecursively( m_dataDir + "presets/", m_workingDir +
 								"presets/",
 					m_presetsCopyRB->isChecked() ?
 								&copyFile :
 								&linkFile );
-	processFilesRecursively( m_lmmsDataDir + "projects/", m_lmmsWorkingDir +
+	processFilesRecursively( m_dataDir + "projects/", m_workingDir +
 								"projects/",
 					m_projectsCopyRB->isChecked() ?
 								&copyFile :
-								&linkFile );
-	setValue( "paths", "workingdir", m_lmmsWorkingDir );
-
+								&linkFile );*/
 	saveConfigFile();
 
 	QDialog::accept();
@@ -767,10 +794,63 @@ bool configManager::loadConfigFile( void )
 
 	// get the head information from the DOM
 	QDomElement root = dom_tree.documentElement();
-/*	if( root.isElement() )
+
+	QDomNode node = root.firstChild();
+
+	// create the settings-map out of the DOM
+	while( !node.isNull() )
+	{
+		if( node.isElement() && node.toElement().hasAttributes () )
+		{
+			stringPairVector attr;
+			QDomNamedNodeMap node_attr =
+						node.toElement().attributes();
+			for( csize i = 0; i < node_attr.count(); ++i )
+			{
+				QDomNode n = node_attr.item( i );
+				if( n.isAttr() )
+				{
+					attr.push_back( qMakePair(
+							n.toAttr().name(),
+							n.toAttr().value() ) );
+				}
+			}
+			m_settings[node.nodeName()] = attr;
+		}
+		node = node.nextSibling();
+	}
+
+	if( value( "paths", "artwork" ) != "" )
+	{
+		m_artworkDir = value( "paths", "artwork" );
+		if( QDir( m_artworkDir ).exists() == FALSE )
+		{
+			m_artworkDir = defaultArtworkDir();
+		}
+		if( m_artworkDir.right( 1 ) != "/" )
+		{
+			m_artworkDir += "/";
+		}
+	}
+	m_workingDir = value( "paths", "workingdir" );
+	m_vstDir = value( "paths", "vstdir" );
+	m_flDir = value( "paths", "fldir" );
+
+	if( m_vstDir == "" )
+	{
+		m_vstDir = QDir::home().absolutePath();
+	}
+
+	if( m_flDir == "" )
+	{
+		m_flDir = QDir::home().absolutePath();
+	}
+
+	if( root.isElement() )
 	{
 		QString cfg_file_ver = root.toElement().attribute( "version" );
 		if( ( cfg_file_ver.length() == 0 || cfg_file_ver != VERSION ) &&
+				value( "app", "nowizard" ).toInt() == FALSE &&
 			QMessageBox::
 #if QT_VERSION >= 0x030200
 				question
@@ -798,35 +878,8 @@ bool configManager::loadConfigFile( void )
 				return( loadConfigFile() );
 			}
 		}
-	}*/
-
-	QDomNode node = root.firstChild();
-
-	// create the settings-map out of the DOM
-	while( !node.isNull() )
-	{
-		if( node.isElement() && node.toElement().hasAttributes () )
-		{
-			stringPairVector attr;
-			QDomNamedNodeMap node_attr =
-						node.toElement().attributes();
-			for( unsigned int i = 0; i < node_attr.count(); ++i )
-			{
-				QDomNode n = node_attr.item( i );
-				if( n.isAttr() )
-				{
-					attr.push_back( qMakePair(
-							n.toAttr().name(),
-							n.toAttr().value() ) );
-				}
-			}
-			m_settings[node.nodeName()] = attr;
-		}
-		node = node.nextSibling();
 	}
 
-	m_lmmsWorkingDir = value( "paths", "workingdir" );
-
 	return( TRUE );
 }
 
@@ -835,6 +888,11 @@ bool configManager::loadConfigFile( void )
 
 void configManager::saveConfigFile( void )
 {
+	setValue( "paths", "artwork", m_artworkDir );
+	setValue( "paths", "workingdir", m_workingDir );
+	setValue( "paths", "vstdir", m_vstDir );
+	setValue( "paths", "fldir", m_flDir );
+
 	QDomDocument doc( "lmms-config-file" );
 
 	QDomElement lmms_config = doc.createElement( "lmms" );
@@ -879,16 +937,17 @@ void configManager::saveConfigFile( void )
 	}
 
 #ifdef QT4
-	outfile.write( xml.toAscii().constData(), xml.length() );
+	outfile.write( xml.toUtf8().constData(), xml.length() );
 #else
-	outfile.writeBlock( xml.ascii(), xml.length() );
+	QCString xml_utf8 = xml.utf8();
+	outfile.writeBlock( xml_utf8.data(), xml_utf8.length() );
 #endif
 	outfile.close();
 }
 
 
 
-
+/*
 void configManager::processFilesRecursively( const QString & _src_dir,
 						const QString & _dst_dir,
 	void( * _proc_func )( const QString & _src, const QString & _dst ) )
@@ -913,9 +972,12 @@ void configManager::processFilesRecursively( const QString & _src_dir,
 		}
 	}
 }
-
+*/
 
 
 
 #include "config_mgr.moc"
+#undef absolutePath
+#undef addButton
 
+#endif
