@@ -1,8 +1,10 @@
+#ifndef SINGLE_SOURCE_COMPILE
+
 /*
- * envelope_widget.cpp - widget which is m_used by envelope/lfo/filter-tab of
- *                       channel-window
+ * envelope_and_lfo_widget.cpp - widget which is m_used by envelope/lfo/filter-
+ *                               tab of channel-window
  *
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -28,16 +30,13 @@
 
 #ifdef QT4
 
-#include <QPainter>
-#include <QPaintEvent>
-#include <QButtonGroup>
-#include <QWhatsThis>
+#include <QtGui/QPainter>
+#include <QtGui/QPaintEvent>
 #include <Qt/QtXml>
-#include <QLabel>
+#include <QtGui/QLabel>
 
 #else
 
-#include <qbuttongroup.h>
 #include <qwhatsthis.h>
 #include <qpainter.h>
 #include <qpen.h>
@@ -100,14 +99,18 @@ const int LFO_SHAPES_Y = LFO_GRAPH_Y + 50;
 
 QPixmap * envelopeAndLFOWidget::s_envGraph = NULL;
 QPixmap * envelopeAndLFOWidget::s_lfoGraph = NULL;
-Uint32 envelopeAndLFOWidget::s_lfoFrame = 0;
+
+QMap<engine *, vvector<envelopeAndLFOWidget *> >
+					envelopeAndLFOWidget::s_EaLWidgets;
 
 
 
 envelopeAndLFOWidget::envelopeAndLFOWidget( float _value_for_zero_amount,
-							QWidget * _parent ) :
+							QWidget * _parent,
+							engine * _engine,
+							track * _track ) :
 	QWidget( _parent ),
-	settings(),
+	journallingObject( _engine ),
 #ifdef QT4
 	specialBgHandlingWidget( palette().color( backgroundRole() ) ),
 #else
@@ -117,8 +120,10 @@ envelopeAndLFOWidget::envelopeAndLFOWidget( float _value_for_zero_amount,
 	m_valueForZeroAmount( _value_for_zero_amount ),
 	m_pahdEnv( NULL ),
 	m_rEnv( NULL ),
+	m_lfoFrame( 0 ),
 	m_lfoAmountIsZero( FALSE ),
 	m_lfoShapeData( NULL ),
+	m_userWave( eng() ),
 	m_lfoShape( SIN ),
 	m_busy( FALSE )
 {
@@ -132,10 +137,14 @@ envelopeAndLFOWidget::envelopeAndLFOWidget( float _value_for_zero_amount,
 		s_lfoGraph = new QPixmap( embed::getIconPixmap( "lfo_graph" ) );
 	}
 
-	m_predelayKnob = new knob( knobBright_26, this, tr( "Predelay-time" ) );
+	s_EaLWidgets[eng()].push_back( this );
+
+
+	m_predelayKnob = new knob( knobBright_26, this, tr( "Predelay-time" ),
+								eng(), _track );
 	m_predelayKnob->setLabel( tr( "DEL" ) );
 	m_predelayKnob->setRange( 0.0, 1.0, 0.001 );
-	m_predelayKnob->setValue( 0.0, TRUE );
+	m_predelayKnob->setInitValue( 0.0 );
 	m_predelayKnob->move( PREDELAY_KNOB_X, ENV_KNOBS_Y );
 	m_predelayKnob->setHintText( tr( "Predelay:" ) + " ", "" );
 #ifdef QT4
@@ -149,10 +158,11 @@ envelopeAndLFOWidget::envelopeAndLFOWidget( float _value_for_zero_amount,
 	connect( m_predelayKnob, SIGNAL( valueChanged( float ) ), this,
 				SLOT( updateAfterKnobChange( float ) ) );
 
-	m_attackKnob = new knob( knobBright_26, this, tr( "Attack-time" ) );
+	m_attackKnob = new knob( knobBright_26, this, tr( "Attack-time" ),
+								eng(), _track );
 	m_attackKnob->setLabel( tr( "ATT" ) );
 	m_attackKnob->setRange( 0.0, 1.0, 0.001 );
-	m_attackKnob->setValue( 0.0, TRUE );
+	m_attackKnob->setInitValue( 0.0 );
 	m_attackKnob->move( ATTACK_KNOB_X, ENV_KNOBS_Y );
 	m_attackKnob->setHintText( tr( "Attack:" )+" ", "" );
 #ifdef QT4
@@ -168,10 +178,11 @@ envelopeAndLFOWidget::envelopeAndLFOWidget( float _value_for_zero_amount,
 	connect( m_attackKnob, SIGNAL( valueChanged( float ) ), this,
 				SLOT( updateAfterKnobChange( float ) ) );
 
-	m_holdKnob = new knob( knobBright_26, this, tr( "Hold-time" ) );
+	m_holdKnob = new knob( knobBright_26, this, tr( "Hold-time" ),
+								eng(), _track );
 	m_holdKnob->setLabel( tr( "HOLD" ) );
 	m_holdKnob->setRange( 0.0, 1.0, 0.001 );
-	m_holdKnob->setValue( 0.5, TRUE );
+	m_holdKnob->setInitValue( 0.5 );
 	m_holdKnob->move( HOLD_KNOB_X, ENV_KNOBS_Y );
 	m_holdKnob->setHintText( tr( "Hold:" ) + " ", "" );
 #ifdef QT4
@@ -186,10 +197,11 @@ envelopeAndLFOWidget::envelopeAndLFOWidget( float _value_for_zero_amount,
 	connect( m_holdKnob, SIGNAL( valueChanged( float ) ), this,
 				SLOT( updateAfterKnobChange( float ) ) );
 
-	m_decayKnob = new knob( knobBright_26, this, tr( "Decay-time" ) );
+	m_decayKnob = new knob( knobBright_26, this, tr( "Decay-time" ),
+								eng(), _track );
 	m_decayKnob->setLabel( tr( "DEC" ) );
 	m_decayKnob->setRange( 0.0, 1.0, 0.001 );
-	m_decayKnob->setValue( 0.5, TRUE );
+	m_decayKnob->setInitValue( 0.5 );
 	m_decayKnob->move( DECAY_KNOB_X, ENV_KNOBS_Y );
 	m_decayKnob->setHintText( tr( "Decay:" ) + " ", "" );
 #ifdef QT4
@@ -205,10 +217,11 @@ envelopeAndLFOWidget::envelopeAndLFOWidget( float _value_for_zero_amount,
 	connect( m_decayKnob, SIGNAL( valueChanged( float ) ), this,
 				SLOT( updateAfterKnobChange( float ) ) );
 
-	m_sustainKnob = new knob( knobBright_26, this, tr( "Sustain-level" ) );
+	m_sustainKnob = new knob( knobBright_26, this, tr( "Sustain-level" ),
+								eng(), _track );
 	m_sustainKnob->setLabel( tr( "SUST" ) );
 	m_sustainKnob->setRange( 0.0, 1.0, 0.001 );
-	m_sustainKnob->setValue( 0.5, TRUE );
+	m_sustainKnob->setInitValue( 0.5 );
 	m_sustainKnob->move( SUSTAIN_KNOB_X, ENV_KNOBS_Y );
 	m_sustainKnob->setHintText( tr( "Sustain:" ) + " ", "" );
 #ifdef QT4
@@ -223,10 +236,11 @@ envelopeAndLFOWidget::envelopeAndLFOWidget( float _value_for_zero_amount,
 	connect( m_sustainKnob, SIGNAL( valueChanged( float ) ), this,
 				SLOT( updateAfterKnobChange( float ) ) );
 
-	m_releaseKnob = new knob( knobBright_26, this, tr( "Release-time" ) );
+	m_releaseKnob = new knob( knobBright_26, this, tr( "Release-time" ),
+								eng(), _track );
 	m_releaseKnob->setLabel( tr( "REL" ) );
 	m_releaseKnob->setRange( 0.0, 1.0, 0.001 );
-	m_releaseKnob->setValue( 0.1, TRUE );
+	m_releaseKnob->setInitValue( 0.1 );
 	m_releaseKnob->move( RELEASE_KNOB_X, ENV_KNOBS_Y );
 	m_releaseKnob->setHintText( tr( "Release:" ) + " ", "" );
 #ifdef QT4
@@ -243,10 +257,11 @@ envelopeAndLFOWidget::envelopeAndLFOWidget( float _value_for_zero_amount,
 				SLOT( updateAfterKnobChange( float ) ) );
 
 	m_amountKnob = new knob( knobBright_26, this,
-						tr( "Modulation amount" ) );
+						tr( "Modulation amount" ),
+						eng(), _track );
 	m_amountKnob->setLabel( tr( "AMT" ) );
 	m_amountKnob->setRange( -1.0, 1.0, 0.005 );
-	m_amountKnob->setValue( 0.0, TRUE );
+	m_amountKnob->setInitValue( 0.0 );
 	m_amountKnob->move( AMOUNT_KNOB_X, ENV_GRAPH_Y );
 	m_amountKnob->setHintText( tr( "Modulation amount:" ) + " ", "" );
 #ifdef QT4
@@ -264,10 +279,11 @@ envelopeAndLFOWidget::envelopeAndLFOWidget( float _value_for_zero_amount,
 
 
 	m_lfoPredelayKnob = new knob( knobBright_26, this,
-						tr( "LFO-predelay-time" ) );
+						tr( "LFO-predelay-time" ),
+						eng(), _track );
 	m_lfoPredelayKnob->setLabel( tr( "DEL" ) );
 	m_lfoPredelayKnob->setRange( 0.0, 1.0, 0.001 );
-	m_lfoPredelayKnob->setValue( 0.0, TRUE );
+	m_lfoPredelayKnob->setInitValue( 0.0 );
 	m_lfoPredelayKnob->move( LFO_PREDELAY_KNOB_X, LFO_KNOB_Y );
 	m_lfoPredelayKnob->setHintText( tr( "LFO-predelay:" ) + " ", "" );
 #ifdef QT4
@@ -282,10 +298,11 @@ envelopeAndLFOWidget::envelopeAndLFOWidget( float _value_for_zero_amount,
 				SLOT( updateAfterKnobChange( float ) ) );
 
 	m_lfoAttackKnob = new knob( knobBright_26, this,
-						tr( "LFO-attack-time" ) );
+						tr( "LFO-attack-time" ),
+						eng(), _track );
 	m_lfoAttackKnob->setLabel( tr( "ATT" ) );
 	m_lfoAttackKnob->setRange( 0.0, 1.0, 0.001 );
-	m_lfoAttackKnob->setValue( 0.0, TRUE );
+	m_lfoAttackKnob->setInitValue( 0.0 );
 	m_lfoAttackKnob->move( LFO_ATTACK_KNOB_X, LFO_KNOB_Y );
 	m_lfoAttackKnob->setHintText( tr( "LFO-attack:" ) + " ", "" );
 #ifdef QT4
@@ -300,10 +317,11 @@ envelopeAndLFOWidget::envelopeAndLFOWidget( float _value_for_zero_amount,
 				SLOT( updateAfterKnobChange( float ) ) );
 
 	m_lfoSpeedKnob = new tempoSyncKnob( knobBright_26, this,
-						tr( "LFO-speed" ), 20000.0 );
+						tr( "LFO-speed" ),
+						eng(), _track, 20000.0 );
 	m_lfoSpeedKnob->setLabel( tr( "SPD" ) );
 	m_lfoSpeedKnob->setRange( 0.01, 1.0, 0.0001 );
-	m_lfoSpeedKnob->setValue( 0.1, TRUE );
+	m_lfoSpeedKnob->setInitValue( 0.1 );
 	m_lfoSpeedKnob->move( LFO_SPEED_KNOB_X, LFO_KNOB_Y );
 	m_lfoSpeedKnob->setHintText( tr( "LFO-speed:" ) + " ", "" );
 #ifdef QT4
@@ -318,10 +336,11 @@ envelopeAndLFOWidget::envelopeAndLFOWidget( float _value_for_zero_amount,
 				SLOT( updateAfterKnobChange( float ) ) );
 
 	m_lfoAmountKnob = new knob( knobBright_26, this,
-						tr( "LFO-modulation-amount" ) );
+						tr( "LFO-modulation-amount" ),
+						eng(), _track );
 	m_lfoAmountKnob->setLabel( tr( "AMT" ) );
 	m_lfoAmountKnob->setRange( -1.0, 1.0, 0.005 );
-	m_lfoAmountKnob->setValue( 0.0, TRUE );
+	m_lfoAmountKnob->setInitValue( 0.0 );
 	m_lfoAmountKnob->move( LFO_AMOUNT_KNOB_X, LFO_KNOB_Y );
 	m_lfoAmountKnob->setHintText( tr( "Modulation amount:" ) + " ", "" );
 #ifdef QT4
@@ -337,101 +356,99 @@ envelopeAndLFOWidget::envelopeAndLFOWidget( float _value_for_zero_amount,
 				SLOT( updateAfterKnobChange( float ) ) );
 
 
-	m_sinLfoBtn = new pixmapButton( this );
-	m_sinLfoBtn->move( LFO_SHAPES_X, LFO_SHAPES_Y );
-	m_sinLfoBtn->setActiveGraphic( embed::getIconPixmap(
+	pixmapButton * sin_lfo_btn = new pixmapButton( this, NULL, eng(),
+									NULL );
+	sin_lfo_btn->move( LFO_SHAPES_X, LFO_SHAPES_Y );
+	sin_lfo_btn->setActiveGraphic( embed::getIconPixmap(
 							"sin_wave_active" ) );
-	m_sinLfoBtn->setInactiveGraphic( embed::getIconPixmap(
+	sin_lfo_btn->setInactiveGraphic( embed::getIconPixmap(
 							"sin_wave_inactive" ) );
-	m_sinLfoBtn->setChecked( TRUE );
 #ifdef QT4
-	m_sinLfoBtn->setWhatsThis(
+	sin_lfo_btn->setWhatsThis(
 #else
-	QWhatsThis::add( m_sinLfoBtn,
+	QWhatsThis::add( sin_lfo_btn,
 #endif
 		tr( "Click here if you want a sine-wave for current "
 							"oscillator." ) );
 
-	m_triangleLfoBtn = new pixmapButton( this );
-	m_triangleLfoBtn->move( LFO_SHAPES_X+15, LFO_SHAPES_Y );
-	m_triangleLfoBtn->setActiveGraphic( embed::getIconPixmap(
+	pixmapButton * triangle_lfo_btn = new pixmapButton( this, NULL, eng(),
+									NULL );
+	triangle_lfo_btn->move( LFO_SHAPES_X+15, LFO_SHAPES_Y );
+	triangle_lfo_btn->setActiveGraphic( embed::getIconPixmap(
 						"triangle_wave_active" ) );
-	m_triangleLfoBtn->setInactiveGraphic( embed::getIconPixmap(
+	triangle_lfo_btn->setInactiveGraphic( embed::getIconPixmap(
 						"triangle_wave_inactive" ) );
 #ifdef QT4
-	m_triangleLfoBtn->setWhatsThis(
+	triangle_lfo_btn->setWhatsThis(
 #else
-	QWhatsThis::add( m_triangleLfoBtn,
+	QWhatsThis::add( triangle_lfo_btn,
 #endif
 		tr( "Click here if you want a triangle-wave for current "
 							"oscillator." ) );
 
-	m_sawLfoBtn = new pixmapButton( this );
-	m_sawLfoBtn->move( LFO_SHAPES_X+30, LFO_SHAPES_Y );
-	m_sawLfoBtn->setActiveGraphic( embed::getIconPixmap(
+	pixmapButton * saw_lfo_btn = new pixmapButton( this, NULL, eng(),
+									NULL );
+	saw_lfo_btn->move( LFO_SHAPES_X+30, LFO_SHAPES_Y );
+	saw_lfo_btn->setActiveGraphic( embed::getIconPixmap(
 							"saw_wave_active" ) );
-	m_sawLfoBtn->setInactiveGraphic( embed::getIconPixmap(
+	saw_lfo_btn->setInactiveGraphic( embed::getIconPixmap(
 							"saw_wave_inactive" ) );
 #ifdef QT4
-	m_sawLfoBtn->setWhatsThis(
+	saw_lfo_btn->setWhatsThis(
 #else
-	QWhatsThis::add( m_sawLfoBtn,
+	QWhatsThis::add( saw_lfo_btn,
 #endif
 		tr( "Click here if you want a saw-wave for current "
 							"oscillator." ) );
 
-	m_sqrLfoBtn = new pixmapButton( this );
-	m_sqrLfoBtn->move( LFO_SHAPES_X+45, LFO_SHAPES_Y );
-	m_sqrLfoBtn->setActiveGraphic( embed::getIconPixmap(
+	pixmapButton * sqr_lfo_btn = new pixmapButton( this, NULL, eng(),
+									NULL );
+	sqr_lfo_btn->move( LFO_SHAPES_X+45, LFO_SHAPES_Y );
+	sqr_lfo_btn->setActiveGraphic( embed::getIconPixmap(
 						"square_wave_active" ) );
-	m_sqrLfoBtn->setInactiveGraphic( embed::getIconPixmap(
+	sqr_lfo_btn->setInactiveGraphic( embed::getIconPixmap(
 						"square_wave_inactive" ) );
 #ifdef QT4
-	m_sqrLfoBtn->setWhatsThis(
+	sqr_lfo_btn->setWhatsThis(
 #else
-	QWhatsThis::add( m_sqrLfoBtn,
+	QWhatsThis::add( sqr_lfo_btn,
 #endif
 		tr( "Click here if you want a square-wave for current "
 							"oscillator." ) );
 
-	m_usrLfoBtn = new pixmapButton( this );
-	m_usrLfoBtn->move( LFO_SHAPES_X+60, LFO_SHAPES_Y );
-	m_usrLfoBtn->setActiveGraphic( embed::getIconPixmap(
+	m_userLfoBtn = new pixmapButton( this, NULL, eng(), NULL );
+	m_userLfoBtn->move( LFO_SHAPES_X+60, LFO_SHAPES_Y );
+	m_userLfoBtn->setActiveGraphic( embed::getIconPixmap(
 							"usr_wave_active" ) );
-	m_usrLfoBtn->setInactiveGraphic( embed::getIconPixmap(
+	m_userLfoBtn->setInactiveGraphic( embed::getIconPixmap(
 							"usr_wave_inactive" ) );
 #ifdef QT4
-	m_usrLfoBtn->setWhatsThis(
+	m_userLfoBtn->setWhatsThis(
 #else
-	QWhatsThis::add( m_usrLfoBtn,
+	QWhatsThis::add( m_userLfoBtn,
 #endif
 		tr( "Click here if you want a user-defined wave for current "
 			"oscillator. Afterwards drag an according sample-"
 			"file into LFO-graph." ) );
 
-	connect( m_sinLfoBtn, SIGNAL( toggled( bool ) ), this,
-					SLOT( lfoSinWaveCh( bool ) ) );
-	connect( m_triangleLfoBtn, SIGNAL( toggled( bool ) ), this,
-					SLOT( lfoTriangleWaveCh( bool ) ) );
-	connect( m_sawLfoBtn, SIGNAL( toggled( bool ) ), this,
-					SLOT( lfoSawWaveCh( bool ) ) );
-	connect( m_sqrLfoBtn, SIGNAL( toggled( bool ) ), this,
-					SLOT( lfoSquareWaveCh( bool ) ) );
-	connect( m_usrLfoBtn, SIGNAL( toggled( bool ) ), this,
+	connect( m_userLfoBtn, SIGNAL( toggled( bool ) ), this,
 					SLOT( lfoUserWaveCh( bool ) ) );
 
-	QButtonGroup * lfo_shapes_algo_group = new QButtonGroup( this );
-	lfo_shapes_algo_group->addButton( m_sinLfoBtn );
-	lfo_shapes_algo_group->addButton( m_triangleLfoBtn );
-	lfo_shapes_algo_group->addButton( m_sawLfoBtn );
-	lfo_shapes_algo_group->addButton( m_sqrLfoBtn );
-	lfo_shapes_algo_group->addButton( m_usrLfoBtn );
-	lfo_shapes_algo_group->setExclusive( TRUE );
-#ifndef QT4
-	lfo_shapes_algo_group->hide();
-#endif
+	m_lfoWaveBtnGrp = new automatableButtonGroup( this,
+							tr( "LFO wave shape" ),
+							eng(), _track );
+	m_lfoWaveBtnGrp->addButton( sin_lfo_btn );
+	m_lfoWaveBtnGrp->addButton( triangle_lfo_btn );
+	m_lfoWaveBtnGrp->addButton( saw_lfo_btn );
+	m_lfoWaveBtnGrp->addButton( sqr_lfo_btn );
+	m_lfoWaveBtnGrp->addButton( m_userLfoBtn );
+	m_lfoWaveBtnGrp->setInitValue( SIN );
 
-	m_x100Cb = new ledCheckBox( tr( "FREQ x 100" ), this );
+	connect( m_lfoWaveBtnGrp, SIGNAL( valueChanged( int ) ),
+						SLOT( lfoWaveCh( int ) ) );
+
+	m_x100Cb = new ledCheckBox( tr( "FREQ x 100" ), this,
+					tr( "Freq x 100" ), eng(), _track );
 	m_x100Cb->setFont( pointSize<6>( m_x100Cb->font() ) );
 	m_x100Cb->move( LFO_PREDELAY_KNOB_X, LFO_GRAPH_Y + 36 );
 #ifdef QT4
@@ -447,7 +464,8 @@ envelopeAndLFOWidget::envelopeAndLFOWidget( float _value_for_zero_amount,
 
 
 	m_controlEnvAmountCb = new ledCheckBox( tr( "MODULATE ENV-AMOUNT" ),
-									this );
+					this, tr( "Modulate Env-Amount" ),
+					eng(), _track );
 	m_controlEnvAmountCb->move( LFO_PREDELAY_KNOB_X, LFO_GRAPH_Y + 54 );
 	m_controlEnvAmountCb->setFont( pointSize<6>(
 					m_controlEnvAmountCb->font() ) );
@@ -468,7 +486,7 @@ envelopeAndLFOWidget::envelopeAndLFOWidget( float _value_for_zero_amount,
 #endif
 	setAcceptDrops( TRUE );
 
-	connect( mixer::inst(), SIGNAL( sampleRateChanged() ), this,
+	connect( eng()->getMixer(), SIGNAL( sampleRateChanged() ), this,
 						SLOT( updateSampleVars() ) );
 
 	updateSampleVars();
@@ -482,29 +500,46 @@ envelopeAndLFOWidget::~envelopeAndLFOWidget()
 	delete[] m_pahdEnv;
 	delete[] m_rEnv;
 	delete[] m_lfoShapeData;
+
+	vvector<envelopeAndLFOWidget *> & v = s_EaLWidgets[eng()];
+	if( qFind( v.begin(), v.end(), this ) != v.end() )
+	{
+		v.erase( qFind( v.begin(), v.end(), this ) );
+	}
 }
 
 
 
 
-void envelopeAndLFOWidget::triggerLFO( void )
+void envelopeAndLFOWidget::triggerLFO( engine * _engine )
 {
-	s_lfoFrame += mixer::inst()->framesPerAudioBuffer();
+	vvector<envelopeAndLFOWidget *> & v = s_EaLWidgets[_engine];
+	for( vvector<envelopeAndLFOWidget *>::iterator it = v.begin();
+							it != v.end(); ++it )
+	{
+		( *it )->m_lfoFrame +=
+				_engine->getMixer()->framesPerAudioBuffer();
+	}
 }
 
 
 
 
-void envelopeAndLFOWidget::resetLFO( void )
+void envelopeAndLFOWidget::resetLFO( engine * _engine )
 {
-	s_lfoFrame = 0;
+	vvector<envelopeAndLFOWidget *> & v = s_EaLWidgets[_engine];
+	for( vvector<envelopeAndLFOWidget *>::iterator it = v.begin();
+							it != v.end(); ++it )
+	{
+		( *it )->m_lfoFrame = 0;
+	}
 }
 
 
 
 
-inline float FASTCALL envelopeAndLFOWidget::lfoLevel( Uint32 _frame,
-						Uint32 _frame_offset ) const
+inline float FASTCALL envelopeAndLFOWidget::lfoLevel( f_cnt_t _frame,
+					const f_cnt_t _frame_offset ) const
 {
 	if( m_lfoAmountIsZero == FALSE && _frame > m_lfoPredelayFrames )
 	{
@@ -514,10 +549,10 @@ inline float FASTCALL envelopeAndLFOWidget::lfoLevel( Uint32 _frame,
 		_frame -= m_lfoPredelayFrames;
 		if( _frame > m_lfoAttackFrames )
 		{
-			return( m_lfoShapeData[( s_lfoFrame + _frame_offset ) %
+			return( m_lfoShapeData[( m_lfoFrame + _frame_offset ) %
 						m_lfoOscillationFrames] );
 		}
-		return( m_lfoShapeData[( s_lfoFrame + _frame_offset ) %
+		return( m_lfoShapeData[( m_lfoFrame + _frame_offset ) %
 						m_lfoOscillationFrames] *
 						_frame / m_lfoAttackFrames );
 	}
@@ -527,87 +562,65 @@ inline float FASTCALL envelopeAndLFOWidget::lfoLevel( Uint32 _frame,
 
 
 
-float FASTCALL envelopeAndLFOWidget::level( Uint32 _frame,
-						Uint32 _release_begin,
-						Uint32 _frame_offset ) const
+float FASTCALL envelopeAndLFOWidget::level( f_cnt_t _frame,
+					const f_cnt_t _release_begin,
+					const f_cnt_t _frame_offset ) const
 {
 	if( m_busy )
 	{
 		return( 0.0f );
 	}
-	if( _frame < m_pahdFrames )
+	const float lfo_level = lfoLevel( _frame, _frame_offset );
+	float env_level;
+	if( _frame < _release_begin && _frame < m_pahdFrames )
 	{
-		if( m_controlEnvAmountCb->isChecked() )
+		env_level = m_pahdEnv[_frame];
+	}
+	else if( _frame >= _release_begin )
+	{
+		if( ( _frame -= _release_begin ) < m_rFrames )
 		{
-			return( m_pahdEnv[_frame] * ( 0.5f +
-					lfoLevel( _frame, _frame_offset ) ) );
+			env_level = m_rEnv[_frame] *
+				( ( _release_begin < m_pahdFrames ) ?
+				m_pahdEnv[_release_begin] : m_sustainLevel );
 		}
 		else
 		{
-			return( m_pahdEnv[_frame] +
-					lfoLevel( _frame, _frame_offset ) );
+			env_level = 0.0f;
 		}
 	}
-	else if( _frame > _release_begin )
+	else
 	{
-		_frame -= _release_begin;
-		if( _frame < m_rFrames )
-		{
-			if( m_controlEnvAmountCb->isChecked() )
-			{
-				return( m_rEnv[_frame] * ( 0.5f +
-					lfoLevel( _frame, _frame_offset ) ) );
-			}
-			else
-			{
-				return( m_rEnv[_frame] +
-					lfoLevel( _frame, _frame_offset ) );
-			}
-		}
-		else
-		{
-			return( 0.0f );
-		}
+		env_level = m_sustainLevel;
 	}
-	if( m_controlEnvAmountCb->isChecked() )
-	{
-		return( m_sustainLevel * ( 0.5f +
-					lfoLevel( _frame, _frame_offset ) ) );
-	}
-	return( m_sustainLevel + lfoLevel( _frame, _frame_offset ) );
+	return( m_controlEnvAmountCb->isChecked() ?
+			env_level * ( 0.5f + lfo_level )
+		:
+			env_level + lfo_level );
 }
 
 
 
 
-void envelopeAndLFOWidget::saveSettings( QDomDocument & ,
+void envelopeAndLFOWidget::saveSettings( QDomDocument & _doc,
 							QDomElement & _parent )
 {
-	_parent.setAttribute( "pdel", QString::number(
-						m_predelayKnob->value() ) );
-	_parent.setAttribute( "att", QString::number( m_attackKnob->value() ) );
-	_parent.setAttribute( "hold", QString::number( m_holdKnob->value() ) );
-	_parent.setAttribute( "dec", QString::number( m_decayKnob->value() ) );
-	_parent.setAttribute( "sus", QString::number(
-						m_sustainKnob->value() ) );
-	_parent.setAttribute( "rel", QString::number(
-						m_releaseKnob->value() ) );
-	_parent.setAttribute( "amt", QString::number( m_amountKnob->value() ) );
-	_parent.setAttribute( "lshp", QString::number( m_lfoShape ) );
-	_parent.setAttribute( "lpdel", QString::number(
-						m_lfoPredelayKnob->value() ) );
-	_parent.setAttribute( "latt", QString::number(
-						m_lfoAttackKnob->value() ) );
-	_parent.setAttribute( "lspd", QString::number(
-						m_lfoSpeedKnob->value() ) );
-	_parent.setAttribute( "lamt", QString::number(
-						m_lfoAmountKnob->value() ) );
-	_parent.setAttribute( "x100", QString::number(
-						m_x100Cb->isChecked() ) );
-	_parent.setAttribute( "ctlenvamt", QString::number(
-					m_controlEnvAmountCb->isChecked() ) );
-	_parent.setAttribute( "lfosyncmode", QString::number(
-				( int ) m_lfoSpeedKnob->getSyncMode() ) );
+	m_predelayKnob->saveSettings( _doc, _parent, "pdel" );
+	m_attackKnob->saveSettings( _doc, _parent, "att" );
+	m_holdKnob->saveSettings( _doc, _parent, "hold" );
+	m_decayKnob->saveSettings( _doc, _parent, "dec" );
+	m_sustainKnob->saveSettings( _doc, _parent, "sus" );
+	m_releaseKnob->saveSettings( _doc, _parent, "rel" );
+	m_amountKnob->saveSettings( _doc, _parent, "amt" );
+	m_lfoWaveBtnGrp->saveSettings( _doc, _parent, "lshp" );
+	m_lfoPredelayKnob->saveSettings( _doc, _parent, "lpdel" );
+	m_lfoAttackKnob->saveSettings( _doc, _parent, "latt" );
+	m_lfoSpeedKnob->saveSettings( _doc, _parent, "lspd" );
+	m_lfoAmountKnob->saveSettings( _doc, _parent, "lamt" );
+	m_x100Cb->saveSettings( _doc, _parent, "x100" );
+	m_controlEnvAmountCb->saveSettings( _doc, _parent, "ctlenvamt" );
+	_parent.setAttribute( "lfosyncmode",
+					( int ) m_lfoSpeedKnob->getSyncMode() );
 	_parent.setAttribute( "userwavefile", m_userWave.audioFile() );
 }
 
@@ -618,50 +631,25 @@ void envelopeAndLFOWidget::loadSettings( const QDomElement & _this )
 {
 	m_busy = TRUE;
 
-	m_predelayKnob->setValue( _this.attribute( "pdel" ).toFloat() );
-	m_attackKnob->setValue( _this.attribute( "att" ).toFloat() );
-	m_holdKnob->setValue( _this.attribute( "hold" ).toFloat() );
-	m_decayKnob->setValue( _this.attribute( "dec" ).toFloat() );
-	m_sustainKnob->setValue( _this.attribute( "sus" ).toFloat() );
-	m_releaseKnob->setValue( _this.attribute( "rel" ).toFloat() );
-	m_amountKnob->setValue( _this.attribute( "amt" ).toFloat() );
-	m_lfoShape = static_cast<lfoShapes>( _this.attribute(
-							"lshp" ).toInt() );
-	m_lfoPredelayKnob->setValue( _this.attribute( "lpdel" ).toFloat() );
-	m_lfoAttackKnob->setValue( _this.attribute( "latt" ).toFloat() );
-	m_lfoSpeedKnob->setValue( _this.attribute( "lspd" ).toFloat() );
-	m_lfoAmountKnob->setValue( _this.attribute( "lamt" ).toFloat() );
-	m_x100Cb->setChecked( _this.attribute( "x100" ).toInt() );
-	m_controlEnvAmountCb->setChecked( _this.attribute(
-							"ctlenvamt" ).toInt() );
+	m_predelayKnob->loadSettings( _this, "pdel" );
+	m_attackKnob->loadSettings( _this, "att" );
+	m_holdKnob->loadSettings( _this, "hold" );
+	m_decayKnob->loadSettings( _this, "dec" );
+	m_sustainKnob->loadSettings( _this, "sus" );
+	m_releaseKnob->loadSettings( _this, "rel" );
+	m_amountKnob->loadSettings( _this, "amt" );
+	m_lfoWaveBtnGrp->loadSettings( _this, "lshp" );
+	m_lfoPredelayKnob->loadSettings( _this, "lpdel" );
+	m_lfoAttackKnob->loadSettings( _this, "latt" );
+	m_lfoSpeedKnob->loadSettings( _this, "lspd" );
+	m_lfoAmountKnob->loadSettings( _this, "lamt" );
+	m_x100Cb->loadSettings( _this, "x100" );
+	m_controlEnvAmountCb->loadSettings( _this, "ctlenvamt" );
 	m_lfoSpeedKnob->setSyncMode(
 		( tempoSyncKnob::tempoSyncMode ) _this.attribute(
 						"lfosyncmode" ).toInt() );
 	m_userWave.setAudioFile( _this.attribute( "userwavefile" ) );
 
-	switch( m_lfoShape )
-	{
-		case SIN:
-			m_sinLfoBtn->setChecked( TRUE );
-			break;
-
-		case TRIANGLE:
-			m_triangleLfoBtn->setChecked( TRUE );
-			break;
-
-		case SAW:
-			m_sawLfoBtn->setChecked( TRUE );
-			break;
-
-		case SQUARE:
-			m_sqrLfoBtn->setChecked( TRUE );
-			break;
-
-		case USER:
-			m_usrLfoBtn->setChecked( TRUE );
-			break;
-	}
-
 	m_busy = FALSE;
 
 	updateSampleVars();
@@ -726,7 +714,7 @@ void envelopeAndLFOWidget::dropEvent( QDropEvent * _de )
 	if( type == "samplefile" )
 	{
 		m_userWave.setAudioFile( stringPairDrag::decodeValue( _de ) );
-		m_usrLfoBtn->setChecked( TRUE );
+		m_userLfoBtn->setChecked( TRUE );
 		lfoUserWaveCh( TRUE );
 		_de->accept();
 	}
@@ -735,7 +723,7 @@ void envelopeAndLFOWidget::dropEvent( QDropEvent * _de )
 		multimediaProject mmp( value, FALSE );
 		m_userWave.setAudioFile( mmp.content().firstChild().toElement().
 							attribute( "src" ) );
-		m_usrLfoBtn->setChecked( TRUE );
+		m_userLfoBtn->setChecked( TRUE );
 		lfoUserWaveCh( TRUE );
 		_de->accept();
 	}
@@ -834,7 +822,7 @@ void envelopeAndLFOWidget::paintEvent( QPaintEvent * )
 	int graph_y_base = LFO_GRAPH_Y + 3 + LFO_GRAPH_H / 2;
 
 	const float frames_for_graph = SECS_PER_LFO_OSCILLATION *
-					mixer::inst()->sampleRate() / 10;
+					eng()->getMixer()->sampleRate() / 10;
 
 	const float lfo_gray_amount = 1.0f - fabsf( m_lfoAmountKnob->value() );
 	p.setPen( QPen( QColor( static_cast<int>( 96 * lfo_gray_amount ),
@@ -860,7 +848,7 @@ void envelopeAndLFOWidget::paintEvent( QPaintEvent * )
 	{
 		float val = 0.0;
 		float cur_sample = x * frames_for_graph / LFO_GRAPH_W;
-		if( static_cast<Uint32>( cur_sample ) > m_lfoPredelayFrames )
+		if( static_cast<f_cnt_t>( cur_sample ) > m_lfoPredelayFrames )
 		{
 			float phase = ( cur_sample -= m_lfoPredelayFrames ) /
 								osc_frames;
@@ -880,11 +868,11 @@ void envelopeAndLFOWidget::paintEvent( QPaintEvent * )
 					val = oscillator::squareSample( phase );
 					break;
 				case USER:
-					val = oscillator::userWaveSample( phase,
-							m_userWave.data(),
-							m_userWave.frames() );
+					val = m_userWave.userWaveSample(
+								phase );
 			}
-			if( (Uint32) cur_sample <= m_lfoAttackFrames )
+			if( static_cast<f_cnt_t>( cur_sample ) <=
+							m_lfoAttackFrames )
 			{
 				val *= cur_sample / m_lfoAttackFrames;
 			}
@@ -927,18 +915,21 @@ void envelopeAndLFOWidget::updateSampleVars( void )
 		m_busy = TRUE;
 
 		const float frames_per_env_seg = SECS_PER_ENV_SEGMENT *
-						mixer::inst()->sampleRate();
-		Uint32 predelay_frames = static_cast<Uint32>(
+						eng()->getMixer()->sampleRate();
+		const f_cnt_t predelay_frames = static_cast<f_cnt_t>(
 							frames_per_env_seg *
 					expKnobVal( m_predelayKnob->value() ) );
 
-		Uint32 attack_frames = static_cast<Uint32>( frames_per_env_seg *
+		const f_cnt_t attack_frames = static_cast<f_cnt_t>(
+							frames_per_env_seg *
 					expKnobVal( m_attackKnob->value() ) );
 
-		Uint32 hold_frames = static_cast<Uint32>( frames_per_env_seg *
+		const f_cnt_t hold_frames = static_cast<f_cnt_t>(
+							frames_per_env_seg *
 					expKnobVal( m_holdKnob->value() ) );
 
-		Uint32 decay_frames = static_cast<Uint32>( frames_per_env_seg *
+		const f_cnt_t decay_frames = static_cast<f_cnt_t>(
+							frames_per_env_seg *
 					expKnobVal( m_decayKnob->value() *
 						m_sustainKnob->value() ) );
 
@@ -956,7 +947,7 @@ void envelopeAndLFOWidget::updateSampleVars( void )
 
 		m_pahdFrames = predelay_frames + attack_frames + hold_frames +
 								decay_frames;
-		m_rFrames = static_cast<Uint32>( frames_per_env_seg *
+		m_rFrames = static_cast<f_cnt_t>( frames_per_env_seg *
 					expKnobVal( m_releaseKnob->value() ) );
 
 		if( static_cast<int>( floorf( m_amount * 1000.0f ) ) == 0 )
@@ -965,30 +956,30 @@ void envelopeAndLFOWidget::updateSampleVars( void )
 			m_rFrames = 0;
 		}
 
-		float * new_pahd_env = new float[m_pahdFrames];
-		float * new_r_env = new float[m_rFrames];
+		sample_t * new_pahd_env = new sample_t[m_pahdFrames];
+		sample_t * new_r_env = new sample_t[m_rFrames];
 
-		for( Uint32 i = 0; i < predelay_frames; ++i )
+		for( f_cnt_t i = 0; i < predelay_frames; ++i )
 		{
 			new_pahd_env[i] = m_amountAdd;
 		}
 
-		Uint32 add = predelay_frames;
+		f_cnt_t add = predelay_frames;
 
-		for( Uint32 i = 0; i < attack_frames; ++i )
+		for( f_cnt_t i = 0; i < attack_frames; ++i )
 		{
 			new_pahd_env[add+i] = ( (float)i / attack_frames ) *
 							m_amount + m_amountAdd;
 		}
 
 		add += attack_frames;
-		for( Uint32 i = 0; i < hold_frames; ++i )
+		for( f_cnt_t i = 0; i < hold_frames; ++i )
 		{
 			new_pahd_env[add+i] = m_amount + m_amountAdd;
 		}
 
 		add += hold_frames;
-		for( Uint32 i = 0; i < decay_frames; ++i )
+		for( f_cnt_t i = 0; i < decay_frames; ++i )
 		{
 			new_pahd_env[add+i] = ( m_sustainLevel + ( 1.0f -
 						(float)i / decay_frames ) *
@@ -1002,10 +993,11 @@ void envelopeAndLFOWidget::updateSampleVars( void )
 		m_pahdEnv = new_pahd_env;
 		m_rEnv = new_r_env;
 
-		for( Uint32 i = 0; i < m_rFrames; ++i )
+		for( f_cnt_t i = 0; i < m_rFrames; ++i )
 		{
-			new_r_env[i] = ( (float)( m_rFrames - i ) / m_rFrames *
-						m_sustainLevel ) * m_amount;
+			new_r_env[i] = ( (float)( m_rFrames - i ) / m_rFrames
+					// * m_sustainLevel
+					 		) * m_amount;
 		}
 
 		// save this calculation in real-time-part
@@ -1014,14 +1006,14 @@ void envelopeAndLFOWidget::updateSampleVars( void )
 
 		const float frames_per_lfo_oscillation =
 						SECS_PER_LFO_OSCILLATION *
-						mixer::inst()->sampleRate();
-		m_lfoPredelayFrames = static_cast<Uint32>(
+						eng()->getMixer()->sampleRate();
+		m_lfoPredelayFrames = static_cast<f_cnt_t>(
 						frames_per_lfo_oscillation *
 				expKnobVal( m_lfoPredelayKnob->value() ) );
-		m_lfoAttackFrames = static_cast<Uint32>(
+		m_lfoAttackFrames = static_cast<f_cnt_t>(
 						frames_per_lfo_oscillation *
 				expKnobVal( m_lfoAttackKnob->value() ) );
-		m_lfoOscillationFrames = static_cast<Uint32>(
+		m_lfoOscillationFrames = static_cast<f_cnt_t>(
 						frames_per_lfo_oscillation *
 						m_lfoSpeedKnob->value() );
 		if( m_x100Cb->isChecked() )
@@ -1048,8 +1040,8 @@ void envelopeAndLFOWidget::updateSampleVars( void )
 		if( m_lfoAmountIsZero == FALSE )
 		{
 			delete[] m_lfoShapeData;
-			m_lfoShapeData = new float[m_lfoOscillationFrames];
-			for( Uint32 frame = 0; frame < m_lfoOscillationFrames;
+			m_lfoShapeData = new sample_t[m_lfoOscillationFrames];
+			for( f_cnt_t frame = 0; frame < m_lfoOscillationFrames;
 								++frame )
 			{
 				const float phase = frame / static_cast<float>(
@@ -1075,9 +1067,7 @@ void envelopeAndLFOWidget::updateSampleVars( void )
 						break;
 					case USER:
 						m_lfoShapeData[frame] =
-				oscillator::userWaveSample( phase,
-							m_userWave.data(),
-							m_userWave.frames() );
+				m_userWave.userWaveSample( phase );
 						break;
 					case SIN:
 					default:
@@ -1097,7 +1087,7 @@ void envelopeAndLFOWidget::updateSampleVars( void )
 
 void envelopeAndLFOWidget::x100Toggled( bool )
 {
-	songEditor::inst()->setModified();
+	eng()->getSongEditor()->setModified();
 	updateSampleVars();
 }
 
@@ -1112,56 +1102,9 @@ void envelopeAndLFOWidget::updateAfterKnobChange( float )
 
 
 
-void envelopeAndLFOWidget::lfoSinWaveCh( bool _on )
+void envelopeAndLFOWidget::lfoWaveCh( int _val )
 {
-	if( _on )
-	{
-		m_lfoShape = SIN;
-	}
-	songEditor::inst()->setModified();
-
-	update();
-}
-
-
-
-
-void envelopeAndLFOWidget::lfoTriangleWaveCh( bool _on )
-{
-	if( _on )
-	{
-		m_lfoShape = TRIANGLE;
-	}
-	songEditor::inst()->setModified();
-
-	update();
-}
-
-
-
-
-void envelopeAndLFOWidget::lfoSawWaveCh( bool _on )
-{
-	if( _on )
-	{
-		m_lfoShape = SAW;
-	}
-	songEditor::inst()->setModified();
-
-	update();
-}
-
-
-
-
-void envelopeAndLFOWidget::lfoSquareWaveCh( bool _on )
-{
-	if( _on )
-	{
-		m_lfoShape = SQUARE;
-	}
-	songEditor::inst()->setModified();
-
+	m_lfoShape = static_cast<lfoShapes>( _val );
 	update();
 }
 
@@ -1170,7 +1113,7 @@ void envelopeAndLFOWidget::lfoSquareWaveCh( bool _on )
 
 void envelopeAndLFOWidget::lfoUserWaveCh( bool _on )
 {
-	if( _on )
+	if( _on && m_lfoShape != USER )
 	{
 		if( m_userWave.frames() <= 1 )
 		{
@@ -1181,7 +1124,7 @@ void envelopeAndLFOWidget::lfoUserWaveCh( bool _on )
 		}
 		m_lfoShape = USER;
 	}
-	songEditor::inst()->setModified();
+	eng()->getSongEditor()->setModified();
 
 	update();
 }
@@ -1191,3 +1134,5 @@ void envelopeAndLFOWidget::lfoUserWaveCh( bool _on )
 
 #include "envelope_and_lfo_widget.moc"
 
+
+#endif
