@@ -1,8 +1,10 @@
+#ifndef SINGLE_SOURCE_COMPILE
+
 /*
  * envelope_tab_widget.cpp - widget for use in envelope/lfo/filter-tab of
- *                           channel-window
+ *                           instrument-track-window
  *
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -28,19 +30,13 @@
 
 #ifdef QT4
 
-#include <QComboBox>
 #include <Qt/QtXml>
-#include <QWhatsThis>
 
 #else
 
-#include <qcombobox.h>
 #include <qdom.h>
 #include <qwhatsthis.h>
 
-#define isChecked isOn
-#define setChecked setOn
-
 #endif
 
 
@@ -53,7 +49,8 @@
 #include "tab_widget.h"
 #include "embed.h"
 #include "gui_templates.h"
-#include "channel_track.h"
+#include "instrument_track.h"
+#include "combobox.h"
 
 
 
@@ -85,9 +82,9 @@ static const QString targetNames[envelopeTabWidget::TARGET_COUNT][2] =
  
 
 
-envelopeTabWidget::envelopeTabWidget( channelTrack * _channel_track ) :
-	QWidget( _channel_track->tabWidgetParent() ),
-	settings()
+envelopeTabWidget::envelopeTabWidget( instrumentTrack * _instrument_track ) :
+	QWidget( _instrument_track->tabWidgetParent() ),
+	journallingObject( _instrument_track->eng() )
 {
 
 	m_targetsTabWidget = new tabWidget( tr( "TARGET" ), this );
@@ -122,7 +119,10 @@ envelopeTabWidget::envelopeTabWidget( channelTrack * _channel_track ) :
 			value_for_zero_amount = 1.0;
 		}
 		m_envLFOWidgets[i] = new envelopeAndLFOWidget(
-				value_for_zero_amount, m_targetsTabWidget );
+							value_for_zero_amount,
+							m_targetsTabWidget,
+							eng(),
+							_instrument_track );
 		m_targetsTabWidget->addTab( m_envLFOWidgets[i],
 						tr( targetNames[i][0]
 #ifdef QT4
@@ -137,24 +137,34 @@ envelopeTabWidget::envelopeTabWidget( channelTrack * _channel_track ) :
 	}
 	
 	
-	m_filterGroupBox = new groupBox( tr( "FILTER" ), this );
+	m_filterGroupBox = new groupBox( tr( "FILTER" ), this, eng(),
+							_instrument_track );
 	m_filterGroupBox->setGeometry( FILTER_GROUPBOX_X, FILTER_GROUPBOX_Y,
 						FILTER_GROUPBOX_WIDTH,
 						FILTER_GROUPBOX_HEIGHT );
 
-	m_filterComboBox = new QComboBox( m_filterGroupBox );
-	m_filterComboBox->setGeometry( 20, 20, 100, 22 );
+	m_filterComboBox = new comboBox( m_filterGroupBox, tr( "Filter type" ),
+						eng(), _instrument_track );
+	m_filterComboBox->setGeometry( 14, 22, 120, 22 );
 	m_filterComboBox->setFont( pointSize<8>( m_filterComboBox->font() ) );
 
 
-	m_filterComboBox->addItem( tr( "LowPass" ) );
-	m_filterComboBox->addItem( tr( "HiPass" ) );
-	m_filterComboBox->addItem( tr( "BandPass csg" ) );
-	m_filterComboBox->addItem( tr( "BandPass czpg" ) );
-	m_filterComboBox->addItem( tr( "Notch" ) );
-	m_filterComboBox->addItem( tr( "Allpass" ) );
-	m_filterComboBox->addItem( tr( "Moog" ) );
-	m_filterComboBox->addItem( tr( "2x LowPass" ) );
+	m_filterComboBox->addItem( tr( "LowPass" ),
+					embed::getIconPixmap( "filter_lp" ) );
+	m_filterComboBox->addItem( tr( "HiPass" ),
+					embed::getIconPixmap( "filter_hp" ) );
+	m_filterComboBox->addItem( tr( "BandPass csg" ),
+					embed::getIconPixmap( "filter_bp" ) );
+	m_filterComboBox->addItem( tr( "BandPass czpg" ),
+					embed::getIconPixmap( "filter_bp" ) );
+	m_filterComboBox->addItem( tr( "Notch" ),
+				embed::getIconPixmap( "filter_notch" ) );
+	m_filterComboBox->addItem( tr( "Allpass" ),
+					embed::getIconPixmap( "filter_ap" ) );
+	m_filterComboBox->addItem( tr( "Moog" ),
+					embed::getIconPixmap( "filter_lp" ) );
+	m_filterComboBox->addItem( tr( "2x LowPass" ),
+					embed::getIconPixmap( "filter_2lp" ) );
 
 #ifdef QT4
 	m_filterComboBox->setWhatsThis(
@@ -162,16 +172,17 @@ envelopeTabWidget::envelopeTabWidget( channelTrack * _channel_track ) :
 	QWhatsThis::add( m_filterComboBox,
 #endif
 		tr( "Here you can select the built-in filter you want to use "
-			"in this channel. Filters are very important for "
-			"changing the characteristics of a sound." ) );
+			"for this instrument-track. Filters are very important "
+			"for changing the characteristics of a sound." ) );
 
 
-	m_filterCutKnob = new knob( knobBright_26, m_filterGroupBox, tr(
-							"cutoff-frequency" ) );
+	m_filterCutKnob = new knob( knobBright_26, m_filterGroupBox,
+						tr( "cutoff-frequency" ),
+						eng(), _instrument_track );
 	m_filterCutKnob->setLabel( tr( "CUTOFF" ) );
-	m_filterCutKnob->setRange( 0.0, 16000.0, 1.0 );
+	m_filterCutKnob->setRange( 0.0, 14000.0, 1.0 );
 	m_filterCutKnob->move( 140, 18 );
-	m_filterCutKnob->setValue( 16000.0, TRUE );
+	m_filterCutKnob->setInitValue( 16000.0 );
 	m_filterCutKnob->setHintText( tr( "cutoff-frequency:" ) + " ", " " +
 								tr( "Hz" ) );
 #ifdef QT4
@@ -186,12 +197,13 @@ envelopeTabWidget::envelopeTabWidget( channelTrack * _channel_track ) :
 			"the cutoff-frequency. A highpass-filter cuts all "
 			"frequencies below cutoff-frequency and so on..." ) );
 
-	m_filterResKnob = new knob( knobBright_26, m_filterGroupBox, tr(
-							"Q/Resonance" ) );
+	m_filterResKnob = new knob( knobBright_26, m_filterGroupBox,
+						tr( "Q/Resonance" ),
+						eng(), _instrument_track );
 	m_filterResKnob->setLabel( tr( "Q/RESO" ) );
 	m_filterResKnob->setRange( 0.01, 10.0, 0.01 );
 	m_filterResKnob->move( 190, 18 );
-	m_filterResKnob->setValue( 0.5, TRUE );
+	m_filterResKnob->setInitValue( 0.5 );
 	m_filterResKnob->setHintText( tr( "Q/Resonance:" ) + " ", "" );
 #ifdef QT4
 	m_filterResKnob->setWhatsThis(
@@ -216,14 +228,14 @@ envelopeTabWidget::~envelopeTabWidget()
 
 
 float FASTCALL envelopeTabWidget::volumeLevel( notePlayHandle * _n,
-								Uint32 _frame )
+							const f_cnt_t _frame )
 {
-	Uint32 release_begin = _frame - _n->releaseFramesDone() +
+	f_cnt_t release_begin = _frame - _n->releaseFramesDone() +
 						_n->framesBeforeRelease();
 
 	if( _n->released() == FALSE )
 	{
-		release_begin += mixer::inst()->framesPerAudioBuffer();
+		release_begin += eng()->getMixer()->framesPerAudioBuffer();
 	}
 
 	return( m_envLFOWidgets[VOLUME]->level( _frame, release_begin, 0 ) );
@@ -232,16 +244,17 @@ float FASTCALL envelopeTabWidget::volumeLevel( notePlayHandle * _n,
 
 
 
-void envelopeTabWidget::processAudioBuffer( sampleFrame * _ab, Uint32 _frames,
+void envelopeTabWidget::processAudioBuffer( sampleFrame * _ab,
+							const fpab_t _frames,
 							notePlayHandle * _n )
 {
-	Uint32 total_frames = _n->totalFramesPlayed();
-	Uint32 release_begin = total_frames - _n->releaseFramesDone() +
+	f_cnt_t total_frames = _n->totalFramesPlayed();
+	f_cnt_t release_begin = total_frames - _n->releaseFramesDone() +
 						_n->framesBeforeRelease();
 
 	if( _n->released() == FALSE )
 	{
-		release_begin += mixer::inst()->framesPerAudioBuffer();
+		release_begin += eng()->getMixer()->framesPerAudioBuffer();
 	}
 
 	// because of optimizations, there's special code for several cases:
@@ -261,27 +274,23 @@ void envelopeTabWidget::processAudioBuffer( sampleFrame * _ab, Uint32 _frames,
 	if( _n->m_filter == NULL )
 	{
 		_n->m_filter = new basicFilters<>(
-						mixer::inst()->sampleRate() );
+					eng()->getMixer()->sampleRate() );
 	}
 
-	if( /*m_filterState->isChecked()*/ m_filterGroupBox->isActive() )
+	if( m_filterGroupBox->isActive() )
 	{
 		int old_filter_cut = 0;
 		int old_filter_res = 0;
 
-		basicFilters<>::filterTypes filter = basicFilters<>::getFilterType( m_filterComboBox->
-#ifdef QT4
-				currentIndex()
-#else
-				currentItem()
-#endif
-			 );
+		basicFilters<>::filterTypes filter =
+				basicFilters<>::getFilterType(
+					m_filterComboBox->value() );
 
 		if( m_envLFOWidgets[VOLUME]->used() &&
 			m_envLFOWidgets[CUT]->used() &&
 			m_envLFOWidgets[RES]->used() )
 		{
-			for( Uint32 frame = 0; frame < _frames;
+			for( fpab_t frame = 0; frame < _frames;
 						++frame, ++total_frames )
 			{
 				float new_cut_val = m_envLFOWidgets[CUT]->level( total_frames, release_begin, frame );
@@ -302,7 +311,7 @@ void envelopeTabWidget::processAudioBuffer( sampleFrame * _ab, Uint32 _frames,
 				float vol_level = m_envLFOWidgets[VOLUME]->level( total_frames, release_begin, frame );
 				vol_level = vol_level*vol_level;
 
-				for( Uint8 chnl = 0; chnl < DEFAULT_CHANNELS; ++chnl )
+				for( ch_cnt_t chnl = 0; chnl < DEFAULT_CHANNELS; ++chnl )
 				{
 					_ab[frame][chnl] = vol_level * _n->m_filter->update( _ab[frame][chnl], chnl );
 				}
@@ -310,7 +319,7 @@ void envelopeTabWidget::processAudioBuffer( sampleFrame * _ab, Uint32 _frames,
 		}
 		else if( m_envLFOWidgets[VOLUME]->used() && m_envLFOWidgets[CUT]->used() )
 		{
-			for( Uint32 frame = 0; frame < _frames; ++frame, ++total_frames )
+			for( fpab_t frame = 0; frame < _frames; ++frame, ++total_frames )
 			{
 				float new_cut_val = m_envLFOWidgets[CUT]->level( total_frames, release_begin, frame );
 				new_cut_val = envelopeAndLFOWidget::expKnobVal( new_cut_val ) * CUT_FREQ_MULTIPLIER +
@@ -325,7 +334,7 @@ void envelopeTabWidget::processAudioBuffer( sampleFrame * _ab, Uint32 _frames,
 				float vol_level = m_envLFOWidgets[VOLUME]->level( total_frames, release_begin, frame );
 				vol_level = vol_level*vol_level;
 
-				for( Uint8 chnl = 0; chnl < DEFAULT_CHANNELS; ++chnl )
+				for( ch_cnt_t chnl = 0; chnl < DEFAULT_CHANNELS; ++chnl )
 				{
 					_ab[frame][chnl] = vol_level * _n->m_filter->update( _ab[frame][chnl], chnl );
 				}
@@ -334,7 +343,7 @@ void envelopeTabWidget::processAudioBuffer( sampleFrame * _ab, Uint32 _frames,
 		else if( m_envLFOWidgets[VOLUME]->used() && m_envLFOWidgets[RES]->used() )
 		{
 
-			for( Uint32 frame = 0; frame < _frames; ++frame, ++total_frames )
+			for( fpab_t frame = 0; frame < _frames; ++frame, ++total_frames )
 			{
 				float new_res_val = m_filterResKnob->value() + RES_MULTIPLIER *
 							m_envLFOWidgets[RES]->level( total_frames, release_begin, frame );
@@ -348,7 +357,7 @@ void envelopeTabWidget::processAudioBuffer( sampleFrame * _ab, Uint32 _frames,
 				float vol_level = m_envLFOWidgets[VOLUME]->level( total_frames, release_begin, frame );
 				vol_level = vol_level*vol_level;
 
-				for( Uint8 chnl = 0; chnl < DEFAULT_CHANNELS; ++chnl )
+				for( ch_cnt_t chnl = 0; chnl < DEFAULT_CHANNELS; ++chnl )
 				{
 					_ab[frame][chnl] = vol_level * _n->m_filter->update( _ab[frame][chnl], chnl );
 				}
@@ -357,7 +366,7 @@ void envelopeTabWidget::processAudioBuffer( sampleFrame * _ab, Uint32 _frames,
 		}
 		else if( m_envLFOWidgets[CUT]->used() )
 		{
-			for( Uint32 frame = 0; frame < _frames; ++frame, ++total_frames )
+			for( fpab_t frame = 0; frame < _frames; ++frame, ++total_frames )
 			{
 				float new_cut_val = m_envLFOWidgets[CUT]->level( total_frames, release_begin, frame );
 				new_cut_val = envelopeAndLFOWidget::expKnobVal( new_cut_val ) * CUT_FREQ_MULTIPLIER +
@@ -369,7 +378,7 @@ void envelopeTabWidget::processAudioBuffer( sampleFrame * _ab, Uint32 _frames,
 					old_filter_cut = static_cast<int>( new_cut_val );
 				}
 
-				for( Uint8 chnl = 0; chnl < DEFAULT_CHANNELS; ++chnl )
+				for( ch_cnt_t chnl = 0; chnl < DEFAULT_CHANNELS; ++chnl )
 				{
 					_ab[frame][chnl] = _n->m_filter->update( _ab[frame][chnl], chnl );
 				}
@@ -377,7 +386,7 @@ void envelopeTabWidget::processAudioBuffer( sampleFrame * _ab, Uint32 _frames,
 		}
 		else if( m_envLFOWidgets[RES]->used() )
 		{
-			for( Uint32 frame = 0; frame < _frames; ++frame, ++total_frames )
+			for( fpab_t frame = 0; frame < _frames; ++frame, ++total_frames )
 			{
 				float new_res_val = m_filterResKnob->value() + RES_MULTIPLIER *
 							m_envLFOWidgets[RES]->level( total_frames, release_begin, frame );
@@ -388,7 +397,7 @@ void envelopeTabWidget::processAudioBuffer( sampleFrame * _ab, Uint32 _frames,
 					old_filter_res = static_cast<int>( new_res_val*RES_PRECISION );
 				}
 
-				for( Uint8 chnl = 0; chnl < DEFAULT_CHANNELS; ++chnl )
+				for( ch_cnt_t chnl = 0; chnl < DEFAULT_CHANNELS; ++chnl )
 				{
 					_ab[frame][chnl] = _n->m_filter->update( _ab[frame][chnl], chnl );
 				}
@@ -398,12 +407,12 @@ void envelopeTabWidget::processAudioBuffer( sampleFrame * _ab, Uint32 _frames,
 		{
 			_n->m_filter->calcFilterCoeffs( filter, m_filterCutKnob->value(), m_filterResKnob->value() );
 
-			for( Uint32 frame = 0; frame < _frames; ++frame, ++total_frames )
+			for( fpab_t frame = 0; frame < _frames; ++frame, ++total_frames )
 			{
 				float vol_level = m_envLFOWidgets[VOLUME]->level( total_frames, release_begin, frame );
 				vol_level = vol_level*vol_level;
 
-				for( Uint8 chnl = 0; chnl < DEFAULT_CHANNELS; ++chnl )
+				for( ch_cnt_t chnl = 0; chnl < DEFAULT_CHANNELS; ++chnl )
 				{
 					_ab[frame][chnl] = vol_level * _n->m_filter->update( _ab[frame][chnl], chnl );
 				}
@@ -413,9 +422,9 @@ void envelopeTabWidget::processAudioBuffer( sampleFrame * _ab, Uint32 _frames,
 		{
 			_n->m_filter->calcFilterCoeffs( filter, m_filterCutKnob->value(), m_filterResKnob->value() );
 
-			for( Uint32 frame = 0; frame < _frames; ++frame, ++total_frames )
+			for( fpab_t frame = 0; frame < _frames; ++frame, ++total_frames )
 			{
-				for( Uint8 chnl = 0; chnl < DEFAULT_CHANNELS; ++chnl )
+				for( ch_cnt_t chnl = 0; chnl < DEFAULT_CHANNELS; ++chnl )
 				{
 					_ab[frame][chnl] = _n->m_filter->update( _ab[frame][chnl], chnl );
 				}
@@ -425,11 +434,11 @@ void envelopeTabWidget::processAudioBuffer( sampleFrame * _ab, Uint32 _frames,
 	else if( m_envLFOWidgets[VOLUME]->used() /*&& m_envLFOWidgets[PANNING]->used() == FALSE*/ )
 	{
 		// only use volume-envelope...
-		for( Uint32 frame = 0; frame < _frames; ++frame, ++total_frames )
+		for( fpab_t frame = 0; frame < _frames; ++frame, ++total_frames )
 		{
 			float vol_level = m_envLFOWidgets[VOLUME]->level( total_frames, release_begin, frame );
 			vol_level = vol_level*vol_level;
-			for( Uint8 chnl = 0; chnl < DEFAULT_CHANNELS; ++chnl )
+			for( ch_cnt_t chnl = 0; chnl < DEFAULT_CHANNELS; ++chnl )
 			{
 				_ab[frame][chnl] = vol_level * _ab[frame][chnl];
 			}
@@ -438,11 +447,11 @@ void envelopeTabWidget::processAudioBuffer( sampleFrame * _ab, Uint32 _frames,
 /*	else if( m_envLFOWidgets[VOLUME]->used() == FALSE && m_envLFOWidgets[PANNING]->used() )
 	{
 		// only use panning-envelope...
-		for( Uint32 frame = 0; frame < _frames; ++frame, ++total_frames )
+		for( fpab_t frame = 0; frame < _frames; ++frame, ++total_frames )
 		{
 			float vol_level = m_envLFOWidgets[PANNING]->level( total_frames, release_begin, frame );
 			vol_level = vol_level*vol_level;
-			for( Uint8 chnl = 0; chnl < DEFAULT_CHANNELS; ++chnl )
+			for( ch_cnt_t chnl = 0; chnl < DEFAULT_CHANNELS; ++chnl )
 			{
 				_ab[frame][chnl] = vol_level * _ab[frame][chnl];
 			}
@@ -453,16 +462,19 @@ void envelopeTabWidget::processAudioBuffer( sampleFrame * _ab, Uint32 _frames,
 
 
 
-Uint32 envelopeTabWidget::envFrames( void )
+f_cnt_t envelopeTabWidget::envFrames( const bool _only_vol )
 {
-	Uint32 ret_val = m_envLFOWidgets[VOLUME]->m_pahdFrames;
+	f_cnt_t ret_val = m_envLFOWidgets[VOLUME]->m_pahdFrames;
 
-	for( int i = VOLUME+1; i < TARGET_COUNT; ++i )
+	if( _only_vol == FALSE )
 	{
-		if( m_envLFOWidgets[i]->used() &&
+		for( int i = VOLUME+1; i < TARGET_COUNT; ++i )
+		{
+			if( m_envLFOWidgets[i]->used() &&
 				m_envLFOWidgets[i]->m_pahdFrames > ret_val )
-		{
-			ret_val = m_envLFOWidgets[i]->m_pahdFrames;
+			{
+				ret_val = m_envLFOWidgets[i]->m_pahdFrames;
+			}
 		}
 	}
 	return( ret_val );
@@ -471,15 +483,19 @@ Uint32 envelopeTabWidget::envFrames( void )
 
 
 
-Uint32 envelopeTabWidget::releaseFrames( void )
+f_cnt_t envelopeTabWidget::releaseFrames( const bool _only_vol )
 {
-	Uint32 ret_val = m_envLFOWidgets[VOLUME]->m_rFrames;
-	for( int i = VOLUME+1; i < TARGET_COUNT; ++i )
+	f_cnt_t ret_val = m_envLFOWidgets[VOLUME]->used() ?
+					m_envLFOWidgets[VOLUME]->m_rFrames : 0;
+	if( m_envLFOWidgets[VOLUME]->used() == FALSE )
 	{
-		if( m_envLFOWidgets[i]->used() &&
-				m_envLFOWidgets[i]->m_rFrames > ret_val )
+		for( int i = VOLUME+1; i < TARGET_COUNT; ++i )
 		{
-			ret_val = m_envLFOWidgets[i]->m_rFrames;
+			if( m_envLFOWidgets[i]->used() &&
+				m_envLFOWidgets[i]->m_rFrames > ret_val )
+			{
+				ret_val = m_envLFOWidgets[i]->m_rFrames;
+			}
 		}
 	}
 	return( ret_val );
@@ -488,34 +504,18 @@ Uint32 envelopeTabWidget::releaseFrames( void )
 
 
 
-void envelopeTabWidget::saveSettings( QDomDocument & _doc,
-							QDomElement & _parent )
+void envelopeTabWidget::saveSettings( QDomDocument & _doc, QDomElement & _this )
 {
-	QDomElement etw_de = _doc.createElement( nodeName() );
-#ifdef QT4
-	etw_de.setAttribute( "ftype", QString::number(
-					m_filterComboBox->currentIndex() ) );
-#else
-	etw_de.setAttribute( "ftype", QString::number(
-					m_filterComboBox->currentItem() ) );
-#endif
-	etw_de.setAttribute( "fcut", QString::number(
-						m_filterCutKnob->value() ) );
-	etw_de.setAttribute( "fres", QString::number(
-						m_filterResKnob->value() ) );
-	etw_de.setAttribute( "fwet", QString::number(
-						/*m_filterState->isChecked()*/
-					m_filterGroupBox->isActive() ) );
-	_parent.appendChild( etw_de );
+	m_filterComboBox->saveSettings( _doc, _this, "ftype" );
+	m_filterCutKnob->saveSettings( _doc, _this, "fcut" );
+	m_filterResKnob->saveSettings( _doc, _this, "fres" );
+	_this.setAttribute( "fwet", m_filterGroupBox->isActive() );
 
 	for( int i = 0; i < TARGET_COUNT; ++i )
 	{
-		QDomElement target_de = _doc.createElement(
-						m_envLFOWidgets[i]->nodeName() +
-					QString(
-						targetNames[i][1] ).toLower() );
-		m_envLFOWidgets[i]->saveSettings( _doc, target_de );
-		etw_de.appendChild( target_de );
+		m_envLFOWidgets[i]->saveState( _doc, _this ).setTagName(
+			m_envLFOWidgets[i]->nodeName() +
+				QString( targetNames[i][1] ).toLower() );
 	}
 }
 
@@ -524,13 +524,9 @@ void envelopeTabWidget::saveSettings( QDomDocument & _doc,
 
 void envelopeTabWidget::loadSettings( const QDomElement & _this )
 {
-#ifdef QT4
-	m_filterComboBox->setCurrentIndex( _this.attribute( "ftype" ).toInt() );
-#else
-	m_filterComboBox->setCurrentItem( _this.attribute( "ftype" ).toInt() );
-#endif
-	m_filterCutKnob->setValue( _this.attribute( "fcut" ).toFloat() );
-	m_filterResKnob->setValue( _this.attribute( "fres" ).toFloat() );
+	m_filterComboBox->loadSettings( _this, "ftype" );
+	m_filterCutKnob->loadSettings( _this, "fcut" );
+	m_filterResKnob->loadSettings( _this, "fres" );
 /*	m_filterState->setChecked( _this.attribute( "fwet" ).toInt() );*/
 	m_filterGroupBox->setState( _this.attribute( "fwet" ).toInt() );
 
@@ -545,7 +541,7 @@ void envelopeTabWidget::loadSettings( const QDomElement & _this )
 						m_envLFOWidgets[i]->nodeName() +
 					QString( targetNames[i][1] ).toLower() )
 				{
-					m_envLFOWidgets[i]->loadSettings(
+					m_envLFOWidgets[i]->restoreState(
 							node.toElement() );
 				}
 			}
@@ -556,9 +552,6 @@ void envelopeTabWidget::loadSettings( const QDomElement & _this )
 
 
 
-#undef isChecked
-#undef setChecked
-
 
 #include "envelope_tab_widget.moc"
 
@@ -630,3 +623,5 @@ return res;
 }
 
 */
+
+#endif
