@@ -1,7 +1,9 @@
+#ifndef SINGLE_SOURCE_COMPILE
+
 /*
  * export_project_dialog.cpp - implementation of dialog for exporting project
  *
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -27,23 +29,19 @@
 
 #ifdef QT4
 
-#include <QFileDialog>
-#include <QMessageBox>
-#include <QProgressBar>
-#include <QComboBox>
-#include <QCheckBox>
-#include <QLabel>
-#include <QPushButton>
-#include <QCloseEvent>
-#include <QApplication>
+#include <QtGui/QFileDialog>
+#include <QtGui/QMessageBox>
+#include <QtGui/QProgressBar>
+#include <QtGui/QLabel>
+#include <QtGui/QPushButton>
+#include <QtGui/QCloseEvent>
+#include <QtGui/QApplication>
 
 #else
 
 #include <qfiledialog.h>
 #include <qmessagebox.h>
 #include <qprogressbar.h>
-#include <qcombobox.h>
-#include <qcheckbox.h>
 #include <qlabel.h>
 #include <qpushbutton.h>
 #include <qapplication.h>
@@ -53,7 +51,9 @@
 
 #include "export_project_dialog.h"
 #include "song_editor.h"
-#include "lmms_main_win.h"
+#include "main_window.h"
+#include "combobox.h"
+#include "led_checkbox.h"
 #include "embed.h"
 
 #include "audio_file_wave.h"
@@ -87,7 +87,7 @@ const int LABEL_MARGIN = 6;
 const int LABEL_X = 10;
 const int LABEL_WIDTH = 48;
 const int TYPE_STUFF_Y = 10;
-const int TYPE_HEIGHT = 26;
+const int TYPE_HEIGHT = 22;
 const int TYPE_COMBO_WIDTH = 256;
 const int KBPS_STUFF_Y = TYPE_STUFF_Y + TYPE_HEIGHT + LABEL_MARGIN + 6;
 const int KBPS_HEIGHT = TYPE_HEIGHT;
@@ -129,8 +129,10 @@ Sint16 exportProjectDialog::s_availableBitrates[] =
 // TODO: rewrite that crap using layouts!!
 
 exportProjectDialog::exportProjectDialog( const QString & _file_name,
-							QWidget * _parent ) :
+						QWidget * _parent,
+						engine * _engine ) :
 	QDialog( _parent ),
+	engineObject( _engine ),
 	m_fileName( _file_name ),
 	m_hourglassLbl( NULL ),
 	m_deleteFile( FALSE )
@@ -151,27 +153,22 @@ exportProjectDialog::exportProjectDialog( const QString & _file_name,
 	m_typeLbl->setGeometry( LABEL_X, TYPE_STUFF_Y, LABEL_WIDTH,
 								TYPE_HEIGHT );
 
-	m_typeCombo = new QComboBox( this );
+	m_typeCombo = new comboBox( this, NULL, eng(), NULL );
 	m_typeCombo->setGeometry( LABEL_X + LABEL_WIDTH+LABEL_MARGIN,
 					TYPE_STUFF_Y, TYPE_COMBO_WIDTH,
 								TYPE_HEIGHT );
 	connect( m_typeCombo, SIGNAL( activated( const QString & ) ), this,
 				SLOT( changedType( const QString & ) ) );
 
-	int idx = 0;
+	Uint8 idx = 0;
 	while( fileEncodeDevices[idx].m_fileType != NULL_FILE )
 	{
 		m_typeCombo->addItem(
 				tr( fileEncodeDevices[idx].m_description ) );
 		++idx;
 	}
-#ifdef QT4
-	m_typeCombo->setCurrentIndex( m_typeCombo->findText( tr(
+	m_typeCombo->setValue( m_typeCombo->findText( tr(
 			fileEncodeDevices[m_fileType].m_description ) ) );
-#else
-	m_typeCombo->setCurrentText( tr(
-				fileEncodeDevices[m_fileType].m_description ) );
-#endif
 
 
 	// kbps-ui-stuff
@@ -179,7 +176,7 @@ exportProjectDialog::exportProjectDialog( const QString & _file_name,
 	m_kbpsLbl->setGeometry( LABEL_X, KBPS_STUFF_Y, LABEL_WIDTH,
 								KBPS_HEIGHT );
 
-	m_kbpsCombo = new QComboBox( this );
+	m_kbpsCombo = new comboBox( this, NULL, eng(), NULL );
 	m_kbpsCombo->setGeometry( LABEL_X + LABEL_WIDTH + LABEL_MARGIN,
 						KBPS_STUFF_Y, KBPS_COMBO_WIDTH,
 								KBPS_HEIGHT );
@@ -191,22 +188,19 @@ exportProjectDialog::exportProjectDialog( const QString & _file_name,
 						s_availableBitrates[idx] ) );
 		++idx;
 	}
-#ifdef QT4
-	m_typeCombo->setCurrentIndex( m_typeCombo->findText(
+	m_typeCombo->setValue( m_typeCombo->findText(
 						QString::number( 128 ) ) );
-#else
-	m_kbpsCombo->setCurrentText( QString::number( 128 ) );
-#endif
 
 
-	m_vbrCb = new QCheckBox( tr( "variable bitrate" ), this );
+	m_vbrCb = new ledCheckBox( tr( "variable bitrate" ), this, NULL, eng(),
+									NULL );
 	m_vbrCb->setGeometry( LABEL_X + LABEL_WIDTH + 3 * LABEL_MARGIN +
 				KBPS_COMBO_WIDTH, KBPS_STUFF_Y + 3, 190, 20 );
 	m_vbrCb->setChecked( TRUE );
 
 
-	m_hqmCb = new QCheckBox( tr( "use high-quality-mode (recommened)" ),
-									this );
+	m_hqmCb = new ledCheckBox( tr( "use high-quality-mode (recommened)" ),
+						this, NULL, eng(), NULL );
 	m_hqmCb->setGeometry( LABEL_X, HQ_MODE_CB_Y + 3, HQ_MODE_CB_WIDTH,
 							HQ_MODE_CB_HEIGHT );
 	m_hqmCb->setChecked( TRUE );
@@ -264,7 +258,7 @@ void exportProjectDialog::keyPressEvent( QKeyEvent * _ke )
 {
 	if( _ke->key() == Qt::Key_Escape )
 	{
-		if( songEditor::inst()->exporting() == FALSE )
+		if( eng()->getSongEditor()->exporting() == FALSE )
 		{
 			accept();
 		}
@@ -278,13 +272,17 @@ void exportProjectDialog::keyPressEvent( QKeyEvent * _ke )
 
 
 
-void exportProjectDialog::closeEvent (QCloseEvent * _ce)
+void exportProjectDialog::closeEvent( QCloseEvent * _ce )
 {
-	if( songEditor::inst()->exporting() == TRUE )
+	if( eng()->getSongEditor()->exporting() == TRUE )
 	{
 		abortProjectExport();
 		_ce->ignore();
 	}
+	else
+	{
+		QDialog::closeEvent( _ce );
+	}
 }
 
 
@@ -333,8 +331,8 @@ void exportProjectDialog::exportBtnClicked( void )
 							m_vbrCb->isChecked(),
 					m_kbpsCombo->currentText().toInt(),
 					m_kbpsCombo->currentText().toInt() - 64,
-					m_kbpsCombo->currentText().toInt() + 64
-				);
+					m_kbpsCombo->currentText().toInt() + 64,
+					eng()->getMixer() );
 	if( success_ful == FALSE )
 	{
 		QMessageBox::information( this,
@@ -358,7 +356,6 @@ void exportProjectDialog::exportBtnClicked( void )
 	delete m_kbpsLbl;
 	delete m_kbpsCombo;
 	delete m_vbrCb;
-	delete m_hqmCb;
 	delete m_exportBtn;
 
 	m_exportProgressBar = new QProgressBar( this );
@@ -380,26 +377,27 @@ void exportProjectDialog::exportBtnClicked( void )
 
 
 
-	mixer::inst()->setAudioDevice( dev, m_hqmCb->isChecked() );
-	songEditor::inst()->startExport();
+	eng()->getMixer()->setAudioDevice( dev, m_hqmCb->isChecked() );
+	eng()->getSongEditor()->startExport();
 
+	delete m_hqmCb;
 
-	songEditor::playPos & pp = songEditor::inst()->getPlayPos(
+	songEditor::playPos & pp = eng()->getSongEditor()->getPlayPos(
 							songEditor::PLAY_SONG );
 
-	while( songEditor::inst()->exportDone() == FALSE &&
-				songEditor::inst()->exporting() == TRUE )
+	while( eng()->getSongEditor()->exportDone() == FALSE &&
+				eng()->getSongEditor()->exporting() == TRUE )
 	{
 		dev->processNextBuffer();
 		int pval = pp * 100 /
-			( ( songEditor::inst()->lengthInTacts() + 1 ) * 64 );
+			( ( eng()->getSongEditor()->lengthInTacts() + 1 ) * 64 );
 #ifdef QT4
 		m_exportProgressBar->setValue( pval );
 #else
 		m_exportProgressBar->setProgress( pval );
 #endif
 		// update lmms-main-win-caption
-		lmmsMainWin::inst()->setWindowTitle( tr( "Rendering:" ) + " " +
+		eng()->getMainWindow()->setWindowTitle( tr( "Rendering:" ) + " " +
 						QString::number( pval ) + "%" );
 		// process paint-events etc.
 		qApp->processEvents();
@@ -420,12 +418,13 @@ void exportProjectDialog::exportBtnClicked( void )
 void exportProjectDialog::cancelBtnClicked( void )
 {
 	// is song-export-thread active?
-	if( songEditor::inst()->exporting() == TRUE )
+	if( eng()->getSongEditor()->exporting() == TRUE )
 	{
 		// then dispose abort of export
 		abortProjectExport();
 		return;
 	}
+	reject();
 }
 
 
@@ -444,7 +443,7 @@ void exportProjectDialog::abortProjectExport( void )
 
 void exportProjectDialog::finishProjectExport( void )
 {
-	mixer::inst()->restoreAudioDevice();
+	eng()->getMixer()->restoreAudioDevice();
 
 	// if the user aborted export-process, the file has to be deleted
 	if( m_deleteFile )
@@ -453,9 +452,9 @@ void exportProjectDialog::finishProjectExport( void )
 	}
 
 	// restore window-title
-	lmmsMainWin::inst()->resetWindowTitle(); 
+	eng()->getMainWindow()->resetWindowTitle(); 
 
-	songEditor::inst()->stopExport();
+	eng()->getSongEditor()->stopExport();
 
 	// if we rendered file from command line, quit after export
 	if( file_to_render != "" )
@@ -473,3 +472,5 @@ void exportProjectDialog::finishProjectExport( void )
 
 #include "export_project_dialog.moc"
 
+
+#endif
