@@ -1,8 +1,10 @@
+#ifndef SINGLE_SOURCE_COMPILE
+
 /*
  * file_browser.cpp - implementation of the project-, preset- and
  *                    sample-file-browser
  *
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -28,16 +30,19 @@
 
 #ifdef QT4
 
-#include <QPushButton>
-#include <QKeyEvent>
-#include <QMenu>
-#include <Q3Header>
+#include <QtGui/QPushButton>
+#include <QtGui/QKeyEvent>
+#include <QtGui/QMenu>
+#include <QtGui/QCursor>
+#include <Qt3Support/Q3Header>
 
 #else
 
 #include <qpushbutton.h>
 #include <qpopupmenu.h>
 #include <qheader.h>
+#include <qcursor.h>
+#include <qworkspace.h>
 
 #endif
 
@@ -46,7 +51,7 @@
 #include "song_editor.h"
 #include "bb_editor.h"
 #include "embed.h"
-#include "channel_track.h"
+#include "instrument_track.h"
 #include "mmp.h"
 #include "preset_preview_play_handle.h"
 #include "sample_play_handle.h"
@@ -55,19 +60,22 @@
 #include "instrument.h"
 #include "text_float.h"
 #include "string_pair_drag.h"
+#include "main_window.h"
+#include "config_mgr.h"
 
 
 
-fileBrowser::fileBrowser( const QString & _path, const QString & _filter,
+fileBrowser::fileBrowser( const QString & _directories, const QString & _filter,
 			const QString & _title, const QPixmap & _pm,
-							QWidget * _parent ) :
+					QWidget * _parent, engine * _engine  ) :
 	sideBarWidget( _title, _pm, _parent ),
+	engineObject( _engine ),
 	m_contextMenuItem( NULL ),
-	m_path( _path ),
+	m_directories( _directories ),
 	m_filter( _filter )
 {
 	setWindowTitle( tr( "Browser" ) );
-	m_l = new listView( contentParent() );
+	m_l = new listView( contentParent(), eng() );
 	addContentWidget( m_l );
 
 #ifdef QT4
@@ -105,7 +113,53 @@ fileBrowser::~fileBrowser()
 void fileBrowser::reloadTree( void )
 {
 	m_l->clear();
-	QDir cdir( m_path );
+#ifndef QT3
+	QStringList paths = m_directories.split( '*' );
+#else
+	QStringList paths = QStringList::split( '*', m_directories );
+#endif
+	for( QStringList::iterator it = paths.begin(); it != paths.end(); ++it )
+	{
+		addItems( *it );
+	}
+
+	Q3ListViewItem * item = m_l->firstChild();
+	bool resort = FALSE;
+
+	// sort merged directories
+	while( item != NULL )
+	{
+		directory * d = dynamic_cast<directory *>( item );
+		if( d == NULL )
+		{
+			resort = TRUE;
+		}
+		else if( resort == TRUE )
+		{
+			Q3ListViewItem * i2 = m_l->firstChild();
+			d->moveItem( i2 );
+			i2->moveItem( d );
+			directory * d2 = NULL;
+			while( ( d2 = dynamic_cast<directory *>( i2 ) ) !=
+									NULL )
+			{
+				if( d->text( 0 ) > d2->text( 0 ) )
+				{
+					d->moveItem( d2 );
+				}
+				i2 = i2->nextSibling();
+			}
+		}
+		item = item->nextSibling();
+	}
+}
+
+
+
+
+void fileBrowser::addItems( const QString & _path )
+{
+	QDir cdir( _path );
 	QStringList files = cdir.entryList( QDir::NoFilter, QDir::Name );
 
 	// TODO: after dropping qt3-support we can use QStringList's iterator
@@ -116,7 +170,8 @@ void fileBrowser::reloadTree( void )
 	{
 		QString cur_file = files[files.size() - i - 1];
 		if( cur_file[0] != '.' &&
-				!QFileInfo( m_path + "/" + cur_file ).isDir()
+			QFileInfo( _path + QDir::separator() +
+						cur_file ).isDir() == FALSE
 #ifdef QT4
 // TBD
 #else
@@ -124,7 +179,9 @@ void fileBrowser::reloadTree( void )
 #endif
 	)
 		{
-			(void) new fileItem( m_l, cur_file, m_path );
+			// remove existing file-items
+			delete m_l->findItem( cur_file, 0 );
+			(void) new fileItem( m_l, cur_file, _path );
 		}
 	}
 
@@ -132,9 +189,20 @@ void fileBrowser::reloadTree( void )
 	{
 		QString cur_file = files[files.size() - i - 1];
 		if( cur_file[0] != '.' &&
-			QFileInfo( m_path + "/" + cur_file ).isDir() )
+			QFileInfo( _path + QDir::separator() +
+							cur_file ).isDir() )
 		{
-			(void) new directory( m_l, cur_file, m_path, m_filter );
+			QListViewItem * item = m_l->findItem( cur_file, 0 );
+			if( item == NULL )
+			{
+				(void) new directory( m_l, cur_file, _path,
+							      	m_filter );
+			}
+			else if( dynamic_cast<directory *>( item ) != NULL )
+			{
+				dynamic_cast<directory *>( item )->
+							addDirectory( _path );
+			}
 		}
 	}
 }
@@ -170,14 +238,17 @@ void fileBrowser::contextMenuRequest( QListViewItem * i, const QPoint &, int )
 	{
 		m_contextMenuItem = f;
 		QMenu * contextMenu = new QMenu( this );
-		contextMenu->addAction( tr( "Send to active channel" ), this,
-						SLOT( sendToActiveChannel() ) );
-		contextMenu->addAction( tr( "Open in new channel/Song-Editor" ),
+		contextMenu->addAction( tr( "Send to active instrument-track" ),
 						this,
-						SLOT( openInNewChannelSE() ) );
-		contextMenu->addAction( tr( "Open in new channel/B+B Editor" ),
+					SLOT( sendToActiveInstrumentTrack() ) );
+		contextMenu->addAction( tr( "Open in new instrument-track/"
+								"Song-Editor" ),
 						this,
-						SLOT( openInNewChannelBBE() ) );
+					SLOT( openInNewInstrumentTrackSE() ) );
+		contextMenu->addAction( tr( "Open in new instrument-track/"
+								"B+B Editor" ),
+						this,
+					SLOT( openInNewInstrumentTrackBBE() ) );
 		contextMenu->exec( QCursor::pos() );
 		m_contextMenuItem = NULL;
 		delete contextMenu;
@@ -188,12 +259,13 @@ void fileBrowser::contextMenuRequest( QListViewItem * i, const QPoint &, int )
 
 
 
-void fileBrowser::sendToActiveChannel( void )
+void fileBrowser::sendToActiveInstrumentTrack( void )
 {
-	if( lmmsMainWin::inst()->workspace() != NULL )
+	if( eng()->getMainWindow()->workspace() != NULL )
 	{
 		// get all windows opened in the workspace
-		QWidgetList pl = lmmsMainWin::inst()->workspace()->windowList(
+		QWidgetList pl =
+			eng()->getMainWindow()->workspace()->windowList(
 #if QT_VERSION >= 0x030200
 						QWorkspace::StackingOrder
 #endif
@@ -201,24 +273,25 @@ void fileBrowser::sendToActiveChannel( void )
 #ifdef QT4
 		QListIterator<QWidget *> w( pl );
 		w.toBack();
-		// now we travel through the window-list until we find a
-		// channel-track
+		// now we travel through the window-list until we find an
+		// instrument-track
 		while( w.hasPrevious() )
 		{
-			channelTrack * ct = dynamic_cast<channelTrack *>(
+			instrumentTrack * ct = dynamic_cast<instrumentTrack *>(
 								w.previous() );
 #else
 		QWidget * w = pl.last();
-		// now we travel through the window-list until we find a
-		// channel-track
+		// now we travel through the window-list until we find an
+		// instrument-track
 		while( w != NULL )
 		{
-			channelTrack * ct = dynamic_cast<channelTrack *>( w );
+			instrumentTrack * ct =
+					dynamic_cast<instrumentTrack *>( w );
 #endif
 			if( ct != NULL && ct->isHidden() == FALSE )
 			{
-				// ok, it's a channel-track, so we can apply the
-				// sample or the preset
+				// ok, it's an instrument-track, so we can apply
+				// the sample or the preset
 				if( m_contextMenuItem->type() ==
 							fileItem::SAMPLE_FILE )
 				{
@@ -226,7 +299,7 @@ void fileBrowser::sendToActiveChannel( void )
 							"audiofileprocessor" );
 					if( afp != NULL )
 					{
-						afp->setParameter( "audiofile",
+						afp->setParameter( "samplefile",
 						m_contextMenuItem->fullName() );
 					}
 				}
@@ -240,7 +313,7 @@ void fileBrowser::sendToActiveChannel( void )
 								firstChild().
 								toElement() );
 				}
-				ct->toggledChannelButton( TRUE );
+				ct->toggledInstrumentTrackButton( TRUE );
 				break;
 			}
 #ifndef QT4
@@ -253,11 +326,11 @@ void fileBrowser::sendToActiveChannel( void )
 
 
 
-void fileBrowser::openInNewChannel( trackContainer * _tc )
+void fileBrowser::openInNewInstrumentTrack( trackContainer * _tc )
 {
 	if( m_contextMenuItem->type() == fileItem::SAMPLE_FILE )
 	{
-		channelTrack * ct = dynamic_cast<channelTrack *>(
+		instrumentTrack * ct = dynamic_cast<instrumentTrack *>(
 			track::create( track::CHANNEL_TRACK, _tc ) );
 #ifdef LMMS_DEBUG
 		assert( ct != NULL );
@@ -265,22 +338,22 @@ void fileBrowser::openInNewChannel( trackContainer * _tc )
 		instrument * afp = ct->loadInstrument( "audiofileprocessor" );
 		if( afp != NULL )
 		{
-			afp->setParameter( "audiofile",
+			afp->setParameter( "samplefile",
 						m_contextMenuItem->fullName() );
 		}
-		ct->toggledChannelButton( TRUE );
+		ct->toggledInstrumentTrackButton( TRUE );
 	}
 	else if( m_contextMenuItem->type() == fileItem::PRESET_FILE )
 	{
 		multimediaProject mmp( m_contextMenuItem->fullName() );
 		track * t = track::create( track::CHANNEL_TRACK, _tc );
-		channelTrack * ct = dynamic_cast<channelTrack *>( t );
+		instrumentTrack * ct = dynamic_cast<instrumentTrack *>( t );
 		if( ct != NULL )
 		{
 			ct->loadTrackSpecificSettings( mmp.content().
 							firstChild().
 							toElement() );
-			ct->toggledChannelButton( TRUE );
+			ct->toggledInstrumentTrackButton( TRUE );
 		}
 	}
 }
@@ -288,17 +361,17 @@ void fileBrowser::openInNewChannel( trackContainer * _tc )
 
 
 
-void fileBrowser::openInNewChannelSE( void )
+void fileBrowser::openInNewInstrumentTrackSE( void )
 {
-	openInNewChannel( songEditor::inst() );
+	openInNewInstrumentTrack( eng()->getSongEditor() );
 }
 
 
 
 
-void fileBrowser::openInNewChannelBBE( void )
+void fileBrowser::openInNewInstrumentTrackBBE( void )
 {
-	openInNewChannel( bbEditor::inst() );
+	openInNewInstrumentTrack( eng()->getBBEditor() );
 }
 
 
@@ -308,8 +381,9 @@ void fileBrowser::openInNewChannelBBE( void )
 
 
 
-listView::listView( QWidget * _parent ) :
+listView::listView( QWidget * _parent, engine * _engine ) :
 	Q3ListView( _parent ),
+	engineObject( _engine ),
 	m_mousePressed( FALSE ),
 	m_pressPos(),
 	m_previewPlayHandle( NULL )
@@ -319,7 +393,7 @@ listView::listView( QWidget * _parent ) :
 	setSorting( -1 );
 	setShowToolTips( TRUE );
 
-	setFont( pointSize<8>( font() ) );
+	setFont( pointSizeF( font(), 7.5f ) );
 }
 
 
@@ -343,41 +417,42 @@ void listView::contentsMouseDoubleClickEvent( QMouseEvent * _me )
 		{
 			// samples are per default opened in bb-editor because
 			// they're likely drum-samples etc.
-			channelTrack * ct = dynamic_cast<channelTrack *>(
-						track::create(
-							track::CHANNEL_TRACK,
-							bbEditor::inst() ) );
+			instrumentTrack * it = dynamic_cast<instrumentTrack *>(
+				track::create( track::CHANNEL_TRACK,
+						eng()->getBBEditor() ) );
 #ifdef LMMS_DEBUG
-			assert( ct != NULL );
+			assert( it != NULL );
 #endif
-			instrument * afp = ct->loadInstrument(
+			instrument * afp = it->loadInstrument(
 							"audiofileprocessor" );
 			if( afp != NULL )
 			{
-				afp->setParameter( "audiofile", f->fullName() );
+				afp->setParameter( "samplefile",
+								f->fullName() );
 			}
-			ct->toggledChannelButton( TRUE );
+			it->toggledInstrumentTrackButton( TRUE );
 		}
 		else if( f->type() == fileItem::PRESET_FILE )
 		{
 			// presets are per default opened in bb-editor
 			multimediaProject mmp( f->fullName() );
 			track * t = track::create( track::CHANNEL_TRACK,
-						bbEditor::inst() );
-			channelTrack * ct = dynamic_cast<channelTrack *>( t );
-			if( ct != NULL )
+						eng()->getBBEditor() );
+			instrumentTrack * it = dynamic_cast<instrumentTrack *>(
+									t );
+			if( it != NULL )
 			{
-				ct->loadTrackSpecificSettings( mmp.content().
+				it->loadTrackSpecificSettings( mmp.content().
 								firstChild().
 								toElement() );
-				ct->toggledChannelButton( TRUE );
+				it->toggledInstrumentTrackButton( TRUE );
 			}
 		}
 		else if( f->type() == fileItem::PROJECT_FILE )
 		{
-			if( songEditor::inst()->mayChangeProject() == TRUE )
+			if( eng()->getSongEditor()->mayChangeProject() == TRUE )
 			{
-				songEditor::inst()->loadProject(
+				eng()->getSongEditor()->loadProject(
 								f->fullName() );
 			}
 		}
@@ -394,6 +469,7 @@ void listView::contentsMousePressEvent( QMouseEvent * _me )
 	{
 		return;
 	}
+
 	QPoint p( contentsToViewport( _me->pos() ) );
 	Q3ListViewItem * i = itemAt( p );
         if ( i )
@@ -408,12 +484,14 @@ void listView::contentsMousePressEvent( QMouseEvent * _me )
 			m_mousePressed = TRUE;
 		}
 	}
+
 	fileItem * f = dynamic_cast<fileItem *>( i );
 	if( f != NULL )
 	{
 		if( m_previewPlayHandle != NULL )
 		{
-			mixer::inst()->removePlayHandle( m_previewPlayHandle );
+			eng()->getMixer()->removePlayHandle(
+							m_previewPlayHandle );
 			m_previewPlayHandle = NULL;
 		}
 		if( f->type() == fileItem::SAMPLE_FILE )
@@ -430,7 +508,7 @@ void listView::contentsMousePressEvent( QMouseEvent * _me )
 			qApp->processEvents();
 #endif
 			samplePlayHandle * s = new samplePlayHandle(
-								f->fullName() );
+							f->fullName(), eng() );
 			s->setDoneMayReturnTrue( FALSE );
 			m_previewPlayHandle = s;
 			delete tf;
@@ -438,11 +516,11 @@ void listView::contentsMousePressEvent( QMouseEvent * _me )
 		else if( f->type() == fileItem::PRESET_FILE )
 		{
 			m_previewPlayHandle = new presetPreviewPlayHandle(
-								f->fullName() );
+							f->fullName(), eng() );
 		}
 		if( m_previewPlayHandle != NULL )
 		{
-			mixer::inst()->addPlayHandle( m_previewPlayHandle );
+			eng()->getMixer()->addPlayHandle( m_previewPlayHandle );
 		}
 	}
 }
@@ -468,7 +546,7 @@ void listView::contentsMouseMoveEvent( QMouseEvent * _me )
 								f->fullName(),
 							embed::getIconPixmap(
 								"preset_file" ),
-									this );
+								this, eng() );
 					break;
 
 				case fileItem::SAMPLE_FILE:
@@ -476,7 +554,7 @@ void listView::contentsMouseMoveEvent( QMouseEvent * _me )
 								f->fullName(),
 							embed::getIconPixmap(
 								"sound_file" ),
-									this );
+								this, eng() );
 					break;
 
 				case fileItem::MIDI_FILE:
@@ -484,7 +562,7 @@ void listView::contentsMouseMoveEvent( QMouseEvent * _me )
 								f->fullName(),
 							embed::getIconPixmap(
 								"midi_file" ),
-									this );
+								this, eng() );
 					break;
 
 				default:
@@ -509,15 +587,15 @@ void listView::contentsMouseReleaseEvent( QMouseEvent * _me )
 		if( s != NULL )
 		{
 			if( s->totalFrames() - s->framesDone() <=
-				static_cast<Uint32>(
-					mixer::inst()->sampleRate() * 3 ) )
+				static_cast<f_cnt_t>(
+					eng()->getMixer()->sampleRate() * 3 ) )
 			{
 				s->setDoneMayReturnTrue( TRUE );
 				m_previewPlayHandle = NULL;
 				return;
 			}
 		}
-		mixer::inst()->removePlayHandle( m_previewPlayHandle );
+		eng()->getMixer()->removePlayHandle( m_previewPlayHandle );
 		m_previewPlayHandle = NULL;
 	}
 }
@@ -538,7 +616,7 @@ directory::directory( directory * _parent, const QString & _name,
 	Q3ListViewItem( _parent, _name ),
 	m_p( _parent ),
 	m_pix( NULL ),
-	m_path( _path ),
+	m_directories( _path ),
 	m_filter( _filter )
 {
 	initPixmapStuff();
@@ -552,7 +630,7 @@ directory::directory( Q3ListView * _parent, const QString & _name,
 	Q3ListViewItem( _parent, _name ),
 	m_p( NULL ),
 	m_pix( NULL ),
-	m_path( _path ),
+	m_directories( _path ),
 	m_filter( _filter )
 {
 	initPixmapStuff();
@@ -569,11 +647,13 @@ void directory::initPixmapStuff( void )
 		s_folderPixmap = new QPixmap(
 					embed::getIconPixmap( "folder" ) );
 	}
+
 	if( s_folderOpenedPixmap == NULL )
 	{
 		s_folderOpenedPixmap = new QPixmap(
 				embed::getIconPixmap( "folder_opened" ) );
 	}
+
 	if( s_folderLockedPixmap == NULL )
 	{
 		s_folderLockedPixmap = new QPixmap(
@@ -593,7 +673,7 @@ void directory::initPixmapStuff( void )
 
 
 
-void directory::setPixmap( QPixmap * _px )
+void directory::setPixmap( const QPixmap * _px )
 {
 	m_pix = _px;
 	setup();
@@ -618,57 +698,18 @@ void directory::setOpen( bool _o )
 
 	if( _o && !childCount() )
 	{
-		QString s( fullName() );
-		QDir thisDir( s );
-		if( !thisDir.isReadable() )
+		for( QStringList::iterator it = m_directories.begin();
+					it != m_directories.end(); ++it )
 		{
-			//readable = FALSE;
-			setExpandable( FALSE );
-			return;
-		}
-
-		listView()->setUpdatesEnabled( FALSE );
-
-		QStringList files = thisDir.entryList( QDir::NoFilter,
-								QDir::Name );
-		for( csize i = 0; i < files.size(); ++i )
-		{
-			QString cur_file = files[files.size()-i-1];
-#ifdef QT4
-			if( cur_file[0] != '.' && !QFileInfo(
-						thisDir.absolutePath() + "/" +
-							cur_file ).isDir() &&
-				thisDir.match( m_filter, cur_file.toLower() )
-				/*QDir::match( FILE_FILTER, cur_file )*/ )
-#else
-			if( cur_file[0] != '.' && !QFileInfo(
-						thisDir.absPath() + "/" +
-							cur_file ).isDir() &&
-				thisDir.match( m_filter, cur_file.lower() )
-				/*QDir::match( FILE_FILTER, cur_file )*/ )
-#endif
+			if( addItems( fullName( *it ) ) &&
+				( *it ).contains(
+					configManager::inst()->dataDir() ) )
 			{
-				(void) new fileItem( this, cur_file, s );
+				( new QListViewItem( this,
+		listView::tr( "--- Factory files ---" ) ) )->setPixmap( 0,
+				embed::getIconPixmap( "factory_files" ) );
 			}
 		}
-		for( csize i = 0; i < files.size(); ++i )
-		{
-			QString cur_file = files[files.size()-i-1];
-#ifdef QT4
-			if( cur_file[0] != '.' && QFileInfo(
-						thisDir.absolutePath() + "/" +
-							cur_file ).isDir() )
-#else
-			if( cur_file[0] != '.' && QFileInfo(
-						thisDir.absPath() + "/" +
-							cur_file ).isDir() )
-#endif
-			{
-				(void) new directory( this, cur_file, s,
-								m_filter );
-			}
-		}
-		listView()->setUpdatesEnabled( TRUE );
 	}
 	Q3ListViewItem::setOpen( _o );
 }
@@ -685,6 +726,90 @@ void directory::setup( void )
 
 
 
+bool directory::addItems( const QString & _path )
+{
+	QDir thisDir( _path );
+	if( !thisDir.isReadable() )
+	{
+		//readable = FALSE;
+		setExpandable( FALSE );
+		return( FALSE );
+	}
+
+	listView()->setUpdatesEnabled( FALSE );
+
+	bool added_something = FALSE;
+
+	QStringList files = thisDir.entryList( QDir::NoFilter, QDir::Name );
+	for( csize i = 0; i < files.size(); ++i )
+	{
+		QString cur_file = files[files.size() - i - 1];
+#ifdef QT4
+		if( cur_file[0] != '.' && !QFileInfo(
+				thisDir.absolutePath() + QDir::separator() +
+						cur_file ).isDir() &&
+			thisDir.match( m_filter, cur_file.toLower() )
+			/*QDir::match( FILE_FILTER, cur_file )*/ )
+#else
+		if( cur_file[0] != '.' && !QFileInfo(
+					thisDir.absPath() + QDir::separator() +
+						cur_file ).isDir() &&
+			thisDir.match( m_filter, cur_file.lower() )
+			/*QDir::match( FILE_FILTER, cur_file )*/ )
+#endif
+		{
+			(void) new fileItem( this, cur_file, _path );
+			added_something = TRUE;
+		}
+	}
+
+	for( csize i = 0; i < files.size(); ++i )
+	{
+		QString cur_file = files[files.size() - i - 1];
+#ifdef QT4
+		if( cur_file[0] != '.' && QFileInfo(
+				thisDir.absolutePath() + QDir::separator() +
+						cur_file ).isDir() )
+#else
+		if( cur_file[0] != '.' && QFileInfo(
+					thisDir.absPath() + QDir::separator() +
+						cur_file ).isDir() )
+#endif
+		{
+			new directory( this, cur_file, _path, m_filter );
+			added_something = TRUE;
+#if 0
+			if( firstChild() == NULL )
+			{
+				continue;
+			}
+			bool moved = FALSE;
+			QListViewItem * item = firstChild();
+			while( item != NULL )
+			{
+				directory * cd =
+					dynamic_cast<directory *>( item );
+				if( cd != NULL )
+				{
+/*					if( moved == FALSE ||
+						cd->text( 0 ) < cur_file )
+					{*/
+						printf( "move item %s after %s\n", d->text(0).ascii(), cd->text(0).ascii());
+						d->moveItem( cd );
+						moved = TRUE;
+					//}
+				}
+				item = item->nextSibling();
+			}
+#endif
+		}
+	}
+
+	listView()->setUpdatesEnabled( TRUE );
+
+	return( added_something );
+}
+
 
 
 
@@ -692,6 +817,7 @@ QPixmap * fileItem::s_projectFilePixmap = NULL;
 QPixmap * fileItem::s_presetFilePixmap = NULL;
 QPixmap * fileItem::s_sampleFilePixmap = NULL;
 QPixmap * fileItem::s_midiFilePixmap = NULL;
+QPixmap * fileItem::s_flpFilePixmap = NULL;
 QPixmap * fileItem::s_unknownFilePixmap = NULL;
 
 
@@ -747,6 +873,12 @@ void fileItem::initPixmapStuff( void )
 							"midi_file", 16, 16 ) );
 	}
 
+	if( s_flpFilePixmap == NULL )
+	{
+		s_flpFilePixmap = new QPixmap( embed::getIconPixmap(
+							"midi_file", 16, 16 ) );
+	}
+
 	if( s_unknownFilePixmap == NULL )
 	{
 		s_unknownFilePixmap = new QPixmap( embed::getIconPixmap(
@@ -759,6 +891,7 @@ void fileItem::initPixmapStuff( void )
 		case PRESET_FILE: m_pix = s_presetFilePixmap; break;
 		case SAMPLE_FILE: m_pix = s_sampleFilePixmap; break;
 		case MIDI_FILE: m_pix = s_midiFilePixmap; break;
+		case FLP_FILE: m_pix = s_flpFilePixmap; break;
 		case UNKNOWN:
 		default:
 			m_pix = s_unknownFilePixmap;
@@ -788,7 +921,7 @@ void fileItem::determineFileType( void )
 		{
 			m_type = PROJECT_FILE;
 		}
-		else if( t == multimediaProject::CHANNEL_SETTINGS )
+		else if( t == multimediaProject::INSTRUMENT_TRACK_SETTINGS )
 		{
 			m_type = PRESET_FILE;
 		}
@@ -811,6 +944,10 @@ void fileItem::determineFileType( void )
 	{
 		m_type = MIDI_FILE;
 	}
+	else if( ext == "flp" )
+	{
+		m_type = FLP_FILE;
+	}
 	else
 	{
 		m_type = UNKNOWN;
@@ -823,3 +960,5 @@ void fileItem::determineFileType( void )
 
 #include "file_browser.moc"
 
+
+#endif
