@@ -1,7 +1,9 @@
+#ifndef SINGLE_SOURCE_COMPILE
+
 /*
  * main.cpp - just main.cpp which is starting up app...
  *
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -27,9 +29,10 @@
 
 #ifdef QT4
 
-#include <QApplication>
-#include <QLocale>
-#include <QSplashScreen>
+#include <QtCore/QLocale>
+#include <QtCore/QTranslator>
+#include <QtGui/QApplication>
+#include <QtGui/QSplashScreen>
 
 #else
 
@@ -38,10 +41,16 @@
 #if QT_VERSION >= 0x030200
 #include <qsplashscreen.h>
 #endif
+#include <qtranslator.h>
 
 #endif
 
-#include "lmms_main_win.h"
+#ifdef HAVE_SCHED_H
+#include <sched.h>
+#endif
+
+
+#include "main_window.h"
 #include "embed.h"
 #include "config_mgr.h"
 #include "export_project_dialog.h"
@@ -49,19 +58,42 @@
 #include "gui_templates.h"
 
 
+QString file_to_render;
+
+
 #if QT_VERSION >= 0x030100
 int splash_alignment_flags = Qt::AlignTop | Qt::AlignLeft;
 #endif
 
-QString file_to_load;
-QString file_to_render;
+inline void loadTranslation( const QString & _tname,
+	const QString & _dir = configManager::inst()->localeDir() )
+{
+	QTranslator * t = new QTranslator( 0 );
+	QString name = _tname + ".qm";
+
+	t->load( name, _dir );
+
+	qApp->installTranslator( t );
+}
+
 
 
 int main( int argc, char * * argv )
 {
+#ifdef HAVE_SCHED_H
+	struct sched_param sparam;
+	sparam.sched_priority = ( sched_get_priority_max( SCHED_FIFO ) +
+				sched_get_priority_min( SCHED_FIFO ) ) / 2;
+	if( sched_setscheduler( 0, SCHED_FIFO, &sparam ) == -1 )
+	{
+		printf( "could not set realtime priority.\n" );
+	}
+#endif
+
 	QApplication app( argc, argv );
 
 	QString extension = "wav";
+	QString file_to_load;
 
 	for( int i = 1; i < app.argc(); ++i )
 	{
@@ -69,7 +101,7 @@ int main( int argc, char * * argv )
 					QString( app.argv()[i] ) == "-v" )
 		{
 			printf( "\nLinux MultiMedia Studio %s\n\n"
-	"Copyright (c) 2004-2005 Tobias Doerffel and others.\n\n"
+	"Copyright (c) 2004-2006 Tobias Doerffel and others.\n\n"
 	"This program is free software; you can redistribute it and/or\n"
 	"modify it under the terms of the GNU General Public\n"
 	"License as published by the Free Software Foundation; either\n"
@@ -83,7 +115,7 @@ int main( int argc, char * * argv )
 					QString( app.argv()[i] ) == "-h" ) )
 		{
 			printf( "\nLinux MultiMedia Studio %s\n"
-	"Copyright (c) 2004-2005 Tobias Doerffel and others.\n\n"
+	"Copyright (c) 2004-2006 Tobias Doerffel and others.\n\n"
 	"usage: lmms [ -r <file_to_render> [ -o <format> ] [ -h ] "
 							"[ <file_to_load> ]\n"
 	"-r, --render			render given file.\n"
@@ -142,9 +174,14 @@ int main( int argc, char * * argv )
 			QString( QTextCodec::locale() ).section( '_', 0, 0 );
 #endif
 	// load translation for Qt-widgets/-dialogs
-	embed::loadTranslation( QString( "qt_" ) + pos );
+#ifdef QT_TRANSLATIONS_DIR
+	loadTranslation( QString( "qt_" ) + pos,
+					QString( QT_TRANSLATIONS_DIR ) );
+#else
+	loadTranslation( QString( "qt_" ) + pos );
+#endif
 	// load actual translation for LMMS
-	embed::loadTranslation( pos );
+	loadTranslation( pos );
 
 #ifdef QT4
 	app.setFont( pointSize<10>( app.font() ) );
@@ -164,7 +201,7 @@ int main( int argc, char * * argv )
 	pal.setColor( QPalette::Text, QColor( 224, 224, 224 ) );
 	pal.setColor( QPalette::Button, QColor( 160, 160, 160 ) );
 	pal.setColor( QPalette::ButtonText, QColor( 255, 255, 255 ) );
-	pal.setColor( QPalette::Highlight, QColor( 255, 224, 224 ) );
+	pal.setColor( QPalette::Highlight, QColor( 224, 224, 224 ) );
 	pal.setColor( QPalette::HighlightedText, QColor( 0, 0, 0 ) );
 #else
 	pal.setColor( QColorGroup::Background, QColor( 128, 128, 128 ) );
@@ -182,10 +219,10 @@ int main( int argc, char * * argv )
 #if QT_VERSION >= 0x030200
 	// init splash screen
 	QPixmap splash = embed::getIconPixmap( "splash" );
-	lmmsMainWin::s_splashScreen = new QSplashScreen( splash );
-	lmmsMainWin::s_splashScreen->show();
+	mainWindow::s_splashScreen = new QSplashScreen( splash );
+	mainWindow::s_splashScreen->show();
 
-	lmmsMainWin::s_splashScreen->showMessage( lmmsMainWin::tr(
+	mainWindow::s_splashScreen->showMessage( mainWindow::tr(
 							"Setting up main-"
 							"window and "
 							"workspace..." ),
@@ -193,33 +230,45 @@ int main( int argc, char * * argv )
 							Qt::white );
 #endif
 
+	engine * main_engine = new engine();
+
+	// we try to load given file
+	if( file_to_load != "" )
+	{
+		main_engine->getSongEditor()->loadProject( file_to_load );
+	}
+	else
+	{
+		main_engine->getSongEditor()->createNewProject();
+	}
 #ifndef QT4
-	app.setMainWidget( lmmsMainWin::inst() );
+	app.setMainWidget( main_engine->getMainWindow() );
 #endif
 	// MDI-mode?
-	if( lmmsMainWin::inst()->workspace() != NULL )
+	if( main_engine->getMainWindow()->workspace() != NULL )
 	{
 		// then maximize
-		lmmsMainWin::inst()->showMaximized();
+		main_engine->getMainWindow()->showMaximized();
 	}
 	else
 	{
 		// otherwise arrange at top-left edge of screen
-		lmmsMainWin::inst()->show();
-		lmmsMainWin::inst()->move( 0, 0 );
-		lmmsMainWin::inst()->resize( 200, 500 );
+		main_engine->getMainWindow()->show();
+		main_engine->getMainWindow()->move( 0, 0 );
+		main_engine->getMainWindow()->resize( 200, 500 );
 	}
 
 
 #if QT_VERSION >= 0x030200
-	delete lmmsMainWin::s_splashScreen;
-	lmmsMainWin::s_splashScreen = NULL;
+	delete mainWindow::s_splashScreen;
+	mainWindow::s_splashScreen = NULL;
 #endif
 	if( file_to_render != "" )
 	{
 		exportProjectDialog * e = new exportProjectDialog(
-							file_to_render,
-							lmmsMainWin::inst() );
+						file_to_render,
+						main_engine->getMainWindow(),
+						main_engine );
 		e->show();
 		e->exportBtnClicked();
 	}
@@ -227,3 +276,5 @@ int main( int argc, char * * argv )
 	return( app.exec() );
 }
 
+
+#endif
