@@ -1,7 +1,9 @@
+#ifndef SINGLE_SOURCE_COMPILE
+
 /*
  * mixer.cpp - audio-device-independent mixer for LMMS
  *
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -23,11 +25,7 @@
  */
 
 
-#include <config.h>
-
-#ifdef HAVE_SYS_TIME_H
-#include <sys/time.h>
-#endif
+#include <math.h>
 
 #include "mixer.h"
 #include "play_handle.h"
@@ -38,6 +36,9 @@
 #include "debug.h"
 #include "config_mgr.h"
 #include "audio_port.h"
+#include "sample_play_handle.h"
+#include "piano_roll.h"
+#include "micro_timer.h"
 
 #include "audio_device.h"
 #include "midi_client.h"
@@ -57,58 +58,23 @@
 
 
 
-Uint32 SAMPLE_RATES[QUALITY_LEVELS] = { 44100, 88200 } ;
+sample_rate_t SAMPLE_RATES[QUALITY_LEVELS] = { 44100, 88200 } ;
 
 
-class microTimer
-{
-public:
-	microTimer( void )
-	{
-		reset();
-	}
-
-	~microTimer()
-	{
-	}
-
-	void reset( void )
-	{
-		gettimeofday( &begin, NULL );
-	}
-
-	Uint32 elapsed( void ) const
-	{
-		struct timeval now;
-		gettimeofday( &now, NULL );
-		return( ( now.tv_sec - begin.tv_sec ) * 1000 * 1000 +
-					( now.tv_usec - begin.tv_usec ) );
-	}
-
-private:
-	struct timeval begin;
-
-} ;
-
-
-
-mixer * mixer::s_instanceOfMe = NULL;
-
-
-mixer::mixer() :
+mixer::mixer( engine * _engine ) :
 	QObject(),
+	engineObject( _engine ),
 	m_framesPerAudioBuffer( DEFAULT_BUFFER_SIZE ),
-	m_curBuf( NULL ),
-	m_nextBuf( NULL ),
+	m_readBuf( NULL ),
+	m_writeBuf( NULL ),
 	m_cpuLoad( 0 ),
 	m_qualityLevel( DEFAULT_QUALITY_LEVEL ),
 	m_masterGain( 1.0f ),
 	m_audioDev( NULL ),
-	m_oldAudioDev( NULL )
+	m_oldAudioDev( NULL ),
+	m_mixMutex(),
+	m_mixMutexLockLevel( 0 )
 {
-	// small hack because code calling mixer::inst() is called out of ctor
-	s_instanceOfMe = this;
-
 	if( configManager::inst()->value( "mixer", "framesperaudiobuffer"
 						).toInt() >= 32 )
 	{
@@ -122,22 +88,16 @@ mixer::mixer() :
 				QString::number( m_framesPerAudioBuffer ) );
 	}
 
-	m_curBuf = bufferAllocator::alloc<surroundSampleFrame>(
-						m_framesPerAudioBuffer );
-	m_nextBuf = bufferAllocator::alloc<surroundSampleFrame>(
-						m_framesPerAudioBuffer );
-
-
-	m_audioDev = tryAudioDevices();
-	m_midiClient = tryMIDIClients();
-
-
-	// now clear our two output-buffers before using them...
-	clearAudioBuffer( m_curBuf, m_framesPerAudioBuffer );
-	clearAudioBuffer( m_nextBuf, m_framesPerAudioBuffer );
-
-	m_audioDev->startProcessing();
-
+	for( Uint8 i = 0; i < 3; i++ )
+	{
+		m_readBuf = bufferAllocator::alloc<surroundSampleFrame>(
+				m_framesPerAudioBuffer );
+		
+		clearAudioBuffer( m_readBuf, m_framesPerAudioBuffer );
+		m_bufferPool.push_back( m_readBuf );
+	}
+	
+	setClipScaling( FALSE );
 }
 
 
@@ -148,8 +108,27 @@ mixer::~mixer()
 	delete m_audioDev;
 	delete m_midiClient;
 
-	bufferAllocator::free( m_curBuf );
-	bufferAllocator::free( m_nextBuf );
+	for( Uint8 i = 0; i < 3; i++ )
+	{
+		bufferAllocator::free( m_bufferPool[i] );
+	}
+}
+
+
+
+
+void mixer::initDevices( void )
+{
+	m_audioDev = tryAudioDevices();
+	m_midiClient = tryMIDIClients();
+}
+
+
+
+
+void mixer::startProcessing( void )
+{
+	m_audioDev->startProcessing();
 }
 
 
@@ -163,10 +142,76 @@ void mixer::stopProcessing( void )
 
 
 
+bool mixer::criticalXRuns( void ) const
+{
+	return( ( m_cpuLoad >= 99 &&
+			eng()->getSongEditor()->realTimeTask() == TRUE ) );
+}
+
+
+
+
+void mixer::setClipScaling( bool _state )
+{
+	m_mixMutex.lock();
+
+	m_scaleClip = _state;
+
+	if( _state )
+	{
+		m_poolDepth = 3;
+		m_readBuffer = 0;
+		m_analBuffer = m_readBuffer + 1;
+		m_writeBuffer = m_poolDepth - 1;
+
+		for( ch_cnt_t chnl=0; chnl < m_audioDev->channels(); ++chnl )
+		{
+			m_clipped[chnl] = FALSE;
+			m_halfStart[chnl] = m_framesPerAudioBuffer;
+			m_maxClip[chnl] = 1.0f;
+			m_previousSample[chnl] = 0.0;
+			m_newBuffer[chnl] = FALSE;
+		}
+		// FIXME: why assign buffer-ptr to m_readBuf just for calling
+		//        another method?
+		//        clearAudioBuffer(m_bufferPool[i],...) would do as well
+		for( Uint8 i = 0; i < 3; i++ )
+		{
+			m_readBuf = m_bufferPool[i];
+			clearAudioBuffer( m_readBuf, m_framesPerAudioBuffer );
+		}
+	}
+	else
+	{
+		m_poolDepth = 2;
+		m_readBuffer = 0;
+		m_writeBuffer = 1;
+		m_analBuffer = 1;
+	}
+
+	m_mixMutex.unlock();
+}
+
+
+
+
 const surroundSampleFrame * mixer::renderNextBuffer( void )
 {
 	microTimer timer;
 
+	static songEditor::playPos last_metro_pos = -1;
+
+	songEditor::playPos p = eng()->getSongEditor()->getPlayPos(
+						songEditor::PLAY_PATTERN );
+	if( eng()->getSongEditor()->playMode() == songEditor::PLAY_PATTERN &&
+		eng()->getPianoRoll()->isRecording() == TRUE &&
+		p != last_metro_pos && p.getTact64th() % 16 == 0 )
+	{
+		addPlayHandle( new samplePlayHandle( "misc/metronome01.ogg",
+								eng() ) );
+		last_metro_pos = p;
+	}
+
 	// now we have to make sure no other thread does anything bad
 	// while we're acting...
 	m_mixMutex.lock();
@@ -189,65 +234,76 @@ const surroundSampleFrame * mixer::renderNextBuffer( void )
 			++it;
 		}
 
-		m_playHandlesToRemove.erase(
-					m_playHandlesToRemove.begin() );
+		m_playHandlesToRemove.erase( m_playHandlesToRemove.begin() );
 	}
 
 	// now swap the buffers... current buffer becomes next (last)
 	// buffer and the next buffer becomes current (first) buffer
-	qSwap( m_curBuf, m_nextBuf );
+//	qSwap( m_curBuf, m_nextBuf );
+	m_writeBuffer++;
+	m_writeBuffer %= m_poolDepth;
+
+	m_readBuffer++;
+	m_readBuffer %= m_poolDepth;
+
+	m_analBuffer++;
+	m_analBuffer %= m_poolDepth;
+
+	m_writeBuf = m_bufferPool[m_writeBuffer];
+	m_readBuf = m_bufferPool[m_readBuffer];
 
 	// clear last audio-buffer
-	clearAudioBuffer( m_curBuf, m_framesPerAudioBuffer );
+	clearAudioBuffer( m_writeBuf, m_framesPerAudioBuffer );
 
-
-	csize idx = 0;
-	while( idx < m_playHandles.size() )
+//	if( criticalXRuns() == FALSE )
 	{
-		register playHandle * n = m_playHandles[idx];
-		if( n->done() )
+		csize idx = 0;
+		while( idx < m_playHandles.size() )
 		{
-			// delete all play-handles which have
-			// played completely now
-			delete n;
-			m_playHandles.erase( m_playHandles.begin() +
-								idx );
+			register playHandle * n = m_playHandles[idx];
+			// delete play-handle if it played completely
+			if( n->done() )
+			{
+				delete n;
+				m_playHandles.erase( m_playHandles.begin() +
+									idx );
+			}
+			else
+			{
+				// play all uncompletely-played play-handles...
+				n->play();
+				++idx;
+			}
 		}
-		else
-		{
-			// play all uncompletely-played play-handles...
-			n->play();
-			++idx;
-		}
-	}
 
-	songEditor::inst()->processNextBuffer();
+		eng()->getSongEditor()->processNextBuffer();
 
-	for( vvector<audioPort *>::iterator it = m_audioPorts.begin();
-					it != m_audioPorts.end(); ++it )
-	{
-		if( ( *it )->m_bufferUsage != audioPort::NONE )
+		for( vvector<audioPort *>::iterator it = m_audioPorts.begin();
+						it != m_audioPorts.end(); ++it )
 		{
-			processBuffer( ( *it )->firstBuffer(),
-					( *it )->nextFxChannel() );
-			( *it )->nextPeriod();
+			if( ( *it )->m_bufferUsage != audioPort::NONE )
+			{
+				processBuffer( ( *it )->firstBuffer(),
+						( *it )->nextFxChannel() );
+				( *it )->nextPeriod();
+			}
 		}
 	}
 
 
-	emit nextAudioBuffer( m_curBuf, m_framesPerAudioBuffer );
+	emit nextAudioBuffer( m_readBuf, m_framesPerAudioBuffer );
 
 	m_mixMutex.unlock();
 
-
 	// and trigger LFOs
-	envelopeAndLFOWidget::triggerLFO();
+	envelopeAndLFOWidget::triggerLFO( eng() );
 
 	const float new_cpu_load = timer.elapsed() / 10000.0f * sampleRate() /
 							m_framesPerAudioBuffer;
-	m_cpuLoad = tLimit( (int) ( new_cpu_load + m_cpuLoad ) / 2, 0, 100 );
+	m_cpuLoad = tLimit( (int) ( new_cpu_load * 0.1f + m_cpuLoad * 0.9f ), 0,
+									100 );
 
-	return( m_curBuf );
+	return( m_readBuf );
 }
 
 
@@ -255,20 +311,29 @@ const surroundSampleFrame * mixer::renderNextBuffer( void )
 
 // removes all play-handles. this is neccessary, when the song is stopped ->
 // all remaining notes etc. would be played until their end
-void mixer::clear( void )
+void mixer::clear( bool _everything )
 {
 	// TODO: m_midiClient->noteOffAll();
 	for( playHandleVector::iterator it = m_playHandles.begin();
 					it != m_playHandles.end(); ++it )
 	{
-		m_playHandlesToRemove.push_back( *it );
+		// we must not delete instrument-play-handles as they exist
+		// during the whole lifetime of an instrument - exception if
+		// parameter _everything is true (which is the case when
+		// clearing song for example)
+		if( _everything == TRUE ||
+			( *it )->type() != playHandle::INSTRUMENT_PLAY_HANDLE )
+		{
+			m_playHandlesToRemove.push_back( *it );
+		}
 	}
 }
 
 
 
 
-void FASTCALL mixer::clearAudioBuffer( sampleFrame * _ab, Uint32 _frames )
+void FASTCALL mixer::clearAudioBuffer( sampleFrame * _ab,
+							const f_cnt_t _frames )
 {
 	memset( _ab, 0, sizeof( *_ab ) * _frames );
 }
@@ -277,7 +342,7 @@ void FASTCALL mixer::clearAudioBuffer( sampleFrame * _ab, Uint32 _frames )
 
 #ifndef DISABLE_SURROUND
 void FASTCALL mixer::clearAudioBuffer( surroundSampleFrame * _ab,
-								Uint32 _frames )
+							const f_cnt_t _frames )
 {
 	memset( _ab, 0, sizeof( *_ab ) * _frames );
 }
@@ -285,17 +350,19 @@ void FASTCALL mixer::clearAudioBuffer( surroundSampleFrame * _ab,
 
 
 
-void FASTCALL mixer::bufferToPort( sampleFrame * _buf, Uint32 _frames,
-						Uint32 _frames_ahead,
-						volumeVector & _volume_vector,
+void FASTCALL mixer::bufferToPort( const sampleFrame * _buf,
+					const fpab_t _frames,
+					const fpab_t _frames_ahead,
+					const volumeVector & _volume_vector,
 						audioPort * _port )
 {
-	Uint32 start_frame = _frames_ahead % m_framesPerAudioBuffer;
-	Uint32 end_frame = start_frame + _frames;
-	Uint32 loop1_frame = tMin( end_frame, m_framesPerAudioBuffer );
-	for( Uint32 frame = start_frame; frame < loop1_frame; ++frame )
+	const fpab_t start_frame = _frames_ahead % m_framesPerAudioBuffer;
+	fpab_t end_frame = start_frame + _frames;
+	const fpab_t loop1_frame = tMin( end_frame, m_framesPerAudioBuffer );
+
+	for( fpab_t frame = start_frame; frame < loop1_frame; ++frame )
 	{
-		for( Uint8 chnl = 0; chnl < SURROUND_CHANNELS; ++chnl )
+		for( ch_cnt_t chnl = 0; chnl < m_audioDev->channels(); ++chnl )
 		{
 			_port->firstBuffer()[frame][chnl] +=
 					_buf[frame - start_frame][chnl %
@@ -303,14 +370,16 @@ void FASTCALL mixer::bufferToPort( sampleFrame * _buf, Uint32 _frames,
 						_volume_vector.vol[chnl];
 		}
 	}
+
 	if( end_frame > m_framesPerAudioBuffer )
 	{
-		Uint32 frames_done = m_framesPerAudioBuffer - start_frame;
+		fpab_t frames_done = m_framesPerAudioBuffer - start_frame;
 		end_frame = tMin( end_frame -= m_framesPerAudioBuffer,
 						m_framesPerAudioBuffer );
-		for( Uint32 frame = 0; frame < end_frame; ++frame )
+		for( fpab_t frame = 0; frame < end_frame; ++frame )
 		{
-			for( Uint8 chnl = 0; chnl < SURROUND_CHANNELS; ++chnl )
+			for( ch_cnt_t chnl = 0; chnl < m_audioDev->channels();
+									++chnl )
 			{
 				_port->secondBuffer()[frame][chnl] +=
 					_buf[frames_done + frame][chnl %
@@ -318,10 +387,12 @@ void FASTCALL mixer::bufferToPort( sampleFrame * _buf, Uint32 _frames,
 						_volume_vector.vol[chnl];
 			}
 		}
+		// we used both buffers so set flags
 		_port->m_bufferUsage = audioPort::BOTH;
 	}
 	else if( _port->m_bufferUsage == audioPort::NONE )
 	{
+		// only first buffer touched
 		_port->m_bufferUsage = audioPort::FIRST;
 	}
 
@@ -336,14 +407,9 @@ void mixer::setHighQuality( bool _hq_on )
 	delete m_audioDev;
 
 	// set new quality-level...
-	if( _hq_on == TRUE )
-	{
-		m_qualityLevel = HIGH_QUALITY_LEVEL;
-	}
-	else
-	{
-		m_qualityLevel = DEFAULT_QUALITY_LEVEL;
-	}
+	m_qualityLevel = ( _hq_on == TRUE ) ? HIGH_QUALITY_LEVEL :
+							DEFAULT_QUALITY_LEVEL;
+
 	// and re-open device
 	m_audioDev = tryAudioDevices();
 	m_audioDev->startProcessing();
@@ -372,7 +438,7 @@ void FASTCALL mixer::setAudioDevice( audioDevice * _dev, bool _hq )
 		m_audioDev = _dev;
 	}
 
-	m_qualityLevel = _hq ? 1 : 0;
+	m_qualityLevel = _hq ? HIGH_QUALITY_LEVEL : DEFAULT_QUALITY_LEVEL;
 	emit sampleRateChanged();
 }
 
@@ -386,11 +452,13 @@ void mixer::restoreAudioDevice( void )
 		delete m_audioDev;	// dtor automatically calls
 					// stopProcessing()
 		m_audioDev = m_oldAudioDev;
-		for( Uint8 qli = 0; qli < QUALITY_LEVELS; ++qli )
+		for( Uint8 qli = DEFAULT_QUALITY_LEVEL;
+						qli < QUALITY_LEVELS; ++qli )
 		{
 			if( SAMPLE_RATES[qli] == m_audioDev->sampleRate() )
 			{
-				m_qualityLevel = qli;
+				m_qualityLevel =
+					static_cast<qualityLevels>( qli );
 				emit sampleRateChanged();
 				break;
 			}
@@ -423,28 +491,29 @@ audioDevice * mixer::tryAudioDevices( void )
 	audioDevice * dev = NULL;
 	QString dev_name = configManager::inst()->value( "mixer", "audiodev" );
 
-#ifdef OSS_SUPPORT
-	if( dev_name == audioOSS::name() || dev_name == "" )
+#ifdef ALSA_SUPPORT
+	if( dev_name == audioALSA::name() || dev_name == "" )
 	{
-		dev = new audioOSS( SAMPLE_RATES[DEFAULT_QUALITY_LEVEL],
-								success_ful );
+		dev = new audioALSA( SAMPLE_RATES[DEFAULT_QUALITY_LEVEL],
+							success_ful, this );
 		if( success_ful )
 		{
-			m_audioDevName = audioOSS::name();
+			m_audioDevName = audioALSA::name();
 			return( dev );
 		}
 		delete dev;
 	}
 #endif
 
-#ifdef ALSA_SUPPORT
-	if( dev_name == audioALSA::name() || dev_name == "" )
+
+#ifdef OSS_SUPPORT
+	if( dev_name == audioOSS::name() || dev_name == "" )
 	{
-		dev = new audioALSA( SAMPLE_RATES[DEFAULT_QUALITY_LEVEL],
-								success_ful );
+		dev = new audioOSS( SAMPLE_RATES[DEFAULT_QUALITY_LEVEL],
+							success_ful, this );
 		if( success_ful )
 		{
-			m_audioDevName = audioALSA::name();
+			m_audioDevName = audioOSS::name();
 			return( dev );
 		}
 		delete dev;
@@ -456,7 +525,7 @@ audioDevice * mixer::tryAudioDevices( void )
 	if( dev_name == audioJACK::name() || dev_name == "" )
 	{
 		dev = new audioJACK( SAMPLE_RATES[DEFAULT_QUALITY_LEVEL],
-								success_ful );
+							success_ful, this );
 		if( success_ful )
 		{
 			m_audioDevName = audioJACK::name();
@@ -471,7 +540,7 @@ audioDevice * mixer::tryAudioDevices( void )
 	if( dev_name == audioSDL::name() || dev_name == "" )
 	{
 		dev = new audioSDL( SAMPLE_RATES[DEFAULT_QUALITY_LEVEL],
-								success_ful );
+							success_ful, this );
 		if( success_ful )
 		{
 			m_audioDevName = audioSDL::name();
@@ -482,7 +551,7 @@ audioDevice * mixer::tryAudioDevices( void )
 #endif
 
 	// add more device-classes here...
-	//dev = new audioXXXX( SAMPLE_RATES[m_qualityLevel], success_ful );
+	//dev = new audioXXXX( SAMPLE_RATES[m_qualityLevel], success_ful, this );
 	//if( sucess_ful )
 	//{
 	//	return( dev );
@@ -495,7 +564,8 @@ audioDevice * mixer::tryAudioDevices( void )
 
 	m_audioDevName = audioDummy::name();
 
-	return( new audioDummy( SAMPLE_RATES[m_qualityLevel], success_ful ) );
+	return( new audioDummy( SAMPLE_RATES[m_qualityLevel], success_ful,
+								this ) );
 }
 
 
@@ -509,7 +579,7 @@ midiClient * mixer::tryMIDIClients( void )
 #ifdef ALSA_SUPPORT
 	if( client_name == midiALSASeq::name() || client_name == "" )
 	{
-		midiALSASeq * malsas = new midiALSASeq();
+		midiALSASeq * malsas = new midiALSASeq( eng() );
 		if( malsas->isRunning() )
 		{
 			m_midiClientName = midiALSASeq::name();
@@ -520,7 +590,7 @@ midiClient * mixer::tryMIDIClients( void )
 
 	if( client_name == midiALSARaw::name() || client_name == "" )
 	{
-		midiALSARaw * malsar = new midiALSARaw();
+		midiALSARaw * malsar = new midiALSARaw( eng() );
 		if( malsar->isRunning() )
 		{
 			m_midiClientName = midiALSARaw::name();
@@ -533,7 +603,7 @@ midiClient * mixer::tryMIDIClients( void )
 #ifdef OSS_SUPPORT
 	if( client_name == midiOSS::name() || client_name == "" )
 	{
-		midiOSS * moss = new midiOSS();
+		midiOSS * moss = new midiOSS( eng() );
 		if( moss->isRunning() )
 		{
 			m_midiClientName = midiOSS::name();
@@ -548,24 +618,107 @@ midiClient * mixer::tryMIDIClients( void )
 
 	m_midiClientName = midiDummy::name();
 
-	return( new midiDummy() );
+	return( new midiDummy( eng() ) );
 }
 
 
 
 
-void mixer::processBuffer( surroundSampleFrame * _buf, fxChnl/* _fx_chnl */ )
+void mixer::processBuffer( const surroundSampleFrame * _buf,
+						fx_ch_t/* _fx_chnl */ )
 {
 	// TODO: effect-implementation
-	for( Uint32 frame = 0; frame < m_framesPerAudioBuffer; ++frame )
+	
+	if( m_scaleClip )
 	{
-		for( Uint8 chnl = 0; chnl < SURROUND_CHANNELS; ++chnl )
+		for( ch_cnt_t chnl=0; 
+			chnl < m_audioDev->channels(); 
+			++chnl )
 		{
-			m_curBuf[frame][chnl] += _buf[frame][chnl];
+			m_newBuffer[chnl] = TRUE;
+		}
+	}
+	
+	for( fpab_t frame = 0; frame < m_framesPerAudioBuffer; ++frame )
+	{
+		for( ch_cnt_t chnl = 0; chnl < m_audioDev->channels(); ++chnl )
+		{
+			m_writeBuf[frame][chnl] += _buf[frame][chnl];
+			
+			if( m_scaleClip )
+			{
+				scaleClip( frame, chnl );
+			}
 		}
 	}
 }
 
 
+
+
+void FASTCALL mixer::scaleClip( fpab_t _frame, ch_cnt_t _chnl )
+{
+	// Check for zero crossing
+	if( ( m_writeBuf[_frame][_chnl] >=0 &&
+		     m_previousSample[_chnl] < 0 ) ||
+		     ( m_writeBuf[_frame][_chnl] <=0 &&
+		     m_previousSample[_chnl] > 0 ) )
+	{
+		// if a clip occurred between the zero
+		// crossings, scale the half-wave
+		if( m_clipped[_chnl] )
+		{
+			if( m_newBuffer[_chnl] )
+			{
+				for( fpab_t i = m_halfStart[_chnl];
+					i < m_framesPerAudioBuffer;
+					i++ )
+				{
+					m_bufferPool[m_analBuffer][i][_chnl] /=
+							m_maxClip[_chnl];
+				}
+					
+				for( fpab_t i = 0;
+					i < _frame;
+					i++ )
+				{
+					m_writeBuf[i][_chnl] /= 
+							m_maxClip[_chnl];
+				}
+			}
+			else
+			{
+				for( fpab_t i = m_halfStart[_chnl];
+					 i < _frame;
+					 i++ )
+				{
+					m_writeBuf[i][_chnl] /= m_maxClip[_chnl];
+				}
+			}
+		}
+		m_halfStart[_chnl] = _frame;
+		m_clipped[_chnl] = FALSE;
+		m_newBuffer[_chnl] = FALSE;
+		m_maxClip[_chnl] = 1.0;
+	}
+			
+	// check for clip
+	if( fabsf( m_writeBuf[_frame][_chnl] ) > 1.0f )
+	{
+		m_clipped[_chnl] = TRUE;
+		if( fabs( m_writeBuf[_frame][_chnl] ) >
+				  m_maxClip[_chnl] )
+		{
+			m_maxClip[_chnl] = fabs( 
+					m_writeBuf[_frame][_chnl] );
+		}
+	}
+			
+	m_previousSample[_chnl] = m_writeBuf[_frame][_chnl];
+}
+
+
 #include "mixer.moc"
 
+
+#endif
