@@ -1,8 +1,10 @@
+#ifndef SINGLE_SOURCE_COMPILE
+
 /*
  * piano_widget.cpp - implementation of piano-widget used in channel-window
  *                    for testing channel
  *
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -28,23 +30,35 @@
 
 #ifdef QT4
 
-#include <QPainter>
-#include <QMouseEvent>
-#include <QKeyEvent>
+#include <QtGui/QCursor>
+#include <QtGui/QKeyEvent>
+#include <QtGui/QLabel>
+#include <QtGui/QMenu>
+#include <QtGui/QMouseEvent>
+#include <QtGui/QPainter>
 
 #else
 
+#include <qcursor.h>
+#include <qlabel.h>
 #include <qpainter.h>
+#include <qpopupmenu.h>
 
 #endif
 
 
 #include "piano_widget.h"
-#include "channel_track.h"
+#include "instrument_track.h"
 #include "midi.h"
 #include "templates.h"
 #include "embed.h"
 
+#ifdef Q_WS_X11
+
+#include <X11/Xlib.h>
+
+#endif
+
 
 const keyTypes KEY_ORDER[] =
 {
@@ -68,18 +82,19 @@ QPixmap * pianoWidget::s_blackKeyPressedPm = NULL;
 
 
 const int PIANO_BASE = 11;
-const int WHITE_KEY_WIDTH = 10;
-const int BLACK_KEY_WIDTH = 8;
-const int WHITE_KEY_HEIGHT = 57;
-const int BLACK_KEY_HEIGHT = 38;
+const int PW_WHITE_KEY_WIDTH = 10;
+const int PW_BLACK_KEY_WIDTH = 8;
+const int PW_WHITE_KEY_HEIGHT = 57;
+const int PW_BLACK_KEY_HEIGHT = 38;
 const int LABEL_TEXT_SIZE = 7;
 
 
-pianoWidget::pianoWidget (channelTrack * _parent ) :
+pianoWidget::pianoWidget( instrumentTrack * _parent ) :
 	QWidget( _parent ),
-	m_channelTrack( _parent ),
+	m_instrumentTrack( _parent ),
 	m_startTone( C ),
-	m_startOctave( OCTAVE_3 )
+	m_startOctave( OCTAVE_3 ),
+	m_lastKey( -1 )
 {
 #ifdef QT4
 	setFocusPolicy( Qt::StrongFocus );
@@ -119,7 +134,7 @@ pianoWidget::pianoWidget (channelTrack * _parent ) :
 									4 );
 	m_pianoScroll->setSingleStep( 1 );
 	m_pianoScroll->setPageStep( 20 );
-	m_pianoScroll->setValue( OCTAVE_3*WHITE_KEYS_PER_OCTAVE );
+	m_pianoScroll->setValue( OCTAVE_3 * WHITE_KEYS_PER_OCTAVE );
 #else
 	// create scrollbar below piano-widget...
 	m_pianoScroll = new QScrollBar( 0, WHITE_KEYS_PER_OCTAVE *
@@ -127,7 +142,8 @@ pianoWidget::pianoWidget (channelTrack * _parent ) :
 					OCTAVE_3 * WHITE_KEYS_PER_OCTAVE,
 							Qt::Horizontal, this );
 #endif
-	m_pianoScroll->setGeometry( 0, PIANO_BASE+WHITE_KEY_HEIGHT, 250, 16 );
+	m_pianoScroll->setGeometry( 0, PIANO_BASE + PW_WHITE_KEY_HEIGHT, 250,
+									16 );
 	// ...and connect it to this widget...
 	connect( m_pianoScroll, SIGNAL( valueChanged( int ) ), this,
 						SLOT( pianoScrolled( int ) ) );
@@ -136,6 +152,14 @@ pianoWidget::pianoWidget (channelTrack * _parent ) :
 	// set background-mode for flicker-free redraw
 	setBackgroundMode( Qt::NoBackground );
 #endif
+
+	m_noteKnob = new knob( knobDark_28, NULL, tr ( "Base note" ),
+						_parent->eng(), _parent );
+	m_noteKnob->setRange( 0, NOTES_PER_OCTAVE * OCTAVES - 1, 1.0f );
+	m_noteKnob->setInitValue( DEFAULT_OCTAVE * NOTES_PER_OCTAVE + A );
+
+	connect( m_noteKnob, SIGNAL( valueChanged( float ) ), this,
+					SLOT( updateBaseNote( void ) ) );
 }
 
 
@@ -143,6 +167,7 @@ pianoWidget::pianoWidget (channelTrack * _parent ) :
 
 pianoWidget::~pianoWidget()
 {
+	delete m_noteKnob;
 }
 
 
@@ -152,7 +177,7 @@ pianoWidget::~pianoWidget()
 int pianoWidget::getKeyFromMouse( const QPoint & _p )
 {
 
-	int key_num = (int)( (float) _p.x() / (float) WHITE_KEY_WIDTH );
+	int key_num = (int)( (float) _p.x() / (float) PW_WHITE_KEY_WIDTH );
 
 	for( int i = 0; i <= key_num; ++i )
 	{
@@ -167,23 +192,25 @@ int pianoWidget::getKeyFromMouse( const QPoint & _p )
 	key_num += m_startOctave * NOTES_PER_OCTAVE + m_startTone;
 
 	// is it a black key?
-	if( _p.y() < PIANO_BASE + BLACK_KEY_HEIGHT )
+	if( _p.y() < PIANO_BASE + PW_BLACK_KEY_HEIGHT )
 	{
 		// then do extra checking whether the mouse-cursor is over
 		// a black key
 		if( key_num > 0 &&
 			KEY_ORDER[( key_num - 1 ) % NOTES_PER_OCTAVE] ==
 								BLACK_KEY &&
-			_p.x() % WHITE_KEY_WIDTH <= ( WHITE_KEY_WIDTH / 2 ) -
-						( BLACK_KEY_WIDTH / 2 ) )
+			_p.x() % PW_WHITE_KEY_WIDTH <=
+					( PW_WHITE_KEY_WIDTH / 2 ) -
+						( PW_BLACK_KEY_WIDTH / 2 ) )
 		{
 			--key_num;
 		}
 		if( key_num < NOTES_PER_OCTAVE * OCTAVES - 1 &&
 			KEY_ORDER[( key_num + 1 ) % NOTES_PER_OCTAVE] ==
 								BLACK_KEY &&
-			_p.x() % WHITE_KEY_WIDTH >=
-				( WHITE_KEY_WIDTH - BLACK_KEY_WIDTH / 2 ) )
+			_p.x() % PW_WHITE_KEY_WIDTH >=
+				( PW_WHITE_KEY_WIDTH -
+				  		PW_BLACK_KEY_WIDTH / 2 ) )
 		{
 			++key_num;
 		}
@@ -210,20 +237,49 @@ void pianoWidget::pianoScrolled( int _new_pos )
 
 
 
+void pianoWidget::contextMenuEvent( QContextMenuEvent * _me )
+{
+	if( _me->pos().y() > PIANO_BASE )
+	{
+		QWidget::contextMenuEvent( _me );
+		return;
+	}
+
+	QMenu contextMenu( this );
+#ifdef QT4
+	contextMenu.setTitle( m_noteKnob->accessibleName() );
+#else
+	QLabel * caption = new QLabel( "<font color=white><b>" +
+			QString( m_noteKnob->accessibleName() ) + "</b></font>",
+			this );
+	caption->setPaletteBackgroundColor( QColor( 0, 0, 192 ) );
+	caption->setAlignment( Qt::AlignCenter );
+	contextMenu.addAction( caption );
+#endif
+	contextMenu.addAction( embed::getIconPixmap( "automation" ),
+					tr( "&Open in automation editor" ),
+					m_noteKnob->getAutomationPattern(),
+					SLOT( openInAutomationEditor() ) );
+	contextMenu.exec( QCursor::pos() );
+}
+
+
+
+
 // handler for mouse-click-event
 void pianoWidget::mousePressEvent( QMouseEvent * _me )
 {
 	if( _me->button() == Qt::LeftButton )
 	{
 		// get pressed key
-		int key_num = getKeyFromMouse( _me->pos() );
+		Uint32 key_num = getKeyFromMouse( _me->pos() );
 		if( _me->pos().y() > PIANO_BASE )
 		{
 			int y_diff = _me->pos().y() - PIANO_BASE;
 			volume vol = (volume)( ( float ) y_diff /
 				( ( KEY_ORDER[key_num % NOTES_PER_OCTAVE] ==
 								WHITE_KEY ) ?
-					WHITE_KEY_HEIGHT : BLACK_KEY_HEIGHT ) *
+				PW_WHITE_KEY_HEIGHT : PW_BLACK_KEY_HEIGHT ) *
 				(float) DEFAULT_VOLUME);
 			if( y_diff < 0 )
 			{
@@ -232,12 +288,12 @@ void pianoWidget::mousePressEvent( QMouseEvent * _me )
 			else if( y_diff > ( ( KEY_ORDER[key_num %
 							NOTES_PER_OCTAVE] ==
 								WHITE_KEY ) ?
-					WHITE_KEY_HEIGHT : BLACK_KEY_HEIGHT ) )
+				PW_WHITE_KEY_HEIGHT : PW_BLACK_KEY_HEIGHT ) )
 			{
 				vol = DEFAULT_VOLUME;
 			}
 			// set note on
-			m_channelTrack->processInEvent(
+			m_instrumentTrack->processInEvent(
 					midiEvent( NOTE_ON, 0, key_num,
 							vol * 127 / 100 ),
 								midiTime() );
@@ -245,10 +301,8 @@ void pianoWidget::mousePressEvent( QMouseEvent * _me )
 		}
 		else
 		{
-			m_channelTrack->setBaseTone( static_cast<tones>(
-						key_num % NOTES_PER_OCTAVE ) );
-			m_channelTrack->setBaseOctave( static_cast<octaves>(
-						key_num / NOTES_PER_OCTAVE ) );
+			m_noteKnob->setInitValue( key_num );
+			m_instrumentTrack->setBaseNote( key_num );
 		}
 
 		// and let the user see that he pressed a key... :)
@@ -264,7 +318,7 @@ void pianoWidget::mouseReleaseEvent( QMouseEvent * _me )
 {
 	int released_key = getKeyFromMouse( _me->pos() );
 
-	m_channelTrack->processInEvent(
+	m_instrumentTrack->processInEvent(
 			midiEvent( NOTE_OFF, 0, released_key, 0 ), midiTime() );
 	m_pressedKeys[released_key] = FALSE;
 
@@ -284,7 +338,7 @@ void pianoWidget::mouseMoveEvent( QMouseEvent * _me )
 	int y_diff = _me->pos().y() - PIANO_BASE;
 	volume vol = (volume)( (float) y_diff /
 		( ( KEY_ORDER[key_num % NOTES_PER_OCTAVE] == WHITE_KEY ) ?
-			WHITE_KEY_HEIGHT : BLACK_KEY_HEIGHT ) *
+			PW_WHITE_KEY_HEIGHT : PW_BLACK_KEY_HEIGHT ) *
 						(float)DEFAULT_VOLUME );
 	// maybe the user moved the mouse-cursor above or under the
 	// piano-widget while holding left button so check that and
@@ -295,7 +349,7 @@ void pianoWidget::mouseMoveEvent( QMouseEvent * _me )
 	}
 	else if( y_diff >
 		( ( KEY_ORDER[key_num % NOTES_PER_OCTAVE] == WHITE_KEY ) ?
-				WHITE_KEY_HEIGHT : BLACK_KEY_HEIGHT ) )
+				PW_WHITE_KEY_HEIGHT : PW_BLACK_KEY_HEIGHT ) )
 	{
 		vol = DEFAULT_VOLUME;
 	}
@@ -304,10 +358,13 @@ void pianoWidget::mouseMoveEvent( QMouseEvent * _me )
 	// user just moved the cursor one pixel left but on the same key)
 	if( key_num != released_key )
 	{
-		m_channelTrack->processInEvent(
+		m_instrumentTrack->processInEvent(
 				midiEvent( NOTE_OFF, 0, released_key, 0 ),
 								midiTime() );
-		m_pressedKeys[released_key] = FALSE;
+		if( released_key >= 0 )
+		{
+			m_pressedKeys[released_key] = FALSE;
+		}
 #ifdef QT4
 		if( _me->buttons() & Qt::LeftButton )
 #else
@@ -316,17 +373,15 @@ void pianoWidget::mouseMoveEvent( QMouseEvent * _me )
 		{
 			if( _me->pos().y() > PIANO_BASE )
 			{
-				m_channelTrack->processInEvent(
+				m_instrumentTrack->processInEvent(
 					midiEvent( NOTE_ON, 0, key_num, vol ),
 								midiTime() );
 				m_pressedKeys[key_num] = TRUE;
 			}
 			else
 			{
-				m_channelTrack->setBaseTone( (tones)
-					( key_num % NOTES_PER_OCTAVE ) );
-				m_channelTrack->setBaseOctave( (octaves)
-					( key_num / NOTES_PER_OCTAVE ) );
+				m_noteKnob->setInitValue( key_num );
+				m_instrumentTrack->setBaseNote( key_num );
 			}
 		}
 		// and let the user see that he pressed a key... :)
@@ -334,7 +389,7 @@ void pianoWidget::mouseMoveEvent( QMouseEvent * _me )
 	}
 	else if( m_pressedKeys[key_num] == TRUE )
 	{
-		m_channelTrack->processInEvent(
+		m_instrumentTrack->processInEvent(
 				midiEvent( KEY_PRESSURE, 0, key_num, vol ),
 								midiTime() );
 	}
@@ -346,37 +401,37 @@ void pianoWidget::mouseMoveEvent( QMouseEvent * _me )
 
 int pianoWidget::getKeyFromKeyboard( int _k ) const
 {
-	switch( _k )
+	switch( m_keycode )
 	{
-		case Qt::Key_Y: return( 0 );
-		case Qt::Key_S: return( 1 );
-		case Qt::Key_X: return( 2 );
-		case Qt::Key_D: return( 3 );
-		case Qt::Key_C: return( 4 );
-		case Qt::Key_V: return( 5 );
-		case Qt::Key_G: return( 6 );
-		case Qt::Key_B: return( 7 );
-		case Qt::Key_H: return( 8 );
-		case Qt::Key_N: return( 9 );
-		case Qt::Key_J: return( 10 );
-		case Qt::Key_M: return( 11 );
-		case Qt::Key_Q: return( 12 );
-		case Qt::Key_2: return( 13 );
-		case Qt::Key_W: return( 14 );
-		case Qt::Key_3: return( 15 );
-		case Qt::Key_E: return( 16 );
-		case Qt::Key_R: return( 17 );
-		case Qt::Key_5: return( 18 );
-		case Qt::Key_T: return( 19 );
-		case Qt::Key_6: return( 20 );
-		case Qt::Key_Z: return( 21 );
-		case Qt::Key_7: return( 22 );
-		case Qt::Key_U: return( 23 );
-		case Qt::Key_I: return( 24 );
-		case Qt::Key_9: return( 25 );
-		case Qt::Key_O: return( 26 );
-		case Qt::Key_0: return( 27 );
-		case Qt::Key_P: return( 28 );
+		case 52: return( 0 ); // Y
+		case 39: return( 1 ); // S
+		case 53: return( 2 ); // X
+		case 40: return( 3 ); // D
+		case 54: return( 4 ); // C
+		case 55: return( 5 ); // V
+		case 42: return( 6 ); // G
+		case 56: return( 7 ); // B
+		case 43: return( 8 ); // H
+		case 57: return( 9 ); // N
+		case 44: return( 10 ); // J
+		case 58: return( 11 ); // M
+		case 24: return( 12 ); // Q
+		case 11: return( 13 ); // 2
+		case 25: return( 14 ); // W
+		case 12: return( 15 ); // 3
+		case 26: return( 16 ); // E
+		case 27: return( 17 ); // R
+		case 14: return( 18 ); // 5
+		case 28: return( 19 ); // T
+		case 15: return( 20 ); // 6
+		case 29: return( 21 ); // Z
+		case 16: return( 22 ); // 7
+		case 30: return( 23 ); // U
+		case 31: return( 24 ); // I
+		case 18: return( 25 ); // 9
+		case 32: return( 26 ); // O
+		case 19: return( 27 ); // 0
+		case 33: return( 28 ); // P
 	}
 
 	return( -100 );
@@ -392,7 +447,7 @@ void pianoWidget::keyPressEvent( QKeyEvent * _ke )
 
 	if( _ke->isAutoRepeat() == FALSE && key_num > -1 )
 	{
-		m_channelTrack->processInEvent(
+		m_instrumentTrack->processInEvent(
 			midiEvent( NOTE_ON, 0, key_num, DEFAULT_VOLUME ),
 								midiTime() );
 		m_pressedKeys[key_num] = TRUE;
@@ -413,7 +468,7 @@ void pianoWidget::keyReleaseEvent( QKeyEvent * _ke )
 				( DEFAULT_OCTAVE - 1 ) * NOTES_PER_OCTAVE;
 	if( _ke->isAutoRepeat() == FALSE && key_num > -1 )
 	{
-		m_channelTrack->processInEvent(
+		m_instrumentTrack->processInEvent(
 					midiEvent( NOTE_OFF, 0, key_num, 0 ),
 								midiTime() );
 		m_pressedKeys[key_num] = FALSE;
@@ -437,7 +492,7 @@ void pianoWidget::focusOutEvent( QFocusEvent * )
 	{
 		if( m_pressedKeys[i] == TRUE )
 		{
-			m_channelTrack->processInEvent(
+			m_instrumentTrack->processInEvent(
 						midiEvent( NOTE_OFF, 0, i, 0 ),
 								midiTime() );
 			m_pressedKeys[i] = FALSE;
@@ -454,7 +509,7 @@ int pianoWidget::getKeyX( int _key_num )
 	int k = m_startOctave*NOTES_PER_OCTAVE + m_startTone;
 	if( _key_num < k )
 	{
-		return( ( _key_num - k ) * WHITE_KEY_WIDTH / 2 );
+		return( ( _key_num - k ) * PW_WHITE_KEY_WIDTH / 2 );
 	}
 
 	int x = 0;
@@ -467,22 +522,22 @@ int pianoWidget::getKeyX( int _key_num )
 			++white_cnt;
 			if( white_cnt > 1 )
 			{
-				x += WHITE_KEY_WIDTH;
+				x += PW_WHITE_KEY_WIDTH;
 			}
 			else
 			{
-				x += WHITE_KEY_WIDTH/2;
+				x += PW_WHITE_KEY_WIDTH/2;
 			}
 		}
 		else
 		{
 			white_cnt = 0;
-			x += WHITE_KEY_WIDTH/2;
+			x += PW_WHITE_KEY_WIDTH/2;
 		}
 		++k;
 	}
 
-	x -= WHITE_KEY_WIDTH / 2;
+	x -= PW_WHITE_KEY_WIDTH / 2;
 
 	return( x );
 
@@ -497,7 +552,7 @@ void pianoWidget::paintEvent( QPaintEvent * )
 	QPainter p( this );
 #else
 	// create pixmap for whole widget
-	QPixmap pm( rect().size() );//width(), PIANO_BASE+WHITE_KEY_HEIGHT);
+	QPixmap pm( rect().size() );//width(), PIANO_BASE+PW_WHITE_KEY_HEIGHT);
 	// and a painter for it
 	QPainter p( &pm, this );
 #endif
@@ -518,18 +573,18 @@ void pianoWidget::paintEvent( QPaintEvent * )
 
 	p.setPen( QColor ( 0xFF, 0xFF, 0xFF ) );
 
-	int base_key = m_channelTrack->baseTone() +
-				m_channelTrack->baseOctave() * NOTES_PER_OCTAVE;
+	int base_key = m_instrumentTrack->baseTone() +
+			m_instrumentTrack->baseOctave() * NOTES_PER_OCTAVE;
 	if( KEY_ORDER[base_key % NOTES_PER_OCTAVE] == WHITE_KEY )
 	{
-		p.fillRect( QRect( getKeyX( base_key ), 1, WHITE_KEY_WIDTH-1,
+		p.fillRect( QRect( getKeyX( base_key ), 1, PW_WHITE_KEY_WIDTH-1,
 								PIANO_BASE-2 ),
 				QColor( 0xFF, 0xBB, 0x00 ) );
 	}
 	else
 	{
 		p.fillRect( QRect( getKeyX( base_key ) + 1, 1,
-					BLACK_KEY_WIDTH - 1, PIANO_BASE - 2 ),
+				PW_BLACK_KEY_WIDTH - 1, PIANO_BASE - 2 ),
 				QColor( 0xFF, 0xBB, 0x00 ) );
 	}
 
@@ -555,13 +610,13 @@ void pianoWidget::paintEvent( QPaintEvent * )
 			p.drawPixmap( x, PIANO_BASE, *s_whiteKeyPm );
 		}
 
-		x += WHITE_KEY_WIDTH;
+		x += PW_WHITE_KEY_WIDTH;
 
 		if( (tones) (cur_key%NOTES_PER_OCTAVE) == C )
 		{
 			// label key of note C with "C" and number of current
 			// octave
-			p.drawText( x - WHITE_KEY_WIDTH, LABEL_TEXT_SIZE + 2,
+			p.drawText( x - PW_WHITE_KEY_WIDTH, LABEL_TEXT_SIZE + 2,
 					QString( "C" ) + QString::number(
 					cur_key / NOTES_PER_OCTAVE, 10 ) );
 		}
@@ -579,12 +634,12 @@ void pianoWidget::paintEvent( QPaintEvent * )
 	{
 		if( m_pressedKeys[s_key] == TRUE )
 		{
-			p.drawPixmap( 0 - WHITE_KEY_WIDTH / 2, PIANO_BASE,
+			p.drawPixmap( 0 - PW_WHITE_KEY_WIDTH / 2, PIANO_BASE,
 							*s_blackKeyPressedPm );
 		}
 		else
 		{
-			p.drawPixmap( 0 - WHITE_KEY_WIDTH / 2, PIANO_BASE,
+			p.drawPixmap( 0 - PW_WHITE_KEY_WIDTH / 2, PIANO_BASE,
 								*s_blackKeyPm );
 		}
 	}
@@ -598,16 +653,16 @@ void pianoWidget::paintEvent( QPaintEvent * )
 			// state of current key
 			if( m_pressedKeys[cur_key] == TRUE )
 			{
-				p.drawPixmap( x + WHITE_KEY_WIDTH / 2,
+				p.drawPixmap( x + PW_WHITE_KEY_WIDTH / 2,
 								PIANO_BASE,
 							*s_blackKeyPressedPm );
 			}
 			else
 			{
-				p.drawPixmap( x + WHITE_KEY_WIDTH / 2,
+				p.drawPixmap( x + PW_WHITE_KEY_WIDTH / 2,
 						PIANO_BASE, *s_blackKeyPm );
 			}
-			x += WHITE_KEY_WIDTH;
+			x += PW_WHITE_KEY_WIDTH;
 			white_cnt = 0;
 		}
 		else
@@ -617,7 +672,7 @@ void pianoWidget::paintEvent( QPaintEvent * )
 			++white_cnt;
 			if( white_cnt > 1 )
 			{
-				x += WHITE_KEY_WIDTH;
+				x += PW_WHITE_KEY_WIDTH;
 			}
 		}
 		++cur_key;
@@ -630,5 +685,60 @@ void pianoWidget::paintEvent( QPaintEvent * )
 
 
 
+
+void pianoWidget::updateBaseNote( void )
+{
+	m_instrumentTrack->setBaseNote( (int)roundf( m_noteKnob->value() ),
+									FALSE );
+	update();
+}
+
+
+
+
+void pianoWidget::saveSettings( QDomDocument & _doc, QDomElement & _this,
+							const QString & _name )
+{
+	m_noteKnob->saveSettings( _doc, _this, _name );
+}
+
+
+
+
+void pianoWidget::loadSettings( const QDomElement & _this,
+							const QString & _name )
+{
+	if( _this.hasAttribute( "baseoct" ) )
+	{
+		m_noteKnob->setInitValue( _this.attribute( "baseoct" ).toInt()
+				* NOTES_PER_OCTAVE
+				+ _this.attribute( "basetone" ).toInt() );
+	}
+	else
+	{
+		m_noteKnob->loadSettings( _this, _name );
+	}
+	updateBaseNote();
+}
+
+
+
+
+bool pianoWidget::x11Event( XEvent * _xe )
+{
+	switch( _xe->type )
+	{
+		case KeyPress:
+		case KeyRelease:
+			m_keycode = _xe->xkey.keycode;
+	}
+	return( FALSE );
+}
+
+
+
+
 #include "piano_widget.moc"
 
+
+#endif
