@@ -1,7 +1,9 @@
+#ifndef SINGLE_SOURCE_COMPILE
+
 /*
  * setup_dialog.cpp - dialog for setting up LMMS
  *
- * Copyright (c) 2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2005-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -27,12 +29,14 @@
 
 #ifdef QT4
 
-#include <QLayout>
-#include <QLabel>
-#include <QSlider>
-#include <QWhatsThis>
-#include <QComboBox>
-#include <QMessageBox>
+#include <QtGui/QComboBox>
+#include <QtGui/QFileDialog>
+#include <QtGui/QLabel>
+#include <QtGui/QLayout>
+#include <QtGui/QLineEdit>
+#include <QtGui/QMessageBox>
+#include <QtGui/QSlider>
+#include <QtGui/QWhatsThis>
 
 #else
 
@@ -42,6 +46,8 @@
 #include <qwhatsthis.h>
 #include <qcombobox.h>
 #include <qmessagebox.h>
+#include <qlineedit.h>
+#include <qfiledialog.h>
 
 #endif
 
@@ -92,18 +98,34 @@ inline void labelWidget( QWidget * _w, const QString & _txt )
 
 
 
-setupDialog::setupDialog( configTabs _tab_to_open ) :
+setupDialog::setupDialog( engine * _engine, configTabs _tab_to_open ) :
 	QDialog(),
-	m_bufferSize( mixer::inst()->framesPerAudioBuffer() ),
+	engineObject( _engine ),
+	m_bufferSize( eng()->getMixer()->framesPerAudioBuffer() ),
 	m_disableToolTips( configManager::inst()->value( "tooltips",
 							"disabled" ).toInt() ),
 	m_classicalKnobUsability( configManager::inst()->value( "knobs",
 					"classicalusability" ).toInt() ),
 	m_gimpLikeWindows( configManager::inst()->value( "app",
-						"gimplikewindows" ).toInt() )
+						"gimplikewindows" ).toInt() ),
+	m_noWizard( configManager::inst()->value( "app", "nowizard" ).toInt() ),
+	m_noMsgAfterSetup( configManager::inst()->value( "app",
+						"nomsgaftersetup" ).toInt() ),
+	m_displaydBV( configManager::inst()->value( "app", 
+		      				"displaydbv" ).toInt() ),
+	m_workingDir( configManager::inst()->workingDir() ),
+	m_vstDir( configManager::inst()->vstDir() ),
+	m_artworkDir( configManager::inst()->artworkDir() ),
+	m_flDir( configManager::inst()->flDir() ),
+	m_disableChActInd( configManager::inst()->value( "ui",
+				"disablechannelactivityindicators" ).toInt() ),
+	m_manualChPiano( configManager::inst()->value( "ui",
+					"manualchannelpiano" ).toInt() )
+					
 {
 	setWindowIcon( embed::getIconPixmap( "setup_general" ) );
 	setWindowTitle( tr( "Setup LMMS" ) );
+	setModal( TRUE );
 
 	QVBoxLayout * vlayout = new QVBoxLayout( this );
 	vlayout->setSpacing( 0 );
@@ -118,10 +140,10 @@ setupDialog::setupDialog( configTabs _tab_to_open ) :
 	m_tabBar->setFixedWidth( 72 );
 
 	QWidget * ws = new QWidget( settings );
-	ws->setFixedSize( 360, 200 );
+	ws->setFixedSize( 360, 300 );
 
 	QWidget * general = new QWidget( ws );
-	general->setFixedSize( 360, 200 );
+	general->setFixedSize( 360, 240 );
 	QVBoxLayout * gen_layout = new QVBoxLayout( general );
 	gen_layout->setSpacing( 0 );
 	gen_layout->setMargin( 0 );
@@ -167,12 +189,12 @@ setupDialog::setupDialog( configTabs _tab_to_open ) :
 
 
 	tabWidget * misc_tw = new tabWidget( tr( "MISC" ), general );
-	misc_tw->setFixedHeight( 76 );
+	misc_tw->setFixedHeight( 128 );
 
 	ledCheckBox * disable_tooltips = new ledCheckBox(
 					tr( "Disable tooltips (no spurious "
 						"interrupts while playing)" ),
-								misc_tw );
+						misc_tw, NULL, eng(), NULL );
 	disable_tooltips->move( 10, 18 );
 	disable_tooltips->setChecked( m_disableToolTips );
 	connect( disable_tooltips, SIGNAL( toggled( bool ) ),
@@ -183,7 +205,7 @@ setupDialog::setupDialog( configTabs _tab_to_open ) :
 					tr( "Classical knob usability (move "
 						"cursor around knob to change "
 						"value)" ),
-								misc_tw );
+						misc_tw, NULL, eng(), NULL );
 	classical_knob_usability->move( 10, 36 );
 	classical_knob_usability->setChecked( m_classicalKnobUsability );
 	connect( classical_knob_usability, SIGNAL( toggled( bool ) ),
@@ -192,13 +214,40 @@ setupDialog::setupDialog( configTabs _tab_to_open ) :
 
 	ledCheckBox * gimp_like_windows = new ledCheckBox(
 					tr( "GIMP-like windows (no MDI)" ),
-								misc_tw );
+						misc_tw, NULL, eng(), NULL );
 	gimp_like_windows->move( 10, 54 );
 	gimp_like_windows->setChecked( m_gimpLikeWindows );
 	connect( gimp_like_windows, SIGNAL( toggled( bool ) ),
 			this, SLOT( toggleGIMPLikeWindows( bool ) ) );
 
 
+	ledCheckBox * no_wizard = new ledCheckBox(
+				tr( "Do not show wizard after up-/downgrade" ),
+						misc_tw, NULL, eng(), NULL );
+	no_wizard->move( 10, 72 );
+	no_wizard->setChecked( m_noWizard );
+	connect( no_wizard, SIGNAL( toggled( bool ) ),
+					this, SLOT( toggleNoWizard( bool ) ) );
+
+
+	ledCheckBox * no_msg = new ledCheckBox(
+					tr( "Do not show message after "
+						"closing this dialog" ),
+						misc_tw, NULL, eng(), NULL );
+	no_msg->move( 10, 90 );
+	no_msg->setChecked( m_noMsgAfterSetup );
+	connect( no_msg, SIGNAL( toggled( bool ) ),
+				this, SLOT( toggleNoMsgAfterSetup( bool ) ) );
+
+
+	ledCheckBox * dbv = new ledCheckBox(
+					tr( "Display volume as dbV " ),
+						misc_tw, NULL, eng(), NULL );
+	dbv->move( 10, 108 );
+	dbv->setChecked( m_displaydBV );
+	connect( dbv, SIGNAL( toggled( bool ) ),
+				this, SLOT( toggleDisplaydBV( bool ) ) );
+
 	gen_layout->addWidget( bufsize_tw );
 	gen_layout->addSpacing( 10 );
 	gen_layout->addWidget( misc_tw );
@@ -206,6 +255,140 @@ setupDialog::setupDialog( configTabs _tab_to_open ) :
 
 
 
+	QWidget * directories = new QWidget( ws );
+	directories->setFixedSize( 360, 260 );
+	QVBoxLayout * dir_layout = new QVBoxLayout( directories );
+	dir_layout->setSpacing( 0 );
+	dir_layout->setMargin( 0 );
+	labelWidget( directories, tr( "Directories" ) );
+
+	// working-dir
+	tabWidget * lmms_wd_tw = new tabWidget( tr(
+					"LMMS working directory" ).toUpper(),
+								directories );
+	lmms_wd_tw->setFixedHeight( 56 );
+
+	m_wdLineEdit = new QLineEdit( m_workingDir, lmms_wd_tw );
+	m_wdLineEdit->setGeometry( 10, 20, 300, 16 );
+	connect( m_wdLineEdit, SIGNAL( textChanged( const QString & ) ), this,
+				SLOT( setWorkingDir( const QString & ) ) );
+
+	QPushButton * workingdir_select_btn = new QPushButton(
+				embed::getIconPixmap( "project_open", 16, 16 ),
+							"", lmms_wd_tw );
+	workingdir_select_btn->setFixedSize( 24, 24 );
+	workingdir_select_btn->move( 320, 20 );
+	connect( workingdir_select_btn, SIGNAL( clicked() ), this,
+						SLOT( openWorkingDir() ) );
+
+	// vst-dir
+	tabWidget * vst_tw = new tabWidget( tr(
+					"VST-plugin directory" ).toUpper(),
+								directories );
+	vst_tw->setFixedHeight( 56 );
+
+	m_vdLineEdit = new QLineEdit( m_vstDir, vst_tw );
+	m_vdLineEdit->setGeometry( 10, 20, 300, 16 );
+	connect( m_vdLineEdit, SIGNAL( textChanged( const QString & ) ), this,
+					SLOT( setVSTDir( const QString & ) ) );
+
+	QPushButton * vstdir_select_btn = new QPushButton(
+				embed::getIconPixmap( "project_open", 16, 16 ),
+								"", vst_tw );
+	vstdir_select_btn->setFixedSize( 24, 24 );
+	vstdir_select_btn->move( 320, 20 );
+	connect( vstdir_select_btn, SIGNAL( clicked() ), this,
+						SLOT( openVSTDir() ) );
+
+	// artwork-dir
+	tabWidget * artwork_tw = new tabWidget( tr(
+					"Artwork directory" ).toUpper(),
+								directories );
+	artwork_tw->setFixedHeight( 56 );
+
+	m_adLineEdit = new QLineEdit( m_artworkDir, artwork_tw );
+	m_adLineEdit->setGeometry( 10, 20, 300, 16 );
+	connect( m_adLineEdit, SIGNAL( textChanged( const QString & ) ), this,
+				SLOT( setArtworkDir( const QString & ) ) );
+
+	QPushButton * artworkdir_select_btn = new QPushButton(
+				embed::getIconPixmap( "project_open", 16, 16 ),
+							"", artwork_tw );
+	artworkdir_select_btn->setFixedSize( 24, 24 );
+	artworkdir_select_btn->move( 320, 20 );
+	connect( artworkdir_select_btn, SIGNAL( clicked() ), this,
+						SLOT( openArtworkDir() ) );
+
+	// FL Studio-dir
+	tabWidget * fl_tw = new tabWidget( tr(
+				"FL Studio installation directory" ).toUpper(),
+								directories );
+	fl_tw->setFixedHeight( 56 );
+
+	m_fdLineEdit = new QLineEdit( m_flDir, fl_tw );
+	m_fdLineEdit->setGeometry( 10, 20, 300, 16 );
+	connect( m_fdLineEdit, SIGNAL( textChanged( const QString & ) ), this,
+					SLOT( setFLDir( const QString & ) ) );
+
+	QPushButton * fldir_select_btn = new QPushButton(
+				embed::getIconPixmap( "project_open", 16, 16 ),
+								"", fl_tw );
+	fldir_select_btn->setFixedSize( 24, 24 );
+	fldir_select_btn->move( 320, 20 );
+	connect( fldir_select_btn, SIGNAL( clicked() ), this,
+						SLOT( openFLDir() ) );
+
+
+	dir_layout->addWidget( lmms_wd_tw );
+	dir_layout->addSpacing( 10 );
+	dir_layout->addWidget( vst_tw );
+	dir_layout->addSpacing( 10 );
+	dir_layout->addWidget( artwork_tw );
+	dir_layout->addSpacing( 10 );
+	dir_layout->addWidget( fl_tw );
+	dir_layout->addStretch();
+
+
+
+
+
+
+	QWidget * performance = new QWidget( ws );
+	performance->setFixedSize( 360, 200 );
+	QVBoxLayout * perf_layout = new QVBoxLayout( performance );
+	perf_layout->setSpacing( 0 );
+	perf_layout->setMargin( 0 );
+	labelWidget( performance, tr( "Performance settings" ) );
+
+	tabWidget * ui_fx_tw = new tabWidget( tr( "UI effects vs. "
+						"performance" ).toUpper(),
+								performance );
+	ui_fx_tw->setFixedHeight( 70 );
+
+	ledCheckBox * disable_ch_act_ind = new ledCheckBox(
+				tr( "Disable channel activity indicators" ),
+						ui_fx_tw, NULL, eng(), NULL );
+	disable_ch_act_ind->move( 10, 20 );
+	disable_ch_act_ind->setChecked( m_disableChActInd );
+	connect( disable_ch_act_ind, SIGNAL( toggled( bool ) ),
+				this, SLOT( toggleDisableChActInd( bool ) ) );
+
+
+	ledCheckBox * manual_ch_piano = new ledCheckBox(
+			tr( "Only press keys on channel-piano manually" ),
+						ui_fx_tw, NULL, eng(), NULL );
+	manual_ch_piano->move( 10, 40 );
+	manual_ch_piano->setChecked( m_manualChPiano );
+	connect( manual_ch_piano, SIGNAL( toggled( bool ) ),
+				this, SLOT( toggleManualChPiano( bool ) ) );
+
+
+
+
+	perf_layout->addWidget( ui_fx_tw );
+	perf_layout->addStretch();
+
+
 
 	QWidget * audio = new QWidget( ws );
 	audio->setFixedSize( 360, 200 );
@@ -215,7 +398,7 @@ setupDialog::setupDialog( configTabs _tab_to_open ) :
 	labelWidget( audio, tr( "Audio settings" ) );
 
 	tabWidget * audioiface_tw = new tabWidget( tr( "AUDIO INTERFACE" ),
-								audio );
+									audio );
 	audioiface_tw->setFixedHeight( 60 );
 
 	m_audioInterfaces = new QComboBox( audioiface_tw );
@@ -278,11 +461,11 @@ setupDialog::setupDialog( configTabs _tab_to_open ) :
 	}
 #ifdef QT4
 	m_audioInterfaces->setCurrentIndex( m_audioInterfaces->findText(
-					mixer::inst()->audioDevName() ) );
+					eng()->getMixer()->audioDevName() ) );
 #else
-	m_audioInterfaces->setCurrentText( mixer::inst()->audioDevName() );
+	m_audioInterfaces->setCurrentText( eng()->getMixer()->audioDevName() );
 #endif
-	m_audioIfaceSetupWidgets[mixer::inst()->audioDevName()]->show();
+	m_audioIfaceSetupWidgets[eng()->getMixer()->audioDevName()]->show();
 
 	connect( m_audioInterfaces, SIGNAL( activated( const QString & ) ),
 		this, SLOT( audioInterfaceChanged( const QString & ) ) );
@@ -359,11 +542,11 @@ setupDialog::setupDialog( configTabs _tab_to_open ) :
 
 #ifdef QT4
 	m_midiInterfaces->setCurrentIndex( m_midiInterfaces->findText(
-					mixer::inst()->midiClientName() ) );
+					eng()->getMixer()->midiClientName() ) );
 #else
-	m_midiInterfaces->setCurrentText( mixer::inst()->midiClientName() );
+	m_midiInterfaces->setCurrentText( eng()->getMixer()->midiClientName() );
 #endif
-	m_midiIfaceSetupWidgets[mixer::inst()->midiClientName()]->show();
+	m_midiIfaceSetupWidgets[eng()->getMixer()->midiClientName()]->show();
 
 	connect( m_midiInterfaces, SIGNAL( activated( const QString & ) ),
 		this, SLOT( midiInterfaceChanged( const QString & ) ) );
@@ -381,9 +564,15 @@ setupDialog::setupDialog( configTabs _tab_to_open ) :
 
 	m_tabBar->addTab( general, tr( "General settings" ), 0, FALSE, TRUE 
 			)->setIcon( embed::getIconPixmap( "setup_general" ) );
-	m_tabBar->addTab( audio, tr( "Audio settings" ), 1, FALSE, TRUE
+	m_tabBar->addTab( directories, tr( "Directories" ), 1, FALSE, TRUE 
+			)->setIcon( embed::getIconPixmap(
+							"setup_directories" ) );
+	m_tabBar->addTab( performance, tr( "Performance settings" ), 2, FALSE,
+				TRUE )->setIcon( embed::getIconPixmap(
+							"setup_performance" ) );
+	m_tabBar->addTab( audio, tr( "Audio settings" ), 3, FALSE, TRUE
 			)->setIcon( embed::getIconPixmap( "setup_audio" ) );
-	m_tabBar->addTab( midi, tr( "MIDI settings" ), 2, TRUE, TRUE
+	m_tabBar->addTab( midi, tr( "MIDI settings" ), 4, TRUE, TRUE
 			)->setIcon( embed::getIconPixmap( "setup_midi" ) );
 
 #undef setIcon
@@ -452,6 +641,23 @@ void setupDialog::accept( void )
 				QString::number( m_classicalKnobUsability ) );
 	configManager::inst()->setValue( "app", "gimplikewindows",
 				QString::number( m_gimpLikeWindows ) );
+	configManager::inst()->setValue( "app", "nowizard",
+				QString::number( m_noWizard ) );
+	configManager::inst()->setValue( "app", "nomsgaftersetup",
+				QString::number( m_noMsgAfterSetup ) );
+	configManager::inst()->setValue( "app", "displaydbv",
+				QString::number( m_displaydBV ) );
+	configManager::inst()->setValue( "ui",
+					"disablechannelactivityindicators",
+					QString::number( m_disableChActInd ) );
+	configManager::inst()->setValue( "ui", "manualchannelpiano",
+					QString::number( m_manualChPiano ) );
+
+	configManager::inst()->setWorkingDir( m_workingDir );
+	configManager::inst()->setVSTDir( m_vstDir );
+	configManager::inst()->setArtworkDir( m_artworkDir );
+	configManager::inst()->setFLDir( m_flDir );
+
 	// tell all audio-settings-widget to save their settings
 	for( aswMap::iterator it = m_audioIfaceSetupWidgets.begin();
 				it != m_audioIfaceSetupWidgets.end(); ++it )
@@ -476,11 +682,14 @@ void setupDialog::accept( void )
 	configManager::inst()->saveConfigFile();
 
 	QDialog::accept();
-	QMessageBox::information( NULL, tr( "Restart LMMS" ),
+	if( m_noMsgAfterSetup == FALSE )
+	{
+		QMessageBox::information( NULL, tr( "Restart LMMS" ),
 					tr( "Please note that most changes "
 						"won't take effect until "
 						"you restart LMMS!" ),
 					QMessageBox::Ok );
+	}
 }
 
 
@@ -494,10 +703,10 @@ void setupDialog::setBufferSize( int _value )
 	}
 
 	m_bufferSize = _value * 64;
-	m_bufSizeLbl->setText( tr( "FRAMES: %1\nLATENCY: %2 ms" ).arg(
+	m_bufSizeLbl->setText( tr( "Frames: %1\nLatency: %2 ms" ).arg(
 					m_bufferSize ).arg(
 						1000.0f * m_bufferSize /
-						mixer::inst()->sampleRate(),
+						eng()->getMixer()->sampleRate(),
 						0, 'f', 1 ) );
 }
 
@@ -531,6 +740,182 @@ void setupDialog::displayBufSizeHelp( void )
 
 
 
+void setupDialog::toggleToolTips( bool _disabled )
+{
+	m_disableToolTips = _disabled;
+}
+
+
+
+
+void setupDialog::toggleKnobUsability( bool _classical )
+{
+	m_classicalKnobUsability = _classical;
+}
+
+
+
+
+void setupDialog::toggleGIMPLikeWindows( bool _enabled )
+{
+	m_gimpLikeWindows = _enabled;
+}
+
+
+
+
+void setupDialog::toggleNoWizard( bool _enabled )
+{
+	m_noWizard = _enabled;
+}
+
+
+
+
+void setupDialog::toggleNoMsgAfterSetup( bool _enabled )
+{
+	m_noMsgAfterSetup = _enabled;
+}
+
+
+
+
+void setupDialog::toggleDisplaydBV( bool _enabled )
+{
+	m_displaydBV = _enabled;
+}
+
+
+
+
+void setupDialog::toggleDisableChActInd( bool _disabled )
+{
+	m_disableChActInd = _disabled;
+}
+
+
+
+
+void setupDialog::toggleManualChPiano( bool _enabled )
+{
+	m_manualChPiano = _enabled;
+}
+
+
+
+
+void setupDialog::openWorkingDir( void )
+{
+#ifdef QT4
+	QString new_dir = QFileDialog::getExistingDirectory( this,
+					tr( "Choose LMMS working directory" ),
+							m_workingDir );
+#else
+	QString new_dir = QFileDialog::getExistingDirectory( m_workingDir,
+									0, 0,
+					tr( "Choose LMMS working directory" ),
+									TRUE );
+#endif
+	if( new_dir != QString::null )
+	{
+		m_wdLineEdit->setText( new_dir );
+	}
+}
+
+
+
+
+void setupDialog::setWorkingDir( const QString & _wd )
+{
+	m_workingDir = _wd;
+}
+
+
+
+
+void setupDialog::openVSTDir( void )
+{
+#ifdef QT4
+	QString new_dir = QFileDialog::getExistingDirectory( this,
+				tr( "Choose your VST-plugin directory" ),
+							m_vstDir );
+#else
+	QString new_dir = QFileDialog::getExistingDirectory( m_vstDir, 0, 0,
+				tr( "Choose your VST-plugin directory" ),
+									TRUE );
+#endif
+	if( new_dir != QString::null )
+	{
+		m_vdLineEdit->setText( new_dir );
+	}
+}
+
+
+
+
+void setupDialog::setVSTDir( const QString & _vd )
+{
+	m_vstDir = _vd;
+}
+
+
+
+
+void setupDialog::openArtworkDir( void )
+{
+#ifdef QT4
+	QString new_dir = QFileDialog::getExistingDirectory( this,
+				tr( "Choose artwork-theme directory" ),
+							m_artworkDir );
+#else
+	QString new_dir = QFileDialog::getExistingDirectory( m_artworkDir,
+									0, 0,
+				tr( "Choose artwork-theme directory" ), TRUE );
+#endif
+	if( new_dir != QString::null )
+	{
+		m_adLineEdit->setText( new_dir );
+	}
+}
+
+
+
+
+void setupDialog::setArtworkDir( const QString & _ad )
+{
+	m_artworkDir = _ad;
+}
+
+
+
+
+void setupDialog::openFLDir( void )
+{
+#ifdef QT4
+	QString new_dir = QFileDialog::getExistingDirectory( this,
+				tr( "Choose FL Studio installation directory" ),
+							m_flDir );
+#else
+	QString new_dir = QFileDialog::getExistingDirectory( m_flDir, 0, 0,
+			tr( "Choose FL Studio installation directory" ), TRUE );
+#endif
+	if( new_dir != QString::null )
+	{
+		m_fdLineEdit->setText( new_dir );
+	}
+}
+
+
+
+
+void setupDialog::setFLDir( const QString & _fd )
+{
+	m_flDir = _fd;
+}
+
+
+
+
 void setupDialog::audioInterfaceChanged( const QString & _iface )
 {
 	for( aswMap::iterator it = m_audioIfaceSetupWidgets.begin();
@@ -607,28 +992,7 @@ void setupDialog::displayMIDIHelp( void )
 
 
 
-void setupDialog::toggleToolTips( bool _disabled )
-{
-	m_disableToolTips = _disabled;
-}
-
-
-
-
-void setupDialog::toggleKnobUsability( bool _classical )
-{
-	m_classicalKnobUsability = _classical;
-}
-
-
-
-
-void setupDialog::toggleGIMPLikeWindows( bool _enabled )
-{
-	m_gimpLikeWindows =  _enabled;
-}
-
-
-
 #include "setup_dialog.moc"
 
+
+#endif
