@@ -1,7 +1,9 @@
+#ifndef SINGLE_SOURCE_COMPILE
+
 /*
  * song_editor.cpp - basic window for editing song
  *
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -35,18 +37,17 @@
 #ifdef QT4
 
 #include <Qt/QtXml>
-#include <QApplication>
-#include <QFile>
-#include <QMessageBox>
-#include <QFileInfo>
-#include <QFileDialog>
-#include <QSlider>
-#include <QKeyEvent>
-#include <QLabel>
-#include <QStatusBar>
-#include <QAction>
-#include <QComboBox>
-#include <QLayout>
+#include <QtCore/QFile>
+#include <QtCore/QFileInfo>
+#include <QtGui/QAction>
+#include <QtGui/QApplication>
+#include <QtGui/QButtonGroup>
+#include <QtGui/QFileDialog>
+#include <QtGui/QKeyEvent>
+#include <QtGui/QLabel>
+#include <QtGui/QLayout>
+#include <QtGui/QMessageBox>
+#include <QtGui/QStatusBar>
 
 #else
 
@@ -56,23 +57,26 @@
 #include <qfiledialog.h>
 #include <qfileinfo.h>
 #include <qdom.h>
-#include <qslider.h>
 #include <qlabel.h>
 #include <qstatusbar.h>
-#include <qcombobox.h>
 #include <qlayout.h>
+#include <qbuttongroup.h>
+
+#define addButton insert
+#define setCheckable setToggleButton
 
 #endif
 
 
 #include "song_editor.h"
+#include "automatable_slider.h"
 #include "bb_editor.h"
 #include "rename_dialog.h"
 #include "embed.h"
 #include "templates.h"
 #include "export_project_dialog.h"
 #include "bb_track.h"
-#include "channel_track.h"
+#include "instrument_track.h"
 #include "mmp.h"
 #include "midi_client.h"
 #include "timeline.h"
@@ -82,28 +86,23 @@
 #include "visualization_widget.h"
 #include "project_notes.h"
 #include "config_mgr.h"
-#include "midi_file.h"
 #include "lcd_spinbox.h"
 #include "tooltip.h"
 #include "tool_button.h"
 #include "cpuload_widget.h"
 #include "text_float.h"
+#include "combobox.h"
+#include "main_window.h"
+#include "import_filter.h"
+#include "project_journal.h"
 
 #include "debug.h"
 
 
 
-extern QString file_to_load;
 
-
-const int SCROLLBAR_SIZE = 16;
-
-
-songEditor * songEditor::s_instanceOfMe = NULL;
-
-
-songEditor::songEditor() :
-	trackContainer(),
+songEditor::songEditor( engine * _engine ) :
+	trackContainer( _engine ),
 	m_fileName( "" ),
 	m_oldFileName( "" ),
 	m_exporting( FALSE ),
@@ -114,22 +113,22 @@ songEditor::songEditor() :
 	m_trackToPlay( NULL ),
 	m_patternToPlay( NULL ),
 	m_loopPattern( FALSE ),
-	m_scrollBack( FALSE )
+	m_scrollBack( FALSE ),
+	m_destroyed( FALSE )
 {
-	// hack, because code called out of this function uses
-	// songEditor::inst(), which assigns s_instanceOfMe this function
-	// returns...
-	s_instanceOfMe = this;
-
 	setWindowTitle( tr( "Song-Editor" ) );
 	setWindowIcon( embed::getIconPixmap( "songeditor" ) );
-	if( lmmsMainWin::inst()->workspace() != NULL )
+
+	QWidget * w = ( parentWidget() != NULL ) ? parentWidget() : this;
+	if( eng()->getMainWindow()->workspace() != NULL )
 	{
-		setGeometry( 10, 10, 680, 300 );
+		resize( 680, 300 );
+		w->move( 10, 10 );
 	}
 	else
 	{
-		setGeometry( 210, 10, 580, 300 );
+		resize( 580, 300 );
+		w->move( 210, 10 );
 	}
 
 #ifdef QT4
@@ -147,23 +146,25 @@ songEditor::songEditor() :
 	timeLine * tl = new timeLine( TRACK_OP_WIDTH +
 					DEFAULT_SETTINGS_WIDGET_WIDTH, 32,
 					pixelsPerTact(), m_playPos[PLAY_SONG],
-					m_currentPosition, cw );
+					m_currentPosition, cw, eng() );
 	connect( this, SIGNAL( positionChanged( const midiTime & ) ),
 				m_playPos[PLAY_SONG].m_timeLine,
 			SLOT( updatePosition( const midiTime & ) ) );
 	connect( tl, SIGNAL( positionChanged( const midiTime & ) ),
 			this, SLOT( updatePosition( const midiTime & ) ) );
 
+	m_automation_track = track::create( track::AUTOMATION_TRACK, this );
 
 	// add some essential widgets to global tool-bar 
-	QWidget * tb = lmmsMainWin::inst()->toolBar();
+	QWidget * tb = eng()->getMainWindow()->toolBar();
 
-	lmmsMainWin::inst()->addSpacingToToolBar( 10 );
+	eng()->getMainWindow()->addSpacingToToolBar( 10 );
 
-	m_bpmSpinBox = new lcdSpinBox( MIN_BPM, MAX_BPM, 3, tb );
+	m_bpmSpinBox = new lcdSpinBox( MIN_BPM, MAX_BPM, 3, tb, tr( "Tempo" ),
+						eng(), m_automation_track );
 	m_bpmSpinBox->setLabel( tr( "TEMPO/BPM" ) );
 	connect( m_bpmSpinBox, SIGNAL( valueChanged( int ) ), this,
-							SLOT( setBPM( int ) ) );
+						SLOT( setTempo( int ) ) );
 	toolTip::add( m_bpmSpinBox, tr( "tempo of song" ) );
 
 #ifdef QT4
@@ -178,45 +179,56 @@ songEditor::songEditor() :
 			"should be played within a minute (or how many tacts "
 			"should be played within four minutes)." ) );
 
-	int col = lmmsMainWin::inst()->addWidgetToToolBar( m_bpmSpinBox, 0 );
+	int col = eng()->getMainWindow()->addWidgetToToolBar( m_bpmSpinBox, 0 );
 
 
 	toolButton * hq_btn = new toolButton( embed::getIconPixmap( "hq_mode" ),
 						tr( "High quality mode" ),
 						NULL, NULL, tb );
 	hq_btn->setCheckable( TRUE );
-	connect( hq_btn, SIGNAL( toggled( bool ) ), mixer::inst(),
+	connect( hq_btn, SIGNAL( toggled( bool ) ), eng()->getMixer(),
 					SLOT( setHighQuality( bool ) ) );
 	hq_btn->setFixedWidth( 42 );
-	lmmsMainWin::inst()->addWidgetToToolBar( hq_btn, 1, col );
+	eng()->getMainWindow()->addWidgetToToolBar( hq_btn, 1, col );
 
 
-	lmmsMainWin::inst()->addSpacingToToolBar( 10 );
+	toolButton * cp_btn = new toolButton( embed::getIconPixmap( "auto_limit" ),
+					      tr( "Auto limiter" ),
+					      NULL, NULL, tb );
+	cp_btn->setCheckable( TRUE );
+	connect( cp_btn, SIGNAL( toggled( bool ) ), eng()->getMixer(),
+		 SLOT( setClipScaling( bool ) ) );
+	cp_btn->setFixedWidth( 30 );
+	eng()->getMainWindow()->addWidgetToToolBar( cp_btn, 1, col + 1 );
+
+
+	eng()->getMainWindow()->addSpacingToToolBar( 10 );
 
 
 
 	QLabel * master_vol_lbl = new QLabel( tb );
 	master_vol_lbl->setPixmap( embed::getIconPixmap( "master_volume" ) );
 
-#ifdef QT4
-	m_masterVolumeSlider = new QSlider( Qt::Vertical, tb );
+	m_masterVolumeSlider = new automatableSlider( tb, tr( "Master volume" ),
+						eng(), m_automation_track );
+	m_masterVolumeSlider->setOrientation( Vertical );
 	m_masterVolumeSlider->setRange( 0, 200 );
 	m_masterVolumeSlider->setPageStep( 1 );
-	m_masterVolumeSlider->setValue( 100 );
+	m_masterVolumeSlider->setInitValue( 100 );
+#ifdef QT4
 	m_masterVolumeSlider->setTickPosition( QSlider::TicksLeft );
 #else
-	m_masterVolumeSlider = new QSlider( 0, 200, 1, 100, Qt::Vertical, tb );
 	m_masterVolumeSlider->setTickPosition( QSlider::Left );
 #endif
 	m_masterVolumeSlider->setFixedSize( 26, 60 );
 	m_masterVolumeSlider->setTickInterval( 50 );
 	toolTip::add( m_masterVolumeSlider, tr( "master volume" ) );
 
-	connect( m_masterVolumeSlider, SIGNAL( valueChanged( int ) ), this,
+	connect( m_masterVolumeSlider, SIGNAL( logicValueChanged( int ) ), this,
 			SLOT( masterVolumeChanged( int ) ) );
 	connect( m_masterVolumeSlider, SIGNAL( sliderPressed() ), this,
 			SLOT( masterVolumePressed() ) );
-	connect( m_masterVolumeSlider, SIGNAL( sliderMoved( int ) ), this,
+	connect( m_masterVolumeSlider, SIGNAL( logicSliderMoved( int ) ), this,
 			SLOT( masterVolumeMoved( int ) ) );
 	connect( m_masterVolumeSlider, SIGNAL( sliderReleased() ), this,
 			SLOT( masterVolumeReleased() ) );
@@ -225,33 +237,35 @@ songEditor::songEditor() :
 	m_mvsStatus->setTitle( tr( "Master volume" ) );
 	m_mvsStatus->setPixmap( embed::getIconPixmap( "master_volume" ) );
 
-	lmmsMainWin::inst()->addWidgetToToolBar( master_vol_lbl );
-	lmmsMainWin::inst()->addWidgetToToolBar( m_masterVolumeSlider );
+	eng()->getMainWindow()->addWidgetToToolBar( master_vol_lbl );
+	eng()->getMainWindow()->addWidgetToToolBar( m_masterVolumeSlider );
 
 
-	lmmsMainWin::inst()->addSpacingToToolBar( 10 );
+	eng()->getMainWindow()->addSpacingToToolBar( 10 );
 
 	QLabel * master_pitch_lbl = new QLabel( tb );
 	master_pitch_lbl->setPixmap( embed::getIconPixmap( "master_pitch" ) );
+	master_pitch_lbl->setFixedHeight( 64 );
 
-#ifdef QT4
-	m_masterPitchSlider = new QSlider( Qt::Vertical, tb );
+	m_masterPitchSlider = new automatableSlider( tb, tr( "Master pitch" ),
+						eng(), m_automation_track );
+	m_masterPitchSlider->setOrientation( Vertical );
 	m_masterPitchSlider->setRange( -12, 12 );
 	m_masterPitchSlider->setPageStep( 1 );
-	m_masterPitchSlider->setValue( 0 );
+	m_masterPitchSlider->setInitValue( 0 );
+#ifdef QT4
 	m_masterPitchSlider->setTickPosition( QSlider::TicksLeft );
 #else
-	m_masterPitchSlider = new QSlider( -12, 12, 1, 0, Qt::Vertical, tb );
 	m_masterPitchSlider->setTickPosition( QSlider::Left );
 #endif
 	m_masterPitchSlider->setFixedSize( 26, 60 );
 	m_masterPitchSlider->setTickInterval( 12 );
 	toolTip::add( m_masterPitchSlider, tr( "master pitch" ) );
-	connect( m_masterPitchSlider, SIGNAL( valueChanged( int ) ), this,
+	connect( m_masterPitchSlider, SIGNAL( logicValueChanged( int ) ), this,
 			SLOT( masterPitchChanged( int ) ) );
 	connect( m_masterPitchSlider, SIGNAL( sliderPressed() ), this,
 			SLOT( masterPitchPressed() ) );
-	connect( m_masterPitchSlider, SIGNAL (sliderMoved( int) ), this,
+	connect( m_masterPitchSlider, SIGNAL( logicSliderMoved( int ) ), this,
 			SLOT( masterPitchMoved( int ) ) );
 	connect( m_masterPitchSlider, SIGNAL( sliderReleased() ), this,
 			SLOT( masterPitchReleased() ) );
@@ -260,10 +274,10 @@ songEditor::songEditor() :
 	m_mpsStatus->setTitle( tr( "Master pitch" ) );
 	m_mpsStatus->setPixmap( embed::getIconPixmap( "master_pitch" ) );
 
-	lmmsMainWin::inst()->addWidgetToToolBar( master_pitch_lbl );
-	lmmsMainWin::inst()->addWidgetToToolBar( m_masterPitchSlider );
+	eng()->getMainWindow()->addWidgetToToolBar( master_pitch_lbl );
+	eng()->getMainWindow()->addWidgetToToolBar( m_masterPitchSlider );
 
-	lmmsMainWin::inst()->addSpacingToToolBar( 10 );
+	eng()->getMainWindow()->addSpacingToToolBar( 10 );
 
 	// create widget for visualization- and cpu-load-widget
 	QWidget * vc_w = new QWidget( tb );
@@ -271,14 +285,14 @@ songEditor::songEditor() :
 	vcw_layout->setMargin( 0 );
 	vcw_layout->setSpacing( 0 );
 
-	vcw_layout->addStretch();
+	//vcw_layout->addStretch();
 	vcw_layout->addWidget( new visualizationWidget(
-			embed::getIconPixmap( "output_graph" ), vc_w ) ); 
+			embed::getIconPixmap( "output_graph" ), vc_w, eng() ) );
 
-	vcw_layout->addWidget( new cpuloadWidget( vc_w ) );
+	vcw_layout->addWidget( new cpuloadWidget( vc_w, eng() ) );
 	vcw_layout->addStretch();
 
-	lmmsMainWin::inst()->addWidgetToToolBar( vc_w );
+	eng()->getMainWindow()->addWidgetToToolBar( vc_w );
 
 
 	// create own toolbar
@@ -286,9 +300,10 @@ songEditor::songEditor() :
 	m_toolBar->setFixedHeight( 32 );
 	m_toolBar->move( 0, 0 );
 #ifdef QT4
+	m_toolBar->setAutoFillBackground( TRUE );
 	QPalette pal;
-	pal.setBrush( m_toolBar->backgroundRole(), QBrush(
-				embed::getIconPixmap( "toolbar_bg" ) ) );
+	pal.setBrush( m_toolBar->backgroundRole(), 
+				embed::getIconPixmap( "toolbar_bg" ) );
 	m_toolBar->setPalette( pal );
 #else
 	m_toolBar->setPaletteBackgroundPixmap( embed::getIconPixmap(
@@ -329,18 +344,28 @@ songEditor::songEditor() :
 					this, SLOT( addSampleTrack() ),
 					m_toolBar );
 
-	m_insertBarButton = new toolButton( embed::getIconPixmap(
-						"insert_bar" ),
-						tr( "Insert bar "
-							"(Shift+Insert)" ),
-						this, SLOT( insertBar() ),
-						m_toolBar );
+	m_drawModeButton = new toolButton( embed::getIconPixmap(
+								"edit_draw" ),
+							tr( "Draw mode" ),
+							NULL, NULL, m_toolBar );
+	m_drawModeButton->setCheckable( TRUE );
+	m_drawModeButton->setChecked( TRUE );
+
+	m_editModeButton = new toolButton( embed::getIconPixmap(
+								"edit_arrow" ),
+					tr( "Edit mode (select and move)" ),
+							NULL, NULL, m_toolBar );
+	m_editModeButton->setCheckable( TRUE );
+
+	QButtonGroup * tool_button_group = new QButtonGroup( this );
+	tool_button_group->addButton( m_drawModeButton );
+	tool_button_group->addButton( m_editModeButton );
+	tool_button_group->setExclusive( TRUE );
+#ifndef QT4
+	tool_button_group->hide();
+#endif
+
 
-	m_removeBarButton = new toolButton( embed::getIconPixmap(
-						"remove_bar" ),
-					tr( "Remove bar (Shift+Delete)" ),
-						this, SLOT( removeBar() ),
-						m_toolBar );
 #ifdef QT4
 #else
 	QWhatsThis::add( m_playButton, tr( "Click here, if you want to play "
@@ -354,13 +379,13 @@ songEditor::songEditor() :
 						"song-position-marker will be "
 						"set to the start of your song."
 			) );
-	QWhatsThis::add( m_insertBarButton, tr( "If you click here, a "
+/*	QWhatsThis::add( m_insertBarButton, tr( "If you click here, a "
 							"bar will "
 							"be inserted at the "
 							"current bar." ) );
 	QWhatsThis::add( m_removeBarButton, tr( "If you click here, the "
 							"current bar will be "
-							"removed." ) );
+							"removed." ) );*/
 #endif
 
 
@@ -368,20 +393,15 @@ songEditor::songEditor() :
 	zoom_lbl->setPixmap( embed::getIconPixmap( "zoom" ) );
 
 	// setup zooming-stuff
-	m_zoomingComboBox = new QComboBox( m_toolBar );
-	m_zoomingComboBox->setGeometry( 580, 4, 80, 24 );
+	m_zoomingComboBox = new comboBox( m_toolBar, NULL, eng(), NULL );
+	m_zoomingComboBox->setFixedSize( 80, 22 );
+	m_zoomingComboBox->move( 580, 4 );
 	for( int i = 0; i < 7; ++i )
 	{
-		m_zoomingComboBox->addItem( QString::number( 25 *
-					static_cast<int>( powf( 2.0f, i ) ) ) +
-									"%" );
+		m_zoomingComboBox->addItem( QString::number( 25 << i ) + "%" );
 	}
-#ifdef QT4
-	m_zoomingComboBox->setCurrentIndex( m_zoomingComboBox->findText(
+	m_zoomingComboBox->setInitValue( m_zoomingComboBox->findText(
 								"100%" ) );
-#else
-	m_zoomingComboBox->setCurrentText( "100%" );
-#endif
 	connect( m_zoomingComboBox, SIGNAL( activated( const QString & ) ),
 			this, SLOT( zoomingChanged( const QString & ) ) );
 
@@ -393,8 +413,8 @@ songEditor::songEditor() :
 	tb_layout->addWidget( m_addBBTrackButton );
 	tb_layout->addWidget( m_addSampleTrackButton );
 	tb_layout->addSpacing( 10 );
-	tb_layout->addWidget( m_insertBarButton );
-	tb_layout->addWidget( m_removeBarButton );
+	tb_layout->addWidget( m_drawModeButton );
+	tb_layout->addWidget( m_editModeButton );
 	tb_layout->addSpacing( 10 );
 	tl->addToolButtons( m_toolBar );
 	tb_layout->addSpacing( 15 );
@@ -407,41 +427,18 @@ songEditor::songEditor() :
 	m_leftRightScroll = new QScrollBar( Qt::Horizontal, cw );
 	m_leftRightScroll->setMinimum( 0 );
 	m_leftRightScroll->setMaximum( 0 );
-#ifdef QT4
+#ifndef QT3
 	m_leftRightScroll->setSingleStep( 1 );
 	m_leftRightScroll->setPageStep( 20 );
 #else
 	m_leftRightScroll->setSteps( 1, 20 );
 #endif
-	connect( m_leftRightScroll, SIGNAL( valueChanged( int ) ), this,
-			SLOT( scrolled( int ) ) );
-
+	connect( m_leftRightScroll, SIGNAL( valueChanged( int ) ),
+					this, SLOT( scrolled( int ) ) );
 
 
 	show();
 
-	m_projectNotes = new projectNotes();
-	m_projectNotes->resize( 300, 200 );
-	if( lmmsMainWin::inst()->workspace() != NULL )
-	{
-		m_projectNotes->move( 700, 10 );
-	}
-	else
-	{
-		m_projectNotes->move( 800, 10 );
-	}
-	m_projectNotes->show();
-
-
-	// we try to load given file
-	if( file_to_load != "" )
-	{
-		loadProject( file_to_load );
-	}
-	else
-	{
-		createNewProject();
-	}
 }
 
 
@@ -449,6 +446,8 @@ songEditor::songEditor() :
 
 songEditor::~songEditor()
 {
+	delete m_automation_track;
+	m_destroyed = TRUE;
 }
 
 
@@ -472,17 +471,31 @@ void songEditor::paintEvent( QPaintEvent * _pe )
 
 
 
+QRect songEditor::scrollAreaRect( void ) const
+{
+	if( centralWidget() == NULL )
+	{
+		return( rect() );
+	}
+	return( QRect( 0, 0, centralWidget()->width(),
+			centralWidget()->height() - m_toolBar->height() -
+			m_playPos[PLAY_SONG].m_timeLine->height() -
+			DEFAULT_SCROLLBAR_SIZE ) );
+}
+
+
+
+
 // responsible for moving scrollbars after resizing
 void songEditor::resizeEvent( QResizeEvent * _re )
 {
 	if( centralWidget() != NULL )
 	{
 		m_leftRightScroll->setGeometry( 0,
-					centralWidget()->height() - 2 -
-								SCROLLBAR_SIZE,
-					centralWidget()->width() -
-								SCROLLBAR_SIZE,
-					SCROLLBAR_SIZE );
+					centralWidget()->height() -
+							DEFAULT_SCROLLBAR_SIZE,
+					centralWidget()->width(),
+					DEFAULT_SCROLLBAR_SIZE );
 
 		m_playPos[PLAY_SONG].m_timeLine->setFixedWidth(
 						centralWidget()->width() );
@@ -497,13 +510,13 @@ void songEditor::resizeEvent( QResizeEvent * _re )
 void songEditor::keyPressEvent( QKeyEvent * _ke )
 {
 	if( /*_ke->modifiers() & Qt::ShiftModifier*/
-		lmmsMainWin::isShiftPressed() == TRUE &&
+		eng()->getMainWindow()->isShiftPressed() == TRUE &&
 						_ke->key() == Qt::Key_Insert )
 	{
 		insertBar();
 	}
 	else if(/* _ke->modifiers() & Qt::ShiftModifier &&*/
-			lmmsMainWin::isShiftPressed() == TRUE &&
+			eng()->getMainWindow()->isShiftPressed() == TRUE &&
 						_ke->key() == Qt::Key_Delete )
 	{
 		removeBar();
@@ -564,7 +577,7 @@ void songEditor::scrolled( int _new_pos )
 
 void songEditor::wheelEvent( QWheelEvent * _we )
 {
-	if( lmmsMainWin::isCtrlPressed() == TRUE )
+	if( eng()->getMainWindow()->isCtrlPressed() == TRUE )
 	{
 		if( _we->delta() > 0 )
 		{
@@ -575,25 +588,18 @@ void songEditor::wheelEvent( QWheelEvent * _we )
 		{
 			setPixelsPerTact( (int) pixelsPerTact() / 2 );
 		}
-#ifdef QT4
 		// update combobox with zooming-factor
-		m_zoomingComboBox->setCurrentIndex(
+		m_zoomingComboBox->setValue(
 				m_zoomingComboBox->findText( QString::number(
 					static_cast<int>( pixelsPerTact() *
 				100 / DEFAULT_PIXELS_PER_TACT ) ) + "%" ) );
-#else
-		// update combobox with zooming-factor
-		m_zoomingComboBox->setCurrentText( QString::number(
-					static_cast<int>( pixelsPerTact() *
-				100 / DEFAULT_PIXELS_PER_TACT ) ) + "%" );
-#endif
 		// update timeline
 		m_playPos[PLAY_SONG].m_timeLine->setPixelsPerTact(
 							pixelsPerTact() );
 		// and make sure, all TCO's are resized and relocated
 		realignTracks( TRUE );
 	} 
-	else if( lmmsMainWin::isShiftPressed() == TRUE )
+	else if( eng()->getMainWindow()->isShiftPressed() == TRUE )
 	{
 		m_leftRightScroll->setValue( m_leftRightScroll->value() -
 							_we->delta() / 30 );
@@ -612,7 +618,8 @@ void songEditor::wheelEvent( QWheelEvent * _we )
 void songEditor::masterVolumeChanged( int _new_val )
 {
 	masterVolumeMoved( _new_val );
-	if( m_mvsStatus->isVisible() == FALSE && m_loadingProject == FALSE )
+	if( m_mvsStatus->isVisible() == FALSE && m_loadingProject == FALSE
+					&& m_masterVolumeSlider->showStatus() )
 	{
 		m_mvsStatus->reparent( m_masterVolumeSlider );
 		m_mvsStatus->move( m_masterVolumeSlider->mapTo(
@@ -621,7 +628,7 @@ void songEditor::masterVolumeChanged( int _new_val )
 			QPoint( m_masterVolumeSlider->width() + 2, -2 ) );
 		m_mvsStatus->setVisibilityTimeOut( 1000 );
 	}
-	mixer::inst()->setMasterGain( 2.0f - _new_val / 100.0f );
+	eng()->getMixer()->setMasterGain( _new_val / 100.0f );
 	setModified();
 }
 
@@ -636,7 +643,7 @@ void songEditor::masterVolumePressed( void )
 							QPoint( 0, 0 ) ) +
 			QPoint( m_masterVolumeSlider->width() + 2, -2 ) );
 	m_mvsStatus->show();
-	masterVolumeMoved( m_masterVolumeSlider->value() );
+	masterVolumeMoved( m_masterVolumeSlider->logicValue() );
 }
 
 
@@ -644,7 +651,7 @@ void songEditor::masterVolumePressed( void )
 
 void songEditor::masterVolumeMoved( int _new_val )
 {
-	m_mvsStatus->setText( tr( "Value: %1%" ).arg( 200 - _new_val ) );
+	m_mvsStatus->setText( tr( "Value: %1%" ).arg( _new_val ) );
 }
 
 
@@ -661,7 +668,8 @@ void songEditor::masterVolumeReleased( void )
 void songEditor::masterPitchChanged( int _new_val )
 {
 	masterPitchMoved( _new_val );
-	if( m_mpsStatus->isVisible() == FALSE && m_loadingProject == FALSE )
+	if( m_mpsStatus->isVisible() == FALSE && m_loadingProject == FALSE
+					&& m_masterPitchSlider->showStatus() )
 	{
 		m_mpsStatus->reparent( m_masterPitchSlider );
 		m_mpsStatus->move( m_masterPitchSlider->mapTo(
@@ -684,7 +692,7 @@ void songEditor::masterPitchPressed( void )
 							QPoint( 0, 0 ) ) +
 			QPoint( m_masterPitchSlider->width() + 2, -2 ) );
 	m_mpsStatus->show();
-	masterPitchMoved( m_masterPitchSlider->value() );
+	masterPitchMoved( m_masterPitchSlider->logicValue() );
 }
 
 
@@ -692,7 +700,7 @@ void songEditor::masterPitchPressed( void )
 
 void songEditor::masterPitchMoved( int _new_val )
 {
-	m_mpsStatus->setText( tr( "Value: %1 semitones").arg( -_new_val ) );
+	m_mpsStatus->setText( tr( "Value: %1 semitones").arg( _new_val ) );
 
 }
 
@@ -742,11 +750,28 @@ void songEditor::zoomingChanged( const QString & _zfac )
 
 
 
-void songEditor::setBPM( int _new_bpm )
+void songEditor::setTempo( int _new_bpm )
 {
-	m_bpmSpinBox->setValue( tLimit( _new_bpm, MIN_BPM, MAX_BPM ) );
+	m_bpmSpinBox->setInitValue(
+				tLimit<bpm_t>( _new_bpm, MIN_BPM, MAX_BPM ) );
 	setModified();
-	emit bpmChanged( _new_bpm );
+	emit tempoChanged( _new_bpm );
+}
+
+
+
+
+void songEditor::setMasterVolume( volume _vol )
+{
+	m_masterVolumeSlider->setInitValue( _vol );
+}
+
+
+
+
+void songEditor::setMasterPitch( int _master_pitch )
+{
+	m_masterPitchSlider->setInitValue( _master_pitch );
 }
 
 
@@ -754,7 +779,7 @@ void songEditor::setBPM( int _new_bpm )
 
 int songEditor::masterPitch( void ) const
 {
-	return( -m_masterPitchSlider->value() );
+	return( m_masterPitchSlider->logicValue() );
 }
 
 
@@ -808,7 +833,7 @@ void songEditor::doActions( void )
 				updateTimeLinePosition();
 
 				// remove all note-play-handles that are active
-				mixer::inst()->clear();
+				eng()->getMixer()->clear();
 
 				break;
 			}
@@ -896,7 +921,7 @@ void songEditor::processNextBuffer( void )
 			// at song-start we have to reset the LFOs
 			if( m_playPos[PLAY_SONG] == 0 )
 			{
-				envelopeAndLFOWidget::resetLFO();
+				envelopeAndLFOWidget::resetLFO( eng() );
 			}
 			break;
 
@@ -905,17 +930,21 @@ void songEditor::processNextBuffer( void )
 			break;
 
 		case PLAY_BB:
-			if( bbEditor::inst()->numOfBBs() > 0 )
+			if( eng()->getBBEditor()->numOfBBs() > 0 )
 			{
-				tco_num = bbEditor::inst()->currentBB();
-				tv.push_back( bbTrack::findBBTrack( tco_num ) );
+				tco_num = eng()->getBBEditor()->currentBB();
+				tv.push_back( bbTrack::findBBTrack( tco_num,
+								eng() ) );
 			}
 			break;
 
 		case PLAY_PATTERN:
-			tco_num = m_patternToPlay->getTrack()->getTCONum(
+			if( m_patternToPlay != NULL )
+			{
+				tco_num = m_patternToPlay->getTrack()->getTCONum(
 							m_patternToPlay );
-			tv.push_back( m_patternToPlay->getTrack() );
+				tv.push_back( m_patternToPlay->getTrack() );
+			}
 			break;
 
 		default:
@@ -946,8 +975,8 @@ void songEditor::processNextBuffer( void )
 		}
 	}
 
-	Uint32 total_frames_played = 0;
-	Uint32 frames_per_tact = static_cast<Uint32>( framesPerTact() );
+	f_cnt_t total_frames_played = 0;
+	f_cnt_t frames_per_tact = static_cast<f_cnt_t>( framesPerTact() );
 	if( m_playPos[m_playMode].currentFrame() == 0 &&
 		m_playPos[m_playMode].getTact64th() > 0 )
 	{
@@ -956,9 +985,9 @@ void songEditor::processNextBuffer( void )
 							frames_per_tact / 64 );
 	}
 
-	while( total_frames_played < mixer::inst()->framesPerAudioBuffer() )
+	while( total_frames_played < eng()->getMixer()->framesPerAudioBuffer() )
 	{
-		Uint32 played_frames = mixer::inst()->framesPerAudioBuffer() -
+		f_cnt_t played_frames = eng()->getMixer()->framesPerAudioBuffer() -
 							total_frames_played;
 
 		// did we play a whole tact?
@@ -973,7 +1002,7 @@ void songEditor::processNextBuffer( void )
 			if( m_playMode == PLAY_BB )
 			{
 				max_tact =
-					bbEditor::inst()->lengthOfCurrentBB();
+					eng()->getBBEditor()->lengthOfCurrentBB();
 			}
 			else if( m_playMode == PLAY_PATTERN &&
 					m_loopPattern == TRUE &&
@@ -998,7 +1027,7 @@ void songEditor::processNextBuffer( void )
 		// or do we have some samples left in this tact but this are 
 		// less then samples we have to play?
 		else if( frames_per_tact - m_playPos[m_playMode].currentFrame()
-				< mixer::inst()->framesPerAudioBuffer() )
+				< eng()->getMixer()->framesPerAudioBuffer() )
 		{
 			// then set played_samples to remaining samples, the 
 			// rest will be played in next loop
@@ -1006,6 +1035,14 @@ void songEditor::processNextBuffer( void )
 					m_playPos[m_playMode].currentFrame();
 		}
 
+		if( m_playMode == PLAY_SONG )
+		{
+			m_automation_track->play( m_playPos[m_playMode],
+					m_playPos[m_playMode].currentFrame(),
+					played_frames, total_frames_played,
+					tco_num );
+		}
+
 		// loop through all tracks and play them if they're not muted
 		for( trackVector::iterator it = tv.begin(); it != tv.end();
 									++it )
@@ -1028,18 +1065,21 @@ void songEditor::processNextBuffer( void )
 						64 / frames_per_tact) % 64 );
 	}
 
-	if( m_playPos[m_playMode].m_timeLine != NULL &&
-		m_playPos[m_playMode].m_timeLineUpdate == TRUE &&
-		m_exporting == FALSE )
+	if( m_exporting == FALSE )
 	{
-		m_playPos[m_playMode].m_timeLine->updatePosition(); 
+		updateTimeLinePosition();
 	}
 }
 
 
 
 
-
+bool songEditor::realTimeTask( void ) const
+{
+	return( !( m_exporting == TRUE || ( m_playMode == PLAY_PATTERN &&
+		  	m_patternToPlay != NULL &&
+			m_patternToPlay->freezing() == TRUE ) ) );
+}
 
 
 
@@ -1051,7 +1091,7 @@ void songEditor::play( void )
 		if( m_playMode != PLAY_SONG )
 		{
 			// make sure, bb-editor updates/resets it play-button
-			bbEditor::inst()->stop();
+			eng()->getBBEditor()->stop();
 			//pianoRoll::inst()->stop();
 		}
 		else
@@ -1105,7 +1145,10 @@ void songEditor::playPattern( pattern * _patternToPlay, bool _loop )
 	}
 	m_patternToPlay = _patternToPlay;
 	m_loopPattern = _loop;
-	m_actions.push_back( ACT_PLAY_PATTERN );
+	if( m_patternToPlay != NULL )
+	{
+		m_actions.push_back( ACT_PLAY_PATTERN );
+	}
 }
 
 
@@ -1131,7 +1174,7 @@ void songEditor::setPlayPos( tact _tact_num, tact64th _t_64th, playModes
 {
 	m_playPos[_play_mode].setTact( _tact_num );
 	m_playPos[_play_mode].setTact64th( _t_64th );
-	m_playPos[_play_mode].setCurrentFrame( static_cast<Uint32>(
+	m_playPos[_play_mode].setCurrentFrame( static_cast<f_cnt_t>(
 					_t_64th * framesPerTact() / 64.0f ) );
 	if( _play_mode == m_playMode )
 	{
@@ -1147,7 +1190,9 @@ void songEditor::updateTimeLinePosition( void )
 	if( m_playPos[m_playMode].m_timeLine != NULL &&
 		m_playPos[m_playMode].m_timeLineUpdate == TRUE )
 	{
-		m_playPos[m_playMode].m_timeLine->updatePosition();
+/*		QTimer::singleShot( 1, m_playPos[m_playMode].m_timeLine,
+						SLOT( updatePosition() ) );*/
+		//m_playPos[m_playMode].m_timeLine->updatePosition();
 	}
 }
 
@@ -1156,6 +1201,10 @@ void songEditor::updateTimeLinePosition( void )
 
 void songEditor::stop( void )
 {
+	if( m_destroyed )
+	{
+		return;
+	}
 	m_actions.push_back( ACT_STOP_PLAY );
 #ifdef QT4
 	m_playButton->setIcon( embed::getIconPixmap( "play" ) );
@@ -1269,14 +1318,17 @@ void songEditor::addSampleTrack( void )
 
 float songEditor::framesPerTact( void ) const
 {
-	return( mixer::inst()->sampleRate() * 60.0f * BEATS_PER_TACT /
-			m_bpmSpinBox->value() );
+	// when fooling around with tempo while playing, we sometimes get
+	// 0 here which leads to FP-exception, so handle it separately
+	const int bpm = tMax( 1, m_bpmSpinBox->value() );
+	return( eng()->getMixer()->sampleRate() * 60.0f * BEATS_PER_TACT /
+								bpm );
 }
 
 
 
 
-int songEditor::getBPM( void )
+bpm_t songEditor::getTempo( void )
 {
 	return( m_bpmSpinBox->value() );
 }
@@ -1297,7 +1349,7 @@ bool songEditor::mayChangeProject( void )
 #else
 		information
 #endif
-				( lmmsMainWin::inst(),
+				( eng()->getMainWindow(),
 						tr( "Project not saved" ),
 						tr( "The current project was "
 							"modified since last "
@@ -1318,12 +1370,12 @@ bool songEditor::mayChangeProject( void )
 				QMessageBox::Yes,
 				QMessageBox::No,
 				QMessageBox::Cancel,
-				lmmsMainWin::inst() );
+				eng()->getMainWindow() );
 	int answer = mb.exec();
 
 	if( answer == QMessageBox::Yes )
 	{
-		return( lmmsMainWin::inst()->saveProject() );
+		return( eng()->getMainWindow()->saveProject() );
 	}
 	else if( answer == QMessageBox::No )
 	{
@@ -1338,6 +1390,8 @@ bool songEditor::mayChangeProject( void )
 
 void songEditor::clearProject( void )
 {
+	eng()->getProjectJournal()->setJournalling( FALSE );
+
 	if( m_playing )
 	{
 		// stop play, because it's dangerous that play-routines try to
@@ -1348,8 +1402,8 @@ void songEditor::clearProject( void )
 
 	// make sure all running notes are cleared, otherwise the whole
 	// thing will end up in a SIGSEGV...
-	//mixer::inst()->clear();
-	while( mixer::inst()->haveNoRunningNotes() == FALSE )
+	eng()->getMixer()->clear( TRUE );
+	while( eng()->getMixer()->haveNoRunningNotes() == FALSE )
 	{
 #ifdef QT4
 		QApplication::processEvents( QEventLoop::AllEvents );
@@ -1358,17 +1412,21 @@ void songEditor::clearProject( void )
 #endif
 	}
 
-	trackVector tv = tracks();
-	for( trackVector::iterator it = tv.begin(); it != tv.end(); ++it )
-	{
-		removeTrack( *it );
-	}
-	tv = bbEditor::inst()->tracks();
-	for( trackVector::iterator it = tv.begin(); it != tv.end(); ++it )
-	{
-		bbEditor::inst()->removeTrack( *it );
-	}
-	m_projectNotes->clear();
+	clearAllTracks();
+
+	eng()->getAutomationEditor()->setCurrentPattern( NULL );
+	m_bpmSpinBox->getAutomationPattern()->clear();
+	m_masterVolumeSlider->clearAutomationValues();
+	m_masterPitchSlider->clearAutomationValues();
+
+	eng()->getBBEditor()->clearAllTracks();
+
+	eng()->getProjectNotes()->clear();
+
+	eng()->getProjectJournal()->clearInvalidJournallingObjects();
+	eng()->getProjectJournal()->clearJournal();
+
+	eng()->getProjectJournal()->setJournalling( TRUE );
 }
 
 
@@ -1378,29 +1436,50 @@ void songEditor::clearProject( void )
 // create new file
 void songEditor::createNewProject( void )
 {
+	QString default_template = configManager::inst()->userProjectsDir()
+						+ "templates/default.mpt";
+	if( QFile::exists( default_template ) )
+	{
+		createNewProjectFromTemplate( default_template );
+		return;
+	}
+
+	default_template = configManager::inst()->factoryProjectsDir()
+						+ "templates/default.mpt";
+	if( QFile::exists( default_template ) )
+	{
+		createNewProjectFromTemplate( default_template );
+		return;
+	}
+
 	clearProject();
 
+	eng()->getProjectJournal()->setJournalling( FALSE );
+
 	track * t;
 	t = track::create( track::CHANNEL_TRACK, this );
-	dynamic_cast< channelTrack * >( t )->loadInstrument(
+	dynamic_cast< instrumentTrack * >( t )->loadInstrument(
 					"tripleoscillator" );
 	track::create( track::SAMPLE_TRACK, this );
-	t = track::create( track::CHANNEL_TRACK, bbEditor::inst() );
-	dynamic_cast< channelTrack * >( t )->loadInstrument(
+	t = track::create( track::CHANNEL_TRACK, eng()->getBBEditor() );
+	dynamic_cast< instrumentTrack * >( t )->loadInstrument(
 						"tripleoscillator" );
 	track::create( track::BB_TRACK, this );
 
 	m_loadingProject = TRUE;
-	setBPM( DEFAULT_BPM );
-	m_masterVolumeSlider->setValue( 100 );
-	m_masterPitchSlider->setValue( 0 );
+	setTempo( DEFAULT_BPM );
+	m_masterVolumeSlider->setInitValue( 100 );
+	m_masterPitchSlider->setInitValue( 0 );
 	m_loadingProject = FALSE;
 
 	m_fileName = m_oldFileName = "";
 
 	m_modified = FALSE;
 
-	lmmsMainWin::inst()->resetWindowTitle( "" );
+	eng()->getMainWindow()->resetWindowTitle( "" );
+
+	eng()->getProjectJournal()->setJournalling( TRUE );
+
 }
 
 
@@ -1421,8 +1500,10 @@ void FASTCALL songEditor::createNewProjectFromTemplate( const QString &
 // load given song
 void FASTCALL songEditor::loadProject( const QString & _file_name )
 {
-
 	clearProject();
+
+	eng()->getProjectJournal()->setJournalling( FALSE );
+
 	m_fileName = _file_name;
 	m_oldFileName = _file_name;
 
@@ -1446,7 +1527,7 @@ void FASTCALL songEditor::loadProject( const QString & _file_name )
 			if( node.nodeName() == "bpm" &&
 		node.toElement().attribute( "value" ).toInt() > 0 )
 			{
-				setBPM( node.toElement().attribute( "value"
+				setTempo( node.toElement().attribute( "value"
 								).toInt() );
 			}
 			else if( node.nodeName() == "mastervol" )
@@ -1454,21 +1535,31 @@ void FASTCALL songEditor::loadProject( const QString & _file_name )
 				if( node.toElement().attribute( "value"
 								).toInt() > 0 )
 				{
-					m_masterVolumeSlider->setValue( 200 -
-				node.toElement().attribute( "value" ).toInt() );
+					m_masterVolumeSlider->setInitValue(
+						node.toElement().attribute(
+							"value" ).toInt() );
 				}
 				else
 				{
-					m_masterVolumeSlider->setValue(
+					m_masterVolumeSlider->setInitValue(
 							DEFAULT_VOLUME );
 				}
 			}
 			else if( node.nodeName() == "masterpitch" )
 			{
-				m_masterPitchSlider->setValue(
-					node.toElement().attribute( "value"
+				m_masterPitchSlider->setInitValue(
+					-node.toElement().attribute( "value"
 								).toInt() );
 			}
+			else if( node.nodeName()
+					== automationPattern::classNodeName() )
+			{
+				m_bpmSpinBox->loadSettings( mmp.head(), "bpm" );
+				m_masterVolumeSlider->loadSettings( mmp.head(),
+								"mastervol" );
+				m_masterPitchSlider->loadSettings( mmp.head(),
+								"masterpitch" );
+			}
 		}
 		node = node.nextSibling();
 	}
@@ -1480,11 +1571,32 @@ void FASTCALL songEditor::loadProject( const QString & _file_name )
 		{
 			if( node.nodeName() == "trackcontainer" )
 			{
-				loadSettings( node.toElement() );
+				( (journallingObject *)( this ) )->
+					restoreState( node.toElement() );
 			}
-			else if( node.nodeName() == m_projectNotes->nodeName() )
+			else if( node.nodeName() ==
+					eng()->getPianoRoll()->nodeName() )
 			{
-				m_projectNotes->loadSettings(
+				eng()->getPianoRoll()->restoreState(
+							node.toElement() );
+			}
+			else if( node.nodeName() ==
+				eng()->getAutomationEditor()->nodeName() )
+			{
+				eng()->getAutomationEditor()->restoreState(
+							node.toElement() );
+			}
+			else if( node.nodeName() ==
+					eng()->getProjectNotes()->nodeName() )
+			{
+				( (journallingObject *)( eng()->
+							getProjectNotes() ) )->
+					restoreState( node.toElement() );
+			}
+			else if( node.nodeName() ==
+				m_playPos[PLAY_SONG].m_timeLine->nodeName() )
+			{
+				m_playPos[PLAY_SONG].m_timeLine->restoreState(
 							node.toElement() );
 			}
 		}
@@ -1496,7 +1608,9 @@ void FASTCALL songEditor::loadProject( const QString & _file_name )
 
 	m_loadingProject = FALSE;
 
-	lmmsMainWin::inst()->resetWindowTitle( "" );
+	eng()->getMainWindow()->resetWindowTitle( "" );
+
+	eng()->getProjectJournal()->setJournalling( TRUE );
 }
 
 
@@ -1507,23 +1621,18 @@ bool songEditor::saveProject( void )
 {
 	multimediaProject mmp( multimediaProject::SONG_PROJECT );
 
-	QDomElement bpm = mmp.createElement( "bpm" );
-	bpm.setAttribute( "value", QString::number( m_bpmSpinBox->value() ) );
-	mmp.head().appendChild( bpm );
-
-	QDomElement mv = mmp.createElement( "mastervol" );
-	mv.setAttribute( "value", QString::number( 200 -
-					m_masterVolumeSlider->value() ) );
-	mmp.head().appendChild( mv );
-
-	QDomElement mp = mmp.createElement( "masterpitch" );
-	mp.setAttribute( "value", QString::number(
-					m_masterPitchSlider->value() ) );
-	mmp.head().appendChild( mp );
+	m_bpmSpinBox->saveSettings( mmp, mmp.head(), "bpm" );
+	m_masterVolumeSlider->saveSettings( mmp, mmp.head(), "mastervol" );
+	m_masterPitchSlider->saveSettings( mmp, mmp.head(), "masterpitch" );
 
 
-	saveSettings( mmp, mmp.content() );
-	m_projectNotes->saveSettings( mmp, mmp.content() );
+	( (journallingObject *)( this ) )->saveState( mmp, mmp.content() );
+
+	eng()->getPianoRoll()->saveState( mmp, mmp.content() );
+	eng()->getAutomationEditor()->saveState( mmp, mmp.content() );
+	( (journallingObject *)( eng()->getProjectNotes() ) )->saveState( mmp,
+								mmp.content() );
+	m_playPos[PLAY_SONG].m_timeLine->saveState( mmp, mmp.content() );
 
 	if( mmp.writeFile( m_fileName, m_oldFileName == "" ||
 					m_fileName != m_oldFileName ) == TRUE )
@@ -1535,7 +1644,7 @@ bool songEditor::saveProject( void )
 							).arg( m_fileName ),
 				embed::getIconPixmap( "project_save", 24, 24 ),
 									2000 );
-		lmmsMainWin::inst()->resetWindowTitle( "" );
+		eng()->getMainWindow()->resetWindowTitle( "" );
 	}
 	else
 	{
@@ -1573,19 +1682,18 @@ bool FASTCALL songEditor::saveProjectAs( const QString & _file_name )
 void songEditor::importProject( void )
 {
 #ifdef QT4
-	QFileDialog ofd( this, tr( "Import file" ), "",
-					tr( "MIDI-files (*.mid)" ) );
+	QFileDialog ofd( this, tr( "Import file" ), ""/*,
+					tr( "MIDI-files (*.mid)" )*/ );
 #else
-	QFileDialog ofd( QString::null, tr( "MIDI-files (*.mid)" ),
+	QFileDialog ofd( QString::null,/* tr( "MIDI-files (*.mid)" )*/ QString::null,
 							this, "", TRUE );
 	ofd.setWindowTitle( tr( "Import file" ) );
 #endif
-	ofd.setDirectory( configManager::inst()->projectsDir() );
+	ofd.setDirectory( configManager::inst()->userProjectsDir() );
 	ofd.setFileMode( QFileDialog::ExistingFiles );
 	if( ofd.exec () == QDialog::Accepted && !ofd.selectedFiles().isEmpty() )
 	{
-		midiFile mf( ofd.selectedFiles()[0] );
-		mf.importToTrackContainer( this );
+		importFilter::import( ofd.selectedFiles()[0], this );
 	}
 }
 
@@ -1606,7 +1714,7 @@ void songEditor::exportProject( void )
 	}
  	base_filename += fileEncodeDevices[0].m_extension;
 
-	QFileDialog efd( lmmsMainWin::inst() );
+	QFileDialog efd( eng()->getMainWindow() );
 	efd.setFileMode( QFileDialog::AnyFile );
 
 	int idx = 0;
@@ -1644,7 +1752,7 @@ void songEditor::exportProject( void )
 		const QString export_file_name = efd.selectedFile();
 #endif
 		if( QFileInfo( export_file_name ).exists() == TRUE &&
-			QMessageBox::warning( lmmsMainWin::inst(),
+			QMessageBox::warning( eng()->getMainWindow(),
 						tr( "File already exists" ),
 						tr( "The file \"%1\" already "
 							"exists. Do you want "
@@ -1660,28 +1768,20 @@ void songEditor::exportProject( void )
 			return;
 		}
 		exportProjectDialog epd( export_file_name,
-							lmmsMainWin::inst() );
+						eng()->getMainWindow(), eng() );
 		epd.exec();
 	}
 }
 
 
 
-
-void songEditor::saveSettings( QDomDocument & _doc, QDomElement & _parent )
-{
-	trackContainer::saveSettings( _doc, _parent );
-}
-
-
-
-
-void songEditor::loadSettings( const QDomElement & _this )
-{
-	trackContainer::loadSettings( _this );
-}
-
-
-
 #include "song_editor.moc"
 
+
+#ifdef QT3
+#undef addButton
+#undef setCheckable
+#endif
+
+
+#endif
