@@ -1,7 +1,9 @@
+#ifndef SINGLE_SOURCE_COMPILE
+
 /*
  * timeline.cpp - class timeLine, representing a time-line with position marker
  *
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -27,11 +29,12 @@
 
 #ifdef QT4
 
-#include <QPainter>
-#include <QApplication>
-#include <QMouseEvent>
-#include <QLayout>
-#include <QTimer>
+#include <Qt/QtXml>
+#include <QtCore/QTimer>
+#include <QtGui/QApplication>
+#include <QtGui/QLayout>
+#include <QtGui/QMouseEvent>
+#include <QtGui/QPainter>
 
 #else
 
@@ -39,6 +42,7 @@
 #include <qapplication.h>
 #include <qlayout.h>
 #include <qtimer.h>
+#include <qdom.h>
 
 #endif
 
@@ -47,7 +51,7 @@
 #include "embed.h"
 #include "templates.h"
 #include "nstate_button.h"
-#include "lmms_main_win.h"
+#include "main_window.h"
 #include "text_float.h"
 
 
@@ -60,8 +64,9 @@ QPixmap * timeLine::s_loopPointDisabledPixmap = NULL;
 
 timeLine::timeLine( const int _xoff, const int _yoff, const float _ppt,
 			songEditor::playPos & _pos, const midiTime & _begin,
-							QWidget * _parent ) :
+					QWidget * _parent, engine * _engine ) :
 	QWidget( _parent ),
+	journallingObject( _engine ),
 	m_autoScroll( AUTOSCROLL_ENABLED ),
 	m_loopPoints( LOOP_POINTS_DISABLED ),
 	m_behaviourAtStop( BACK_TO_ZERO ),
@@ -79,23 +84,23 @@ timeLine::timeLine( const int _xoff, const int _yoff, const float _ppt,
 	m_loopPos[0] = 0;
 	m_loopPos[1] = 64;
 
-	if( s_timeLinePixmap == NULL)
+	if( s_timeLinePixmap == NULL )
 	{
 		s_timeLinePixmap = new QPixmap( embed::getIconPixmap(
 								"timeline" ) );
 	}
-	if( s_posMarkerPixmap == NULL)
+	if( s_posMarkerPixmap == NULL )
 	{
 		s_posMarkerPixmap = new QPixmap( embed::getIconPixmap(
 							"playpos_marker" ) );
 	}
-	if( s_loopPointPixmap == NULL)
+	if( s_loopPointPixmap == NULL )
 	{
 		s_loopPointPixmap = new QPixmap( embed::getIconPixmap(
 							"loop_point" ) );
 	}
 
-	if( s_loopPointDisabledPixmap == NULL)
+	if( s_loopPointDisabledPixmap == NULL )
 	{
 		s_loopPointDisabledPixmap = new QPixmap( embed::getIconPixmap(
 						"loop_point_disabled" ) );
@@ -112,10 +117,9 @@ timeLine::timeLine( const int _xoff, const int _yoff, const float _ppt,
 
 	m_pos.m_timeLine = this;
 
-	updatePosition();
 	QTimer * update_timer = new QTimer( this );
-	connect( update_timer, SIGNAL( timeout() ), this,
-					SLOT( checkForUpdatedPosition() ) );
+	connect( update_timer, SIGNAL( timeout() ),
+					this, SLOT( updatePosition() ) );
 	update_timer->start( 50 );
 }
 
@@ -172,6 +176,28 @@ void timeLine::addToolButtons( QWidget * _tool_bar )
 
 
 
+void timeLine::saveSettings( QDomDocument & _doc, QDomElement & _this )
+{
+	_this.setAttribute( "lp0pos", (int) loopBegin() );
+	_this.setAttribute( "lp1pos", (int) loopEnd() );
+	_this.setAttribute( "lpstate", m_loopPoints );
+}
+
+
+
+
+void timeLine::loadSettings( const QDomElement & _this )
+{
+	m_loopPos[0] = _this.attribute( "lp0pos" ).toInt();
+	m_loopPos[1] = _this.attribute( "lp1pos" ).toInt();
+	m_loopPoints = static_cast<loopPointStates>(
+					_this.attribute( "lpstate" ).toInt() );
+	update();
+}
+
+
+
+
 void timeLine::updatePosition( const midiTime & )
 {
 	const int new_x = markerX( m_pos );
@@ -184,6 +210,7 @@ void timeLine::updatePosition( const midiTime & )
 		{
 			emit positionChanged( m_pos );
 		}
+		update();
 	}
 }
 
@@ -215,18 +242,6 @@ void timeLine::toggleBehaviourAtStop( int _n )
 
 
 
-void timeLine::checkForUpdatedPosition( void )
-{
-	if( m_changedPosition == TRUE )
-	{
-		repaint();
-		m_changedPosition = FALSE;
-	}
-}
-
-
-
-
 void timeLine::paintEvent( QPaintEvent * )
 {
 #ifdef QT4
@@ -288,49 +303,7 @@ void timeLine::mousePressEvent( QMouseEvent * _me )
 	{
 		return;
 	}
-	if( _me->button() == Qt::RightButton )
-	{
-		if( _me->x() >= markerX( loopBegin() ) &&
-				_me->x() <= markerX( loopBegin() ) +
-						s_loopPointPixmap->width() )
-		{
-			m_action = MOVE_LOOP_BEGIN;
-		}
-		else if( _me->x() >= markerX( loopEnd() ) &&
-				_me->x() <= markerX( loopEnd() ) +
-						s_loopPointPixmap->width() )
-		{
-			m_action = MOVE_LOOP_END;
-		}
-		if( m_action != NONE )
-		{
-			m_moveXOff = s_loopPointPixmap->width() / 2;
-		}
-	}
-	else if( _me->button() == Qt::MidButton )
-	{
-		const midiTime t = m_begin +
-				static_cast<Sint32>( _me->x() * 64 / m_ppt );
-		Uint8 pmin = 0;
-		Uint8 pmax = 1;
-		m_action = MOVE_LOOP_BEGIN;
-		if( m_loopPos[pmin] > m_loopPos[pmax] )
-		{
-			qSwap( pmin, pmax );
-			m_action = MOVE_LOOP_END;
-		}
-		if( lmmsMainWin::isShiftPressed() == TRUE )
-		{
-			m_loopPos[pmax] = t;
-			m_action = ( m_action == MOVE_LOOP_BEGIN ) ?
-						MOVE_LOOP_END : MOVE_LOOP_BEGIN;
-		}
-		else
-		{
-			m_loopPos[pmin] = t;
-		}
-	}
-	else
+	if( _me->button() == Qt::LeftButton )
 	{
 		m_action = MOVE_POS_MARKER;
 		if( _me->x() - m_xOffset < s_posMarkerPixmap->width() )
@@ -342,6 +315,21 @@ void timeLine::mousePressEvent( QMouseEvent * _me )
 			m_moveXOff = s_posMarkerPixmap->width() / 2;
 		}
 	}
+	else
+	{
+		const midiTime t = m_begin +
+				static_cast<Sint32>( _me->x() * 64 / m_ppt );
+		m_action = MOVE_LOOP_BEGIN;
+		if( m_loopPos[0] > m_loopPos[1]  )
+		{
+			qSwap( m_loopPos[0], m_loopPos[1] );
+		}
+		if( _me->button() == Qt::RightButton )
+		{
+			m_action = MOVE_LOOP_END;
+		}
+		m_loopPos[( m_action == MOVE_LOOP_BEGIN ) ? 0 : 1] = t;
+	}
 	if( m_action == MOVE_LOOP_BEGIN || m_action == MOVE_LOOP_END )
 	{
 		delete m_hint;
@@ -372,8 +360,8 @@ void timeLine::mouseMoveEvent( QMouseEvent * _me )
 		case MOVE_LOOP_BEGIN:
 		case MOVE_LOOP_END:
 		{
-			Uint8 i = m_action - MOVE_LOOP_BEGIN;
-			if( lmmsMainWin::isCtrlPressed() == TRUE )
+			const Uint8 i = m_action - MOVE_LOOP_BEGIN;
+			if( eng()->getMainWindow()->isCtrlPressed() == TRUE )
 			{
 				// no ctrl-press-hint when having ctrl pressed
 				delete m_hint;
@@ -382,11 +370,7 @@ void timeLine::mouseMoveEvent( QMouseEvent * _me )
 			}
 			else
 			{
-				m_loopPos[i] = t.getTact() * 64;
-				if( t.getTact64th() >= 32 )
-				{
-					m_loopPos[i] += 64;
-				}
+				m_loopPos[i] = t.toNearestTact();
 			}
 			update();
 			break;
@@ -412,3 +396,5 @@ void timeLine::mouseReleaseEvent( QMouseEvent * _me )
 
 #include "timeline.moc"
 
+
+#endif
