@@ -1,8 +1,10 @@
+#ifndef SINGLE_SOURCE_COMPILE
+
 /*
  * track_container.cpp - implementation of base-class for all track-containers
  *                       like Song-Editor, BB-Editor...
  *
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -29,9 +31,9 @@
 #ifdef QT4
 
 #include <Qt/QtXml>
-#include <QApplication>
-#include <QProgressDialog>
-#include <QWheelEvent>
+#include <QtGui/QApplication>
+#include <QtGui/QProgressDialog>
+#include <QtGui/QWheelEvent>
 
 #else
 
@@ -49,36 +51,42 @@
 #include "track.h"
 #include "templates.h"
 #include "bb_track.h"
-#include "lmms_main_win.h"
+#include "main_window.h"
 #include "mixer.h"
 #include "song_editor.h"
 #include "string_pair_drag.h"
-#include "channel_track.h"
+#include "instrument_track.h"
 #include "mmp.h"
 #include "config_mgr.h"
-#include "midi_file.h"
+#include "import_filter.h"
+#include "instrument.h"
+#include "rubberband.h"
+#include "project_journal.h"
+#include "debug.h"
 
 
-
-trackContainer::trackContainer() :
-	QMainWindow( lmmsMainWin::inst()->workspace()
-#ifndef QT4
+trackContainer::trackContainer( engine * _engine ) :
+	QMainWindow( _engine->getMainWindow()->workspace()
+#ifdef QT3
 				, 0, Qt::WStyle_Title
 #endif
 			 ),
-	settings(),
+	journallingObject( _engine ),
 	m_currentPosition( 0, 0 ),
-	m_ppt( DEFAULT_PIXELS_PER_TACT )
+	m_scrollArea( new scrollArea( this ) ),
+	m_ppt( DEFAULT_PIXELS_PER_TACT ),
+	m_rubberBand( new rubberBand( m_scrollArea ) ),
+	m_origin()
 {
 #ifdef QT4
-	if( lmmsMainWin::inst()->workspace() != NULL )
+	if( eng()->getMainWindow()->workspace() != NULL )
 	{
-		lmmsMainWin::inst()->workspace()->addWindow( this );
+		eng()->getMainWindow()->workspace()->addWindow( this );
 	}
 #endif
 
-	m_scrollArea = new scrollArea( this );
 	m_scrollArea->show();
+	m_rubberBand->hide();
 
 	setAcceptDrops( TRUE );
 }
@@ -88,26 +96,30 @@ trackContainer::trackContainer() :
 
 trackContainer::~trackContainer()
 {
+	eng()->getProjectJournal()->setJournalling( FALSE );
+
 	while( m_trackWidgets.size() )
 	{
 		removeTrack( m_trackWidgets.front()->getTrack() );
 	}
+
+	eng()->getProjectJournal()->setJournalling( TRUE );
 }
 
 
 
 
-void trackContainer::saveSettings( QDomDocument & _doc, QDomElement & _parent )
+void trackContainer::saveSettings( QDomDocument & _doc, QDomElement & _this )
 {
-	QDomElement tc_de = _doc.createElement( "trackcontainer" );
-	tc_de.setAttribute( "type", nodeName() );
-	_parent.appendChild( tc_de );
+	_this.setTagName( "trackcontainer" );
+	_this.setAttribute( "type", nodeName() );
+	mainWindow::saveWidgetState( this, _this );
 
 	// save settings of each track
 	for( trackWidgetVector::iterator it = m_trackWidgets.begin();
 					it != m_trackWidgets.end(); ++it )
 	{
-		( *it )->getTrack()->saveSettings( _doc, tc_de );
+		( *it )->getTrack()->saveState( _doc, _this );
 	}
 }
 
@@ -161,13 +173,16 @@ void trackContainer::loadSettings( const QDomElement & _this )
 			break;
 		}
 
-		if( node.isElement() )
+		if( node.isElement() &&
+			!node.toElement().attribute( "metadata" ).toInt() )
 		{
 			track::create( node.toElement(), this );
 		}
 		node = node.nextSibling();
 	}
 
+	mainWindow::restoreWidgetState( this, _this );
+
 	pd->setValue( start_val + _this.childNodes().count() );
 
 	if( was_null )
@@ -182,7 +197,9 @@ void trackContainer::loadSettings( const QDomElement & _this )
 
 void trackContainer::cloneTrack( track * _track )
 {
+	eng()->getMixer()->pause();
 	track::clone( _track );
+	eng()->getMixer()->play();
 }
 
 
@@ -190,6 +207,10 @@ void trackContainer::cloneTrack( track * _track )
 
 void trackContainer::addTrack( track * _track )
 {
+	QMap<QString, QVariant> map;
+	map["id"] = _track->id();
+	addJournalEntry( journalEntry( ADD_TRACK, map ) );
+
 	m_trackWidgets.push_back( _track->getTrackWidget() );
 #ifndef QT4
 	m_scrollArea->addChild( _track->getTrackWidget() );
@@ -209,7 +230,14 @@ void trackContainer::removeTrack( track * _track )
 			m_trackWidgets.end(), _track->getTrackWidget() );
 	if( it != m_trackWidgets.end() )
 	{
-		mixer::inst()->pause();
+		QMap<QString, QVariant> map;
+		multimediaProject mmp( multimediaProject::JOURNAL_DATA );
+		_track->saveState( mmp, mmp.content() );
+		map["id"] = _track->id();
+		map["state"] = mmp.toString();
+		addJournalEntry( journalEntry( REMOVE_TRACK, map ) );
+
+		eng()->getMixer()->pause();
 #ifndef QT4
 		m_scrollArea->removeChild( _track->getTrackWidget() );
 #endif
@@ -217,10 +245,13 @@ void trackContainer::removeTrack( track * _track )
 
 		delete _track;
 
-		mixer::inst()->play();
+		eng()->getMixer()->play();
 
 		realignTracks();
-		songEditor::inst()->setModified();
+		if( eng()->getSongEditor() )
+		{
+			eng()->getSongEditor()->setModified();
+		}
 	}
 }
 
@@ -280,23 +311,20 @@ void trackContainer::realignTracks( bool _complete_update )
 	for( trackWidgetVector::iterator it = m_trackWidgets.begin();
 					it != m_trackWidgets.end(); ++it )
 	{
-		if( _complete_update )
-		{
-			( *it )->hide();
-		}
 		( *it )->show();
+		( *it )->repaint();
 #ifdef QT4
 		( *it )->move( 0, y );
 #else
 		m_scrollArea->moveChild( *it, 0, y );
 #endif
-		( *it )->resize( width(), ( *it )->height() );
+		( *it )->resize( width() - DEFAULT_SCROLLBAR_SIZE,
+				 			( *it )->height() );
 		( *it )->changePosition( m_currentPosition );
 		y += ( *it )->height();
 	}
-#ifndef QT4
-	m_scrollArea->resizeContents( m_scrollArea->parentWidget()->width(),
-									y );
+#ifdef QT3
+	m_scrollArea->resizeContents( width() - DEFAULT_SCROLLBAR_SIZE, y );
 #endif
 	updateScrollArea();
 }
@@ -304,9 +332,47 @@ void trackContainer::realignTracks( bool _complete_update )
 
 
 
-unsigned int trackContainer::countTracks( track::trackTypes _tt ) const
+void trackContainer::clearAllTracks( void )
 {
-	unsigned int cnt = 0;
+	while( m_trackWidgets.size() )
+	{
+		removeTrack( m_trackWidgets.front()->getTrack() );
+	}
+}
+
+
+
+
+const trackWidget * trackContainer::trackWidgetAt( const int _y ) const
+{
+	int y_cnt = 0;
+	for( trackWidgetVector::const_iterator it = m_trackWidgets.begin();
+					it != m_trackWidgets.end(); ++it )
+	{
+		const int y_cnt1 = y_cnt;
+		y_cnt += ( *it )->height();
+		if( _y >= y_cnt1 && _y < y_cnt )
+		{
+			return( *it );
+		}
+	}
+	return( NULL );
+}
+
+
+
+
+bool trackContainer::allowRubberband( void ) const
+{
+	return( FALSE );
+}
+
+
+
+
+Uint16 trackContainer::countTracks( track::trackTypes _tt ) const
+{
+	Uint16 cnt = 0;
 	for( trackWidgetVector::const_iterator it = m_trackWidgets.begin();
 					it != m_trackWidgets.end(); ++it )
 	{
@@ -327,7 +393,7 @@ void trackContainer::setMutedOfAllTracks( bool _muted )
 	for( trackWidgetVector::iterator it = m_trackWidgets.begin();
 					it != m_trackWidgets.end(); ++it )
 	{
-		( *it )->setMuted( _muted );
+		( *it )->getTrack()->setMuted( _muted );
 	}
 }
 
@@ -370,9 +436,55 @@ void trackContainer::setPixelsPerTact( Uint16 _ppt )
 
 
 
-void trackContainer::resizeEvent( QResizeEvent * )
+void trackContainer::undoStep( journalEntry & _je )
 {
-	realignTracks();
+	saveJournallingState( FALSE );
+	switch( _je.actionID() )
+	{
+		case ADD_TRACK:
+		{
+			QMap<QString, QVariant> map = _je.data().toMap();
+			track * tr =
+dynamic_cast<track *>(
+	eng()->getProjectJournal()->getJournallingObject( map["id"].toInt() ) );
+			assert( tr != NULL );
+			multimediaProject mmp(
+					multimediaProject::JOURNAL_DATA );
+			tr->saveState( mmp, mmp.content() );
+			map["state"] = mmp.toString();
+			_je.data() = map;
+			removeTrack( tr );
+			break;
+		}
+
+		case REMOVE_TRACK:
+		{
+			multimediaProject mmp(
+				_je.data().toMap()["state"].toString(), FALSE );
+			track::create( mmp.content().firstChild().toElement(),
+									this );
+			break;
+		}
+	}
+	restoreJournallingState();
+}
+
+
+
+
+void trackContainer::redoStep( journalEntry & _je )
+{
+	switch( _je.actionID() )
+	{
+		case ADD_TRACK:
+		case REMOVE_TRACK:
+			_je.actionID() = ( _je.actionID() == ADD_TRACK ) ?
+						REMOVE_TRACK : ADD_TRACK;
+			undoStep( _je );
+			_je.actionID() = ( _je.actionID() == ADD_TRACK ) ?
+						REMOVE_TRACK : ADD_TRACK;
+			break;
+	}
 }
 
 
@@ -381,7 +493,8 @@ void trackContainer::resizeEvent( QResizeEvent * )
 void trackContainer::dragEnterEvent( QDragEnterEvent * _dee )
 {
 	stringPairDrag::processDragEnterEvent( _dee,
-		QString( "presetfile,instrument,midifile,track_%1,track_%2" ).
+		QString( "presetfile,sampledata,samplefile,instrument,midifile,"
+					"track_%1,track_%2" ).
 						arg( track::CHANNEL_TRACK ).
 						arg( track::SAMPLE_TRACK ) );
 }
@@ -395,28 +508,37 @@ void trackContainer::dropEvent( QDropEvent * _de )
 	QString value = stringPairDrag::decodeValue( _de );
 	if( type == "instrument" )
 	{
-		channelTrack * ct = dynamic_cast<channelTrack *>(
+		instrumentTrack * it = dynamic_cast<instrumentTrack *>(
 				track::create( track::CHANNEL_TRACK,
 								this ) );
-		ct->loadInstrument( value );
-		ct->toggledChannelButton( TRUE );
+		it->loadInstrument( value );
+		it->toggledInstrumentTrackButton( TRUE );
+		_de->accept();
+	}
+	else if( type == "sampledata" || type == "samplefile" )
+	{
+		instrumentTrack * it = dynamic_cast<instrumentTrack *>(
+				track::create( track::CHANNEL_TRACK,
+								this ) );
+		instrument * i = it->loadInstrument( "audiofileprocessor" );
+		i->setParameter( type, value );
+		it->toggledInstrumentTrackButton( TRUE );
 		_de->accept();
 	}
 	else if( type == "presetfile" )
 	{
 		multimediaProject mmp( value );
-		channelTrack * ct = dynamic_cast<channelTrack *>(
+		instrumentTrack * it = dynamic_cast<instrumentTrack *>(
 				track::create( track::CHANNEL_TRACK,
 								this ) );
-		ct->loadTrackSpecificSettings( mmp.content().firstChild().
+		it->loadTrackSpecificSettings( mmp.content().firstChild().
 								toElement() );
-		ct->toggledChannelButton( TRUE );
+		it->toggledInstrumentTrackButton( TRUE );
 		_de->accept();
 	}
 	else if( type == "midifile" )
 	{
-		midiFile mf( value );
-		mf.importToTrackContainer( this );
+		importFilter::import( value, this );
 		_de->accept();
 	}
 	else if( type.left( 6 ) == "track_" )
@@ -433,13 +555,56 @@ void trackContainer::dropEvent( QDropEvent * _de )
 
 
 
+void trackContainer::mousePressEvent( QMouseEvent * _me )
+{
+	if( allowRubberband() == TRUE )
+	{
+		m_origin = m_scrollArea->mapFromParent( _me->pos() );
+		m_rubberBand->setGeometry( QRect( m_origin, QSize() ) );
+		m_rubberBand->show();
+	}
+}
+
+
+
+
+void trackContainer::mouseMoveEvent( QMouseEvent * _me )
+{
+	if( rubberBandActive() == TRUE )
+	{
+		m_rubberBand->setGeometry( QRect( m_origin,
+				m_scrollArea->mapFromParent( _me->pos() ) ).
+								normalized() );
+	}
+}
+
+
+
+
+void trackContainer::mouseReleaseEvent( QMouseEvent * _me )
+{
+	m_rubberBand->hide();
+}
+
+
+
+
+
+void trackContainer::resizeEvent( QResizeEvent * )
+{
+	realignTracks();
+}
+
+
+
+
 void trackContainer::updateScrollArea( void )
 {
-	m_scrollArea->resize( tMax( m_scrollArea->parentWidget()->width() - 
+	m_scrollArea->resize( width(), scrollAreaRect().height() );
+/*	m_scrollArea->resize( tMax( m_scrollArea->parentWidget()->width() - 
 					m_scrollArea->x() - 2, 0 ),
 				tMax( m_scrollArea->parentWidget()->height() -
-					m_scrollArea->y() - 2, 0 ) );
-	//m_scrollArea->updateContents();
+					m_scrollArea->y() - 2, 0 ) );*/
 }
 
 
@@ -455,6 +620,13 @@ trackContainer::scrollArea::scrollArea( trackContainer * _parent ) :
 					Qt::ScrollBarAlwaysOff
 #else
 					QScrollArea::AlwaysOff
+#endif
+					);
+	setVerticalScrollBarPolicy( 
+#ifdef QT4
+					Qt::ScrollBarAlwaysOn
+#else
+					QScrollArea::AlwaysOn
 #endif
 					);
 }
@@ -489,3 +661,5 @@ void trackContainer::scrollArea::wheelEvent( QWheelEvent * _we )
 
 #undef setValue
 #undef maximum
+
+#endif
