@@ -1,3 +1,5 @@
+#ifndef SINGLE_SOURCE_COMPILE
+
 /*
  * ladspa_manager.cpp - a class to manage loading and instantiation
  *                      of ladspa plugins
@@ -31,9 +33,9 @@
 
 #ifdef QT4
 
-#include <QDir>
-#include <QFileInfo>
-#include <QLibrary>
+#include <QtCore/QDir>
+#include <QtCore/QFileInfo>
+#include <QtCore/QLibrary>
 
 #else
 
@@ -917,3 +919,5 @@ void FASTCALL ladspaManager::cleanup( const ladspaKey & _plugin,
 
 #endif
 
+
+#endif
