@@ -1,7 +1,9 @@
+#ifndef SINGLE_SOURCE_COMPILE
+
 /*
  * mmp.cpp - implementation of class multimediaProject
  *
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -27,8 +29,8 @@
 
 #ifdef QT4
 
-#include <QFile>
-#include <QMessageBox>
+#include <QtCore/QFile>
+#include <QtGui/QMessageBox>
 
 #else
 
@@ -53,11 +55,14 @@ multimediaProject::typeDescStruct
 	{ multimediaProject::UNKNOWN, "unknown" },
 	{ multimediaProject::SONG_PROJECT, "song" },
 	{ multimediaProject::SONG_PROJECT_TEMPLATE, "songtemplate" },
-	{ multimediaProject::CHANNEL_SETTINGS, "channelsettings" },
+#warning compat-code, remove in 0.3.0
+	{ multimediaProject::INSTRUMENT_TRACK_SETTINGS,
+				"instrumenttracksettings,channelsettings" },
 	{ multimediaProject::DRAG_N_DROP_DATA, "dnddata" },
+	{ multimediaProject::JOURNAL_DATA, "journaldata" },
 	{ multimediaProject::EFFECT_SETTINGS, "effectsettings" },
-	{ multimediaProject::VIDEO_PROJECT, "video" },
-	{ multimediaProject::BURN_PROJECT, "burn" },
+	{ multimediaProject::VIDEO_PROJECT, "videoproject" },
+	{ multimediaProject::BURN_PROJECT, "burnproject" },
 	{ multimediaProject::PLAYLIST, "playlist" }
 } ;
 
@@ -147,10 +152,8 @@ multimediaProject::multimediaProject( const QString & _in_file_name,
 		}
 	}
 
-
 	QDomElement root = documentElement();
 	m_type = type( root.attribute( "type" ) );
-
 	QDomNode node = root.firstChild();
 	while( !node.isNull() )
 	{
@@ -160,7 +163,9 @@ multimediaProject::multimediaProject( const QString & _in_file_name,
 			{
 				m_head = node.toElement();
 			}
-			else if( node.nodeName() == typeName( m_type ) )
+			else if( node.nodeName() == typeName( m_type ) ||
+#warning compat-code, remove in 0.3.0
+					node.nodeName() == "channelsettings" )
 			{
 				m_content = node.toElement();
 			}
@@ -181,25 +186,24 @@ multimediaProject::~multimediaProject()
 
 bool multimediaProject::writeFile( const QString & _fn, bool _overwrite_check )
 {
-	QString xml = "<?xml version=\"1.0\"?>\n" + toString(
-#if QT_VERSION >= 0x030100
-								2
-#endif
-									);
+	bool clean_meta_nodes = FALSE;
 	QString fn = _fn;
-	if( type() == CHANNEL_SETTINGS )
+	if( type() == INSTRUMENT_TRACK_SETTINGS )
 	{
 		if( fn.section( '.', -2, -1 ) != "cs.xml" )
 		{
 			fn += ".cs.xml";
 		}
+		clean_meta_nodes = TRUE;
 	}
 	else if( type() == SONG_PROJECT )
 	{
-		if( fn.section( '.',-1 ) != "mmp" )
+		if( fn.section( '.', -1 ) != "mmp" &&
+						fn.section( '.', -1 ) != "mpt" )
 		{
 			fn += ".mmp";
 		}
+		clean_meta_nodes = TRUE;
 	}
 	else if( type() == SONG_PROJECT_TEMPLATE )
 	{
@@ -207,6 +211,12 @@ bool multimediaProject::writeFile( const QString & _fn, bool _overwrite_check )
 		{
 			fn += ".mpt";
 		}
+		clean_meta_nodes = TRUE;
+	}
+
+	if( clean_meta_nodes == TRUE )
+	{
+		cleanMetaNodes( documentElement() );
 	}
 
 
@@ -251,10 +261,16 @@ bool multimediaProject::writeFile( const QString & _fn, bool _overwrite_check )
 						).arg( fn ) );
 		return( FALSE );
 	}
+	QString xml = "<?xml version=\"1.0\"?>\n" + toString(
+#if QT_VERSION >= 0x030100
+								2
+#endif
+									);
 #ifdef QT4
-	outfile.write( xml.toAscii().constData(), xml.length() );
+	outfile.write( xml.toUtf8().constData(), xml.length() );
 #else
-	outfile.writeBlock( xml.ascii(), xml.length() );
+	QCString xml_utf8 = xml.utf8();
+	outfile.writeBlock( xml_utf8.data(), xml_utf8.length() );
 #endif
 	outfile.close();
 
@@ -279,7 +295,12 @@ multimediaProject::projectTypes multimediaProject::type(
 {
 	for( int i = 0; i < PROJ_TYPE_COUNT; ++i )
 	{
-		if( s_types[i].m_name == _type_name )
+		if( s_types[i].m_name == _type_name || (
+			s_types[i].m_name.contains( "," ) && (
+			s_types[i].m_name.section( ',', 0, 0 ) == _type_name ||
+			s_types[i].m_name.section( ',', 1, 1 ) == _type_name ) )
+							)
+
 		{
 			return( static_cast<multimediaProject::projectTypes>(
 									i ) );
@@ -295,8 +316,40 @@ QString multimediaProject::typeName( projectTypes _project_type )
 {
 	if( _project_type >= UNKNOWN && _project_type < PROJ_TYPE_COUNT )
 	{
-		return( s_types[_project_type].m_name );
+		return( s_types[_project_type].m_name
+#warning compat-code, remove in 0.3.0
+				.section( ',', 0, 0 )
+				);
 	}
 	return( s_types[UNKNOWN].m_name );
 }
 
+
+
+
+void multimediaProject::cleanMetaNodes( QDomElement _de )
+{
+	QDomNode node = _de.firstChild();
+	while( !node.isNull() )
+	{
+		if( node.isElement() )
+		{
+			if( node.toElement().attribute( "metadata" ).toInt() )
+			{
+				QDomNode ns = node.nextSibling();
+				_de.removeChild( node );
+				node = ns;
+				continue;
+			}
+			if( node.hasChildNodes() )
+			{
+				cleanMetaNodes( node.toElement() );
+			}
+		}
+		node = node.nextSibling();
+	}
+}
+
+
+
+#endif
