@@ -1,7 +1,9 @@
+#ifndef SINGLE_SOURCE_COMPILE
+
 /*
  * midi_alsa_raw.cpp - midi-client for RawMIDI via ALSA
  *
- * Copyright (c) 2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2005-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -27,8 +29,8 @@
 
 #ifdef QT4
 
-#include <QLineEdit>
-#include <QLabel>
+#include <QtGui/QLabel>
+#include <QtGui/QLineEdit>
 
 #else
 
@@ -47,8 +49,8 @@
 #ifdef ALSA_SUPPORT
 
 
-midiALSARaw::midiALSARaw( void ) :
-	midiClientRaw(),
+midiALSARaw::midiALSARaw( engine * _engine ) :
+	midiClientRaw( _engine ),
 	QThread(),
 	m_inputp( &m_input ),
 	m_outputp( &m_output ),
@@ -75,7 +77,7 @@ midiALSARaw::midiALSARaw( void ) :
 	snd_rawmidi_poll_descriptors( m_input, m_pfds, m_npfds );
 
 	start( 
-#if QT_VERSION >= 0x030200	
+#if QT_VERSION >= 0x030505
 	    	QThread::LowPriority 
 #endif
 					);
@@ -230,3 +232,5 @@ void midiALSARaw::setupWidget::saveSettings( void )
 
 #endif
 
+
+#endif
