@@ -1,7 +1,9 @@
+#ifndef SINGLE_SOURCE_COMPILE
+
 /*
  * midi_oss.cpp - OSS-raw-midi-client
  *
- * Copyright (c) 2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2005-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -32,8 +34,8 @@
 
 #ifdef QT4
 
-#include <QLineEdit>
-#include <QLabel>
+#include <QtGui/QLabel>
+#include <QtGui/QLineEdit>
 
 #else
 
@@ -53,8 +55,8 @@
 
 
 
-midiOSS::midiOSS( void ) :
-	midiClientRaw(),
+midiOSS::midiOSS( engine * _engine ) :
+	midiClientRaw( _engine ),
 	QThread(),
 	m_midiDev( probeDevice() ),
 	m_quit( FALSE )
@@ -69,7 +71,7 @@ midiOSS::midiOSS( void ) :
 #endif
 	{
 		start(
-#if QT_VERSION >= 0x030200
+#if QT_VERSION >= 0x030505
 				QThread::LowPriority
 #endif
 							);
@@ -173,3 +175,5 @@ void midiOSS::setupWidget::saveSettings( void )
 
 #endif
 
+
+#endif
