@@ -1,7 +1,9 @@
+#ifndef SINGLE_SOURCE_COMPILE
+
 /*
- * bb_track.cpp - implementation of class bbTrack
+ * bb_track.cpp - implementation of class bbTrack and bbTCO
  *
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -28,9 +30,9 @@
 #ifdef QT4
 
 #include <Qt/QtXml>
-#include <QPainter>
-#include <QColorDialog>
-#include <QMenu>
+#include <QtGui/QColorDialog>
+#include <QtGui/QMenu>
+#include <QtGui/QPainter>
 
 #else
 
@@ -49,10 +51,11 @@
 #include "name_label.h"
 #include "embed.h"
 #include "rename_dialog.h"
+#include "templates.h"
 
 
 
-QMap<bbTrack *, bbTrack::bbInfoStruct> bbTrack::s_bbNums;
+bbTrack::infoMap bbTrack::s_infoMap;
 
 
 bbTCO::bbTCO( track * _track, const QColor & _c ) :
@@ -62,15 +65,16 @@ bbTCO::bbTCO( track * _track, const QColor & _c ) :
 								QString( "" ) ),
 	m_color( _c.isValid() ? _c : QColor( 64, 128, 255 ) )
 {
-	//setPaletteBackgroundColor( QColor( 64, 128, 255 ) );
 #ifndef QT4
 	setBackgroundMode( Qt::NoBackground );
 #endif
-	tact t = bbEditor::inst()->lengthOfBB(
+	tact t = eng()->getBBEditor()->lengthOfBB(
 					bbTrack::numOfBBTrack( getTrack() ) );
 	if( t > 0 )
 	{
+		saveJournallingState( FALSE );
 		changeLength( midiTime( t, 0 ) );
+		restoreJournallingState();
 	}
 }
 
@@ -84,36 +88,18 @@ bbTCO::~bbTCO()
 
 
 
-void bbTCO::movePosition( const midiTime & _pos )
-{
-	// bb-playlist-entries are always aligned at tact-boundaries
-	trackContentObject::movePosition( midiTime( _pos.getTact(), 0 ) );
-}
-
-
-
-
-void bbTCO::changeLength( const midiTime & _length )
-{
-	// the length of a bb-playlist-entry is always a multiple of one tact
-	trackContentObject::changeLength( midiTime( _length.getTact(), 0 ) );
-}
-
-
-
-
 void bbTCO::constructContextMenu( QMenu * _cm )
 {
 #ifdef QT4
 	QAction * a = new QAction( embed::getIconPixmap( "bb_track" ),
-					tr( "Open in Beat+Bassline-Editor" ),
+					tr( "Open in Beat+Baseline-Editor" ),
 					_cm );
 	_cm->insertAction( _cm->actions()[0], a );
 	connect( a, SIGNAL( triggered( bool ) ), this,
 					SLOT( openInBBEditor( bool ) ) );
 #else
 	_cm->insertItem( embed::getIconPixmap( "bb_track" ),
-					tr( "Open in Beat+Bassline-Editor" ),
+					tr( "Open in Beat+Baseline-Editor" ),
 					this, SLOT( openInBBEditor() ),
 								0, -1, 0 );
 #endif
@@ -149,14 +135,22 @@ void bbTCO::mouseDoubleClickEvent( QMouseEvent * )
 void bbTCO::paintEvent( QPaintEvent * )
 {
 	QColor col = m_color;
-	if( getTrack()->muted() )
+	if( getTrack()->muted() || muted() )
 	{
 		col = QColor( 160, 160, 160 );
 	}
+	if( isSelected() == TRUE )
+	{
+		col = QColor( tMax( col.red() - 128, 0 ),
+					tMax( col.green() - 128, 0 ), 255 );
+	}
 #ifdef QT4
 	QPainter p( this );
-	// TODO: set according brush/pen for gradient!
-	p.fillRect( rect(), col );
+
+	QLinearGradient lingrad( 0, 0, 0, height() );
+	lingrad.setColorAt( 0, col.light( 130 ) );
+	lingrad.setColorAt( 1, col.light( 70 ) );
+	p.fillRect( rect(), lingrad );
 #else
 	// create pixmap for whole widget
 	QPixmap pm( rect().size() );
@@ -169,15 +163,14 @@ void bbTCO::paintEvent( QPaintEvent * )
 		p.setPen( col.light( 130 - y * 60 / height() ) );
 		p.drawLine( 1, y, width() - 1, y );
 	}
-	//pm.fill( col );
 #endif
 
-	tact t = bbEditor::inst()->lengthOfBB( bbTrack::numOfBBTrack(
+	tact t = eng()->getBBEditor()->lengthOfBB( bbTrack::numOfBBTrack(
 								getTrack() ) );
-	if( length().getTact() > 1 && t > 0 )
+	if( length() > 64 && t > 0 )
 	{
-		for( int x = TCO_BORDER_WIDTH + static_cast<int>( t *
-						pixelsPerTact() ); x < width();
+		for( int x = static_cast<int>( t * pixelsPerTact() );
+								x < width()-2;
 			x += static_cast<int>( t * pixelsPerTact() ) )
 		{
 			p.setPen( col.light( 80 ) );
@@ -188,11 +181,21 @@ void bbTCO::paintEvent( QPaintEvent * )
 	}
 
 	p.setPen( col.dark() );
-	p.drawRect( 0, 0, width(), height() );
+#ifndef QT3
+	p.drawRect( 0, 0, rect().right(), rect().bottom() );
+#else
+	p.drawRect( rect() );
+#endif
 
 	p.setFont( pointSize<7>( p.font() ) );
 	p.setPen( QColor( 0, 0, 0 ) );
-	p.drawText( 2, QFontMetrics( p.font() ).height() - 1, m_name );
+	p.drawText( 2, p.fontMetrics().height() - 1, m_name );
+
+	if( muted() )
+	{
+		p.drawPixmap( 3, p.fontMetrics().height() + 1,
+				embed::getIconPixmap( "muted", 16, 16 ) );
+	}
 
 #ifndef QT4
 	bitBlt( this, rect().topLeft(), &pm );
@@ -202,22 +205,20 @@ void bbTCO::paintEvent( QPaintEvent * )
 
 
 
-void bbTCO::saveSettings( QDomDocument & _doc, QDomElement & _parent )
+void bbTCO::saveSettings( QDomDocument & _doc, QDomElement & _this )
 {
-	QDomElement bbtco_de = _doc.createElement( nodeName() );
-	bbtco_de.setAttribute( "name", m_name );
-	if( _parent.nodeName() == "clipboard" )
+	_this.setAttribute( "name", m_name );
+	if( _this.parentNode().nodeName() == "clipboard" )
 	{
-		bbtco_de.setAttribute( "pos", QString::number( -1 ) );
+		_this.setAttribute( "pos", -1 );
 	}
 	else
 	{
-		bbtco_de.setAttribute( "pos",
-					QString::number( startPosition() ) );
+		_this.setAttribute( "pos", startPosition() );
 	}
-	bbtco_de.setAttribute( "len", QString::number( length() ) );
-	bbtco_de.setAttribute( "color", QString::number( m_color.rgb() ) );
-	_parent.appendChild( bbtco_de );
+	_this.setAttribute( "len", length() );
+	_this.setAttribute( "muted", muted() );
+	_this.setAttribute( "color", m_color.rgb() );
 }
 
 
@@ -231,6 +232,11 @@ void bbTCO::loadSettings( const QDomElement & _this )
 		movePosition( _this.attribute( "pos" ).toInt() );
 	}
 	changeLength( _this.attribute( "len" ).toInt() );
+	if( _this.attribute( "muted" ).toInt() != muted() )
+	{
+		toggleMute();
+	}
+
 	if( _this.attribute( "color" ).toUInt() != 0 )
 	{
 		m_color.setRgb( _this.attribute( "color" ).toUInt() );
@@ -242,9 +248,10 @@ void bbTCO::loadSettings( const QDomElement & _this )
 
 void bbTCO::openInBBEditor( bool )
 {
-	bbEditor::inst()->setCurrentBB( bbTrack::numOfBBTrack( getTrack() ) );
-	bbEditor::inst()->show();
-	bbEditor::inst()->setFocus();
+	eng()->getBBEditor()->setCurrentBB( bbTrack::numOfBBTrack(
+								getTrack() ) );
+	eng()->getBBEditor()->show();
+	eng()->getBBEditor()->setFocus();
 }
 
 
@@ -285,7 +292,7 @@ void bbTCO::changeColor( void )
 	if( _new_color.isValid() && _new_color != m_color )
 	{
 		m_color = _new_color;
-		songEditor::inst()->setModified();
+		eng()->getSongEditor()->setModified();
 		update();
 	}
 }
@@ -294,25 +301,33 @@ void bbTCO::changeColor( void )
 
 
 
-bbTrack::bbTrack( trackContainer * _tc )
-	: track( _tc )
+bbTrack::bbTrack( trackContainer * _tc ) :
+	track( _tc )
 {
 	getTrackWidget()->setFixedHeight( 32 );
 	// drag'n'drop with bb-tracks only causes troubles (and makes no sense
 	// too), so disable it
 	getTrackWidget()->setAcceptDrops( FALSE );
 
-	csize bbNum = s_bbNums.size();
-	bbInfoStruct bis = { bbNum, "" };
-	s_bbNums[this] = bis;
-	m_trackLabel = new nameLabel( tr( "Beat/Bassline %1" ).arg( bbNum ),
-					getTrackSettingsWidget(),
-					embed::getIconPixmap( "bb_track" ) );
-	m_trackLabel->setGeometry( 1, 1, DEFAULT_SETTINGS_WIDGET_WIDTH-2, 29 );
+	csize bbNum = s_infoMap.size();
+	s_infoMap[this] = bbNum;
+
+	m_trackLabel = new nameLabel( tr( "Beat/Baseline %1" ).arg( bbNum ),
+					getTrackSettingsWidget(), eng() );
+	m_trackLabel->setPixmap( embed::getIconPixmap( "bb_track" ) );
+	m_trackLabel->setGeometry( 1, 1, DEFAULT_SETTINGS_WIDGET_WIDTH - 2,
+									29 );
 	m_trackLabel->show();
-	connect( m_trackLabel, SIGNAL( clicked() ), this,
-						SLOT( clickedTrackLabel() ) );
-	bbEditor::inst()->setCurrentBB( bbNum );
+	connect( m_trackLabel, SIGNAL( clicked() ),
+			this, SLOT( clickedTrackLabel() ) );
+	connect( m_trackLabel, SIGNAL( nameChanged() ),
+			eng()->getBBEditor(), SLOT( updateComboBox() ) );
+	connect( m_trackLabel, SIGNAL( pixmapChanged() ),
+			eng()->getBBEditor(), SLOT( updateComboBox() ) );
+
+
+	eng()->getBBEditor()->setCurrentBB( bbNum );
+	eng()->getBBEditor()->updateComboBox();
 
 	_tc->updateAfterTrackAdd();
 }
@@ -322,24 +337,25 @@ bbTrack::bbTrack( trackContainer * _tc )
 
 bbTrack::~bbTrack()
 {
-	csize bb = s_bbNums[this].num;
-	bbEditor::inst()->removeBB( bb );
-	for( QMap<bbTrack *, bbTrack::bbInfoStruct>::iterator it =
-				s_bbNums.begin(); it != s_bbNums.end(); ++it )
+	csize bb = s_infoMap[this];
+	eng()->getBBEditor()->removeBB( bb );
+	for( infoMap::iterator it = s_infoMap.begin(); it != s_infoMap.end();
+									++it )
 	{
 #ifdef QT4
-		if( it.value().num > bb )
+		if( it.value() > bb )
 		{
-			--it.value().num;
+			--it.value();
 		}
 #else
-		if( it.data().num > bb )
+		if( it.data() > bb )
 		{
-			--it.data().num;
+			--it.data();
 		}
 #endif
 	}
-	s_bbNums.remove( this );
+	s_infoMap.remove( this );
+	eng()->getBBEditor()->updateComboBox();
 }
 
 
@@ -353,20 +369,23 @@ track::trackTypes bbTrack::type( void ) const
 
 
 // play _frames frames of given TCO within starting with _start/_start_frame
-bool FASTCALL bbTrack::play( const midiTime & _start, Uint32 _start_frame,
-					Uint32 _frames, Uint32 _frame_base,
+bool FASTCALL bbTrack::play( const midiTime & _start,
+						const f_cnt_t _start_frame,
+						const fpab_t _frames,
+						const f_cnt_t _frame_base,
 							Sint16 _tco_num )
 {
 	if( _tco_num >= 0 )
 	{
-		return( bbEditor::inst()->play( _start, _start_frame, _frames,
+		return( eng()->getBBEditor()->play( _start, _start_frame,
+							_frames,
 							_frame_base,
-							s_bbNums[this].num ) );
+							s_infoMap[this] ) );
 	}
 
 	vlist<trackContentObject *> tcos;
 	getTCOsInRange( tcos, _start, _start +static_cast<Sint32>( _frames *
-				64 / songEditor::inst()->framesPerTact() ) );
+			64 / eng()->getSongEditor()->framesPerTact() ) );
 	
 	if ( tcos.size() == 0 )
 	{
@@ -378,7 +397,8 @@ bool FASTCALL bbTrack::play( const midiTime & _start, Uint32 _start_frame,
 	for( vlist<trackContentObject *>::iterator it = tcos.begin();
 							it != tcos.end(); ++it )
 	{
-		if( ( *it )->startPosition() >= lastPosition )
+		if( !( *it )->muted() &&
+				( *it )->startPosition() >= lastPosition )
 		{
 			lastPosition = ( *it )->startPosition();
 			lastLen = ( *it )->length();
@@ -386,10 +406,10 @@ bool FASTCALL bbTrack::play( const midiTime & _start, Uint32 _start_frame,
 	}
 	if( _start - lastPosition < lastLen )
 	{
-		return( bbEditor::inst()->play( _start - lastPosition,
+		return( eng()->getBBEditor()->play( _start - lastPosition,
 							_start_frame, _frames,
 							_frame_base,
-							s_bbNums[this].num ) );
+							s_infoMap[this] ) );
 	}
 	return( FALSE );
 }
@@ -419,17 +439,18 @@ trackContentObject * bbTrack::createTCO( const midiTime & _pos )
 
 
 void bbTrack::saveTrackSpecificSettings( QDomDocument & _doc,
-							QDomElement & _parent )
+							QDomElement & _this )
 {
-	QDomElement bbt_de = _doc.createElement( nodeName() );
-	bbt_de.setAttribute( "name", m_trackLabel->text() );
-/*	bbt_de.setAttribute( "current", s_bbNums[this].num ==
-					bbEditor::inst()->currentBB() );*/
-	_parent.appendChild( bbt_de );
-	if( s_bbNums[this].num == 0 &&
-				_parent.parentNode().nodeName() != "clone" )
+	_this.setAttribute( "name", m_trackLabel->text() );
+	_this.setAttribute( "icon", m_trackLabel->pixmapFile() );
+/*	_this.setAttribute( "current", s_infoMap[this] ==
+					eng()->getBBEditor()->currentBB() );*/
+	if( s_infoMap[this] == 0 &&
+			_this.parentNode().nodeName() != "clone" &&
+			_this.parentNode().nodeName() != "journaldata" )
 	{
-		bbEditor::inst()->saveSettings( _doc, bbt_de );
+		( (journallingObject *)( eng()->getBBEditor() ) )->saveState(
+								_doc, _this );
 	}
 }
 
@@ -439,9 +460,13 @@ void bbTrack::saveTrackSpecificSettings( QDomDocument & _doc,
 void bbTrack::loadTrackSpecificSettings( const QDomElement & _this )
 {
 	m_trackLabel->setText( _this.attribute( "name" ) );
+	if( _this.attribute( "icon" ) != "" )
+	{
+		m_trackLabel->setPixmapFile( _this.attribute( "icon" ) );
+	}
 	if( _this.firstChild().isElement() )
 	{
-		bbEditor::inst()->loadSettings(
+		( (journallingObject *)( eng()->getBBEditor() ) )->restoreState(
 					_this.firstChild().toElement() );
 	}
 /*	doesn't work yet because bbTrack-ctor also sets current bb so if
@@ -449,7 +474,7 @@ void bbTrack::loadTrackSpecificSettings( const QDomElement & _this )
 	help at all....
 	if( _this.attribute( "current" ).toInt() )
 	{
-		bbEditor::inst()->setCurrentBB( s_bbNums[this].num );
+		eng()->getBBEditor()->setCurrentBB( s_infoMap[this] );
 	}*/
 }
 
@@ -457,16 +482,15 @@ void bbTrack::loadTrackSpecificSettings( const QDomElement & _this )
 
 
 // return pointer to bbTrack specified by _bb_num
-bbTrack * bbTrack::findBBTrack( csize _bb_num )
+bbTrack * bbTrack::findBBTrack( csize _bb_num, engine * _engine )
 {
-	for( QMap<bbTrack *, bbTrack::bbInfoStruct>::iterator it =
-							s_bbNums.begin();
-		it != s_bbNums.end(); ++it )
+	for( infoMap::iterator it = s_infoMap.begin(); it != s_infoMap.end();
+									++it )
 	{
 #ifdef QT4
-		if( it.value().num == _bb_num )
+		if( it.value() == _bb_num && it.key()->eng() == _engine )
 #else
-		if( it.data().num == _bb_num )
+		if( it.data() == _bb_num && it.key()->eng() == _engine )
 #endif
 		{
 			return( it.key() );
@@ -482,7 +506,7 @@ csize bbTrack::numOfBBTrack( track * _track )
 {
 	if( dynamic_cast<bbTrack *>( _track ) != NULL )
 	{
-		return( s_bbNums[dynamic_cast<bbTrack *>( _track )].num );
+		return( s_infoMap[dynamic_cast<bbTrack *>( _track )] );
 	}
 	return( 0 );
 }
@@ -496,9 +520,10 @@ void bbTrack::swapBBTracks( track * _track1, track * _track2 )
 	bbTrack * t2 = dynamic_cast<bbTrack *>( _track2 );
 	if( t1 != NULL && t2 != NULL )
 	{
-		qSwap( s_bbNums[t1].num, s_bbNums[t2].num );
-		bbEditor::inst()->swapBB( s_bbNums[t1].num, s_bbNums[t2].num );
-		bbEditor::inst()->setCurrentBB( s_bbNums[t2].num );
+		qSwap( s_infoMap[t1], s_infoMap[t2] );
+		_track1->eng()->getBBEditor()->swapBB( s_infoMap[t1],
+								s_infoMap[t2] );
+		_track1->eng()->getBBEditor()->setCurrentBB( s_infoMap[t1] );
 	}
 }
 
@@ -507,8 +532,8 @@ void bbTrack::swapBBTracks( track * _track1, track * _track2 )
 
 void bbTrack::clickedTrackLabel( void )
 {
-	bbEditor::inst()->setCurrentBB( s_bbNums[this].num );
-	bbEditor::inst()->show();
+	eng()->getBBEditor()->setCurrentBB( s_infoMap[this] );
+	eng()->getBBEditor()->show();
 }
 
 
@@ -516,3 +541,5 @@ void bbTrack::clickedTrackLabel( void )
 
 #include "bb_track.moc"
 
+
+#endif
