@@ -1,8 +1,10 @@
+#ifndef SINGLE_SOURCE_COMPILE
+
 /*
  * pattern.cpp - implementation of class pattern which holds notes
  *
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
- * Copyright (c) 2005 Danny McRae <khjklujn@yahoo.com>
+ * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2005 Danny McRae <khjklujn/at/yahoo.com>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -29,13 +31,12 @@
 #ifdef QT4
 
 #include <Qt/QtXml>
-#include <QMenu>
-#include <QProgressBar>
-#include <QPushButton>
-#include <QMessageBox>
-#include <QImage>
-#include <QMouseEvent>
-#include <QTimer>
+#include <QtCore/QTimer>
+#include <QtGui/QMenu>
+#include <QtGui/QMessageBox>
+#include <QtGui/QMouseEvent>
+#include <QtGui/QProgressBar>
+#include <QtGui/QPushButton>
 
 #else
 
@@ -54,7 +55,7 @@
 
 
 #include "pattern.h"
-#include "channel_track.h"
+#include "instrument_track.h"
 #include "templates.h"
 #include "gui_templates.h"
 #include "embed.h"
@@ -67,9 +68,10 @@
 #include "tooltip.h"
 #include "bb_editor.h"
 #include "string_pair_drag.h"
+#include "buffer_allocator.h"
+#include "main_window.h"
 
 
-QPixmap * pattern::s_patternBg = NULL;
 QPixmap * pattern::s_stepBtnOn = NULL;
 QPixmap * pattern::s_stepBtnOverlay = NULL;
 QPixmap * pattern::s_stepBtnOff = NULL;
@@ -78,12 +80,13 @@ QPixmap * pattern::s_frozen = NULL;
 
 
 
-
-pattern::pattern ( channelTrack * _channel_track ) :
-	trackContentObject( _channel_track ),
-	m_channelTrack( _channel_track ),
+pattern::pattern ( instrumentTrack * _instrument_track ) :
+	trackContentObject( _instrument_track ),
+	m_paintPixmap(),
+	m_needsUpdate( TRUE ),
+	m_instrumentTrack( _instrument_track ),
 	m_patternType( BEAT_PATTERN ),
-	m_name( _channel_track->name() ),
+	m_name( _instrument_track->name() ),
 	m_steps( DEFAULT_STEPS_PER_TACT ),
 	m_frozenPatternMutex(),
 	m_frozenPattern( NULL ),
@@ -97,8 +100,10 @@ pattern::pattern ( channelTrack * _channel_track ) :
 
 
 pattern::pattern( const pattern & _pat_to_copy ) :
-	trackContentObject( _pat_to_copy.m_channelTrack ),
-	m_channelTrack( _pat_to_copy.m_channelTrack ),
+	trackContentObject( _pat_to_copy.m_instrumentTrack ),
+	m_paintPixmap(),
+	m_needsUpdate( TRUE ),
+	m_instrumentTrack( _pat_to_copy.m_instrumentTrack ),
 	m_patternType( _pat_to_copy.m_patternType ),
 	m_name( "" ),
 	m_steps( _pat_to_copy.m_steps ),
@@ -120,10 +125,19 @@ pattern::pattern( const pattern & _pat_to_copy ) :
 
 pattern::~pattern()
 {
-	if( pianoRoll::inst()->currentPattern() == this )
+	if( eng()->getPianoRoll()->currentPattern() == this )
 	{
-		pianoRoll::inst()->setCurrentPattern( NULL );
+		eng()->getPianoRoll()->setCurrentPattern( NULL );
+		// we have to have the song-editor to stop playing if it played
+		// us before
+		if( eng()->getSongEditor()->playing() &&
+			eng()->getSongEditor()->playMode() ==
+						songEditor::PLAY_PATTERN )
+		{
+			eng()->getSongEditor()->playPattern( NULL );
+		}
 	}
+
 	for( noteVector::iterator it = m_notes.begin();
 						it != m_notes.end(); ++it )
 	{
@@ -142,46 +156,48 @@ pattern::~pattern()
 
 void pattern::init( void )
 {
-	if( s_patternBg == NULL )
-	{
-		s_patternBg = new QPixmap( embed::getIconPixmap(
-							"pattern_bg" ) );
-	}
 	if( s_stepBtnOn == NULL )
 	{
 		s_stepBtnOn = new QPixmap( embed::getIconPixmap(
 							"step_btn_on_100" ) );
 	}
+
 	if( s_stepBtnOverlay == NULL )
 	{
 		s_stepBtnOverlay = new QPixmap( embed::getIconPixmap(
 						"step_btn_on_yellow" ) );
 	}
+
 	if( s_stepBtnOff == NULL )
 	{
 		s_stepBtnOff = new QPixmap( embed::getIconPixmap(
 							"step_btn_off" ) );
 	}
+
 	if( s_stepBtnOffLight == NULL )
 	{
 		s_stepBtnOffLight = new QPixmap( embed::getIconPixmap(
 						"step_btn_off_light" ) );
 	}
+
 	if( s_frozen == NULL )
 	{
 		s_frozen = new QPixmap( embed::getIconPixmap( "frozen" ) );
 	}
 
+	saveJournallingState( FALSE );
+
 	ensureBeatNotes();
 
-#ifndef QT4
+	changeLength( length() );
+	restoreJournallingState();
+
+#ifdef QT3
 	// set background-mode for flicker-free redraw
 	setBackgroundMode( Qt::NoBackground );
 #endif
 
-	setFixedHeight( s_patternBg->height() + 4 );
-	changeLength( length() );
-
+	setFixedHeight( parentWidget()->height() - 2 );
 	setAutoResizeEnabled( FALSE );
 
 	toolTip::add( this,
@@ -192,15 +208,6 @@ void pattern::init( void )
 
 
 
-void pattern::movePosition( const midiTime & _pos )
-{
-	// patterns are always aligned on tact-boundaries
-	trackContentObject::movePosition( midiTime( _pos.getTact(), 0 ) );
-}
-
-
-
-
 midiTime pattern::length( void ) const
 {
 	if( m_patternType == BEAT_PATTERN )
@@ -232,9 +239,13 @@ midiTime pattern::length( void ) const
 
 
 
-note * pattern::addNote( const note & _new_note )
+note * pattern::addNote( const note & _new_note, const bool _quant_pos )
 {
 	note * new_note = new note( _new_note );
+	if( _quant_pos )
+	{
+		new_note->quantizePos( eng()->getPianoRoll()->quantization() );
+	}
 
 	if( m_notes.size() == 0 || m_notes.back()->pos() <= new_note->pos() )
 	{
@@ -295,14 +306,15 @@ void pattern::removeNote( const note * _note_to_del )
 
 
 
-note * pattern::rearrangeNote( const note * _note_to_proc )
+note * pattern::rearrangeNote( const note * _note_to_proc,
+							const bool _quant_pos )
 {
 	// just rearrange the position of the note by removing it and adding 
 	// a copy of it -> addNote inserts it at the correct position
 	note copy_of_note( *_note_to_proc );
 	removeNote( _note_to_proc );
 
-	return( addNote( copy_of_note ) );
+	return( addNote( copy_of_note, _quant_pos ) );
 }
 
 
@@ -319,6 +331,10 @@ void pattern::clearNotes( void )
 	m_notes.clear();
 	checkType();
 	update();
+	if( eng()->getPianoRoll()->currentPattern() == this )
+	{
+		eng()->getPianoRoll()->update();
+	}
 }
 
 
@@ -380,8 +396,8 @@ void pattern::checkType( void )
 
 
 
-void pattern::playFrozenData( sampleFrame * _ab, Uint32 _start_frame,
-								Uint32 _frames )
+void pattern::playFrozenData( sampleFrame * _ab, const f_cnt_t _start_frame,
+							const fpab_t _frames )
 {
 	m_frozenPatternMutex.lock();
 	if( m_frozenPattern != NULL )
@@ -394,28 +410,26 @@ void pattern::playFrozenData( sampleFrame * _ab, Uint32 _start_frame,
 
 
 
-void pattern::saveSettings( QDomDocument & _doc, QDomElement & _parent )
+void pattern::saveSettings( QDomDocument & _doc, QDomElement & _this )
 {
-	QDomElement pattern_de = _doc.createElement( nodeName() );
-	pattern_de.setAttribute( "type", QString::number( m_patternType ) );
-	pattern_de.setAttribute( "name", m_name );
+	_this.setAttribute( "type", m_patternType );
+	_this.setAttribute( "name", m_name );
 	// as the target of copied/dragged pattern is always an existing
 	// pattern, we must not store actual position, instead we store -1
 	// which tells loadSettings() not to mess around with position
-	if( _parent.nodeName() == "clipboard" ||
-					_parent.nodeName() == "dnddata" )
+	if( _this.parentNode().nodeName() == "clipboard" ||
+			_this.parentNode().nodeName() == "dnddata" )
 	{
-		pattern_de.setAttribute( "pos", QString::number( -1 ) );
+		_this.setAttribute( "pos", -1 );
 	}
 	else
 	{
-		pattern_de.setAttribute( "pos", QString::number(
-							startPosition() ) );
+		_this.setAttribute( "pos", startPosition() );
 	}
-	pattern_de.setAttribute( "len", QString::number( length() ) );
-	pattern_de.setAttribute( "frozen", QString::number(
-						m_frozenPattern != NULL ) );
-	_parent.appendChild( pattern_de );
+	_this.setAttribute( "len", length() );
+	_this.setAttribute( "muted", muted() );
+	_this.setAttribute( "steps", m_steps );
+	_this.setAttribute( "frozen", m_frozenPattern != NULL );
 
 	// now save settings of all notes
 	for( noteVector::iterator it = m_notes.begin();
@@ -423,7 +437,7 @@ void pattern::saveSettings( QDomDocument & _doc, QDomElement & _parent )
 	{
 		if( ( *it )->length() )
 		{
-			( *it )->saveSettings( _doc, pattern_de );
+			( *it )->saveState( _doc, _this );
 		}
 	}
 }
@@ -443,16 +457,21 @@ void pattern::loadSettings( const QDomElement & _this )
 		movePosition( _this.attribute( "pos" ).toInt() );
 	}
 	changeLength( midiTime( _this.attribute( "len" ).toInt() ) );
+	if( _this.attribute( "muted" ).toInt() != muted() )
+	{
+		toggleMute();
+	}
 
 	clearNotes();
 
 	QDomNode node = _this.firstChild();
 	while( !node.isNull() )
 	{
-		if( node.isElement() )
+		if( node.isElement() &&
+			!node.toElement().attribute( "metadata" ).toInt() )
 		{
-			note * n = new note();
-			n->loadSettings( node.toElement() );
+			note * n = new note( eng() );
+			n->restoreState( node.toElement() );
 			m_notes.push_back( n );
 		}
 		node = node.nextSibling();
@@ -475,6 +494,15 @@ void pattern::loadSettings( const QDomElement & _this )
 
 
 
+void pattern::update( void )
+{
+	m_needsUpdate = TRUE;
+	changeLength( length() );
+	trackContentObject::update();
+}
+
+
+
 
 void pattern::openInPianoRoll( void )
 {
@@ -486,10 +514,9 @@ void pattern::openInPianoRoll( void )
 
 void pattern::openInPianoRoll( bool )
 {
-	pianoRoll::inst()->setCurrentPattern( this );
-	pianoRoll::inst()->show();
-	pianoRoll::inst()->setFocus();
-	return;
+	eng()->getPianoRoll()->setCurrentPattern( this );
+	eng()->getPianoRoll()->show();
+	eng()->getPianoRoll()->setFocus();
 }
 
 
@@ -506,7 +533,7 @@ void pattern::clear( void )
 
 void pattern::resetName( void )
 {
-	m_name = m_channelTrack->name();
+	m_name = m_instrumentTrack->name();
 }
 
 
@@ -523,7 +550,7 @@ void pattern::changeName( void )
 
 void pattern::freeze( void )
 {
-	if( songEditor::inst()->playing() )
+	if( eng()->getSongEditor()->playing() )
 	{
 		QMessageBox::information( 0, tr( "Cannot freeze pattern" ),
 						tr( "The pattern currently "
@@ -534,7 +561,7 @@ void pattern::freeze( void )
 						QMessageBox::Ok );
 		return;
 	}
-	if( m_channelTrack->muted() )
+	if( m_instrumentTrack->muted() || muted() )
 	{
 		if( QMessageBox::
 #if QT_VERSION >= 0x030200		
@@ -543,9 +570,10 @@ void pattern::freeze( void )
 				 information
 #endif				 
 		
-					    ( 0, tr( "Channel muted" ),
-						tr( "The channel this pattern "
-							"belongs to is "
+					    ( 0, tr( "Pattern muted" ),
+						tr( "The track this pattern "
+							"belongs to or the "
+							"pattern itself is "
 							"currently muted "
 							"therefore "
 							"freezing makes no "
@@ -583,6 +611,7 @@ void pattern::unfreeze( void )
 		delete m_frozenPattern;
 		m_frozenPattern = NULL;
 		m_frozenPatternMutex.unlock();
+		update();
 	}
 }
 
@@ -763,11 +792,6 @@ void pattern::mouseDoubleClickEvent( QMouseEvent * _me )
 
 void pattern::mousePressEvent( QMouseEvent * _me )
 {
-/*	if( _me->button() != Qt::LeftButton )
-	{
-	return;
-}*/
-
 	if( _me->button() == Qt::LeftButton &&
 		   m_patternType == pattern::BEAT_PATTERN &&
 		   ( pixelsPerTact() >= 192 ||
@@ -789,16 +813,21 @@ void pattern::mousePressEvent( QMouseEvent * _me )
 		{
 			n->setLength( -64 );
 		}
-		songEditor::inst()->setModified();
+		eng()->getSongEditor()->setModified();
 		update();
+		if( eng()->getPianoRoll()->currentPattern() == this )
+		{
+			eng()->getPianoRoll()->update();
+		}
 	}
 	else if( m_frozenPattern != NULL && _me->button() == Qt::LeftButton &&
-			lmmsMainWin::isShiftPressed() == TRUE )
+			eng()->getMainWindow()->isShiftPressed() == TRUE )
 	{
+		QString s;
 		new stringPairDrag( "sampledata",
-				    m_frozenPattern->toBase64(),
-				    embed::getIconPixmap( "freeze" ),
-				    this );
+					m_frozenPattern->toBase64( s ),
+					    embed::getIconPixmap( "freeze" ),
+					    this, eng() );
 	}
 	else
 	{
@@ -848,8 +877,17 @@ void pattern::wheelEvent( QWheelEvent * _we )
 				n->setLength( 0 );
 			}
 		}
-		songEditor::inst()->setModified();
+		eng()->getSongEditor()->setModified();
 		update();
+		if( eng()->getPianoRoll()->currentPattern() == this )
+		{
+			eng()->getPianoRoll()->update();
+		}
+		_we->accept();
+	}
+	else
+	{
+		trackContentObject::wheelEvent( _we );
 	}
 }
 
@@ -858,22 +896,60 @@ void pattern::wheelEvent( QWheelEvent * _we )
 
 void pattern::paintEvent( QPaintEvent * )
 {
+	if( m_needsUpdate == FALSE )
+	{
+		QPainter p( this );
+		p.drawPixmap( 0, 0, m_paintPixmap );
+		return;
+	}
+
 	changeLength( length() );
 
-#ifdef QT4
-	QPainter p( this );
-#else
-	// create pixmap for whole widget
-	QPixmap pm( rect().size() );
+	m_needsUpdate = FALSE;
 
-	// and a painter for it
-	QPainter p( &pm );
+	if( m_paintPixmap.isNull() == TRUE || m_paintPixmap.size() != size() )
+	{
+		m_paintPixmap = QPixmap( size() );
+	}
+
+	QPainter p( &m_paintPixmap );
+#ifdef QT4
+	QLinearGradient lingrad( 0, 0, 0, height() );
+	const QColor c = isSelected() ? QColor( 0, 0, 224 ) :
+							QColor( 96, 96, 96 );
+	lingrad.setColorAt( 0, c );
+	lingrad.setColorAt( 0.5, Qt::black );
+	lingrad.setColorAt( 1, c );
+	p.fillRect( QRect( 1, 1, width() - 2, height() - 2 ), lingrad );
+#else
+	for( int y = 1; y < height() / 2; ++y )
+	{
+		const int gray = 96 - y * 192 / height();
+		if( isSelected() == TRUE )
+		{
+			p.setPen( QColor( 0, 0, 128 + gray ) );
+		}
+		else
+		{
+			p.setPen( QColor( gray, gray, gray ) );
+		}
+		p.drawLine( 1, y, width() - 1, y );
+	}
+	for( int y = height() / 2; y < height() - 1; ++y )
+	{
+		const int gray = ( y - height() / 2 ) * 192 / height();
+		if( isSelected() == TRUE )
+		{
+			p.setPen( QColor( 0, 0, 128 + gray ) );
+		}
+		else
+		{
+			p.setPen( QColor( gray, gray, gray ) );
+		}
+		p.drawLine( 1, y, width() - 1, y );
+	}
 #endif
 
-	for( Sint16 x = 2; x < width() - 1; x += 2 )
-	{
-		p.drawPixmap( x, 2, *s_patternBg );
-	}
 	p.setPen( QColor( 57, 69, 74 ) );
 	p.drawLine( 0, 0, width(), 0 );
 	p.drawLine( 0, 0, 0, height() );
@@ -884,7 +960,7 @@ void pattern::paintEvent( QPaintEvent * )
 	p.setPen( QColor( 0, 0, 0 ) );
 	p.drawRect( 1, 1, width() - 2, height() - 2 );
 
-	float ppt = pixelsPerTact();
+	const float ppt = pixelsPerTact();
 
 	if( m_patternType == pattern::MELODY_PATTERN )
 	{
@@ -908,7 +984,7 @@ void pattern::paintEvent( QPaintEvent * )
 			{
 				central_key = central_key / total_notes;
 
-				Sint16 central_y = s_patternBg->height() / 2;
+				Sint16 central_y = height() / 2;
 				Sint16 y_base = central_y + TCO_BORDER_WIDTH -1;
 
 				const Sint16 x_base = TCO_BORDER_WIDTH;
@@ -917,15 +993,16 @@ void pattern::paintEvent( QPaintEvent * )
 				for( tact tact_num = 1; tact_num <
 						length().getTact(); ++tact_num )
 				{
-					p.drawLine( x_base + static_cast<int>(
-								ppt*tact_num ),
-							TCO_BORDER_WIDTH,
-							x_base +
-					static_cast<int>( ppt * tact_num ),
-							height() - 2 *
+					p.drawLine(
+						x_base + static_cast<int>(
+							ppt * tact_num ) - 1,
+						TCO_BORDER_WIDTH,
+						x_base + static_cast<int>(
+							ppt * tact_num ) - 1,
+						height() - 2 *
 							TCO_BORDER_WIDTH );
 				}
-				if( getTrack()->muted() )
+				if( getTrack()->muted() || muted() )
 				{
 					p.setPen( QColor( 160, 160, 160 ) );
 				}
@@ -961,42 +1038,42 @@ void pattern::paintEvent( QPaintEvent * )
 		}
 	}
 	else if( m_patternType == pattern::BEAT_PATTERN &&
-			( ppt >= 192 || m_steps != DEFAULT_STEPS_PER_TACT ) )
+			( ppt >= 96 || m_steps != DEFAULT_STEPS_PER_TACT ) )
 	{
 		QPixmap stepon;
 		QPixmap stepoverlay;
 		QPixmap stepoff;
 		QPixmap stepoffl;
-		int steps = length() / BEATS_PER_TACT;
-		
+		const int steps = length() / BEATS_PER_TACT;
+		const int w = width() - 2 * TCO_BORDER_WIDTH;
 #ifdef QT4
-		stepon = s_stepBtnOn->scaled( width() / steps,
+		stepon = s_stepBtnOn->scaled( w / steps,
 					      s_stepBtnOn->height(),
 					      Qt::IgnoreAspectRatio,
 					      Qt::SmoothTransformation );
-		stepoverlay = s_stepBtnOverlay->scaled( width() / steps,
+		stepoverlay = s_stepBtnOverlay->scaled( w / steps,
 					      s_stepBtnOn->height(),
 					      Qt::IgnoreAspectRatio,
 					      Qt::SmoothTransformation );
-		stepoff = s_stepBtnOff->scaled( width() / steps,
+		stepoff = s_stepBtnOff->scaled( w / steps,
 						s_stepBtnOff->height(),
 						Qt::IgnoreAspectRatio,
 						Qt::SmoothTransformation );
-		stepoffl = s_stepBtnOffLight->scaled( width() / steps,
+		stepoffl = s_stepBtnOffLight->scaled( w / steps,
 						s_stepBtnOffLight->height(),
 						Qt::IgnoreAspectRatio,
 						Qt::SmoothTransformation );
 #else
 		stepon.convertFromImage( 
 				s_stepBtnOn->convertToImage().scale(
-				width() / steps, s_stepBtnOn->height() ) );
+					w / steps, s_stepBtnOn->height() ) );
 		stepoverlay.convertFromImage( 
 				s_stepBtnOverlay->convertToImage().scale(
-				width() / steps, s_stepBtnOverlay->height() ) );
+				w / steps, s_stepBtnOverlay->height() ) );
 		stepoff.convertFromImage( s_stepBtnOff->convertToImage().scale(
-				width() / steps, s_stepBtnOff->height() ) );
+					w / steps, s_stepBtnOff->height() ) );
 		stepoffl.convertFromImage( s_stepBtnOffLight->convertToImage().
-					scale( width() / steps,
+					scale( w / steps,
 						s_stepBtnOffLight->height() ) );
 #endif
 		for( noteVector::iterator it = m_notes.begin();
@@ -1004,7 +1081,7 @@ void pattern::paintEvent( QPaintEvent * )
 		{
 			Sint16 no = it - m_notes.begin();
 			Sint16 x = TCO_BORDER_WIDTH + static_cast<int>( no *
-							width() / steps );
+								w / steps );
 			Sint16 y = height() - s_stepBtnOff->height() - 1;
 			
 			Uint8 vol = ( *it )->getVolume();
@@ -1034,19 +1111,32 @@ void pattern::paintEvent( QPaintEvent * )
 	}
 
 	p.setFont( pointSize<7>( p.font() ) );
-	p.setPen( QColor( 32, 240, 32 ) );
-	p.drawText( 2, 9, m_name );
-	if( m_frozenPattern != NULL )
+	if( muted() || getTrack()->muted() )
+	{
+		p.setPen( QColor( 192, 192, 192 ) );
+	}
+	else
+	{
+		p.setPen( QColor( 32, 240, 32 ) );
+	}
+	p.drawText( 2, p.fontMetrics().height() - 1, m_name );
+	if( muted() )
+	{
+		p.drawPixmap( 3, p.fontMetrics().height() + 1,
+				embed::getIconPixmap( "muted", 16, 16 ) );
+	}
+	else if( m_frozenPattern != NULL )
 	{
 		p.setPen( QColor( 0, 224, 255 ) );
 		p.drawRect( 0, 0, width(), height() - 1 );
 		p.drawPixmap( 3, height() - s_frozen->height() - 4, *s_frozen );
 	}
 
-#ifndef QT4
-	// blit drawn pixmap to actual widget
-	bitBlt( this, rect().topLeft(), &pm );
-#endif
+	p.end();
+
+	p.begin( this );
+	p.drawPixmap( 0, 0, m_paintPixmap );
+
 }
 
 
@@ -1070,7 +1160,7 @@ void pattern::ensureBeatNotes( void )
 		}
 		if( found == FALSE )
 		{
-			addNote( note( midiTime( 0 ), midiTime( i *
+			addNote( note( eng(), midiTime( 0 ), midiTime( i *
 							BEATS_PER_TACT ) ) );
 		}
 	}
@@ -1081,9 +1171,9 @@ void pattern::ensureBeatNotes( void )
 
 void pattern::updateBBTrack( void )
 {
-	if( getTrack()->getTrackContainer() == bbEditor::inst() )
+	if( getTrack()->getTrackContainer() == eng()->getBBEditor() )
 	{
-		bbEditor::inst()->updateBBTrack( this );
+		eng()->getBBEditor()->updateBBTrack( this );
 	}
 }
 
@@ -1095,7 +1185,8 @@ void pattern::updateBBTrack( void )
 
 patternFreezeStatusDialog::patternFreezeStatusDialog( QThread * _thread ) :
 	QDialog(),
-	m_freezeThread( _thread )
+	m_freezeThread( _thread ),
+	m_progress( 0 )
 {
 	setWindowTitle( tr( "Freezing pattern..." ) );
 #if QT_VERSION >= 0x030200
@@ -1197,8 +1288,10 @@ void patternFreezeStatusDialog::updateProgress( void )
 
 patternFreezeThread::patternFreezeThread( pattern * _pattern ) :
 	QThread(),
+	engineObject( _pattern->eng() ),
 	m_pattern( _pattern )
 {
+	// create status-dialog
 	m_statusDlg = new patternFreezeStatusDialog( this );
 	QObject::connect( m_statusDlg, SIGNAL( aborted() ),
 					m_pattern, SLOT( abortFreeze() ) );
@@ -1211,6 +1304,7 @@ patternFreezeThread::patternFreezeThread( pattern * _pattern ) :
 
 patternFreezeThread::~patternFreezeThread()
 {
+	m_pattern->update();
 }
 
 
@@ -1218,29 +1312,33 @@ patternFreezeThread::~patternFreezeThread()
 
 void patternFreezeThread::run( void )
 {
+	bufferAllocator::disableAutoCleanup( TRUE );
+
 	// create and install audio-sample-recorder
 	bool b;
 	// we cannot create local copy, because at a later stage
 	// mixer::restoreAudioDevice(...) deletes old audio-dev and thus
 	// audioSampleRecorder would be destroyed two times...
 	audioSampleRecorder * freeze_recorder = new audioSampleRecorder(
-			mixer::inst()->sampleRate(), DEFAULT_CHANNELS, b );
-	mixer::inst()->setAudioDevice( freeze_recorder,
-						mixer::inst()->highQuality() );
+						eng()->getMixer()->sampleRate(),
+							DEFAULT_CHANNELS, b,
+							eng()->getMixer() );
+	eng()->getMixer()->setAudioDevice( freeze_recorder,
+					eng()->getMixer()->highQuality() );
 
 	// prepare stuff for playing correct things later
-	songEditor::inst()->playPattern( m_pattern, FALSE );
-	songEditor::playPos & ppp = songEditor::inst()->getPlayPos(
+	eng()->getSongEditor()->playPattern( m_pattern, FALSE );
+	songEditor::playPos & ppp = eng()->getSongEditor()->getPlayPos(
 						songEditor::PLAY_PATTERN );
 	ppp.setTact( 0 );
 	ppp.setTact64th( 0 );
 	ppp.setCurrentFrame( 0 );
 	ppp.m_timeLineUpdate = FALSE;
 
-	// create status-dialog
 	m_pattern->m_freezeAborted = FALSE;
 	m_pattern->m_freezing = TRUE;
 
+
 	// now render everything
 	while( ppp < m_pattern->length() &&
 					m_pattern->m_freezeAborted == FALSE )
@@ -1249,10 +1347,11 @@ void patternFreezeThread::run( void )
 		m_statusDlg->setProgress( ppp * 100 / m_pattern->length() );
 	}
 
+
 	m_pattern->m_freezing = FALSE;
 
 	// reset song-editor settings
-	songEditor::inst()->stop();
+	eng()->getSongEditor()->stop();
 	ppp.m_timeLineUpdate = TRUE;
 
 	// create final sample-buffer if freezing was successful
@@ -1264,8 +1363,10 @@ void patternFreezeThread::run( void )
 		m_pattern->m_frozenPatternMutex.unlock();
 	}
 
+	bufferAllocator::disableAutoCleanup( FALSE );
+
 	// restore original audio-device
-	mixer::inst()->restoreAudioDevice();
+	eng()->getMixer()->restoreAudioDevice();
 
 	m_statusDlg->setProgress( -1 );	// we're finished
 
@@ -1277,3 +1378,5 @@ void patternFreezeThread::run( void )
 
 #include "pattern.moc"
 
+
+#endif
