@@ -1,8 +1,10 @@
+#ifndef SINGLE_SOURCE_COMPILE
+
 /*
  * sample_track.cpp - implementation of class sampleTrack, a track which
  *                    provides arrangement of samples
  *
- * Copyright (c) 2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2005-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -28,16 +30,17 @@
 
 #ifdef QT4
 
-#include <QPushButton>
-#include <QPainter>
 #include <Qt/QtXml>
-#include <QDropEvent>
+#include <QtGui/QDropEvent>
+#include <QtGui/QPainter>
+#include <QtGui/QPushButton>
 
 #else
 
 #include <qpushbutton.h>
 #include <qpainter.h>
 #include <qdom.h>
+#include <qwhatsthis.h>
 
 #endif
 
@@ -51,23 +54,26 @@
 #include "tooltip.h"
 #include "audio_port.h"
 #include "string_pair_drag.h"
-
+#include "knob.h"
+#include "volume.h"
 
 
 sampleTCO::sampleTCO( track * _track ) :
 	trackContentObject( _track ),
-	m_sampleBuffer()
+	m_sampleBuffer( eng() )
 {
 #ifndef QT4
 	setBackgroundMode( Qt::NoBackground );
 #endif
 
+	saveJournallingState( FALSE );
 	setSampleFile( "" );
+	restoreJournallingState();
 
 	// we need to receive bpm-change-events, because then we have to
 	// change length of this TCO
-	connect( songEditor::inst(), SIGNAL( bpmChanged( int ) ), this,
-						SLOT( updateLength( int ) ) );
+	connect( eng()->getSongEditor(), SIGNAL( tempoChanged( bpm_t ) ), this,
+						SLOT( updateLength( bpm_t ) ) );
 }
 
 
@@ -89,12 +95,12 @@ void sampleTCO::changeLength( const midiTime & _length )
 
 
 
-void FASTCALL sampleTCO::play( sampleFrame * _ab, Uint32 _start_frame,
-								Uint32 _frames )
+void FASTCALL sampleTCO::play( sampleFrame * _ab, f_cnt_t _start_frame,
+							const fpab_t _frames )
 {
 	_start_frame = static_cast<Uint32>( tMax( 0.0f, _start_frame -
 							startPosition() *
-				songEditor::inst()->framesPerTact() / 64 ) );
+			eng()->getSongEditor()->framesPerTact() / 64 ) );
 	m_sampleBuffer.play( _ab, _start_frame, _frames );
 }
 
@@ -124,7 +130,7 @@ void sampleTCO::setSampleFile( const QString & _sf )
 
 
 
-void sampleTCO::updateLength( int )
+void sampleTCO::updateLength( bpm_t )
 {
 	changeLength( getSampleLength() );
 }
@@ -155,7 +161,7 @@ void sampleTCO::dropEvent( QDropEvent * _de )
 	{
 		m_sampleBuffer.loadFromBase64(
 					stringPairDrag::decodeValue( _de ) );
-		songEditor::inst()->setModified();
+		eng()->getSongEditor()->setModified();
 		updateLength();
 		update();
 		_de->accept();
@@ -175,31 +181,44 @@ void sampleTCO::mouseDoubleClickEvent( QMouseEvent * )
 	if( af != "" && af != m_sampleBuffer.audioFile() )
 	{
 		setSampleFile( af );
-		songEditor::inst()->setModified();
+		eng()->getSongEditor()->setModified();
 	}
 }
 
 
 
 
-void sampleTCO::paintEvent( QPaintEvent * )
+void sampleTCO::paintEvent( QPaintEvent * _pe )
 {
 #ifdef QT4
 	QPainter p( this );
+#warning TODO: set according brush for gradient!
+	p.fillRect( _pe->rect(), QColor( 64, 64, 64 ) );
 #else
-	// create pixmap for whole widget
-	QPixmap pm( size() );
+	// create pixmap for our widget
+	QPixmap pm( _pe->rect().size() );
 	// and a painter for it
 	QPainter p( &pm );
-#endif
-	QPixmap bg = embed::getIconPixmap( "sample_track_bg" );
-	for( Sint16 x = 1; x < width() - 1; x += 10 )
+	p.translate( -_pe->rect().x(), -_pe->rect().y() );
+
+	for( int y = 1; y < height() - 1; ++y )
 	{
-		p.drawPixmap( x, 1, bg );
+		const int gray = 96 - y * 96 / height();
+		if( isSelected() == TRUE )
+		{
+			p.setPen( QColor( 0, 0, 128 + gray ) );
+		}
+		else
+		{
+			p.setPen( QColor( gray, gray, gray ) );
+		}
+		p.drawLine( 1, y, width() - 1, y );
 	}
+#endif
+
 	p.setPen( QColor( 0, 0, 0 ) );
 	p.drawRect( 0, 0, width(), height() );
-	if( getTrack()->muted() )
+	if( getTrack()->muted() || muted() )
 	{
 		p.setPen( QColor( 128, 128, 128 ) );
 	}
@@ -211,14 +230,21 @@ void sampleTCO::paintEvent( QPaintEvent * )
 			tMax( static_cast<int>( getSampleLength() *
 				pixelsPerTact() / 64 ), 1 ), height() - 4 );
 	p.setClipRect( QRect( 1, 1, width() - 2, height() - 2 ) );
-	m_sampleBuffer.drawWaves( p, r );
+	m_sampleBuffer.visualize( p, r, _pe->rect() );
 	if( r.width() < width() - 1 )
 	{
 		p.drawLine( r.x() + r.width(), r.y() + r.height() / 2,
 				width() - 2, r.y() + r.height() / 2 );
 	}
+
+	p.translate( 0, 0 );
+	if( muted() )
+	{
+		p.drawPixmap( 3, 8, embed::getIconPixmap( "muted", 16, 16 ) );
+	}
+
 #ifndef QT4
-	bitBlt( this, rect().topLeft(), &pm );
+	bitBlt( this, _pe->rect().topLeft(), &pm );
 #endif
 }
 
@@ -228,7 +254,7 @@ void sampleTCO::paintEvent( QPaintEvent * )
 midiTime sampleTCO::getSampleLength( void ) const
 {
 	return( static_cast<Sint32>( m_sampleBuffer.frames() /
-					songEditor::inst()->framesPerTact() *
+					eng()->getSongEditor()->framesPerTact() *
 									64 ) );
 }
 
@@ -236,26 +262,25 @@ midiTime sampleTCO::getSampleLength( void ) const
 
 
 void FASTCALL sampleTCO::saveSettings( QDomDocument & _doc,
-							QDomElement & _parent )
+							QDomElement & _this )
 {
-	QDomElement sampletco_de = _doc.createElement( nodeName() );
-	if( _parent.nodeName() == "clipboard" )
+	if( _this.parentNode().nodeName() == "clipboard" )
 	{
-		sampletco_de.setAttribute( "pos", QString::number( -1 ) );
+		_this.setAttribute( "pos", -1 );
 	}
 	else
 	{
-		sampletco_de.setAttribute( "pos", QString::number(
-							startPosition() ) );
+		_this.setAttribute( "pos", startPosition() );
 	}
-	sampletco_de.setAttribute( "len", QString::number( length() ) );
-	sampletco_de.setAttribute( "src", sampleFile() );
+	_this.setAttribute( "len", length() );
+	_this.setAttribute( "muted", muted() );
+	_this.setAttribute( "src", sampleFile() );
 	if( sampleFile() == "" )
 	{
-		sampletco_de.setAttribute( "data", m_sampleBuffer.toBase64() );
+		QString s;
+		_this.setAttribute( "data", m_sampleBuffer.toBase64( s ) );
 	}
 	// TODO: start- and end-frame
-	_parent.appendChild( sampletco_de );
 }
 
 
@@ -273,6 +298,10 @@ void FASTCALL sampleTCO::loadSettings( const QDomElement & _this )
 		m_sampleBuffer.loadFromBase64( _this.attribute( "data" ) );
 	}
 	changeLength( _this.attribute( "len" ).toInt() );
+	if( _this.attribute( "muted" ).toInt() != muted() )
+	{
+		toggleMute();
+	}
 }
 
 
@@ -334,7 +363,7 @@ void sampleTCOSettingsDialog::setSampleFile( const QString & _f )
 {
 	m_fileLbl->setText( _f );
 	m_sampleTCO->setSampleFile( _f );
-	songEditor::inst()->setModified();
+	eng()->getSongEditor()->setModified();
 }
 */
 
@@ -344,17 +373,36 @@ void sampleTCOSettingsDialog::setSampleFile( const QString & _f )
 
 sampleTrack::sampleTrack( trackContainer * _tc ) :
 	track( _tc ),
-	m_audioPort( new audioPort( tr( "Sample track" ) ) )
+	m_audioPort( new audioPort( tr( "Sample track" ), eng() ) ),
+	m_volume( 1.0f )
 {
 	getTrackWidget()->setFixedHeight( 32 );
 
 	m_trackLabel = new nameLabel( tr( "Sample track" ),
-						getTrackSettingsWidget(),
-						embed::getIconPixmap(
-							"sample_track" ) );
-	m_trackLabel->setGeometry( 1, 1, DEFAULT_SETTINGS_WIDGET_WIDTH-2, 29 );
+					getTrackSettingsWidget(), eng() );
+	m_trackLabel->setPixmap( embed::getIconPixmap( "sample_track" ) );
+	m_trackLabel->setGeometry( 26, 1, DEFAULT_SETTINGS_WIDGET_WIDTH-2, 29 );
 	m_trackLabel->show();
 
+	m_volumeKnob = new volumeKnob( knobSmall_17, getTrackSettingsWidget(),
+				    tr( "Channel volume" ), eng(), this );
+	m_volumeKnob->setRange( MIN_VOLUME, MAX_VOLUME, 1.0f );
+	m_volumeKnob->setInitValue( DEFAULT_VOLUME );
+	m_volumeKnob->setHintText( tr( "Channel volume:" ) + " ", "%" );
+	m_volumeKnob->move( 4, 4 );
+	m_volumeKnob->setLabel( tr( "VOL" ) );
+	m_volumeKnob->show();
+	connect( m_volumeKnob, SIGNAL( valueChanged( float ) ), 
+		 this, SLOT( setVolume( float ) ) );
+#ifdef QT4
+	m_volumeKnob->setWhatsThis(
+#else
+	QWhatsThis::add( m_volumeKnob,
+#endif
+		tr( "With this knob you can set "
+			"the volume of the opened "
+			"channel." ) ); 
+	
 	_tc->updateAfterTrackAdd();
 }
 
@@ -368,6 +416,16 @@ sampleTrack::~sampleTrack()
 
 
 
+void sampleTrack::setVolume( float _new_volume )
+{
+	if( _new_volume <= MAX_VOLUME )
+	{
+		m_volume = _new_volume / 100.0f;
+	}
+}
+
+
+
 
 track::trackTypes sampleTrack::type( void ) const
 {
@@ -377,13 +435,17 @@ track::trackTypes sampleTrack::type( void ) const
 
 
 
-bool FASTCALL sampleTrack::play( const midiTime & _start, Uint32 _start_frame,
-					Uint32 _frames, Uint32 _frame_base,
+bool FASTCALL sampleTrack::play( const midiTime & _start,
+						const f_cnt_t _start_frame,
+						const fpab_t _frames,
+						const f_cnt_t _frame_base,
 							Sint16 /*_tco_num*/ )
 {
+	sendMidiTime( _start );
+
 	vlist<trackContentObject *> tcos;
 	getTCOsInRange( tcos, _start, _start+static_cast<Sint32>( _frames * 64 /
-					songEditor::inst()->framesPerTact() ) );
+				eng()->getSongEditor()->framesPerTact() ) );
 
 	if ( tcos.size() == 0 )
 	{
@@ -392,24 +454,25 @@ bool FASTCALL sampleTrack::play( const midiTime & _start, Uint32 _start_frame,
 
 	sampleFrame * buf = bufferAllocator::alloc<sampleFrame>( _frames );
 
-	volumeVector v = { 1.0f, 1.0f
+	volumeVector v = { m_volume, m_volume
 #ifndef DISABLE_SURROUND
-					, 1.0f, 1.0f
+					, m_volume, m_volume
 #endif
 			} ;
-	float fpt = songEditor::inst()->framesPerTact();
+	float fpt = eng()->getSongEditor()->framesPerTact();
 
 	for( vlist<trackContentObject *>::iterator it = tcos.begin();
 							it != tcos.end(); ++it )
 	{
 		sampleTCO * st = dynamic_cast<sampleTCO *>( *it );
-		if( st != NULL )
+		if( st != NULL && !st->muted() )
 		{
 			st->play( buf, _start_frame +
 					static_cast<Uint32>( _start.getTact() *
 									fpt ),
 					_frames );
-			mixer::inst()->bufferToPort( buf, _frames, _frame_base +
+			eng()->getMixer()->bufferToPort( buf, _frames,
+							_frame_base +
 							static_cast<Uint32>(
 					st->startPosition().getTact64th() *
 							fpt / 64.0f ), v,
@@ -435,11 +498,11 @@ trackContentObject * sampleTrack::createTCO( const midiTime & )
 
 
 void sampleTrack::saveTrackSpecificSettings( QDomDocument & _doc,
-							QDomElement & _parent )
+							QDomElement & _this )
 {
-	QDomElement st_de = _doc.createElement( nodeName() );
-	st_de.setAttribute( "name", m_trackLabel->text() );
-	_parent.appendChild( st_de );
+	_this.setAttribute( "name", m_trackLabel->text() );
+	_this.setAttribute( "icon", m_trackLabel->pixmapFile() );
+	m_volumeKnob->saveSettings( _doc, _this, "vol" );
 }
 
 
@@ -448,6 +511,30 @@ void sampleTrack::saveTrackSpecificSettings( QDomDocument & _doc,
 void sampleTrack::loadTrackSpecificSettings( const QDomElement & _this )
 {
 	m_trackLabel->setText( _this.attribute( "name" ) );
+	if( _this.attribute( "icon" ) != "" )
+	{
+		m_trackLabel->setPixmapFile( _this.attribute( "icon" ) );
+	}
+	if( _this.attribute( "vol" ) != "" )
+	{
+		m_volume = _this.attribute( "vol" ).toFloat();
+		m_volumeKnob->setInitValue( m_volume * 100.0f );
+	}
+	else
+	{
+		QDomNode node = _this.namedItem(
+					automationPattern::classNodeName() );
+		if( node.isElement() && node.namedItem( "vol" ).isElement() )
+		{
+			m_volumeKnob->loadSettings( _this, "vol" );
+			m_volume = m_volumeKnob->value() / 100.0f;
+		}
+		else
+		{
+			m_volume = 1.0;
+			m_volumeKnob->setInitValue( 100.0f );
+		}
+	}
 }
 
 
@@ -455,3 +542,5 @@ void sampleTrack::loadTrackSpecificSettings( const QDomElement & _this )
 
 #include "sample_track.moc"
 
+
+#endif
