@@ -1,7 +1,9 @@
+#ifndef SINGLE_SOURCE_COMPILE
+
 /*
  * group_box.cpp - groupbox for LMMS
  *
- * Copyright (c) 2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2005-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -27,9 +29,9 @@
 
 #ifdef QT4
 
-#include <QPainter>
-#include <QTimer>
-#include <QMouseEvent>
+#include <QtCore/QTimer>
+#include <QtGui/QPainter>
+#include <QtGui/QMouseEvent>
 
 #else
 
@@ -37,8 +39,6 @@
 #include <qtimer.h>
 #include <qobjectlist.h>
 
-#define setChecked setOn
-
 #endif
 
 #ifndef __USE_XOPEN
@@ -57,20 +57,28 @@
 QPixmap * groupBox::s_ledBg = NULL;
 
 
-groupBox::groupBox( const QString & _caption, QWidget * _parent ) :
+groupBox::groupBox( const QString & _caption, QWidget * _parent,
+					engine * _engine, track * _track ) :
 	QWidget( _parent ),
+	engineObject( _engine ),
 	m_caption( _caption ),
 	m_origHeight( height() ),
 	m_animating( FALSE )
 {
+#ifndef QT3
+	setAutoFillBackground( TRUE );
+#endif
+
 	if( s_ledBg == NULL )
 	{
 		s_ledBg = new QPixmap( embed::getIconPixmap(
 							"groupbox_led_bg" ) );
 	}
+
 	updatePixmap();
 
-	m_led = new pixmapButton( this );
+	m_led = new pixmapButton( this, _caption, eng(), _track );
+	m_led->setCheckable( TRUE );
 	m_led->move( 2, 3 );
 	m_led->setActiveGraphic( embed::getIconPixmap( "led_green" ) );
 	m_led->setInactiveGraphic( embed::getIconPixmap( "led_off" ) );
@@ -188,17 +196,25 @@ void groupBox::animate( void )
 
 void groupBox::updatePixmap( void )
 {
+#ifndef QT3
+	const int c = 0;
+	QColor bg_color = QApplication::palette().color( QPalette::Active,
+							QPalette::Background );
+#else
+	const int c = 1;
+	QColor bg_color = QApplication::palette().active().background();
+#endif
 	QPixmap pm( size() );
-	pm.fill( QColor( 96, 96, 96 ) );
+	pm.fill( bg_color.dark( 132 ) );
 
 	QPainter p( &pm );
 
 	// outer rect
-	p.setPen( QColor( 64, 64, 64 ) );
-	p.drawRect( 0, 0, width(), height() );
+	p.setPen( bg_color.dark( 200 ) );
+	p.drawRect( 0, 0, width() - 1 + c, height() - 1 + c );
 
 	// brighter line at bottom/right
-	p.setPen( QColor( 160, 160, 160 ) );
+	p.setPen( bg_color.light( 125 ) );
 	p.drawLine( width() - 1, 0, width() - 1, height() - 1 );
 	p.drawLine( 0, height() - 1, width() - 1, height() - 1 );
 
@@ -206,34 +222,38 @@ void groupBox::updatePixmap( void )
 	p.drawPixmap( 2, 2, *s_ledBg );
 
 	// draw groupbox-titlebar
-	p.fillRect( 2, 2, width() - 4, 9, QColor( 30, 45, 60 ) );
+	p.fillRect( 2, 2, width() - 4, 9, bg_color.dark( 300 ) );
 
 	// draw line below titlebar
-	p.setPen( QColor( 0, 0, 0 ) );
+	p.setPen( bg_color.dark( 400 ) );
 	p.drawLine( 2 + s_ledBg->width(), 11, width() - 3, 11 );
 
 	// black inner rect
-	p.drawRect( 1, 1, width() - 2, height() - 2 );
+	p.drawRect( 1, 1, width() - 3 + c, height() - 3 + c );
 
 
-	p.setPen( QColor( 255, 255, 255 ) );
+	//p.setPen( QColor( 255, 255, 255 ) );
+#ifndef QT3
+	p.setPen( palette().color( QPalette::Active, QPalette::ButtonText ) );
+#else
+	p.setPen( colorGroup().buttonText() );
+#endif
 	p.setFont( pointSize<7>( font() ) );
 	p.drawText( 22, 10, m_caption );
 
 #ifdef QT4
 	QPalette pal = palette();
 	pal.setBrush( backgroundRole(), QBrush( pm ) );
-/*	pal.setColor( QPalette::Background, QColor( 96, 96, 96 ) );*/
 	setPalette( pal );
 #else
-	setPaletteBackgroundColor( QColor( 96, 96, 96 ) );
+	setPaletteBackgroundColor( bg_color.dark( 132 ) );
 	setErasePixmap( pm );
 #endif
 }
 
 
 
-#undef setChecked
-
 #include "group_box.moc"
 
+
+#endif
