@@ -1,10 +1,12 @@
+#ifndef SINGLE_SOURCE_COMPILE
+
 /*
  * knob.cpp - powerful knob-widget
  *
- * This file is based on the knob-widget of the Qwt Widget Library from
- * Josef Wilgen
+ * This file is partly based on the knob-widget of the Qwt Widget Library by
+ * Josef Wilgen.
  *
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -30,17 +32,15 @@
 
 #ifdef QT4
 
-#include <QPainter>
-#include <QPalette>
-#include <QBitmap>
-#include <QLabel>
-#include <QStatusBar>
-#include <QMouseEvent>
-#include <QMenu>
-#include <QStatusBar>
-#include <QFontMetrics>
-#include <QApplication>
-#include <QInputDialog>
+#include <QtGui/QApplication>
+#include <QtGui/QBitmap>
+#include <QtGui/QFontMetrics>
+#include <QtGui/QInputDialog>
+#include <QtGui/QLabel>
+#include <QtGui/QMenu>
+#include <QtGui/QMouseEvent>
+#include <QtGui/QPainter>
+#include <QtGui/QPalette>
 
 #else
 
@@ -53,6 +53,8 @@
 #include <qfontmetrics.h>
 #include <qapplication.h>
 #include <qinputdialog.h>
+#include <qcursor.h>
+#include <qwhatsthis.h>
 
 #define addSeparator insertSeparator
 
@@ -64,7 +66,6 @@
 #include <math.h>
 
 #include "knob.h"
-#include "song_editor.h"
 /*#include "midi_client.h"*/
 #include "embed.h"
 #include "spc_bg_hndl_widget.h"
@@ -74,25 +75,24 @@
 #include "gui_templates.h"
 #include "templates.h"
 #include "string_pair_drag.h"
+#include "main_window.h"
 
 
 
-static double MinRelStep = 1.0e-10;
-static double DefaultRelStep = 1.0e-2;
-static double MinEps = 1.0e-10;
-
 
 float knob::s_copiedValue = 0.0f;
 textFloat * knob::s_textFloat = NULL;
 
 
 
-knob::knob( int _knob_num, QWidget * _parent, const QString & _name ) :
+knob::knob( int _knob_num, QWidget * _parent, const QString & _name,
+					engine * _engine, track * _track ) :
 	QWidget( _parent
 #ifndef QT4
 			, _name.ascii()
 #endif
 		),
+	autoObj( _engine, _track ),
 	m_mouseOffset( 0.0f ),
 	m_buttonPressed( FALSE ),
 	m_angle( 0.0f ),
@@ -100,12 +100,6 @@ knob::knob( int _knob_num, QWidget * _parent, const QString & _name ) :
 	m_hintTextBeforeValue( "" ),
 	m_hintTextAfterValue( "" ),
 	m_label( "" ),
-	m_minValue( 0.0f ),
-	m_maxValue( 100.0f ),
-	m_value( 0.0f ),
-	m_exactValue( 0.0f ),
-	m_exactPrevValue( 0.0f ),
-	m_prevValue( 0.0f ),
 	m_initValue( 0.0f )
 {
 	if( s_textFloat == NULL )
@@ -124,12 +118,16 @@ knob::knob( int _knob_num, QWidget * _parent, const QString & _name ) :
 	m_knobPixmap = new QPixmap( embed::getIconPixmap( "knob0" +
 					QString::number( m_knobNum + 1 ) ) );
 #endif
+	if( _track != NULL )
+	{
+		getAutomationPattern();
+	}
+
 	setRange( 0.0f, 100.0f, 1.0f );
 
 	setFixedSize( m_knobPixmap->width(), m_knobPixmap->height() );
 	setTotalAngle( 270.0f );
 	recalcAngle();
-
 }
 
 
@@ -140,9 +138,9 @@ knob::~knob()
 {
 /*	// make sure pointer to this knob isn't used anymore in active
 	// midi-device-class
-	if( mixer::inst()->getMIDIClient()->pitchBendKnob() == this )
+	if( eng()->getMixer()->getMIDIClient()->pitchBendKnob() == this )
 	{
-		mixer::inst()->getMIDIClient()->setPitchBendKnob( NULL );
+		eng()->getMixer()->getMIDIClient()->setPitchBendKnob( NULL );
 	}*/
 }
 
@@ -233,6 +231,14 @@ void knob::drawKnob( QPainter * _p )
 					(int)( ym - ca*re ) + 1 );
 			break;
 		}
+		case knobGreen_17:
+		{
+			_p->setPen( QPen( QColor( 0, 200, 0 ), 2 ) );
+			_p->drawLine( (int)( xm-sa ), (int)( ym-ca ),
+					(int)( xm - sa*radius ),
+					(int)( ym - ca*radius ) );
+			break;
+		}
 	}
 }
 
@@ -244,6 +250,7 @@ void knob::valueChange( void )
 	recalcAngle();
 	update();
 	emit valueChanged( value() );
+	emit valueChanged( data() );
 }
 
 
@@ -261,11 +268,11 @@ float knob::getValue( const QPoint & _p )
 
 		const float arc = atan2( -dx, dy ) * 180.0 / M_PI;
 
-		float new_value = 0.5 * ( m_minValue + m_maxValue ) +
-					arc * ( m_maxValue - m_minValue ) /
+		float new_value = 0.5 * ( minValue() + maxValue() ) +
+					arc * ( maxValue() - minValue() ) /
 								m_totalAngle;
 
-		const float oneTurn = tAbs<float>( m_maxValue - m_minValue ) *
+		const float oneTurn = tAbs<float>( maxValue() - minValue() ) *
 							360.0 / m_totalAngle;
 		const float eqValue = value() + m_mouseOffset;
 
@@ -282,6 +289,10 @@ float knob::getValue( const QPoint & _p )
 		}
 		return( new_value );
 	}
+	if( eng()->getMainWindow()->isShiftPressed() )
+	{
+		return( ( _p.y() - m_origMousePos.y() ) * step() );
+	}
 	return( ( _p.y() - m_origMousePos.y() ) * m_pageSize );
 }
 
@@ -316,14 +327,14 @@ void knob::recalcAngle( void )
 	//
 	// calculate the angle corresponding to the value
 	//
-	if( m_maxValue == m_minValue )
+	if( maxValue() == minValue() )
 	{
 		m_angle = 0;
 	}
 	else
 	{
-		m_angle = ( value() - 0.5 * ( m_minValue + m_maxValue ) ) /
-				( m_maxValue - m_minValue ) * m_totalAngle;
+		m_angle = ( value() - 0.5 * ( minValue() + maxValue() ) ) /
+				( maxValue() - minValue() ) * m_totalAngle;
 		m_angle = static_cast<int>( m_angle ) % 360;
 	}
 }
@@ -364,6 +375,14 @@ void knob::contextMenuEvent( QContextMenuEvent * )
 							m_hintTextAfterValue ),
 				this, SLOT( pasteValue() ) );
 	contextMenu.addSeparator();
+	if( !nullTrack() )
+	{
+		contextMenu.addAction( embed::getIconPixmap( "automation" ),
+					tr( "&Open in automation editor" ),
+					getAutomationPattern(),
+					SLOT( openInAutomationEditor() ) );
+		contextMenu.addSeparator();
+	}
 	contextMenu.addAction( tr( "Connect to MIDI-device" ), this,
 						SLOT( connectToMidiDevice() ) );
 	contextMenu.addSeparator();
@@ -401,8 +420,11 @@ void knob::dropEvent( QDropEvent * _de )
 void knob::mousePressEvent( QMouseEvent * _me )
 {
 	if( _me->button() == Qt::LeftButton &&
-				lmmsMainWin::inst()->isCtrlPressed() == FALSE )
+			eng()->getMainWindow()->isCtrlPressed() == FALSE )
 	{
+		setJournalling( FALSE );
+		m_oldValue = value();
+
 		const QPoint & p = _me->pos();
 		m_origMousePos = p;
 
@@ -428,10 +450,10 @@ void knob::mousePressEvent( QMouseEvent * _me )
 		m_buttonPressed = TRUE;
 	}
 	else if( _me->button() == Qt::LeftButton &&
-					lmmsMainWin::isCtrlPressed() == TRUE )
+			eng()->getMainWindow()->isCtrlPressed() == TRUE )
 	{
 		new stringPairDrag( "float_value", QString::number( value() ),
-							QPixmap(), this );
+						QPixmap(), this, eng() );
 	}
 	else if( _me->button() == Qt::MidButton )
 	{
@@ -449,12 +471,12 @@ void knob::mouseMoveEvent( QMouseEvent * _me )
 	{
 		setPosition( _me->pos() );
 		emit sliderMoved( value() );
+		emit valueChanged( data() );
 		if( !configManager::inst()->value( "knobs",
 						"classicalusability").toInt() )
 		{
 			QCursor::setPos( mapToGlobal( m_origMousePos ) );
 		}
-		songEditor::inst()->setModified();
 	}
 
 	s_textFloat->setText( m_hintTextBeforeValue +
@@ -468,6 +490,9 @@ void knob::mouseMoveEvent( QMouseEvent * _me )
 //! Mouse Release Event handler
 void knob::mouseReleaseEvent( QMouseEvent * /* _me*/ )
 {
+	setJournalling( TRUE );
+	addJournalEntryFromOldToCurVal();
+
 	if( m_buttonPressed )
 	{
 		m_buttonPressed = TRUE;
@@ -516,9 +541,13 @@ void knob::paintEvent( QPaintEvent * _me )
 		if( m_label != "" )
 		{
 			p.setFont( pointSize<6>( p.font() ) );
+			p.setPen( QColor( 64, 64, 64 ) );
+			p.drawText( width() / 2 -
+				p.fontMetrics().width( m_label ) / 2 + 1,
+					height() - 1, m_label );
 			p.setPen( QColor( 255, 255, 255 ) );
 			p.drawText( width() / 2 -
-				QFontMetrics( p.font() ).width( m_label ) / 2,
+					p.fontMetrics().width( m_label ) / 2,
 					height() - 2, m_label );
 		}
 #ifndef QT4
@@ -546,7 +575,6 @@ void knob::wheelEvent( QWheelEvent * _we )
 	_we->accept();
 	const int inc = ( _we->delta() > 0 ) ? 1 : -1;
 	incValue( inc );
-	songEditor::inst()->setModified();
 
 
 	s_textFloat->reparent( this );
@@ -557,22 +585,17 @@ void knob::wheelEvent( QWheelEvent * _we )
 				QPoint( m_knobPixmap->width() + 2, 0 ) );
 	s_textFloat->setVisibilityTimeOut( 1000 );
 
-	if( value() != m_prevValue )
-	{
-		emit sliderMoved( value() );
-	}
+	emit sliderMoved( value() );
+	emit valueChanged( data() );
 }
 
 
 
 
-//! Emits a valueChanged() signal if necessary
 void knob::buttonReleased( void )
 {
-	if( value() != m_prevValue )
-	{
-		emit valueChanged( value() );
-	}
+	emit valueChanged( value() );
+	emit valueChanged( data() );
 }
 
 
@@ -587,74 +610,36 @@ void knob::setPosition( const QPoint & _p )
 	}
 	else
 	{
-		setValue( m_value - getValue( _p ) );
+		setValue( value() - getValue( _p ) );
 	}
 }
 
 
 
 
-void knob::setValue( float _val, bool _is_init_value )
+void knob::setValue( const float _x )
 {
-	if( _is_init_value )
+	const float prev_value = value();
+	autoObj::setValue( _x );
+	setFirstValue();
+	if( prev_value != value() )
 	{
-		m_initValue = _val;
+		valueChange();
 	}
-
-	setNewValue( _val, TRUE );
 }
 
 
 
 
-void knob::fitValue( float _val )
+void knob::setRange( const float _min, const float _max, const float _step )
 {
-	setValue( _val );
-}
+	bool rchg = ( ( maxValue() != _max ) || ( minValue() != _min ) );
+	autoObj::setRange( _min, _max, _step );
 
+	m_pageSize = tMax<float>( ( maxValue() - minValue() ) / 100.0f,
+								step() );
 
-
-
-void knob::incValue( int _steps )
-{
-	setValue( m_value + float( _steps ) * m_step );
-}
-
-
-
-
-void knob::setRange( float _vmin, float _vmax, float _vstep, int _page_size )
-{
-	int rchg = ( ( m_maxValue != _vmax ) || ( m_minValue != _vmin ) );
-
-	if( rchg )
-	{
-		m_minValue = _vmin;
-		m_maxValue = _vmax;
-	}
-
-	//
-	// look if the step width has an acceptable 
-	// value or otherwise change it.
-	//
-	setStep( _vstep );
-
-	//
-	// limit page size
-	//
-/*	m_pageSize = tLimit( pageSize, 0, int( tAbs<float>( ( m_maxValue -
-						m_minValue ) / m_step ) ) ); */
-	m_pageSize = tMax<float>( ( m_maxValue - m_minValue ) / 100.0f,
-								m_step );
-
-	// 
-	// If the value lies out of the range, it 
-	// will be changed. Note that it will not be adjusted to 
-	// the new step width.
-	setNewValue( m_value, FALSE );
-
-	// call notifier after the step width has been 
-	// adjusted.
+	// call notifier after the step width has been adjusted.
 	if( rchg )
 	{
 		rangeChange();
@@ -664,90 +649,9 @@ void knob::setRange( float _vmin, float _vmax, float _vstep, int _page_size )
 
 
 
-void knob::setNewValue( float _x, bool _align )
-{
-	m_prevValue = m_value;
-
-	m_value = tLimit( _x, m_minValue, m_maxValue );
-
-	m_exactPrevValue = m_exactValue;
-	m_exactValue = m_value;
-
-	// align to grid
-	if( _align )
-	{
-		if( m_step != 0.0 )
-		{
-			m_value = floorf( m_value / m_step ) * m_step;
-		}
-		else
-		{
-			m_value = m_minValue;
-		}
-
-		// correct rounding error at the border
-		if( tAbs<float>( m_value - m_maxValue ) < MinEps *
-							tAbs<float>( m_step ) )
-		{
-			m_value = m_maxValue;
-		}
-
-		// correct rounding error if value = 0
-		if( tAbs<float>( m_value ) < MinEps * tAbs<float>( m_step ) )
-		{
-			m_value = 0.0;
-		}
-	}
-
-	if( m_prevValue != m_value )
-	{
-		valueChange();
-	}
-}
-
-
-
-
-void knob::setStep( float _vstep )
-{
-	float intv = m_maxValue - m_minValue;
-
-	float newStep;
-
-	if( _vstep == 0.0 )
-	{
-		newStep = intv * DefaultRelStep;
-	}
-	else
-	{
-		if( ( intv > 0 ) && ( _vstep < 0 ) || ( intv < 0 ) &&
-								( _vstep > 0 ) )
-		{
-			newStep = -_vstep;
-		}
-		else
-		{
-			newStep = _vstep;
-		}
-		if( tAbs<float>( newStep ) <
-					tAbs<float>( MinRelStep * intv ) )
-		{
-			newStep = MinRelStep * intv;
-		}
-	}
-	if( newStep != m_step )
-	{
-		m_step = newStep;
-	}
-}
-
-
-
-
 void knob::reset( void )
 {
 	setValue( m_initValue );
-	songEditor::inst()->setModified();
 	s_textFloat->reparent( this );
 	s_textFloat->setText( m_hintTextBeforeValue +
 					QString::number( value() ) +
@@ -771,7 +675,6 @@ void knob::copyValue( void )
 void knob::pasteValue( void )
 {
 	setValue( s_copiedValue );
-	songEditor::inst()->setModified();
 	s_textFloat->reparent( this );
 	s_textFloat->setText( m_hintTextBeforeValue +
 					QString::number( value() ) +
@@ -812,7 +715,7 @@ void knob::enterValue( void )
 
 void knob::connectToMidiDevice( void )
 {
-	//mixer::inst()->getMIDIDevice()->setPitchBendKnob( this );
+	//eng()->getMixer()->getMIDIDevice()->setPitchBendKnob( this );
 }
 
 
@@ -838,3 +741,5 @@ void knob::displayHelp( void )
 #ifndef QT4
 #undef addSeparator
 #endif
+
+#endif
