@@ -1,7 +1,9 @@
+#ifndef SINGLE_SOURCE_COMPILE
+
 /*
  * lcd_spinbox.cpp - class lcdSpinBox, an improved QLCDNumber
  *
- * Copyright (c) 2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2005-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -27,31 +29,37 @@
 
 #ifdef QT4
 
-#include <QApplication>
-#include <QMouseEvent>
-#include <QCursor>
-#include <QLabel>
+#include <QtGui/QApplication>
+#include <QtGui/QCursor>
+#include <QtGui/QLabel>
+#include <QtGui/QMenu>
+#include <QtGui/QMouseEvent>
 
 #else
 
 #include <qapplication.h>
 #include <qcursor.h>
 #include <qlabel.h>
+#include <qpopupmenu.h>
 
 #endif
 
 
 #include "lcd_spinbox.h"
+#include "embed.h"
 #include "gui_templates.h"
 #include "templates.h"
 
 
-lcdSpinBox::lcdSpinBox( int _min, int _max, int _num_digits,
-							QWidget * _parent ) :
-	QWidget( _parent ),
-	m_minValue( _min ),
-	m_maxValue( _max ),
-	m_step( 1 ),
+lcdSpinBox::lcdSpinBox( int _min, int _max, int _num_digits, QWidget * _parent,
+					const QString & _name,
+					engine * _engine, track * _track ) :
+	QWidget( _parent
+#ifndef QT4
+			, _name.ascii()
+#endif
+		),
+	autoObj( _engine, _track, 0, _min, _max ),
 	m_label( NULL ),
 	m_origMousePos()
 {
@@ -59,10 +67,27 @@ lcdSpinBox::lcdSpinBox( int _min, int _max, int _num_digits,
 	m_number->setFrameShape( QFrame::Panel );
 	m_number->setFrameShadow( QFrame::Sunken );
 	m_number->setSegmentStyle( QLCDNumber::Flat );
+#ifndef QT3
+	QPalette pal;
+	pal.setColor( QPalette::Light, Qt::gray );
+	pal.setColor( QPalette::Mid, Qt::darkGray );
+	pal.setColor( QPalette::Dark, Qt::black );
+	pal.setColor( m_number->backgroundRole(), Qt::black );
+	m_number->setPalette( pal );
+	m_number->setAutoFillBackground( TRUE );
+#endif
 	setEnabled( TRUE );
 
+	if( _track != NULL )
+	{
+		getAutomationPattern();
+	}
+
 	// value is automatically limited to given range
-	setValue( 0 );
+	setInitValue( 0 );
+#ifdef QT4
+	setAccessibleName( _name );
+#endif
 
 	m_number->setFixedSize( m_number->sizeHint() * 0.9 );
 	setFixedSize( m_number->size() );
@@ -77,22 +102,22 @@ lcdSpinBox::~lcdSpinBox()
 
 
 
-void lcdSpinBox::setStep( int _step )
+
+void lcdSpinBox::setStep( const int _step )
 {
-	m_step = tMax( _step, 1 );
+	autoObj::setStep( tMax( _step, 1 ) );
 }
 
 
 
 
-void lcdSpinBox::setValue( int _value )
+void lcdSpinBox::setValue( const int _value )
 {
-	_value = ( ( tLimit( _value, m_minValue, m_maxValue ) - m_minValue ) /
-						m_step ) * m_step + m_minValue;
-	QString s = m_textForValue[_value];
+	autoObj::setValue( _value );
+	QString s = m_textForValue[value()];
 	if( s == "" )
 	{
-		s = QString::number( _value );
+		s = QString::number( value() );
 		while( (int) s.length() < m_number->numDigits() )
 		{
 			s = "0" + s;
@@ -146,12 +171,49 @@ void lcdSpinBox::setEnabled( bool _on )
 
 
 
+void lcdSpinBox::contextMenuEvent( QContextMenuEvent * _me )
+{
+	m_origMousePos = _me->globalPos();
+
+	if( nullTrack() )
+	{
+		QWidget::contextMenuEvent( _me );
+		return;
+	}
+
+	// for the case, the user clicked right while pressing left mouse-
+	// button, the context-menu appears while mouse-cursor is still hidden
+	// and it isn't shown again until user does something which causes
+	// an QApplication::restoreOverrideCursor()-call...
+	mouseReleaseEvent( NULL );
+
+	QMenu contextMenu( this );
+#ifdef QT4
+	contextMenu.setTitle( accessibleName() );
+#else
+	QLabel * caption = new QLabel( "<font color=white><b>" +
+			QString( accessibleName() ) + "</b></font>", this );
+	caption->setPaletteBackgroundColor( QColor( 0, 0, 192 ) );
+	caption->setAlignment( Qt::AlignCenter );
+	contextMenu.addAction( caption );
+#endif
+	contextMenu.addAction( embed::getIconPixmap( "automation" ),
+					tr( "&Open in automation editor" ),
+					getAutomationPattern(),
+					SLOT( openInAutomationEditor() ) );
+	contextMenu.exec( QCursor::pos() );
+}
+
+
+
+
 void lcdSpinBox::mousePressEvent( QMouseEvent * _me )
 {
 	if( _me->button() == Qt::LeftButton && _me->y() < m_number->height()  )
 	{
 		m_origMousePos = _me->globalPos();
 		QApplication::setOverrideCursor( Qt::BlankCursor );
+		m_oldValue = value();
 	}
 }
 
@@ -169,8 +231,10 @@ void lcdSpinBox::mouseMoveEvent( QMouseEvent * _me )
 		int dy = _me->globalY() - m_origMousePos.y();
 		if( dy > 1 || dy < -1 )
 		{
-			setValue( value() - dy / 2 * m_step );
+			setJournalling( FALSE );// why is this neccessary?!
+			setInitValue( value() - dy / 2 * step() );
 			emit valueChanged( value() );
+			setJournalling( TRUE );
 			QCursor::setPos( m_origMousePos );
 		}
 	}
@@ -181,6 +245,8 @@ void lcdSpinBox::mouseMoveEvent( QMouseEvent * _me )
 
 void lcdSpinBox::mouseReleaseEvent( QMouseEvent * _me )
 {
+	addJournalEntryFromOldToCurVal();
+
 	QCursor::setPos( m_origMousePos );
 	QApplication::restoreOverrideCursor();
 }
@@ -191,7 +257,7 @@ void lcdSpinBox::mouseReleaseEvent( QMouseEvent * _me )
 void lcdSpinBox::wheelEvent( QWheelEvent * _we )
 {
 	_we->accept();
-	setValue( value() + ( ( _we->delta() > 0 ) ? 1 : -1 ) * m_step );
+	setInitValue( value() + ( ( _we->delta() > 0 ) ? 1 : -1 ) * step() );
 	emit valueChanged( value() );
 }
 
@@ -200,3 +266,5 @@ void lcdSpinBox::wheelEvent( QWheelEvent * _we )
 
 #include "lcd_spinbox.moc"
 
+
+#endif
