@@ -1,7 +1,9 @@
+#ifndef SINGLE_SOURCE_COMPILE
+
 /*
  * led_checkbox.cpp - class ledCheckBox, an improved QCheckBox
  *
- * Copyright (c) 2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2005-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -27,9 +29,9 @@
 
 #ifdef QT4
 
-#include <QPaintEvent>
-#include <QFontMetrics>
-#include <QPainter>
+#include <QtGui/QFontMetrics>
+#include <QtGui/QPainter>
+#include <QtGui/QPaintEvent>
 
 #else
 
@@ -53,11 +55,14 @@ static const QString names[ledCheckBox::TOTAL_COLORS] =
 
 
 ledCheckBox::ledCheckBox( const QString & _text, QWidget * _parent,
-							ledColors _color ) :
-	QWidget( _parent ),
-	m_checked( FALSE ),
+					const QString & _name,
+					engine * _engine, track * _track,
+					ledColors _color ) :
+	automatableButton( _parent, _name, _engine, _track ),
 	m_text( _text )
 {
+	setCheckable( TRUE );
+
 	if( _color >= TOTAL_COLORS || _color < YELLOW )
 	{
 		_color = YELLOW;
@@ -73,8 +78,8 @@ ledCheckBox::ledCheckBox( const QString & _text, QWidget * _parent,
 	setBackgroundMode( Qt::PaletteBackground );
 #endif
 
-	setFont( pointSize<7>( font() ) );
-	setFixedSize( m_ledOffPixmap->width() + 6 +
+	setFont( pointSizeF( font(), 7.5f ) );
+	setFixedSize( m_ledOffPixmap->width() + 4 +
 					QFontMetrics( font() ).width( text() ),
 			m_ledOffPixmap->height() );
 }
@@ -91,42 +96,6 @@ ledCheckBox::~ledCheckBox()
 
 
 
-void ledCheckBox::mousePressEvent( QMouseEvent * _me )
-{
-	if( _me->button() == Qt::LeftButton )
-	{
-		toggle();
-	}
-}
-	
-	
-	
-	
-void ledCheckBox::toggle( void )
-{
-	m_checked = !m_checked;
-	update();
-	emit( toggled( m_checked ) );
-}
-
-
-
-
-void ledCheckBox::setChecked( bool _on )
-{
-	if( _on != isChecked() )
-	{
-		toggle();
-	}
-	else
-	{
-		emit( toggled( m_checked ) );
-	}
-}
-
-
-
-
 void ledCheckBox::paintEvent( QPaintEvent * )
 {
 #ifdef QT4
@@ -148,7 +117,10 @@ void ledCheckBox::paintEvent( QPaintEvent * )
 		p.drawPixmap( 0, 0, *m_ledOffPixmap );
 	}
 
-	p.drawText( m_ledOffPixmap->width() + 2, 10, text() );
+	p.setPen( QColor( 64, 64, 64 ) );
+	p.drawText( m_ledOffPixmap->width() + 3, 9, text() );
+	p.setPen( QColor( 255, 255, 255 ) );
+	p.drawText( m_ledOffPixmap->width() + 2, 8, text() );
 
 #ifndef QT4
 	// and blit all the drawn stuff on the screen...
@@ -159,3 +131,5 @@ void ledCheckBox::paintEvent( QPaintEvent * )
 
 
 #include "led_checkbox.moc"
+
+#endif
