@@ -1,7 +1,9 @@
+#ifndef SINGLE_SOURCE_COMPILE
+
 /*
  * nstate_button.cpp - implementation of n-state-button
  *
- * Copyright (c) 2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2005-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -27,7 +29,7 @@
 
 #ifdef QT4
 
-#include <QMouseEvent>
+#include <QtGui/QMouseEvent>
 
 #endif
 
@@ -109,3 +111,5 @@ void nStateButton::mousePressEvent( QMouseEvent * _me )
 
 #include "nstate_button.moc"
 
+
+#endif
