@@ -1,8 +1,10 @@
+#ifndef SINGLE_SOURCE_COMPILE
+
 /*
  * pixmap_button.cpp - implementation of pixmap-button (often used as "themed"
  *                     checkboxes/radiobuttons etc)
  *
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -28,8 +30,8 @@
 
 #ifdef QT4
 
-#include <QPainter>
-#include <QMouseEvent>
+#include <QtGui/QMouseEvent>
+#include <QtGui/QPainter>
 
 #else
 
@@ -43,11 +45,12 @@
 
 
 
-pixmapButton::pixmapButton( QWidget * _parent ) :
-	QPushButton( _parent ),
-	m_activePixmap( NULL ),
-	m_inactivePixmap( NULL ),
-	m_bgPixmap( NULL )
+pixmapButton::pixmapButton( QWidget * _parent, const QString & _name,
+					engine * _engine, track * _track ) :
+	automatableButton( _parent, _name, _engine, _track ),
+	m_activePixmap(),
+	m_inactivePixmap(),
+	m_bgPixmap()
 {
 	setActiveGraphic( embed::getIconPixmap( "led_yellow" ) );
 	setInactiveGraphic( embed::getIconPixmap( "led_off" ), FALSE );
@@ -56,7 +59,7 @@ pixmapButton::pixmapButton( QWidget * _parent ) :
 	setBackgroundMode( Qt::NoBackground );
 #endif
 
-	setCheckable( TRUE );
+	//setCheckable( TRUE );
 }
 
 
@@ -64,9 +67,6 @@ pixmapButton::pixmapButton( QWidget * _parent ) :
 
 pixmapButton::~pixmapButton()
 {
-	delete m_activePixmap;
-	delete m_inactivePixmap;
-	delete m_bgPixmap;
 }
 
 
@@ -83,29 +83,23 @@ void pixmapButton::paintEvent( QPaintEvent * )
 	QPainter p( &draw_pm, this );
 #endif
 
-	if( m_bgPixmap != NULL )
+	if( !m_bgPixmap.isNull() )
 	{
-		p.drawPixmap( 0, 0, *m_bgPixmap );
+		p.drawPixmap( 0, 0, m_bgPixmap );
 	}
 
-#ifdef QT4
-	if( isChecked() || isDown() )
-#else
-	if( isOn() || isDown() )
-#endif
+	if( isChecked() )
 	{
-		if( m_activePixmap != NULL )
+		if( !m_activePixmap.isNull() )
 		{
-			p.drawPixmap( 0, 0, *m_activePixmap );
+			p.drawPixmap( 0, 0, m_activePixmap );
 		}
 	}
-	else
+	else if( !m_inactivePixmap.isNull() )
 	{
-		if( m_inactivePixmap != NULL )
-		{
-			p.drawPixmap( 0, 0, *m_inactivePixmap );
-		}
+		p.drawPixmap( 0, 0, m_inactivePixmap );
 	}
+
 #ifndef QT4
 	// and blit all the drawn stuff on the screen...
 	bitBlt( this, rect().topLeft(), &draw_pm );
@@ -116,24 +110,26 @@ void pixmapButton::paintEvent( QPaintEvent * )
 
 
 
-void pixmapButton::mousePressEvent( QMouseEvent * _me)
+void pixmapButton::mousePressEvent( QMouseEvent * _me )
 {
 	if( _me->button() == Qt::RightButton )
 	{
 		emit( clickedRight() );
+		_me->accept();
 	}
 	else
 	{
-		QPushButton::mousePressEvent( _me );
+		automatableButton::mousePressEvent( _me );
 	}
 }
 
 
 
 
-void pixmapButton::mouseDoubleClickEvent( QMouseEvent * )
+void pixmapButton::mouseDoubleClickEvent( QMouseEvent * _me )
 {
 	emit doubleClicked();
+	_me->accept();
 }
 
 
@@ -141,10 +137,8 @@ void pixmapButton::mouseDoubleClickEvent( QMouseEvent * )
 
 void pixmapButton::setActiveGraphic( const QPixmap & _pm )
 {
-	delete m_activePixmap;
-
-	m_activePixmap = new QPixmap( _pm );
-	resize( m_activePixmap->width(), m_activePixmap->height() );
+	m_activePixmap = _pm;
+	resize( m_activePixmap.width(), m_activePixmap.height() );
 }
 
 
@@ -152,9 +146,7 @@ void pixmapButton::setActiveGraphic( const QPixmap & _pm )
 
 void pixmapButton::setInactiveGraphic( const QPixmap & _pm, bool _update )
 {
-	delete m_inactivePixmap;
-
-	m_inactivePixmap = new QPixmap( _pm );
+	m_inactivePixmap = _pm;
 	if( _update )
 	{
 		update();
@@ -166,12 +158,12 @@ void pixmapButton::setInactiveGraphic( const QPixmap & _pm, bool _update )
 
 void pixmapButton::setBgGraphic( const QPixmap & _pm )
 {
-	delete m_bgPixmap;
-
-	m_bgPixmap = new QPixmap( _pm );
+	m_bgPixmap = _pm;
 }
 
 
 
 #include "pixmap_button.moc"
 
+
+#endif
