@@ -1,7 +1,9 @@
+#ifndef SINGLE_SOURCE_COMPILE
+
 /*
  * project_notes.cpp - implementation of project-notes-editor
  *
- * Copyright (c) 2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2005-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -28,15 +30,15 @@
 #ifdef QT4
 
 #include <Qt/QtXml>
-#include <QApplication>
-#include <QTextEdit>
-#include <QLineEdit>
-#include <QAction>
-#include <QToolBar>
-#include <QFontDatabase>
-#include <QComboBox>
-#include <QColorDialog>
-#include <QTextCursor>
+#include <QtGui/QAction>
+#include <QtGui/QApplication>
+#include <QtGui/QColorDialog>
+#include <QtGui/QComboBox>
+#include <QtGui/QFontDatabase>
+#include <QtGui/QLineEdit>
+#include <QtGui/QTextCursor>
+#include <QtGui/QTextEdit>
+#include <QtGui/QToolBar>
 
 #else
 
@@ -58,7 +60,6 @@
 #define setFontFamily setFamily
 #define setFontPointSize setPointSize
 #define setTextColor setColor
-#define textColor color
 #define setHtml setText
 #define toHtml text
 
@@ -67,24 +68,26 @@
 
 #include "project_notes.h"
 #include "embed.h"
-#include "lmms_main_win.h"
+#include "main_window.h"
 #include "song_editor.h"
 
 
 
-projectNotes::projectNotes() :
-	QMainWindow( lmmsMainWin::inst()->workspace()
+projectNotes::projectNotes( engine * _engine) :
+	QMainWindow( _engine->getMainWindow()->workspace()
 #ifndef QT4
 				, 0, Qt::WStyle_Title
 #endif
-		)
+		),
+	journallingObject( _engine )
 {
 #ifdef QT4
-	lmmsMainWin::inst()->workspace()->addWindow( this );
+	eng()->getMainWindow()->workspace()->addWindow( this );
 #endif
 
 	m_edit = new QTextEdit( this );
 #ifdef QT4
+	m_edit->setAutoFillBackground( TRUE );
 	QPalette pal;
 	pal.setColor( m_edit->backgroundRole(), QColor( 64, 64, 64 ) );
 	m_edit->setPalette( pal );
@@ -101,13 +104,35 @@ projectNotes::projectNotes() :
 	connect( m_edit, SIGNAL( currentAlignmentChanged( int ) ),
 			this, SLOT( alignmentChanged( int ) ) );
 	connect( m_edit, SIGNAL( textChanged() ),
-				songEditor::inst(), SLOT( setModified() ) );
+				eng()->getSongEditor(), SLOT( setModified() ) );
 
 	setupActions();
 
 	setCentralWidget( m_edit );
 	setWindowTitle( tr( "Project notes" ) );
 	setWindowIcon( embed::getIconPixmap( "project_notes" ) );
+
+	resize( 300, 200 );
+
+	QWidget * w = ( parentWidget() != NULL ) ? parentWidget() : this;
+	if( eng()->getMainWindow()->workspace() != NULL )
+	{
+		w->move( 700, 10 );
+	}
+	else
+	{
+		w->move( 800, 10 );
+	}
+
+	//show();
+
+}
+
+
+
+
+projectNotes::~projectNotes()
+{
 }
 
 
@@ -130,6 +155,14 @@ void projectNotes::clear( void )
 
 
 
+void projectNotes::setText( const QString & _text )
+{
+	m_edit->setHtml( _text );
+}
+
+
+
+
 void projectNotes::setupActions()
 {
 	QToolBar * tb = new QToolBar( tr( "Edit Actions" ), this );
@@ -405,7 +438,7 @@ void projectNotes::setupActions()
 void projectNotes::textBold()
 {
 	m_edit->setFontWeight( m_actionTextBold->isChecked() );
-	songEditor::inst()->setModified();
+	eng()->getSongEditor()->setModified();
 }
 
 
@@ -414,7 +447,7 @@ void projectNotes::textBold()
 void projectNotes::textUnderline()
 {
 	m_edit->setFontUnderline( m_actionTextUnderline->isChecked() );
-	songEditor::inst()->setModified();
+	eng()->getSongEditor()->setModified();
 }
 
 
@@ -423,7 +456,7 @@ void projectNotes::textUnderline()
 void projectNotes::textItalic()
 {
 	m_edit->setFontItalic( m_actionTextItalic->isChecked() );
-	songEditor::inst()->setModified();
+	eng()->getSongEditor()->setModified();
 }
 
 
@@ -433,7 +466,7 @@ void projectNotes::textFamily( const QString & _f )
 {
 	m_edit->setFontFamily( _f );
 	m_edit->viewport()->setFocus();
-	songEditor::inst()->setModified();
+	eng()->getSongEditor()->setModified();
 }
 
 
@@ -443,7 +476,7 @@ void projectNotes::textSize( const QString & _p )
 {
 	m_edit->setFontPointSize( _p.toInt() );
 	m_edit->viewport()->setFocus();
-	songEditor::inst()->setModified();
+	eng()->getSongEditor()->setModified();
 }
 
 
@@ -499,7 +532,7 @@ void projectNotes::fontChanged( const QFont & _f )
 	m_actionTextBold->setChecked( _f.bold() );
 	m_actionTextItalic->setChecked( _f.italic() );
 	m_actionTextUnderline->setChecked( _f.underline() );
-	songEditor::inst()->setModified();
+	eng()->getSongEditor()->setModified();
 }
 
 
@@ -514,7 +547,7 @@ void projectNotes::colorChanged( const QColor & _c )
 #else
 	m_actionTextColor->setIconSet( pix );
 #endif
-	songEditor::inst()->setModified();
+	eng()->getSongEditor()->setModified();
 }
 
 
@@ -542,18 +575,18 @@ void projectNotes::alignmentChanged( int _a )
 	{
 		m_actionAlignJustify->setChecked( TRUE );
 	}
-	songEditor::inst()->setModified();
+	eng()->getSongEditor()->setModified();
 }
 
 
 
 
-void projectNotes::saveSettings( QDomDocument & _doc, QDomElement & _parent )
+void projectNotes::saveSettings( QDomDocument & _doc, QDomElement & _this )
 {
-	QDomElement pn_de = _doc.createElement( nodeName() );
+	mainWindow::saveWidgetState( this, _this );
+
 	QDomCDATASection ds = _doc.createCDATASection( m_edit->toHtml() );
-	pn_de.appendChild( ds );
-	_parent.appendChild( pn_de );
+	_this.appendChild( ds );
 }
 
 
@@ -561,6 +594,7 @@ void projectNotes::saveSettings( QDomDocument & _doc, QDomElement & _parent )
 
 void projectNotes::loadSettings( const QDomElement & _this )
 {
+	mainWindow::restoreWidgetState( this, _this );
 	m_edit->setHtml( _this.text() );
 }
 
@@ -576,8 +610,9 @@ void projectNotes::loadSettings( const QDomElement & _this )
 #undef setFontItalic
 #undef setFontFamily
 #undef setFontPointSize
-#undef textColor
 #undef setTextColor
 #undef setHtml
 #undef toHtml
 
+
+#endif
