@@ -1,7 +1,9 @@
+#ifndef SINGLE_SOURCE_COMPILE
+
 /*
  * side_bar_widget.cpp - implementation of base-widget for side-bar
  *
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -27,9 +29,9 @@
 
 #ifdef QT4
 
-#include <QApplication>
-#include <QPainter>
-#include <QFontMetrics>
+#include <QtGui/QApplication>
+#include <QtGui/QFontMetrics>
+#include <QtGui/QPainter>
 
 #else
 
@@ -55,6 +57,7 @@ sideBarWidget::sideBarWidget( const QString & _title, const QPixmap & _icon,
 	m_contents = new QWidget( this );
 	m_layout = new QVBoxLayout( m_contents );
 	m_layout->setSpacing( 5 );
+	m_layout->setMargin( 0 );
 }
 
 
@@ -122,3 +125,5 @@ void sideBarWidget::resizeEvent( QResizeEvent * )
 #include "side_bar.moc"
 #include "side_bar_widget.moc"
 
+
+#endif
