@@ -1,7 +1,9 @@
+#ifndef SINGLE_SOURCE_COMPILE
+
 /*
  * text_float.cpp - class textFloat, a floating text-label
  *
- * Copyright (c) 2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2005-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -27,8 +29,8 @@
 
 #ifdef QT4
 
-#include <QPainter>
-#include <QTimer>
+#include <QtCore/QTimer>
+#include <QtGui/QPainter>
 
 #else
 
@@ -43,7 +45,7 @@
 
 #include "text_float.h"
 #include "gui_templates.h"
-#include "lmms_main_win.h"
+#include "main_window.h"
 
 
 
@@ -154,10 +156,20 @@ textFloat * textFloat::displayMessage( const QString & _msg, int _timeout,
 	QWidget * mw = QApplication::activeWindow();
 	if( mw == NULL )
 	{
-		mw = lmmsMainWin::inst();
+		foreach( QWidget * w, QApplication::topLevelWidgets() )
+		{
+			if( mw == NULL || dynamic_cast<mainWindow *>( w ) )
+			{
+				mw = w;
+			}
+		}
 	}
 #else
 	QWidget * mw = qApp->mainWidget();
+	if( mw == NULL )
+	{
+		mw = qApp->desktop();
+	}
 #endif
 	textFloat * tf = new textFloat( ( _parent == NULL ) ? mw : _parent );
 	if( _parent != NULL )
@@ -214,7 +226,11 @@ void textFloat::paintEvent( QPaintEvent * _pe )
 
 	p.setFont( pointSize<8>( p.font() ) );
 
+#ifndef QT3
+	p.drawRect( 0, 0, rect().right(), rect().bottom() );
+#else
 	p.drawRect( rect() );
+#endif
 
 //	p.setPen( Qt::black );
 	// small message?
@@ -281,3 +297,5 @@ void textFloat::updateSize( void )
 
 #undef setParent
 
+
+#endif
