@@ -1,7 +1,9 @@
+#ifndef SINGLE_SOURCE_COMPILE
+
 /*
  * tooltip.cpp - namespace toolTip, a tooltip-wrapper for LMMS
  *
- * Copyright (c) 2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2005-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -27,7 +29,7 @@
 
 #ifdef QT4
 
-#include <QToolTip>
+#include <QtGui/QToolTip>
 
 #else
 
@@ -51,3 +53,5 @@ void toolTip::add( QWidget * _w, const QString & _txt )
 	}
 }
 
+
+#endif
