@@ -1,7 +1,9 @@
+#ifndef SINGLE_SOURCE_COMPILE
+
 /*
  * visualization_widget.cpp - widget for visualization of sound-data
  *
- * Copyright (c) 2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2005-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -27,10 +29,10 @@
 
 #ifdef QT4
 
-#include <QPainter>
-#include <QPixmap>
-#include <QTimer>
-#include <QMouseEvent>
+#include <QtCore/QTimer>
+#include <QtGui/QMouseEvent>
+#include <QtGui/QPainter>
+#include <QtGui/QPixmap>
 
 #else
 
@@ -53,41 +55,36 @@ const int UPDATE_TIME = 1000 / 20;	// 20 fps
 
 
 visualizationWidget::visualizationWidget( const QPixmap & _bg, QWidget * _p,
+						engine * _engine,
 						visualizationTypes _vtype ) :
 	QWidget( _p ),
+	engineObject( _engine ),
 	s_background( _bg ),
-	m_enabled( FALSE )
+	m_enabled( TRUE )
 {
 #ifndef QT4
 	setBackgroundMode( Qt::NoBackground );
 #endif
+	setFixedSize( s_background.width(), s_background.height() );
 
-	const Uint32 frames = mixer::inst()->framesPerAudioBuffer();
+
+	const fpab_t frames = eng()->getMixer()->framesPerAudioBuffer();
 	m_buffer = bufferAllocator::alloc<surroundSampleFrame>( frames );
 
-	mixer::inst()->clearAudioBuffer( m_buffer, frames );
-/*	for( Uint32 frame = 0; frame < frames; ++frame )
-	{
-		for( Uint8 chnl = 0; chnl < SURROUND_CHANNELS; ++chnl )
-		{
-			m_buffer[frame][chnl] = 0.0f;
-		}
-	}*/
-
-	setFixedSize( s_background.width(), s_background.height() );
+	eng()->getMixer()->clearAudioBuffer( m_buffer, frames );
 
 
 	m_updateTimer = new QTimer( this );
 	connect( m_updateTimer, SIGNAL( timeout() ), this, SLOT( update() ) );
-	if( m_enabled )
+	if( m_enabled == TRUE )
 	{
 		m_updateTimer->start( UPDATE_TIME );
 	}
 
-	connect( mixer::inst(), SIGNAL( nextAudioBuffer(
-				const surroundSampleFrame *, Uint32 ) ),
+	connect( eng()->getMixer(), SIGNAL( nextAudioBuffer(
+					const surroundSampleFrame *, int ) ),
 		this, SLOT( setAudioBuffer(
-				const surroundSampleFrame *, Uint32 ) ) );
+					const surroundSampleFrame *, int ) ) );
 
 	toolTip::add( this, tr( "click to enable/disable visualization of "
 							"master-output" ) );
@@ -104,9 +101,9 @@ visualizationWidget::~visualizationWidget()
 
 
 void visualizationWidget::setAudioBuffer( const surroundSampleFrame * _ab,
-								Uint32 _frames )
+								int _frames )
 {
-	if( m_enabled )
+	if( m_enabled == TRUE )
 	{
 		memcpy( m_buffer, *_ab, _frames * BYTES_PER_SURROUND_FRAME);
 	}
@@ -121,15 +118,14 @@ void visualizationWidget::paintEvent( QPaintEvent * )
 	QPainter p( this );
 #else
 	QPixmap draw_pm( rect().size() );
-	//draw_pm.fill( this, rect().topLeft() );
 
 	QPainter p( &draw_pm, this );
 #endif
 	p.drawPixmap( 0, 0, s_background );
 
-	if( m_enabled )
+	if( m_enabled == TRUE )
 	{
-		float master_output = mixer::inst()->masterGain();
+		float master_output = eng()->getMixer()->masterGain();
 		Uint16 w = width()-4;
 		float half_h = -( height() - 6 ) / 3.0 * master_output - 1;
 		Uint16 x_base = 2;
@@ -144,12 +140,13 @@ void visualizationWidget::paintEvent( QPaintEvent * )
 
 		float max_level = 0.0;
 
-		const Uint32 frames = mixer::inst()->framesPerAudioBuffer();
+		const fpab_t frames = eng()->getMixer()->framesPerAudioBuffer();
 
 		// analyse wave-stream for max-level
-		for( Uint32 frame = 0; frame < frames; ++frame )
+		for( fpab_t frame = 0; frame < frames; ++frame )
 		{
-			for( Uint8 chnl = 0; chnl < SURROUND_CHANNELS; ++chnl )
+			for( ch_cnt_t chnl = 0; chnl < SURROUND_CHANNELS;
+									++chnl )
 			{
 				if( tAbs<float>( m_buffer[frame][chnl] ) >
 								max_level )
@@ -174,7 +171,7 @@ void visualizationWidget::paintEvent( QPaintEvent * )
 		}
 
 		// now draw all that stuff
-		for( Uint32 frame = 0; frame < frames; ++frame )
+		for( fpab_t frame = 0; frame < frames; ++frame )
 		{
 			for( Uint8 chnl = 0; chnl < DEFAULT_CHANNELS; ++chnl )
 			{
@@ -203,7 +200,7 @@ void visualizationWidget::mousePressEvent( QMouseEvent * _me )
 	if( _me->button() == Qt::LeftButton )
 	{
 		m_enabled = !m_enabled;
-		if( m_enabled )
+		if( m_enabled == TRUE )
 		{
 			m_updateTimer->start( UPDATE_TIME );
 		}
@@ -219,5 +216,7 @@ void visualizationWidget::mousePressEvent( QMouseEvent * _me )
 
 
 
-#include <visualization_widget.moc>
+#include "visualization_widget.moc"
 
+
+#endif
